// BergFrame.java, used for
// JDubiel.java: computer program for pc-set analysis
// by Akira Takaoka, akira.takaoka@columbia.edu
// Copyright (C) 1994-2021 Akira Takaoka
// All rights reserved.



// package jdubielApplication;

import java.lang.reflect.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;



public class BergFrame extends JFrame implements ActionListener {

    static final int xOffset = 450, yOffset = 170;
    static int openFrameCount = 0;

    JPanel bergPanel;
    JPanel buttonPanel;
    JScrollPane bergScrollPane;

    JComboBox<String> pieceJComboBox;
    JTextField toneRowTextField;

    JComboBox<String> oderUnorderJComboBox;
    JTextField pcSetTextField;

    JEditorPane editorPane;
    JScrollPane editorScrollPane;

    JCheckBox tnCheckBox;
    JCheckBox tnICheckBox;
    JCheckBox mnCheckBox;
    JCheckBox rkTnCheckBox;
    JCheckBox rkTnICheckBox;
    JCheckBox RTnCheckBox;
    JCheckBox RTnICheckBox;
    JCheckBox rkRTnCheckBox;
    JCheckBox rkRTnICheckBox;

    JCheckBox primeFormsCheckBox;
    JCheckBox matrixCheckBox;

    StringBuffer results;






    public BergFrame() {

	//...Create the GUI and put it in the window...
	createWidgets();

	//Put everything together.
	JPanel devidedPane = new JPanel(new BorderLayout());
	devidedPane.add(bergPanel, BorderLayout.NORTH);
	devidedPane.add(buttonPanel, BorderLayout.WEST);
	devidedPane.add(bergScrollPane, BorderLayout.CENTER);
	devidedPane.add(editorScrollPane, BorderLayout.SOUTH);

	getContentPane().add(devidedPane, BorderLayout.CENTER);

	setSize(440, 620);
	openFrameCount++;
	setTitle("Berg's Twelve-Tone Compositions " + openFrameCount);
	setLocation(xOffset + 20 * openFrameCount,
		    yOffset + 20 * openFrameCount);
    }
	



    // Create and add the widgets.
    private void createWidgets() {

	JLabel piecesLabel = new JLabel("Choose a piece: ", 
					SwingConstants.RIGHT);

	String[] pieceStrings = {
	    " Chamber Concerto (1925), Movements 1 and 3 ",
	    " Chamber Concerto (1925), Movements 2 and 3 (1) ",
	    " Chamber Concerto (1925), Movements 2 and 3 (2) ",
	    " Wozzeck (1914-1922), Act 1, Scene 4, \"Passacaglia\" ",
	    " Lyric Suite (1925-1926), Movement 1, also used for \"Schliesse mir die Augen beide\" (1925) ",
	    " Lyric Suite (1925-1926), Movement 3 ",
	    " Lyric Suite (1925-1926), Movements 5 and 6 (1) ",
	    " Lyric Suite (1925-1926), Movements 5 and 6 (2) ",
	    " Der Wein (1929) ",
	    " Lulu (1927-1935), main row ",
	    " Lulu (1927-1935), Lulu's row ",
	    " Lulu (1927-1935), whole-tone row ",
	    " Lulu (1927-1935), main row with intertwined middle tetrachord ",
	    " Lulu (1927-1935), Schigolch row ",
	    " Lulu (1927-1935), permutation of main row (1) ",
	    " Lulu (1927-1935), permutation of main row (2) ",
	    " Lulu (1927-1935), Alwa's row (every 7th note of transposed main row) ",
	    " Lulu (1927-1935), Countess' row ",
	    " Lulu (1927-1935), Acrobat's row (alternate notes of transposed main row) ",
	    " Lulu (1927-1935), Schoolboy's row (every 3rd note of transposed main row) ",
	    " Lulu (1927-1935), Dr Schoen's row ",
	    " Lulu (1927-1935), Chorale row ",
	    " Concerto for Violin and Orchestra (1935) ",
	};


	pieceJComboBox = new JComboBox<>(pieceStrings);
	pieceJComboBox.setSelectedIndex(9);
	pieceJComboBox.setBackground(Color.white);
	pieceJComboBox.setFont(new Font("SansSerif", Font.ITALIC, 14));
	pieceJComboBox.setActionCommand("piece");
	pieceJComboBox.addActionListener(this);
	piecesLabel.setLabelFor(pieceJComboBox);

	JLabel toneRowLabel = new JLabel("Its tone row: ", 
					 SwingConstants.RIGHT);
	toneRowLabel.setToolTipText("Insert a space(s) to segment the "
				    + "tone row. Ex. 037B25 91468A ");
	toneRowTextField = new JTextField(12);
	toneRowTextField.setBackground(Color.white);
	toneRowLabel.setLabelFor(toneRowTextField);
	toneRowTextField.setText(" 037B2591468A ");
	toneRowTextField.setToolTipText("Insert a space(s) to segment the "
					+ "tone row. Ex. 037B25 91468A ");
	toneRowTextField.setDragEnabled(true);

	//Add the widgets to the container.
	bergPanel = new JPanel(new GridLayout(2, 2));

	bergPanel.add(piecesLabel);
	bergPanel.add(pieceJComboBox);
	bergPanel.add(toneRowLabel);
	bergPanel.add(toneRowTextField);
	bergPanel.setBorder(BorderFactory.createCompoundBorder(
                                BorderFactory.createTitledBorder(
					"Berg's Twelve-Tone composition: "),
                                BorderFactory.createEmptyBorder(5,5,5,5)));







	// Find button
	JButton findButton = new JButton("Find");
	findButton.setActionCommand("find");
	findButton.addActionListener(this);
	findButton.setBackground(Color.lightGray);
	findButton.setFont(new Font("SansSerif", Font.BOLD, 12));

	// Clear button
	JButton clearButton = new JButton("Clear");
	clearButton.setActionCommand("clear");
	clearButton.addActionListener(this);
	clearButton.setBackground(Color.lightGray);

	JPanel buttonPanel1 = new JPanel(new SpringLayout());
	buttonPanel1.add(findButton);
	buttonPanel1.add(clearButton);
	SpringUtilities.makeCompactGrid(buttonPanel1,
					2, 1,  //rows, cols
					1, 1,  //initX, initY
					1, 1); //xPad, yPad

	buttonPanel = new JPanel(new BorderLayout());
	buttonPanel.add(buttonPanel1, BorderLayout.NORTH);







		String[] oderUnorderStrings = {"Ordered PC-Set:",
					       "Unordered PC-Set:"};
		oderUnorderJComboBox = new JComboBox<>(oderUnorderStrings);
		oderUnorderJComboBox.setSelectedIndex(0);
		oderUnorderJComboBox.setAlignmentY(RIGHT_ALIGNMENT);
		oderUnorderJComboBox.setBackground(Color.white);
		pcSetTextField = new JTextField(10);
		pcSetTextField.setBackground(Color.white);
		pcSetTextField.setDragEnabled(true);

		JPanel pcSetPanel = new JPanel(new SpringLayout());
		pcSetPanel.add(oderUnorderJComboBox);
		pcSetPanel.add(pcSetTextField);
		SpringUtilities.makeCompactGrid(pcSetPanel,
						1, 2,  //rows, cols
						1, 1,  //initX, initY
						1, 1); //xPad, yPad
//		pcSetPanel.setBorder(
//			BorderFactory.createCompoundBorder(
//                                BorderFactory.createTitledBorder(""),
//                                BorderFactory.createEmptyBorder(5,5,5,5)));





		// Check buttons on a panel.
		tnCheckBox = new JCheckBox(" T ");
		tnCheckBox.setToolTipText(
			"Pn: Transposition by n");
		tnCheckBox.setActionCommand("tn");
		tnCheckBox.addActionListener(this);

		tnICheckBox = new JCheckBox(" TI ");
		tnICheckBox.setToolTipText(
			"In: inversion at n");
		tnICheckBox.setActionCommand("tnI");
		tnICheckBox.addActionListener(this);

		mnCheckBox = new JCheckBox(" M ");
		mnCheckBox.setToolTipText(
			"Mn: multiplication by n");
		mnCheckBox.setActionCommand("mn");
		mnCheckBox.addActionListener(this);

		RTnCheckBox = new JCheckBox(" RT "); 
		RTnCheckBox.setToolTipText(
			"Rn: retrograde");
		RTnCheckBox.setActionCommand("RTn");
		RTnCheckBox.addActionListener(this);

		// RIn (retrograde inversion)
		RTnICheckBox = new JCheckBox(" RTI "); 
		RTnICheckBox.setToolTipText(
			"RIn: retrograde inversion");
		RTnICheckBox.setActionCommand("RTnI");
		RTnICheckBox.addActionListener(this);

		JPanel checkBoxesPanel1 = new JPanel(new GridLayout(1, 5));
		checkBoxesPanel1.add(tnCheckBox);
		checkBoxesPanel1.add(tnICheckBox);
		checkBoxesPanel1.add(mnCheckBox);
		checkBoxesPanel1.add(RTnCheckBox);
		checkBoxesPanel1.add(RTnICheckBox);

		checkBoxesPanel1.setBorder(
			BorderFactory.createCompoundBorder(
                                BorderFactory.createTitledBorder(
					"in the tone row under: "),
                                BorderFactory.createEmptyBorder(5,5,5,5)));






	primeFormsCheckBox = new JCheckBox("Tn/TnI types of subsequences"); 
	primeFormsCheckBox.setToolTipText(
		"Shows the names of the Tn/TnI types of subsequences of the tone row.");
	primeFormsCheckBox.setActionCommand("primeForms");
	primeFormsCheckBox.addActionListener(this);





		matrixCheckBox = new JCheckBox("Matrix of the tone row"); 
		matrixCheckBox.setToolTipText(
			"Shows the forty-eight forms of the tone row.");
		matrixCheckBox.setActionCommand("matrix");
		matrixCheckBox.addActionListener(this);



		//Lay out the panel.
		JPanel checkBoxesPanel = new JPanel(new SpringLayout());
		checkBoxesPanel.add(pcSetPanel);
		checkBoxesPanel.add(checkBoxesPanel1);
		checkBoxesPanel.add(primeFormsCheckBox);
		checkBoxesPanel.add(matrixCheckBox);
		SpringUtilities.makeCompactGrid(checkBoxesPanel,
						4, 1,  //rows, cols
						1, 1,  //initX, initY
						1, 1); //xPad, yPad
		checkBoxesPanel.setBorder(
			BorderFactory.createCompoundBorder(
                                BorderFactory.createTitledBorder(
					""),
                                BorderFactory.createEmptyBorder(5,5,5,5)));


	bergScrollPane = new JScrollPane(checkBoxesPanel);
	bergScrollPane.setVerticalScrollBarPolicy(
		JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
	bergScrollPane.setHorizontalScrollBarPolicy(
		JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
	bergScrollPane.setPreferredSize(new Dimension(390, 200));
	bergScrollPane.setMinimumSize(new Dimension(390, 200));









		// "Found Relations" area
		editorPane = new JEditorPane();
		editorScrollPane = new JScrollPane(editorPane);
		editorScrollPane.setVerticalScrollBarPolicy(
                        JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		editorScrollPane.setHorizontalScrollBarPolicy(
			JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		editorScrollPane.setPreferredSize(new Dimension(250, 350));
		editorScrollPane.setMinimumSize(new Dimension(250, 350));
		editorScrollPane.setBorder(
			BorderFactory.createCompoundBorder(
				BorderFactory.createTitledBorder(
					"Found: "),
				BorderFactory.createEmptyBorder(5,5,5,5)));
		editorScrollPane.getBorder();
	}









    public int[] initSeries(String piece) { // Initialize a 12-tone row.

	// Berg's twelve-tone rows
	if(piece.equals(" Chamber Concerto (1925), Movements 1 and 3 ")) {
	    int [] toneRow01 = {5, 6, 8, 1, 9, 2, 3, 0, 11, 10, 4, 7};
	    return toneRow01;
 	} else if(piece.equals(" Chamber Concerto (1925), Movements 2 and 3 (1) ")) {
	    int [] toneRow02 = {11, 7, 5, 9, 2, 3, 6, 8, 0, 1, 4, 10};
	    return toneRow02;
 	} else if(piece.equals(" Chamber Concerto (1925), Movements 2 and 3 (2) ")) {
	    int [] toneRow03 = {9, 8, 7, 1, 2, 6, 4, 10, 11, 3, 5, 0};
	    return toneRow03;
 	} else if(piece.equals(" Wozzeck (1914-1922), Act 1, Scene 4, \"Passacaglia\" ")) {
	    int [] toneRow04 = {3, 11, 7, 1, 0, 6, 4, 10, 9, 5, 8, 2};
	    return toneRow04;
 	} else if(piece.equals(" Lyric Suite (1925-1926), Movement 1, also used for \"Schliesse mir die Augen beide\" (1925) ")) {
	    int [] toneRow05 = {5, 4, 0, 9, 7, 2, 8, 1, 3, 6, 10, 11};
	    return toneRow05;
 	} else if(piece.equals(" Lyric Suite (1925-1926), Movement 3 ")) {
	    int [] toneRow06 = {5, 4, 0, 6, 7, 2, 8, 1, 3, 9, 10, 11};
	    return toneRow06;
 	} else if(piece.equals(" Lyric Suite (1925-1926), Movements 5 and 6 (1) ")) {
	    int [] toneRow07 = {5, 4, 0, 6, 9, 1, 8, 2, 3, 7, 10, 11};
	    return toneRow07;
 	} else if(piece.equals(" Lyric Suite (1925-1926), Movements 5 and 6 (2) ")) {
	    int [] toneRow08 = {5, 6, 9, 8, 10, 11, 4, 0, 1, 2, 3, 7};
	    return toneRow08;
 	} else if(piece.equals(" Der Wein (1929) ")) {
	    int [] toneRow09 = {2, 4, 5, 7, 9, 10, 1, 6, 8, 0, 11, 3};
	    return toneRow09;
 	} else if(piece.equals(" Lulu (1927-1935), main row ")) {
	    int [] toneRow10 = {0, 4, 5, 2, 7, 9, 6, 8, 11, 10, 3, 1};
	    return toneRow10;
 	} else if(piece.equals(" Lulu (1927-1935), Lulu's row ")) {
	    int [] toneRow11 = {5, 7, 8, 10, 0, 2, 6, 3, 4, 9, 11, 1};
	    return toneRow11;
 	} else if(piece.equals(" Lulu (1927-1935), whole-tone row ")) {
	    int [] toneRow12 = {5, 7, 9, 11, 3, 1, 4, 0, 2, 6, 8, 10};
	    return toneRow12;
 	} else if(piece.equals(" Lulu (1927-1935), main row with intertwined middle tetrachord ")) {
	    int [] toneRow13 = {0, 4, 5, 2, 6, 7, 8, 9, 11, 10, 3, 1};
	    return toneRow13;
 	} else if(piece.equals(" Lulu (1927-1935), Schigolch row ")) {
	    int [] toneRow14 = {0, 2, 3, 1, 4, 5, 6, 7, 8, 9, 11, 10};
	    return toneRow14;
 	} else if(piece.equals(" Lulu (1927-1935), permutation of main row (1) ")) {
	    int [] toneRow15 = {0, 4, 2, 7, 9, 6, 8, 11, 3, 1, 5, 10};
	    return toneRow15;
 	} else if(piece.equals(" Lulu (1927-1935), permutation of main row (2) ")) {
	    int [] toneRow16 = {0, 5, 2, 7, 9, 6, 8, 10, 3, 1, 4, 11};
	    return toneRow16;
 	} else if(piece.equals(" Lulu (1927-1935), Alwa's row (every 7th note of transposed main row) ")) {
	    int [] toneRow17 = {4, 0, 9, 2, 11, 5, 10, 8, 3, 6, 7, 1};
	    return toneRow17;
 	} else if(piece.equals(" Lulu (1927-1935), Countess' row ")) {
	    int [] toneRow18 = {4, 9, 6, 2, 5, 11, 0, 10, 3, 8, 1, 7};
	    return toneRow18;
 	} else if(piece.equals(" Lulu (1927-1935), Acrobat's row (alternate notes of transposed main row) ")) {
	    int [] toneRow19 = {4, 9, 11, 10, 3, 7, 8, 6, 1, 0, 2, 5};
	    return toneRow19;
 	} else if(piece.equals(" Lulu (1927-1935), Schoolboy's row (every 3rd note of transposed main row) ")) {
	    int [] toneRow20 = {2, 4, 8, 0, 6, 9, 10, 5, 7, 11, 1, 3};
	    return toneRow20;
 	} else if(piece.equals(" Lulu (1927-1935), Dr Schoen's row ")) {
	    int [] toneRow21 = {4, 9, 1, 2, 8, 11, 10, 3, 5, 6, 0, 7};
	    return toneRow21;
 	} else if(piece.equals(" Lulu (1927-1935), Chorale row ")) {
	    int [] toneRow22 = {5, 10, 6, 7, 8, 2, 1, 3, 9, 4, 11, 0};
	    return toneRow22;
 	} else if(piece.equals(" Concerto for Violin and Orchestra (1935) ")) {
	    int [] toneRow23 = {7, 10, 2, 6, 9, 0, 4, 8, 11, 1, 3, 5};
	    return toneRow23;
	} else {
	    int [] toneRow24 = {};
	    return toneRow24;
	}

/*
	// Berg's twelve-tone rows
	if(piece.equals(" CHAMBER CONCERTO ")) {
	    int [] toneRow01 = {0, 8, 6, 10, 3, 4, 7, 9, 1, 2, 5, 11};
	    return toneRow01;
 	} else if(piece.equals(" WOZZECK, ACT I, SCENE 4 \"PASSACAGLIA\" ")) {
	    int [] toneRow02 = {0, 8, 4, 10, 9, 3, 1, 7, 6, 2, 5, 11};
	    return toneRow02;
 	} else if(piece.equals(" LYRIC SUITE PRIMARY ROW ")) {
	    int [] toneRow03 = {0, 11, 7, 4, 2, 9, 3, 8, 19, 5, 6};
	    return toneRow03;
 	} else if(piece.equals(" LYRIC SUITE, LAST MVT. PERMUTATION ")) {
	    int [] toneRow04 = {0, 1, 5, 11, 8, 4, 9, 3, 2, 10, 7, 6};
	    return toneRow04;
 	} else if(piece.equals(" DER WEIN ")) {
	    int [] toneRow05 = {0, 2, 3, 5, 7, 8, 11, 4, 6, 10, 9, 1};
	    return toneRow05;
 	} else if(piece.equals(" LULU: PRIMARY ROW ")) {
	    int [] toneRow06 = {0, 4, 5, 2, 7, 9, 6, 8, 11, 10, 3, 1};
	    return toneRow06;
 	} else if(piece.equals(" LULU, ACT I , SCENE XX -- PERM. (EVERY 7TH NOTE OF PRIMARY ROW) ")) {
	    int [] toneRow07 = {0, 8, 5, 10, 7, 1, 6, 4, 11, 2, 3, 9};
	    return toneRow07;
 	} else if(piece.equals(" LULU, ACT II, SCENE 1 -- PERM. (EVERY 5th NOTE OF PRIMARY ROW) ")) {
	    int [] toneRow08 = {0, 9, 3, 2, 11, 4, 6, 1, 7, 10, 5, 8};
	    return toneRow08;
 	} else if(piece.equals(" CONCERTO FOR VIOLIN AND ORCHESTRA ")) {
	    int [] toneRow09 = {0, 3, 7, 11, 2, 5, 9, 1, 4, 6, 8, 10};
	    return toneRow09;
	} else {
	    int [] toneRow10 = {};
	    return toneRow10;
	}
*/

    }
    // The end of initSeries()




    public void actionPerformed(ActionEvent e) {

	if("piece".equals(e.getActionCommand())) {
	    Set s = new Set();
	    toneRowTextField.setText(" " 
	     + s.pcSetArrayToString(initSeries((String) pieceJComboBox.getSelectedItem())));
	}


	if("find".equals(e.getActionCommand())) {
	    editorPane.setText("");		

	    results = new StringBuffer();
	    PcSet pq = new PcSet();

	    // Get a twelve-tone row and a typed-in pc set and
	    // remove space(s) at the beginning and the end of the row.
	    String typedToneRowString 
		= pq.removeFistLastSpaces(toneRowTextField.getText());

	    // Remove consecutive spaces.
	    String noConsecutiveSpacesString 
		= pq.removeConsecutiveSpaces(typedToneRowString);

	    // Eliminate duplicates
	    String duplicatesFreeToneRowString 
		= pq.removeDuplicatesInString(noConsecutiveSpacesString);

	    // Check if there are non-numerical characters
	    String illegalChars 
		= pq.checkNonNumerals(duplicatesFreeToneRowString);



	    // Puts each subsequence of the segmented tone row in an array.
	    char[] toneRowChars 
		= new char[duplicatesFreeToneRowString.length()];
	    duplicatesFreeToneRowString.getChars(
				 0, 
				 duplicatesFreeToneRowString.length(), 
				 toneRowChars, 
				 0);

	    char[][] tmpSegmentsCharArray = new char[24][24];
	    int[] sizeOfEachSegment = new int[24];
	    int numberOfSegments = 0;

	    for(int i = 0; i < toneRowChars.length; i++) {
		while(toneRowChars[i] == ' ' && i < toneRowChars.length)
		    i++;

		sizeOfEachSegment[numberOfSegments] = 0;
		while(toneRowChars[i] != ' ') {
   tmpSegmentsCharArray[numberOfSegments][sizeOfEachSegment[numberOfSegments]] 
       = toneRowChars[i];
		    sizeOfEachSegment[numberOfSegments]++;
		    i++;

		    if(i > toneRowChars.length - 1)
			break;
		}
		numberOfSegments++;
	    }

	    char[][] rowSegmentsCharArray = new char[numberOfSegments][];
	    for (int i = 0; i < rowSegmentsCharArray.length; i++) {
		rowSegmentsCharArray[i] = new char[sizeOfEachSegment[i]];
		System.arraycopy(tmpSegmentsCharArray[i], 
				 0, 
				 rowSegmentsCharArray[i], 
				 0, 
				 sizeOfEachSegment[i]);
	    }


			
	    // Get typed-in pc-set.
	    String readPcSetString = pcSetTextField.getText();

	    // Eliminate spaces
	    StringBuffer withoutSpaceStringBuffer = new StringBuffer();
	    for (int i = 0; i < readPcSetString.length(); i++)
		if (readPcSetString.charAt(i) != ' ')
		    withoutSpaceStringBuffer.append(readPcSetString.charAt(i));

	    // Eliminate duplicates
	    String pcSet2String = pq.removeDuplicatesInString(
				     withoutSpaceStringBuffer.toString());

	    // Check if there are non-numerical characters
	    String pcSetIllegalChars
		= pq.checkNonNumerals(pcSet2String);



	    // Check errors.
	    boolean goAhead = true;

	    // Check if the size of each segmented subsequence is at least 3.
	    for (int iiii = 0; iiii < numberOfSegments; iiii++)
		if (sizeOfEachSegment[iiii] < 3) {
		    JOptionPane.showMessageDialog(
			bergPanel, 
			"The size of each subsequence must be at least 3.",
			"Warning",
			JOptionPane.WARNING_MESSAGE);
		goAhead = false;
		}

	    // Any operations chosen?
	    if (tnCheckBox.getSelectedObjects() == null &&
		tnICheckBox.getSelectedObjects() == null &&
		mnCheckBox.getSelectedObjects() == null &&
		RTnCheckBox.getSelectedObjects() == null &&
		RTnICheckBox.getSelectedObjects() == null &&
		primeFormsCheckBox.getSelectedObjects() == null &&
		matrixCheckBox.getSelectedObjects() == null) {
		JOptionPane.showMessageDialog(
				bergPanel,
				"You need to choose at least "
				+ "one operation.\n",
				"Warning",
				JOptionPane.WARNING_MESSAGE);
		goAhead = false;
	    } else if (pcSet2String.length() < 2
		       && (tnCheckBox.getSelectedObjects() != null
			   || tnICheckBox.getSelectedObjects() != null
			   || mnCheckBox.getSelectedObjects() != null
			   || RTnCheckBox.getSelectedObjects() != null
			   || RTnICheckBox.getSelectedObjects() != null)) {
		JOptionPane.showMessageDialog(
			bergPanel, 
			"The size of the pc-set must be at least 2.",
			"Warning",
			JOptionPane.WARNING_MESSAGE);
		goAhead = false;
	    } else if (illegalChars.length() > 1) {
		JOptionPane.showMessageDialog(
		      bergPanel, 
	      "Non-numerical characters are found in the twelve-tone row:"
		      + illegalChars.toString(),
		      "Warning",
		      JOptionPane.WARNING_MESSAGE);
		goAhead = false;
	    } else if (pq.numberOfNonSpaces(duplicatesFreeToneRowString) 
		      != pq.numberOfNonSpaces(noConsecutiveSpacesString)) {
		    JOptionPane.showMessageDialog(
				  bergPanel, 
				"There are duplicates in the tone row.",
				"Warning",
				JOptionPane.WARNING_MESSAGE);
		goAhead = false;
	    } else if (pq.numberOfNonSpaces(duplicatesFreeToneRowString) 
		       != 12) {
		JOptionPane.showMessageDialog(
			bergPanel, 
			"The size of the tone row is not 12.",
			"Warning",
			JOptionPane.WARNING_MESSAGE);
	    } else if (pcSetIllegalChars.length() > 1) {
		JOptionPane.showMessageDialog(
		      bergPanel,
		      "Non-numerical characters are found:"
		      + pcSetIllegalChars.toString(),
		      "Warning",
		      JOptionPane.WARNING_MESSAGE);
		goAhead = false;
	    } else if(pcSet2String.length() 
			  != withoutSpaceStringBuffer.length()) {
		    JOptionPane.showMessageDialog(
				  bergPanel, 
				"There are duplicates in the pc-set.",
				"Warning",
				JOptionPane.WARNING_MESSAGE);
		goAhead = false;
	    } else {
		if (goAhead == false)
		    goAhead = false;
		else
		    goAhead = true;
	    }


	    
	    // If no error is found, the following will be carried out.
	    if (goAhead == true) {

		PcSet [] rowSegmentsPcSets = new PcSet[numberOfSegments];

		for (int i = 0; i < rowSegmentsPcSets.length; i++) {
		    rowSegmentsPcSets[i] = new PcSet(sizeOfEachSegment[i]);
		    rowSegmentsPcSets[i].set 
	    = rowSegmentsPcSets[i].charArrayToIntOne(rowSegmentsCharArray[i]);
		}


		// Gets an ordered or unordered PC-Set 2.
		char[] pcSet2Chars = new char[pcSet2String.length()];
		pcSet2String.getChars(0, 
				      pcSet2String.length(), 
				      pcSet2Chars, 
				      0);
		PcSet pcSet2 = new PcSet(pcSet2Chars.length);
		pcSet2.set = pcSet2.charArrayToIntOne(pcSet2Chars);


		if (tnCheckBox.getSelectedObjects() != null ||
		    tnICheckBox.getSelectedObjects() != null ||
		    mnCheckBox.getSelectedObjects() != null ||
		    RTnCheckBox.getSelectedObjects() != null ||
		    RTnICheckBox.getSelectedObjects() != null) {

		    // Is PC-Set 2 ordered or unordered?
		    String ordered 
			= (String)oderUnorderJComboBox.getSelectedItem();

		    if(ordered.equals("Ordered PC-Set:")) {
			OrderedInclusion[] inclusion 
			    = new OrderedInclusion[numberOfSegments];
		
			for(int ii = 0; ii < numberOfSegments; ii++)
			    inclusion[ii] 
				= new OrderedInclusion(rowSegmentsPcSets[ii],
						       pcSet2);


			if (tnCheckBox.getSelectedObjects() != null) {
			    int noRelations = 0;
			    int someRelations = 0;
			    for(int ii = 0; ii < inclusion.length; ii++) {
				String nru = inclusion[ii].listFitTns();
				if (nru.equals("noRelationsUnderT"))
				    noRelations++;
				else {
				    results.append(nru);
				    someRelations++;
				}
			    }
			    results.append("\n");
			    if (noRelations != 0 && someRelations == 0)
				results.append(
				       "No relations under T.\n\n");
			}


			if (tnICheckBox.getSelectedObjects() != null) {
			    int noRelations = 0;
			    int someRelations = 0;
			    for(int ii = 0; ii < inclusion.length; ii++) {
				String nru = inclusion[ii].listFitTnIs();
				if (nru.equals("noRelationsUnderTI"))
				    noRelations++;
				else {
				    results.append(nru);
				    someRelations++;
				}
			    }
			    results.append("\n");
			    if (noRelations != 0 && someRelations == 0)
				results.append(
				       "No relations under TI.\n\n");
			}
			    

			if (mnCheckBox.getSelectedObjects() != null) {
			    int noRelations = 0;
			    int someRelations = 0;
			    for(int ii = 0; ii < inclusion.length; ii++) {
				String nru = inclusion[ii].listFitMns();
				if (nru.equals("noRelationsUnderM"))
				    noRelations++;
				else {
				    results.append(nru);
				    someRelations++;
				}
			    }
			    results.append("\n");
			    if (noRelations != 0 && someRelations == 0)
				results.append(
				       "No relations under M.\n\n");
			}
		    

			if (RTnCheckBox.getSelectedObjects() != null) {
			    int noRelations = 0;
			    int someRelations = 0;
			    for(int ii = 0; ii < inclusion.length; ii++) {
				String nru = inclusion[ii].listFitRTs();
				if (nru.equals("noRelationsUnderRT"))
				    noRelations++;
				else {
				    results.append(nru);
				    someRelations++;
				}
			    }
			    results.append("\n");
			    if (noRelations != 0 && someRelations == 0)
				results.append(
				       "No relations under RT.\n\n");
			}


			if (RTnICheckBox.getSelectedObjects() != null) {
			    int noRelations = 0;
			    int someRelations = 0;
			    for(int ii = 0; ii < inclusion.length; ii++) {
				String nru = inclusion[ii].listFitRTnIs();
				if (nru.equals("noRelationsUnderRTI"))
				    noRelations++;
				else {
				    results.append(nru);
				    someRelations++;
				}
			    }
			    results.append("\n");
			    if (noRelations != 0 && someRelations == 0)
				results.append(
				       "No relations under RTI.\n\n");
			}

		    } else {
			if(rowSegmentsPcSets.length == 1)
			    JOptionPane.showMessageDialog(
				bergPanel,
				"You need to segment the tone row\n by inserting a space(s)\n",
				"Warning",
				JOptionPane.WARNING_MESSAGE);
			else {
			    UnorderedInclusion[] inclusion 
				= new UnorderedInclusion[numberOfSegments];
		    
			    for(int ii = 0; ii < numberOfSegments; ii++)
				inclusion[ii] 
				    = new UnorderedInclusion(
						     rowSegmentsPcSets[ii],
						     pcSet2);

			    if (tnCheckBox.getSelectedObjects() != null)
				for(int i = 0; i < inclusion.length; i++)
				    results.append(inclusion[i].listFitTns());
			    
			    if (tnICheckBox.getSelectedObjects() != null)
				for(int i = 0; i < inclusion.length; i++)
				    results.append(inclusion[i].listFitTnIs());

			    if (mnCheckBox.getSelectedObjects() != null)
				for(int i = 0; i < inclusion.length; i++)
				    results.append(inclusion[i].listFitMns());
			}
			if (RTnCheckBox.getSelectedObjects() != null ||
			    RTnICheckBox.getSelectedObjects() != null)
			    JOptionPane.showMessageDialog(
				  bergPanel,
				"Retrograde can not be "
				+ "applied to an unordered pc-set.\n",
				"Warning",
				JOptionPane.WARNING_MESSAGE);
		    }
		}
	




		// Chooses operations.
		if (primeFormsCheckBox.getSelectedObjects() != null) {

		    if (rowSegmentsPcSets.length < 2)
			JOptionPane.showMessageDialog(
			bergPanel,
			"Insert a space(s) in the twelve-tone row to make subsequences.\n",
			"Warning",
			JOptionPane.WARNING_MESSAGE);
		else {
		    TnTnIType[] subSequences 
			= new TnTnIType[rowSegmentsPcSets.length];
		    Set[] prime = new Set[rowSegmentsPcSets.length];
		    for(int i = 0; i < rowSegmentsPcSets.length; i++) {
			subSequences[i] 
			    = new TnTnIType(rowSegmentsPcSets[i].set);
			subSequences[i].base12String 
			    = rowSegmentsPcSets[i].base12String;
			subSequences[i].getTnType();
			subSequences[i].getTnIType();
			subSequences[i].getTnTnIType();
			prime[i] = subSequences[i].getPrimeByTnTnI();
		    }

		    results.append("\n");
		    results.append("Subsequence: ");
		    for(int i = 0; i < rowSegmentsPcSets.length; i++) {
			results.append("<");
			results.append(
	       subSequences[i].pcSetArrayToString(rowSegmentsPcSets[i].set));
			results.append("> ");
		    }

		    results.append("\n");
		    results.append("Tn/TnI type:      ");
		    for(int i = 0; i < rowSegmentsPcSets.length; i++) {
			results.append("[");
			results.append(subSequences[i].tntnitypeToString());
			results.append("]   ");
		    }

		    results.append("\n");
		    results.append("I-Vector:          ");
		    for(int i = 0; i < rowSegmentsPcSets.length; i++) {
			results.append("<");
			results.append(prime[i].ivectorToString());
			results.append("> ");
		    }

		    results.append("\n");
		    results.append("Forte Name:             ");
		    for(int i = 0; i < rowSegmentsPcSets.length; i++) {
			results.append(prime[i].name);
			results.append("           ");
		    }

		    results.append("\n");
		}
	    }
		






	    // Chooses operations.
	    if (matrixCheckBox.getSelectedObjects() != null) {

		    results.append("\n");
		    results.append("Matrix: \n\n");
		    Set st = new Set();
		    for(int i = 0; i < rowSegmentsPcSets.length; i++) {
			st.base12String = rowSegmentsPcSets[i].base12String;

			int[][] mtrx = st.matrix(rowSegmentsPcSets[i].set);

			results.append("        ");
			for(int h = 0; h < mtrx.length; h++) {
			    if (10 == (((mtrx[0][h] + 12) - mtrx[0][0]) % 12))
				results.append("IA  ");
			    else if (11
			     == (((mtrx[0][h] + 12) - mtrx[0][0]) % 12))
				results.append("IB  ");
			    else
				results.append("I"
			       + (((mtrx[0][h] + 12) - mtrx[0][0]) % 12)
					       + "  ");
			}
			results.append("\n");
			results.append("        -----------------------------");
			results.append("\n");
			
			for(int ii = 0; ii < mtrx.length; ii++) {
			    if (10 == (((mtrx[ii][0] + 12) - mtrx[0][0]) % 12))
				results.append("PA: ");
			    else if (11
			     == (((mtrx[ii][0] + 12) - mtrx[0][0]) % 12))
				results.append("PB: ");
			    else
				results.append("P"
			       + (((mtrx[ii][0] + 12) - mtrx[0][0]) % 12)
					       + ": ");
			    
			    for(int iii = 0; iii < mtrx.length; iii++) {
				results.append("   ");
				results.append(st.intToChar(mtrx[ii][iii]));
			    }
			    results.append("\n");
			}
			results.append("\n");
		    }
		}
	    }

	    editorPane.setText(results.toString());
	}

	if ("clear".equals(e.getActionCommand())) {
	    pcSetTextField.setText("");
	    editorPane.setText("");		
	}
    }
}


