// InvariantsList.java, used for
// JDubiel.java: computer program for pc-set analysis
// by Akira Takaoka, akira.takaoka@columbia.edu
// Copyright (C) 1994-2017 Akira Takaoka
// All rights reserved.




// package jdubielApplication;

import javax.swing.*;


public class InvariantsList extends PcSet {

    PcSet pcSetInvarList;
    int [] st;
    int min, max;

    // Constructor 1
    public InvariantsList(int [] s, int in, int ax) {
	st = s; 
	min = in;
	max = ax;
    }
    // Constructor 2
    public InvariantsList(Set ss, int in, int ax) {
	pcSetInvarList = new PcSet(ss.set.length);
	System.arraycopy(ss.set, 0, pcSetInvarList.set, 0, ss.set.length);
	min = in;
	max = ax;

	// Characters used for base 12 system, 
	// a & b, A & B, t & e, or T & E
	if (ss.base12String != null) {
	    pcSetInvarList.base12String = new String(ss.base12String);
	    base12String = new String(ss.base12String);
	} else {
	    pcSetInvarList.base12String = new String("AB");
	    base12String = new String("AB");
	}
    }
    // Constructor 3
    public InvariantsList(Set ss, int setSize, int in, int ax) {
	pcSetInvarList = new PcSet(setSize);
	System.arraycopy(ss.set, 0, pcSetInvarList.set, 0, setSize);
	min = in;
	max = ax;

	// Characters used for base 12 system, 
	// a & b, A & B, t & e, or T & E
	if (ss.base12String != null) {
	    pcSetInvarList.base12String = new String(ss.base12String);
	    base12String = new String(ss.base12String);
	} else {
	    pcSetInvarList.base12String = new String("AB");
	    base12String = new String("AB");
	}
    }
    // The end of constructors



    public String unorderedTnInvariants() {

	StringBuffer invariantsList = new StringBuffer();
	int [][] unordrdTns = returnTns(pcSetInvarList.set, min, max);

	invariantsList.append("Invariant subsets under Tk to Tn: \n\n");
	for (int n = min; n <= max; n++) { // n: that of Tn
	    int count = 0;
	    int [] invariants = new int[pcSetInvarList.set.length];
	    for (int j = 0; j < unordrdTns[n - min].length; j++)
		for (int k = 0; k < pcSetInvarList.set.length; k++)
		    if (unordrdTns[n - min][j] == pcSetInvarList.set[k])
			invariants[count++] = pcSetInvarList.set[k];

	    if (count == 0)
		invariantsList.append("There is no subset of {"
				      + pcSetArrayToString(pcSetInvarList.set)
				      + "} under:\n"
				      + "    T" 
				      + n 
				      + ": {"
				      + pcSetArrayToString(pcSetInvarList.set)
				      + "} --> {"
				      + pcSetArrayToString(unordrdTns[n - min])
				      + "}\n\n");
	    else {
		Set s = new Set(invariants);
		s.base12String = pcSetInvarList.base12String;
		char [] charInvariants = s.intArrayToCharOne(s.set);
		invariantsList.append(" {");
		for (int c = 0; c < count; c++)
		    invariantsList.append(charInvariants[c]);

		invariantsList.append("} is a subset of {"
				      + pcSetArrayToString(pcSetInvarList.set)
				      + "} under:\n"
				      + "    T" 
				      + n 
				      + ": {"
				      + pcSetArrayToString(pcSetInvarList.set)
				      + "} --> {"
				      + pcSetArrayToString(unordrdTns[n - min])
				      + "}\n\n");
	    }
	}
	invariantsList.append("\n");
	return invariantsList.toString();
    }






    public String unorderedTnIInvariants() {

	StringBuffer invariantsList = new StringBuffer();
	int [][] unordrdTnIs = returnTnIs(pcSetInvarList.set, min, max);

	invariantsList.append("Invariant subsets under TkI to TnI: \n\n");
	for (int n = min; n <= max; n++) { // n: that of Tn
	    int count = 0;
	    int [] invariants = new int[pcSetInvarList.set.length];
	    for (int j = 0; j < unordrdTnIs[n - min].length; j++)
		for (int k = 0; k < pcSetInvarList.set.length; k++)
		    if (unordrdTnIs[n - min][j] == pcSetInvarList.set[k])
			invariants[count++] = pcSetInvarList.set[k];

	    if (count == 0)
		invariantsList.append("There is no subset of {"
				      + pcSetArrayToString(pcSetInvarList.set)
				      + "} under:\n"
				      + "    T" 
				      + n 
				      + "I: {"
				      + pcSetArrayToString(pcSetInvarList.set)
				      + "} --> {"
				      + pcSetArrayToString(unordrdTnIs[n - min])
				      + "}\n\n");
	    else {
		Set s = new Set(invariants);
		s.base12String = pcSetInvarList.base12String;
		char [] charInvariants = s.intArrayToCharOne(s.set);
		invariantsList.append("{");
		for (int c = 0; c < count; c++)
		    invariantsList.append(charInvariants[c]);

		invariantsList.append("} is a subset of {"
				      + pcSetArrayToString(pcSetInvarList.set)
				      + "} under:\n"
				      + "    T" 
				      + n 
				      + "I: {"
				      + pcSetArrayToString(pcSetInvarList.set)
				      + "} --> {"
				      + pcSetArrayToString(unordrdTnIs[n - min])
				      + "}\n\n");
	    }
	}
	invariantsList.append("\n");
	return invariantsList.toString();
    }






    public String unorderedMnInvariants() {

	StringBuffer invariantsList = new StringBuffer();
	int [][] unordrdMns = returnMns(pcSetInvarList.set, min, max);

	invariantsList.append("Invariant subsets under Mk to Mn: \n\n");
	for (int n = min; n <= max; n++) { // n: that of Tn
	    int count = 0;
	    int [] invariants = new int[pcSetInvarList.set.length];
	    for (int j = 0; j < unordrdMns[n - min].length; j++)
		for (int k = 0; k < pcSetInvarList.set.length; k++)
		    if (unordrdMns[n - min][j] == pcSetInvarList.set[k])
			invariants[count++] = pcSetInvarList.set[k];

	    if (count == 0)
		invariantsList.append("There is no subset of {"
				      + pcSetArrayToString(pcSetInvarList.set)
				      + "} under:\n"
				      + "    M" 
				      + n 
				      + ": {"
				      + pcSetArrayToString(pcSetInvarList.set)
				      + "} --> {"
				      + pcSetArrayToString(unordrdMns[n - 1])
				      + "}\n\n");
	    else {
		int [] iv = checkDuplicates(sortPcSet(invariants));
		Set s = new Set(iv);
		s.base12String = pcSetInvarList.base12String;
		char [] charInvariants = s.intArrayToCharOne(s.set);
		invariantsList.append("{");
		for (int c = 0; c < iv.length; c++)
		     invariantsList.append(charInvariants[c]);

		invariantsList.append("} is a subset of {"
				      + pcSetArrayToString(pcSetInvarList.set)
				      + "} under:\n"
				      + "    M" 
				      + n 
				      + ": {"
				      + pcSetArrayToString(pcSetInvarList.set)
				      + "} --> {"
				      + pcSetArrayToString(unordrdMns[n - 1])
				      + "}\n\n");
	    }
	}
	invariantsList.append("\n");
	return invariantsList.toString();
    }
    // The end of unorderedMnInvariants()









    public String orderedTnInvariants(int sizeMin, int sizeMax) {

	StringBuffer invariantsList = new StringBuffer();
	int noRelations = 0;
	int someRelations = 0;
	invariantsList.append("\nInvariant sequences under T: \n");

	for(int sizeOfSubsequence = sizeMin; 
	    sizeOfSubsequence <= sizeMax; 
	    sizeOfSubsequence++) {

	    int[][] subsequences 
   = new int[pcSetInvarList.set.length-sizeOfSubsequence+1][sizeOfSubsequence];

	    int iii = 0;
	    for(int i = 0; i < subsequences.length; i++) {
		for(int ii = 0; ii < subsequences[i].length; ii++)
		    subsequences[i][ii] = pcSetInvarList.set[ii + iii];

		iii++;
	    }

	    for(int i = 0; i < subsequences.length; i++) {
		OrderedInclusion oi 
		    = new OrderedInclusion(pcSetInvarList, subsequences[i]);

		String nrut = oi.listFitTns();
		if (nrut.equals("noRelationsUnderT"))
		    noRelations++;
		else {
		    invariantsList.append("\n");
		    invariantsList.append(oi.listFitTns());
		    someRelations++;
		}
	    }
	    invariantsList.append("\n");
	}
	if (noRelations != 0 && someRelations == 0)
	    invariantsList.append("    No invariants under T.\n\n");

	return invariantsList.toString();
    }
    // The end of orderedTnInvariants()








    public String orderedTnIInvariants(int sizeMin, int sizeMax) {

	StringBuffer invariantsList = new StringBuffer();
	int noRelations = 0;
	int someRelations = 0;
	invariantsList.append("\nInvariant sequences under TI: \n");

	for(int sizeOfSubsequence = sizeMin; 
	    sizeOfSubsequence <= sizeMax; 
	    sizeOfSubsequence++) {

	    int[][] subsequences 
   = new int[pcSetInvarList.set.length-sizeOfSubsequence+1][sizeOfSubsequence];

	    int iii = 0;
	    for(int i = 0; i < subsequences.length; i++) {
		for(int ii = 0; ii < subsequences[i].length; ii++)
		    subsequences[i][ii] = pcSetInvarList.set[ii + iii];

		iii++;
	    }

	    for(int i = 0; i < subsequences.length; i++) {
		OrderedInclusion oi 
		    = new OrderedInclusion(pcSetInvarList, subsequences[i]);

		String nruti = oi.listFitTnIs();
		if (nruti.equals("noRelationsUnderTI"))
		    noRelations++;
		else {
		    invariantsList.append("\n");
		    invariantsList.append(oi.listFitTnIs());
		    someRelations++;
		}
	    }
	    invariantsList.append("\n");
	}
	if (noRelations != 0 && someRelations == 0)
	    invariantsList.append("    No invariants under TI.\n\n");

	return invariantsList.toString();
    }
    // The end of orderedTnIInvariants()








    public String orderedMnInvariants(int sizeMin, int sizeMax) {

	StringBuffer invariantsList = new StringBuffer();
	int noRelations = 0;
	int someRelations = 0;
	invariantsList.append("\nInvariant sequences under M: \n");

	for(int sizeOfSubsequence = sizeMin; 
	    sizeOfSubsequence <= sizeMax; 
	    sizeOfSubsequence++) {

	    int[][] subsequences 
   = new int[pcSetInvarList.set.length-sizeOfSubsequence+1][sizeOfSubsequence];

	    int iii = 0;
	    for(int i = 0; i < subsequences.length; i++) {
		for(int ii = 0; ii < subsequences[i].length; ii++)
		    subsequences[i][ii] = pcSetInvarList.set[ii + iii];

		iii++;
	    }

	    for(int i = 0; i < subsequences.length; i++) {
		OrderedInclusion oi 
		    = new OrderedInclusion(pcSetInvarList, subsequences[i]);

		String nrum = oi.listFitMns();
		if (nrum.equals("noRelationsUnderM"))
		    noRelations++;
		else {
		    invariantsList.append("\n");
		    invariantsList.append(oi.listFitMns());
		    someRelations++;
		}
	    }
	    invariantsList.append("\n");
	}
	if (noRelations != 0 && someRelations == 0)
	    invariantsList.append("    No invariants under M.\n\n");

	return invariantsList.toString();
    }
    // The end of orderedMnInvariants()







    public String orderedrTnInvariants(int sizeMin, int sizeMax) {

	StringBuffer invariantsList = new StringBuffer();
	int noRelations = 0;
	int someRelations = 0;
	invariantsList.append("\nInvariant sequences under rT: \n");

	for(int sizeOfSubsequence = sizeMin; 
	    sizeOfSubsequence <= sizeMax; 
	    sizeOfSubsequence++) {

	    int[][] subsequences 
   = new int[pcSetInvarList.set.length-sizeOfSubsequence+1][sizeOfSubsequence];

	    int iii = 0;
	    for(int i = 0; i < subsequences.length; i++) {
		for(int ii = 0; ii < subsequences[i].length; ii++)
		    subsequences[i][ii] = pcSetInvarList.set[ii + iii];

		iii++;
	    }

	    for(int i = 0; i < subsequences.length; i++) {
		OrderedInclusion oi 
		    = new OrderedInclusion(pcSetInvarList, subsequences[i]);

		String nrurt = oi.listFitrTns();
		if (nrurt.equals("noRelationsUnderrT"))
		    noRelations++;
		else {
		    invariantsList.append("\n");
		    invariantsList.append(oi.listFitrTns());
		    someRelations++;
		}
	    }
	    invariantsList.append("\n");
	}
	if (noRelations != 0 && someRelations == 0)
	    invariantsList.append("    No invariants under rT.\n\n");

	return invariantsList.toString();
    }
    // The end of orderedrTnInvariants()








    public String orderedrTnIInvariants(int sizeMin, int sizeMax) {

	StringBuffer invariantsList = new StringBuffer();
	int noRelations = 0;
	int someRelations = 0;
	invariantsList.append("\nInvariant sequences under rTI: \n");

	for(int sizeOfSubsequence = sizeMin; 
	    sizeOfSubsequence <= sizeMax; 
	    sizeOfSubsequence++) {

	    int[][] subsequences 
   = new int[pcSetInvarList.set.length-sizeOfSubsequence+1][sizeOfSubsequence];

	    int iii = 0;
	    for(int i = 0; i < subsequences.length; i++) {
		for(int ii = 0; ii < subsequences[i].length; ii++)
		    subsequences[i][ii] = pcSetInvarList.set[ii + iii];

		iii++;
	    }

	    for(int i = 0; i < subsequences.length; i++) {
		OrderedInclusion oi 
		    = new OrderedInclusion(pcSetInvarList, subsequences[i]);

		String nrurti = oi.listFitrTnIs();
		if (nrurti.equals("noRelationsUnderrTI"))
		    noRelations++;
		else {
		    invariantsList.append("\n");
		    invariantsList.append(oi.listFitrTnIs());
		    someRelations++;
		}
	    }
	    invariantsList.append("\n");
	}
	if (noRelations != 0 && someRelations == 0)
	    invariantsList.append("    No invariants under rTI.\n\n");

	return invariantsList.toString();
    }
    // The end of orderedrTnIInvariants()









    public String orderedRTnInvariants(int sizeMin, int sizeMax) {

	StringBuffer invariantsList = new StringBuffer();
	int noRelations = 0;
	int someRelations = 0;
	invariantsList.append("\nInvariant sequences under RT: \n");

	for(int sizeOfSubsequence = sizeMin; 
	    sizeOfSubsequence <= sizeMax; 
	    sizeOfSubsequence++) {

	    int[][] subsequences 
   = new int[pcSetInvarList.set.length-sizeOfSubsequence+1][sizeOfSubsequence];

	    int iii = 0;
	    for(int i = 0; i < subsequences.length; i++) {
		for(int ii = 0; ii < subsequences[i].length; ii++)
		    subsequences[i][ii] = pcSetInvarList.set[ii + iii];

		iii++;
	    }

	    for(int i = 0; i < subsequences.length; i++) {
		OrderedInclusion oi 
		    = new OrderedInclusion(pcSetInvarList, subsequences[i]);

		String nrurt = oi.listFitRTs();
		if (nrurt.equals("noRelationsUnderRT"))
		    noRelations++;
		else {
		    invariantsList.append("\n");
		    invariantsList.append(oi.listFitRTs());
		    someRelations++;
		}
	    }
	    invariantsList.append("\n");
	}
	if (noRelations != 0 && someRelations == 0)
	    invariantsList.append("    No invariants under RT.\n\n");

	return invariantsList.toString();
    }
    // The end of orderedRTnInvariants()








    public String orderedRTnIInvariants(int sizeMin, int sizeMax) {

	StringBuffer invariantsList = new StringBuffer();
	int noRelations = 0;
	int someRelations = 0;
	invariantsList.append("\nInvariant sequences under RTI: \n");

	for(int sizeOfSubsequence = sizeMin; 
	    sizeOfSubsequence <= sizeMax; 
	    sizeOfSubsequence++) {

	    int[][] subsequences 
   = new int[pcSetInvarList.set.length-sizeOfSubsequence+1][sizeOfSubsequence];

	    int iii = 0;
	    for(int i = 0; i < subsequences.length; i++) {
		for(int ii = 0; ii < subsequences[i].length; ii++)
		    subsequences[i][ii] = pcSetInvarList.set[ii + iii];

		iii++;
	    }

	    for(int i = 0; i < subsequences.length; i++) {
		OrderedInclusion oi 
		    = new OrderedInclusion(pcSetInvarList, subsequences[i]);

		String nrurti = oi.listFitRTnIs();
		if (nrurti.equals("noRelationsUnderRTI"))
		    noRelations++;
		else {
		    invariantsList.append("\n");
		    invariantsList.append(oi.listFitRTnIs());
		    someRelations++;
		}
	    }
	    invariantsList.append("\n");
	}
	if (noRelations != 0 && someRelations == 0)
	    invariantsList.append("    No invariants under RTI.\n\n");

	return invariantsList.toString();
    }
    // The end of orderedRTnIInvariants()










    public String orderedrRTnInvariants(int sizeMin, int sizeMax) {

	StringBuffer invariantsList = new StringBuffer();
	int noRelations = 0;
	int someRelations = 0;
	invariantsList.append("\nInvariant sequences under rRT: \n");

	for(int sizeOfSubsequence = sizeMin; 
	    sizeOfSubsequence <= sizeMax; 
	    sizeOfSubsequence++) {

	    int[][] subsequences 
   = new int[pcSetInvarList.set.length-sizeOfSubsequence+1][sizeOfSubsequence];

	    int iii = 0;
	    for(int i = 0; i < subsequences.length; i++) {
		for(int ii = 0; ii < subsequences[i].length; ii++)
		    subsequences[i][ii] = pcSetInvarList.set[ii + iii];

		iii++;
	    }

	    for(int i = 0; i < subsequences.length; i++) {
		OrderedInclusion oi 
		    = new OrderedInclusion(pcSetInvarList, subsequences[i]);

		String nrurrt = oi.listFitrRTs();
		if (nrurrt.equals("noRelationsUnderrRT"))
		    noRelations++;
		else {
		    invariantsList.append("\n");
		    invariantsList.append(oi.listFitrRTs());
		    someRelations++;
		}
	    }
	    invariantsList.append("\n");
	}
	if (noRelations != 0 && someRelations == 0)
	    invariantsList.append("    No invariants under rRT.\n\n");

	return invariantsList.toString();
    }
    // The end of orderedrRTnInvariants()






    public String orderedrRTnIInvariants(int sizeMin, int sizeMax) {

	StringBuffer invariantsList = new StringBuffer();
	int noRelations = 0;
	int someRelations = 0;
	invariantsList.append("\nInvariant sequences under rRTI: \n");

	for(int sizeOfSubsequence = sizeMin; 
	    sizeOfSubsequence <= sizeMax; 
	    sizeOfSubsequence++) {

	    int[][] subsequences 
   = new int[pcSetInvarList.set.length-sizeOfSubsequence+1][sizeOfSubsequence];

	    int iii = 0;
	    for(int i = 0; i < subsequences.length; i++) {
		for(int ii = 0; ii < subsequences[i].length; ii++)
		    subsequences[i][ii] = pcSetInvarList.set[ii + iii];

		iii++;
	    }

	    for(int i = 0; i < subsequences.length; i++) {
		OrderedInclusion oi 
		    = new OrderedInclusion(pcSetInvarList, subsequences[i]);

		String nrurrti = oi.listFitrRTnIs();
		if (nrurrti.equals("noRelationsUnderrRTI"))
		    noRelations++;
		else {
		    invariantsList.append("\n");
		    invariantsList.append(oi.listFitrRTnIs());
		    someRelations++;
		}
	    }
	    invariantsList.append("\n");
	}
	if (noRelations != 0 && someRelations == 0)
	    invariantsList.append("    No invariants under rRTI.\n\n");

	return invariantsList.toString();
    }
    // The end of orderedrRTnIInvariants()

}
