// MenuFrame.java, used for
// JDubiel.java: computer program for pc-set analysis
// by Akira Takaoka, akira.takaoka@columbia.edu
// Copyright (C) 1994-2021 Akira Takaoka
// All rights reserved.





// package jdubielApplication;


import java.lang.reflect.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;



public class MenuFrame extends JFrame 
    implements ActionListener {

    static final int xOffset = 5, yOffset = 25;
    static int openFrameCount = 0;

    JMenuBar menuBar;





    public MenuFrame() {

	createMenuBar();

	//Put everything together.
	JPanel menuPane = new JPanel(new BorderLayout());
	menuPane.add(menuBar);

	getContentPane().add(menuPane, BorderLayout.CENTER);

	setSize(400, 75);
	openFrameCount++;
	setTitle("JDubiel v0.9: Menu " + openFrameCount);
	setLocation(xOffset + 10 * openFrameCount, 
		    yOffset + 2 * openFrameCount);
    }
	




    protected JMenuBar createMenuBar() {
	menuBar = new JMenuBar();
	menuBar.setFont(new Font("SansSerif", Font.BOLD, 14));

	// Sets up the JDubiel menu.
	JMenu jdubielMenu = new JMenu("  JDubiel");
	menuBar.add(jdubielMenu);

	//Set up the first JDubiel menu item.
	JMenuItem aboutMenuItem = new JMenuItem("About");
	aboutMenuItem.setActionCommand("about");
	aboutMenuItem.addActionListener(this);
	jdubielMenu.add(aboutMenuItem);

	//Set up the second JDubiel menu item.
	JMenuItem acknowledgmentsMenuItem 
	    = new JMenuItem("Acknowledgments");
	acknowledgmentsMenuItem.setActionCommand("acknowledgments");
	acknowledgmentsMenuItem.addActionListener(this);
	jdubielMenu.add(acknowledgmentsMenuItem);

	//Set up the third JDubiel menu item.
	JMenuItem helpMenuItem = new JMenuItem("Help");
	helpMenuItem.setActionCommand("help");
	helpMenuItem.addActionListener(this);
	jdubielMenu.add(helpMenuItem);

	//Set up the fourth JDubiel menu item.
	JMenuItem licenseMenuItem = new JMenuItem("License");
	licenseMenuItem.setActionCommand("license");
	licenseMenuItem.addActionListener(this);
	jdubielMenu.add(licenseMenuItem);



	// Sets up the Edit menu
	JMenu editMenu = new JMenu("Edit");
	menuBar.add(editMenu);

	JMenuItem cutMenuItem 
	    = new JMenuItem(new DefaultEditorKit.CutAction());
	cutMenuItem.setText("Cut");
	editMenu.add(cutMenuItem);

	JMenuItem copyMenuItem 
	    = new JMenuItem(new DefaultEditorKit.CopyAction());
	copyMenuItem.setText("Copy");
	editMenu.add(copyMenuItem);

	JMenuItem pasteMenuItem 
	    = new JMenuItem(new DefaultEditorKit.PasteAction());
	pasteMenuItem.setText("Paste");
	editMenu.add(pasteMenuItem);




	//Set up the Unordered PC-Sets menu.
	JMenu unorderedSetsMenu = new JMenu("Unordered");
	unorderedSetsMenu.setToolTipText(
					 "Unordered pc-sets.");
	menuBar.add(unorderedSetsMenu);

	//Set up the Properties unorderedSets menu item.
	JMenuItem unorderedSetPropertiesItem 
	    = new JMenuItem("Properties");
	unorderedSetPropertiesItem.setActionCommand(
						    "unorderedSetProperties");
	unorderedSetPropertiesItem.addActionListener(this);
	unorderedSetsMenu.add(unorderedSetPropertiesItem);

	//Set up the Relations frames menu item.
	JMenuItem unorderedSetRelationsItem 
	    = new JMenuItem("Relations");
	unorderedSetRelationsItem.setActionCommand(
						   "unorderedSetRelations");
	unorderedSetRelationsItem.addActionListener(this);
	unorderedSetsMenu.add(unorderedSetRelationsItem);

	//Set up the Lists frames menu item.
	JMenuItem unorderedSetListsItem 
	    = new JMenuItem("Lists");
	unorderedSetListsItem.setActionCommand(
					       "unorderedSetLists");
	unorderedSetListsItem.addActionListener(this);
	unorderedSetsMenu.add(unorderedSetListsItem);


	//Set up the Tables frame menu item.
	//Tables submenu
	unorderedSetsMenu.addSeparator();
	JMenu tablesSubmenu = new JMenu("Tables of Tn/TnI Types");
	tablesSubmenu.setToolTipText("Tables of set classes");

	JMenuItem trichordMenuItem = new JMenuItem("Trichords");
	trichordMenuItem.setActionCommand("trichords");
	trichordMenuItem.addActionListener(this);
	tablesSubmenu.add(trichordMenuItem);

	JMenuItem tetrachordMenuItem = new JMenuItem("Tetrachords");
	tetrachordMenuItem.setActionCommand("tetrachords");
	tetrachordMenuItem.addActionListener(this);
	tablesSubmenu.add(tetrachordMenuItem);

	JMenuItem pentachordMenuItem = new JMenuItem("Pentachords");
	pentachordMenuItem.setActionCommand("pentachords");
	pentachordMenuItem.addActionListener(this);
	tablesSubmenu.add(pentachordMenuItem);

	JMenuItem hexachordMenuItem = new JMenuItem("Hexachords");
	hexachordMenuItem.setActionCommand("hexachords");
	hexachordMenuItem.addActionListener(this);
	tablesSubmenu.add(hexachordMenuItem);

	JMenuItem septachordMenuItem = new JMenuItem("Septachords");
	septachordMenuItem.setActionCommand("septachords");
	septachordMenuItem.addActionListener(this);
	tablesSubmenu.add(septachordMenuItem);

	JMenuItem octachordMenuItem = new JMenuItem("Octachords");
	octachordMenuItem.setActionCommand("octachords");
	octachordMenuItem.addActionListener(this);
	tablesSubmenu.add(octachordMenuItem);

	JMenuItem nonachordMenuItem = new JMenuItem("Nonachords");
	nonachordMenuItem.setActionCommand("nonachords");
	nonachordMenuItem.addActionListener(this);
	tablesSubmenu.add(nonachordMenuItem);

	unorderedSetsMenu.add(tablesSubmenu);


	//Set up the Ordered PC-Sets menu.
	JMenu orderedSetsMenu = new JMenu("Ordered");
	orderedSetsMenu.setToolTipText(
				       "Ordered pc-sets, sequences, "
				       + "or twelve-tone rows.");
	menuBar.add(orderedSetsMenu);

	//Set up the Relations frame menu item.
	JMenuItem orderedSetRelationsItem = new JMenuItem("Relations");
	orderedSetRelationsItem.setActionCommand(
						 "orderedSetRelations");
	orderedSetRelationsItem.addActionListener(this);
	orderedSetsMenu.add(orderedSetRelationsItem);

	//Set up the Lists frame menu item.
	JMenuItem orderedSetListsItem = new JMenuItem("Lists");
	orderedSetListsItem.setActionCommand("orderedSetLists");
	orderedSetListsItem.addActionListener(this);
	orderedSetsMenu.add(orderedSetListsItem);

	//Set up the Twelve-Tone frame menu item.
	JMenuItem twelveToneItem 
	    = new JMenuItem("Twelve-Tone Operations");
	twelveToneItem.setActionCommand("twelveTone");
	twelveToneItem.addActionListener(this);
	orderedSetsMenu.add(twelveToneItem);


	//Set up the Composers menu.
	JMenu composersMenu = new JMenu("Composers");
	composersMenu.setToolTipText(
			     "Stravinsky, Schoenberg, Webern, and Berg.");
	menuBar.add(composersMenu);

	JMenuItem stravinskyMenuItem = new JMenuItem("Stravinsky");
	stravinskyMenuItem.setActionCommand("stravinsky");
	stravinskyMenuItem.addActionListener(this);
	composersMenu.add(stravinskyMenuItem);

	JMenuItem schoenbergMenuItem = new JMenuItem("Schoenberg");
	schoenbergMenuItem.setActionCommand("schoenberg");
	schoenbergMenuItem.addActionListener(this);
	composersMenu.add(schoenbergMenuItem);

	JMenuItem webernMenuItem = new JMenuItem("Webern");
	webernMenuItem.setActionCommand("webern");
	webernMenuItem.addActionListener(this);
	composersMenu.add(webernMenuItem);

	JMenuItem bergMenuItem = new JMenuItem("Berg");
	bergMenuItem.setActionCommand("berg");
	bergMenuItem.addActionListener(this);
	composersMenu.add(bergMenuItem);


	//Set up the MIDI menu.
	JMenu midiMenu = new JMenu("MIDI  ");
//	menuBar.add(midiMenu);

	// Set up Import a SMF sub menu
	JMenu importSubMenu = new JMenu("Import MIDI data");

	//Set up SMF menu item.
/*
	JMenuItem fromSMFMenuItem 
	    = new JMenuItem("from a Standard MIDI File");
	fromSMFMenuItem.setActionCommand("fromSMF");
	fromSMFMenuItem.addActionListener(this);
	importSubMenu.add(fromSMFMenuItem);
*/

	//Set up Import from a MIDI Keyboard menu item.
	JMenuItem fromKeyboardMenuItem 
	    = new JMenuItem("from a MIDI Keyboard");
	fromKeyboardMenuItem.setActionCommand("fromKeyboard");
	fromKeyboardMenuItem.addActionListener(this);
	importSubMenu.add(fromKeyboardMenuItem);

	midiMenu.add(importSubMenu);

	return menuBar;
    }





    // React to menu selections.
    public void actionPerformed(ActionEvent e) {
	if ("about".equals(e.getActionCommand())) {
	    createJDubielAbout();
	} else if ("acknowledgments".equals(e.getActionCommand())) {
	    createJDubielAcknowledgments();
	} else if ("help".equals(e.getActionCommand())) {
	    createJDubielHelp();
	} else if ("license".equals(e.getActionCommand())) {
	    createJDubielLicense();
	} else if (
		   "unorderedSetProperties".equals(e.getActionCommand())) {
	    createUnorderedSetProperties();
	} else if (
		   "unorderedSetRelations".equals(e.getActionCommand())) {
	    createUnorderedSetRelations();
	} else if (
		   "unorderedSetLists".equals(e.getActionCommand())) {
	    createUnorderedSetLists();
	} else if (
		   "twelveTone".equals(e.getActionCommand())) {
	    createTwelveTone();
	} else if (
		   "orderedSetRelations".equals(e.getActionCommand())) {
	    createOrderedSetRelations();
	} else if (
		   "orderedSetLists".equals(e.getActionCommand())) {
	    createOrderedSetLists();
	} else if ("trichords".equals(e.getActionCommand())) {
	    createTrichordList();
	} else if ("tetrachords".equals(e.getActionCommand())) {
	    createTetrachordList();
	} else if ("pentachords".equals(e.getActionCommand())) {
	    createPentachordList();
	} else if ("hexachords".equals(e.getActionCommand())) {
	    createHexachordList();
	} else if ("septachords".equals(e.getActionCommand())) {
	    createSeptachordList();
	} else if ("octachords".equals(e.getActionCommand())) {
	    createOctachordList();
	} else if ("nonachords".equals(e.getActionCommand())) {
	    createNonachordList();
	} else if ("schoenberg".equals(e.getActionCommand())) {
	    createSchoenberg();
	} else if ("webern".equals(e.getActionCommand())) {
	    createWebern();
	} else if ("berg".equals(e.getActionCommand())) {
	    createBerg();
	} else if ("stravinsky".equals(e.getActionCommand())) {
	    createStravinsky();
	} else if ("fromSMF".equals(e.getActionCommand())) {
	    JOptionPane.showMessageDialog(
					  null, 
					  "For security reasons MIDI capabilities are disabled in this applet version. \n",
					  "Warning",
					  JOptionPane.WARNING_MESSAGE);

	    //			createSMF();
	} else if ("fromKeyboard".equals(e.getActionCommand())) {
	    JOptionPane.showMessageDialog(
					  null, 
					  "For security reasons MIDI capabilities are disabled in this applet version. \n",
					  "Warning",
					  JOptionPane.WARNING_MESSAGE);

	    //			createKeyboard();
	} else {
	    quit();
	}
    }






    //Quit the application.
    protected void quit() {
	System.exit(0);
    }

    //Create an internal frame to work on set properties.
    protected void createUnorderedSetProperties() {
	UnorderedSetProperties unorderedSetProperties 
	    = new UnorderedSetProperties();
	unorderedSetProperties.setVisible(true);
    }

    //Create another internal frame to work on set relations.
    protected void createUnorderedSetRelations() {
	UnorderedSetRelations unorderedSetRelations 
	    = new UnorderedSetRelations();
	unorderedSetRelations.setVisible(true);
    }

    //Create another internal frame to work on set Lists.
    protected void createUnorderedSetLists() {
	UnorderedSetLists unorderedSetLists 
	    = new UnorderedSetLists();
	unorderedSetLists.setVisible(true);
    }

    //Create another frame to work on ordered set relations.
    protected void createOrderedSetRelations() {
	OrderedSetRelations orderedSetRelations 
	    = new OrderedSetRelations();
	orderedSetRelations.setVisible(true);
    }

    //Create another frame to work on ordered set lists.
    protected void createOrderedSetLists() {
	OrderedSetLists orderedSetLists 
	    = new OrderedSetLists();
	orderedSetLists.setVisible(true);
    }

    //Create another internal frame to work on twelve-tone operations.
    protected void createTwelveTone() {
	TwelveTones twelveTone 
	    = new TwelveTones();
	twelveTone.setVisible(true);
    }

    //Create another internal frame to print a list.
    protected void createTrichordList() {
	TrichordTableFrame trichordTableFrame 
	    = new TrichordTableFrame();
	trichordTableFrame.setVisible(true); //necessary as of 1.3
    }
    //Create another internal frame to print a list.
    protected void createTetrachordList() {
	TetrachordTableFrame tetrachordTableFrame 
	    = new TetrachordTableFrame();
	tetrachordTableFrame.setVisible(true); //necessary as of 1.3
    }
    //Create another internal frame to print a list.
    protected void createPentachordList() {
	PentachordTableFrame pentachordTableFrame 
	    = new PentachordTableFrame();
	pentachordTableFrame.setVisible(true); //necessary as of 1.3
    }
    //Create another internal frame to print a list.
    protected void createHexachordList() {
	HexachordTableFrame hexachordTableFrame 
	    = new HexachordTableFrame();
	hexachordTableFrame.setVisible(true); //necessary as of 1.3
    }
    //Create another internal frame to print a list.
    protected void createSeptachordList() {
	SeptachordTableFrame septachordTableFrame 
	    = new SeptachordTableFrame();
	septachordTableFrame.setVisible(true); //necessary as of 1.3
    }
    //Create another internal frame to print a list.
    protected void createOctachordList() {
	OctachordTableFrame octachordTableFrame 
	    = new OctachordTableFrame();
	octachordTableFrame.setVisible(true); //necessary as of 1.3
    }
    //Create another internal frame to print a list.
    protected void createNonachordList() {
	NonachordTableFrame nonachordTableFrame 
	    = new NonachordTableFrame();
	nonachordTableFrame.setVisible(true); //necessary as of 1.3
    }

    //Create an internal frame for Schoengerg.
    protected void createSchoenberg() {
	SchoenbergFrame SchoenbergFrame = new SchoenbergFrame();
	SchoenbergFrame.setVisible(true); //necessary as of 1.3
    }

    //Create an internal frame for Webern.
    protected void createWebern() {
	WebernFrame WebernFrame = new WebernFrame();
	WebernFrame.setVisible(true); //necessary as of 1.3
    }

    //Create an internal frame for Berg.
    protected void createBerg() {
	BergFrame BergFrame = new BergFrame();
	BergFrame.setVisible(true); //necessary as of 1.3
    }

    //Create an internal frame for Stravinsky.
    protected void createStravinsky() {
	StravinskyFrame stravinskyFrame = new StravinskyFrame();
	stravinskyFrame.setDefaultLookAndFeelDecorated(true);
	stravinskyFrame.setVisible(true); //necessary as of 1.3

    }

    //Create another internal frame for MIDI.
    protected void createSMF() {
	MidiData midiSMFFrame = new MidiData();
	midiSMFFrame.setVisible(true);
    }

    //Create another internal frame for MIDI.
    protected void createKeyboard() {
	MidiData midiKeyboard = new MidiData();
    }



    // Help
    protected void createJDubielAbout() {
	JOptionPane.showMessageDialog(
				      null, 
				      "\n"
				      + "This is JDubiel version 0.9.\n"
				      + "\n"
				      + "JDubiel is named after Professor Joseph Dubiel of Columbia University, \n"
				      + "with whom I was privileged to study pitch-class set theory.\n"
				      + "\n"
				      + "JDubial was originally written as DUBIEL in Objective-C, C, and C++\n"
				      + "under NeXTSTEP and Linux as part of my dissertation project,\n"
				      + "_Stravinsky's 12-Tone Music: Some Principles of Atonal Pitch Organization_.\n"
				      + "\n" 
				      + "Joe generously granted me permission to use his name when I was writing \n"
				      + "DUBIEL in 1995. Then he allowed me to use it again summer 2005\n"
				      + "when I showed him a prototype of this new GUI version written in Java. \n"
				      + "\n" 
				      + "If you are a student currently studying pitch-class set theory, I strongly \n"
				      + "recommend that you should not use this program but should use just a pen \n"
				      + "and music sheets until you acquire solid skills in the theory.\n"
				      + "\n"
				      + "Please send me bug reports, suggestions, comments, and requests for\n"
				      + " new functions at:\n"
				      + "\n"
				      + "akira@music.columbia.edu \n"
				      + "\n"
				      + "Thank you very much for very kind, encouraging comments and suggestions! \n"
				      + "\n"
				      + "Akira Takaoka \n"
				      + "New York, October, 2017\n"
				      + "\n",
				      "About JDubiel",
				      JOptionPane.INFORMATION_MESSAGE);
    }




    protected void createJDubielAcknowledgments() {
	JOptionPane.showMessageDialog(
				      null, 
				      "\n"
				      + "The lists of twelve-tone rows of Schoenberg, Webern, and Berg come from:\n"
				      + "\n"
				      + "London, J., von Hippel, P., Huron, D., Cartano, J., Kingery, K., Olsen, B., &\n"
				      + "Santelli, T. (2000/2001). (von Hippel, P., ed.). Row forms in the serial\n"
				      + "works of Schoenberg, Berg, and Webern. [Computer database.]\n"
				      + "Stanford, CA: Center for Computer Assisted Research in the Humanities \n"
				      + "(www.ccarh.org/publications/data/humdrum/tonerow/).\n"
				      + "\n"
				      + "In addition to Joe's excellent teaching, JDubiel could not have been written \n"
				      + "without the following works:\n"
				      + "\n"
				      + "Brinkman, Alexander R.  1990.  _Pascal Programming for Music Research_.\n"
				      + "\tChicago: The University of Chicago Press.\n"
				      + "Forte, Allen.  1973.   _The Structure of Atonal Music_.\n"
				      + "\tNew Haven: Yale University Press.\n"
				      + "Morris, Robert.  1987.  _Composition with Pitch-Classes: "
				      + "A Theory of\n"
				      + "\tCompositional Design_.  New Haven: Yale University Press.\n"
				      + "Rahn, John.  1980.  _Basic Atonal Theory_. \n"
				      + "\tNew York: Schirmer.\n"
				      + "Straus, Joseph.  1990.  _Introduction to Post-Tonal Theory_.\n"
				      + "\tEnglewood Cliffs: Prentice Hall.\n"
				      + "\n"
				      + "Finally, I am grateful to the entire GNU/Linux community for excellent \n"
				      + "development tools.\n"
				      + "\n"
				      + "Akira Takaoka\n"
				      + "New York, August, 2005",
				      "Acknowledgments",
				      JOptionPane.INFORMATION_MESSAGE);
    }



    protected void createJDubielHelp() {
	JOptionPane.showMessageDialog(
				      null, 
				      "\n"
				      + "Drag copy of pc-sets is enabled between frames.\n"
				      + "\n"
				      + "Operations and forms of pc-sets are designated as follows (Morris 1987, Rahn 1980):\n"
				      + "\n"
				      + "\trkTn: rotation by k positions and transposition by n \n"
				      + "\tRTn: retrograde-transposition by n \n"
				      + "\trkRTn: rotation by k positions and retrograde-transposition by n \n"
				      + "\tPn: nth vertical made of a P rotational array \n"
				      + "\t    or the Pn form of a ordered pc-set, or sequence \n"
				      + "\tRn: nth vertical made of an R rotational array \n"
				      + "\t    or the Rn form of a ordered pc-set, or sequence \n"
				      + "\tIn: nth vertical made of an I rotational array \n"
				      + "\t    or the In form of a ordered pc-set, or sequence \n"
				      + "\tIRn: nth vertical made of an IR rotational array\n"
				      + "\t     or the IRn form of a ordered pc-set, or sequence \n"
				      + "\n"
				      + "When sequence <24579B> is transformed by T2I, for example, JDubiel shows \n"
				      + "T2I(I10): <0A9753>, by which it means that, if <24579B> is prime series form P0, \n"
				      + "<0A9753> is its inverted series form at the 10th transpositional level \n"
				      + " just like I10 of twelve-tone row P0:<24579B01368A> is <0A975321B864>. \n"
				      + "\n"
				      + "The following Tn/TnI types have variants (Brinkman 1990:628-9):\n"
				      + "\n"
				      + "\tName \tForte      \tRahn    \n"
				      + "\t5-20 \t01378    \t01568   \n"
				      + "\t6-29 \t013689   \t023679  \n"
				      + "\t6-31 \t013589   \t014579  \n"
				      + "\t7-18 \t0123589  \t0145679 \n"
				      + "\t7-20 \t0124789  \t0125679 \n"
				      + "\t8-26 \t0124579A \t0134578A\n"
				      + "\n"
				      + "Forte's prime forms are employed in JDubiel. \n",
				      "Help",
				      JOptionPane.INFORMATION_MESSAGE);
    }







    protected void createJDubielLicense() {
	JOptionPane.showMessageDialog(
				      null, 
				      "\n"
				      + "JDubiel: Computer Program for PC-Set Analysis\n"
				      + "Copyright (C) 1994-2021 Akira Takaoka\n"
				      + "All rights reserved.\n"
				      + "\n"
				      + "All the code was written by me except for SpringUtilities.java, \n"
				      + "the copyright of which belongs to Sun Microsystems, Inc.\n"
				      + "\n"
				      + "This program is free software; you can redistribute it and/or modify it under \n"
				      + "the terms of the GNU General Public License as published by \n"
				      + "the Free Software Foundation; either version 2 of the License, or (at your option)\n"
				      + "any later version.\n"
				      + "\n"
				      + "This program is distributed in the hope that it will be useful, but WITHOUT \n" 
				      + "ANY WARRANTY; without even the implied warranty of MERCHANTABILITY\n"
				      + " or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License \n"
				      + "for more details.\n"
				      + "\n"
				      + "A copy of GNU GENERAL PUBLIC LICENSE Version 2, June 1991 is available at:\n"
				      + "\n"
				      + "http://www.fsf.org/licensing/licenses/gpl.html\n"
				      + "\n"
				      + "Also it can be obtained from: \n"
				      + "\n"
				      + "Free Software Foundation, Inc. \n"
				      + "59 Temple Place, Suite 330, Boston, MA  02111-1307  USA\n",
				      "License",
				      JOptionPane.INFORMATION_MESSAGE);
    }
}

