// MidiData.java, used for
// JDubiel.java: computer program for pc-set analysis
// by Akira Takaoka, akira.takaoka@columbia.edu
// Copyright (C) 1994-2020 Akira Takaoka
// All rights reserved.




// package jdubielApplication;

import java.lang.reflect.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;
import java.io.*;
import javax.sound.midi.*;



//public class MidiData extends JInternalFrame
//                      implements ActionListener, 
//				 ScrollPaneConstants {

public class MidiData extends JInternalFrame
                      implements ScrollPaneConstants {

    Sequence openedSequence;
    JTextArea displayTextArea;

    static final int xOffset = 10, yOffset = 320;
    static int openFrameCount = 0;



    public MidiData() {

	super(" ", 
	      true, //resizable
	      true, //closable
	      true, //maximizable
	      true);//iconifiable

	//Create a file chooser
	JFileChooser fc = new JFileChooser();
	//fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
	fc.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);

	int returnVal = fc.showOpenDialog(MidiData.this);

	if (returnVal == JFileChooser.APPROVE_OPTION) {
	    File midiFile;
	    try {
		midiFile = fc.getSelectedFile();
		openedSequence = MidiSystem.getSequence(midiFile);

		displaySMF();

		JScrollPane displayScrollPane 
		    = new JScrollPane(displayTextArea,
				      VERTICAL_SCROLLBAR_AS_NEEDED,
				      HORIZONTAL_SCROLLBAR_AS_NEEDED);
		displayScrollPane.setPreferredSize(new Dimension(1000, 200));
		displayScrollPane.setMinimumSize(new Dimension(600, 50));
		displayScrollPane.setBorder(
			BorderFactory.createCompoundBorder(
				BorderFactory.createTitledBorder(
					""),
				BorderFactory.createEmptyBorder(5,5,5,5)));
		displayScrollPane.getBorder();

		getContentPane().add(displayScrollPane, 
				     BorderLayout.CENTER);

		setTitle(midiFile.getName());
		setSize(800, 200);
		setLocation(xOffset + 20 * openFrameCount, 
			    yOffset + 20 * openFrameCount);
	    } catch (Exception e) { 
		JOptionPane.showMessageDialog(
					      MidiData.this, 
					      e.toString(),
					      "Warning",
					      JOptionPane.WARNING_MESSAGE);
	    }

	    //This is where a real application would open the file.

	}
    }






    protected void displaySMF() throws Exception {
	Track [] trackArray = openedSequence.getTracks();

	displayTextArea = new JTextArea();
	displayTextArea.setBackground(Color.WHITE);
	displayTextArea.setRows(16);
	displayTextArea.setLineWrap(false);
	displayTextArea.setEditable(true);
	StringBuffer midiNoteOn = new StringBuffer();

	for (int j = 0; j < trackArray.length; j++) {
	    midiNoteOn.append("Channel " + (j + 1));
	    midiNoteOn.append("\n\n");
	    int startTime = 0, previousStartTime = 0;
	    for (int i = 0; i < trackArray[j].size(); i++) {
		MidiEvent me = trackArray[j].get(i);
		MidiMessage mm = me.getMessage();
		byte [] b = mm.getMessage();
		if (0x90 == mm.getStatus()) { // 0x90: "note on"
		    startTime = (int)(me.getTick()/60);
		    for (int k = 0; k < startTime - previousStartTime; k++)
			midiNoteOn.append(" "); 

		    midiNoteOn.append(noteNumber2NoteName((int)(b[1] & 0xFF)));
		    previousStartTime = startTime;
		}
	    }
	    midiNoteOn.append("\n\n");
	}
	displayTextArea.insert("Akira Takaoka", 1);

	displayTextArea.append(midiNoteOn.toString());
    }











	// Converting MIDI notenumber to ASA note name.
	public String noteNumber2NoteName(int noteNumber) throws Exception {

		int octave, divide = 0;
		String name;

		int oct = (int) (noteNumber / 12) + 3;
		int pc = noteNumber - (12 * (oct - 3));

		if(pc > 11)
			divide = (int)(pc/12);

		switch(oct + divide) {
		case 4:
			octave = 0;
			break;
		case 5:
			octave = 1;
			break;
		case 6:
			octave = 2;
			break;
		case 7:
			octave = 3;
			break;
		case 8:
			octave = 4;
			break;
		case 9:
			octave = 5;
			break;
		case 10:
			octave = 6;
			break;
		case 11:
			octave = 7;
			break;
		case 12:
			octave = 8;
			break;
		case 13:
			octave = 9;
			break;
		case 14:
			octave = 10;
			break;
		case 15:
			octave = 11;
			break;
		default:
			octave = 100;
//			System.err.println("octave: " + octave 
//					   + " is too low or high.");
//			System.exit(-1);
		}

		switch(pc % 12) {
		case 0:
			name = "C";
			break;
		case 1:
			name = "C#";
			break;
		case 2:
			name = "D";
			break;
		case 3:
			name = "D#";
			break;
		case 4:
			name = "E";
			break;
		case 5:
			name = "F";
			break;
		case 6:
			name = "F#";
			break;
		case 7:
			name = "G";
			break;
		case 8:
			name = "G#";
			break;
		case 9:
			name = "A";
			break;
		case 10:
			name = "A#";
			break;
		case 11:
			name = "B";
			break;
		default:
			name = " ";
			System.err.println("Pitch-class: "
					   + pc 
					   + " is not a pitch class.");
			System.exit(-1);
		}

//		Integer otv = new Integer(octave); // a deprecated API
		Integer otv = Integer.valueOf(octave);

		return name.concat(otv.toString());
	}







    protected void readSMF() throws Exception {

	int chan = 0, vel = 96, numOfTracks = 1, fileType = 0;
	int firstPitch = 60, numOfNotes = 48;
	int program = 52; // Program 52 is Voice Aahs.
	int resolution = 20; // The number of ticks per quarter note.
	double MM = 60.0; //  Metronome: quarter notes per minute
	long quarterNote = resolution; 
	long halfNote = resolution * 2;
	long wholeNote = resolution * 4;
	long startTime = 0, endTime;

	// Create Sequence and Track instances.
	Sequence sequence = new Sequence(Sequence.PPQ, resolution);
	Track track = sequence.createTrack();

	// Fill the array pitch[] with pitches.
	//		EnoPitches eno = new EnoPitches(firstPitch, numOfNotes);
	//		int [] pitch = eno.generatePitches();
    
	// Set a tempo meta-message.
	MetaMessage metaMessage = new MetaMessage();
	byte [] tempoData = {0x0C, 0x7F, 0x35};
	metaMessage.setMessage(0x51, tempoData, tempoData.length);
	MidiEvent tempoEvent = new MidiEvent(metaMessage, 0);
	track.add(tempoEvent);

	// Send a program change
	ShortMessage pChange = new ShortMessage();
	pChange.setMessage(ShortMessage.PROGRAM_CHANGE + chan, 
			   program, 0);
	MidiEvent programEvent = new MidiEvent(pChange, 0);
	track.add(programEvent);
/*		
	for(int i = 0; i < pitch.length; i++) {
	// Note on message
	ShortMessage onMssg = new ShortMessage();
			onMssg.setMessage(ShortMessage.NOTE_ON + chan, 
					  pitch[i], vel);
			MidiEvent noteOnEvent = new MidiEvent(onMssg, startTime);
			track.add(noteOnEvent);
			// Add the pitch perfect 5th above.
			ShortMessage onMssg5th = new ShortMessage();
			onMssg5th.setMessage(ShortMessage.NOTE_ON + chan, 
					     pitch[i]+7, vel);
			MidiEvent noteOnEvent5th = new MidiEvent(onMssg5th, 
								 startTime);
			track.add(noteOnEvent5th);
			
			endTime = startTime + (wholeNote * 2);

			// Note off message
			ShortMessage offMssg = new ShortMessage();
			offMssg.setMessage(ShortMessage.NOTE_OFF + chan, 
					   pitch[i], vel);
			MidiEvent noteOffEvent = new MidiEvent(offMssg, endTime);
			track.add(noteOffEvent);
			ShortMessage offMssg5th = new ShortMessage();
			offMssg5th.setMessage(ShortMessage.NOTE_OFF + chan, 
					      pitch[i]+7, vel);
			MidiEvent noteOffEvent5th = new MidiEvent(offMssg5th, 
								  endTime);
			track.add(noteOffEvent5th);
			
			startTime += wholeNote;
	}
		// Write a MIDI file.
		if (args.length >= 1)
			MidiSystem.write(sequence, fileType, 
					 new FileOutputStream(args[0]));
		else {
			System.err.print("\nUsage: Java Eno " + 
					 "file_name.mid\n\n");
			System.exit(-1);
		}
*/


    }





/*
  public void actionPerformed(ActionEvent e) {
  }
*/
}





