// OrderedInclusion.java, used for
// JDubiel.java: computer program for pc-set analysis
// by Akira Takaoka, akira.takaoka@columbia.edu
// Copyright (C) 1994-2017 Akira Takaoka
// All rights reserved.





// package jdubielApplication;


public class OrderedInclusion extends PcSet {


// include a method to check out a cyclic subset (Berg).

    PcSet pcSet1, pcSet2;

    // Constructor 1
    public OrderedInclusion(int[] pc1, int[] pc2) {
	if(pc1.length >= pc2.length) {
	    pcSet1 = new PcSet(pc1.length);
	    pcSet2 = new PcSet(pc2.length);
	    for (int i = 0; i < pcSet1.set.length; i++)
		pcSet1.set[i] = pc1[i];
	    for (int i = 0; i < pcSet2.set.length; i++)
		pcSet2.set[i] = pc2[i];
	} else {
	    pcSet1 = new PcSet(pc2.length);
	    pcSet2 = new PcSet(pc1.length);
	    for (int i = 0; i < pcSet1.set.length; i++)
		pcSet1.set[i] = pc2[i];
	    for (int i = 0; i < pcSet2.set.length; i++)
		pcSet2.set[i] = pc1[i];
	}
	// So the cardinality of pcSet1.set is always 
	// greater than or equal to that of pcSet2.set.
    }
    // Constructor 2
    public OrderedInclusion(Set s1, Set s2) {
	if(s1.set.length >= s2.set.length) {
	    pcSet1 = new PcSet(s1.set.length);
	    pcSet2 = new PcSet(s2.set.length);
	    for (int i = 0; i < pcSet1.set.length; i++)
		pcSet1.set[i] = s1.set[i];
	    for (int i = 0; i < pcSet2.set.length; i++)
		pcSet2.set[i] = s2.set[i];
	} else {
	    pcSet1 = new PcSet(s2.set.length);
	    pcSet2 = new PcSet(s1.set.length);
	    for (int i = 0; i < pcSet1.set.length; i++)
		pcSet1.set[i] = s2.set[i];
	    for (int i = 0; i < pcSet2.set.length; i++)
		pcSet2.set[i] = s1.set[i];

	    // So the size of pcSet1.set is always greater than 
	    // or equal to that of pcSet2.set.
	}

	// Characters used for base 12 system, 
	// a & b, A & B, t & e, or T & E
	if (s1.base12String != null) {
	    pcSet1.base12String = new String(s1.base12String);
	    base12String = new String(s1.base12String);
	} else if (s2.base12String != null) {
	    pcSet2.base12String = new String(s2.base12String);
	    base12String = new String(s2.base12String);
	} else {
	    pcSet1.base12String = new String("AB");
	    pcSet2.base12String = new String("AB");
	    base12String = new String("AB");
	}
    }
    // Constructor 3
    public OrderedInclusion(Set s1, int[] pc2) {
	if(s1.set.length >= pc2.length) {
	    pcSet1 = new PcSet(s1.set.length);
	    pcSet2 = new PcSet(pc2.length);
	    for (int i = 0; i < pcSet1.set.length; i++)
		pcSet1.set[i] = s1.set[i];
	    for (int i = 0; i < pcSet2.set.length; i++)
		pcSet2.set[i] = pc2[i];
	} else {
	    pcSet1 = new PcSet(pc2.length);
	    pcSet2 = new PcSet(s1.set.length);
	    for (int i = 0; i < pcSet1.set.length; i++)
		pcSet1.set[i] = pc2[i];
	    for (int i = 0; i < pcSet2.set.length; i++)
		pcSet2.set[i] = s1.set[i];

	    // So the size of pcSet1.set is always greater than 
	    // or equal to that of pcSet2.set.
	}

	// Characters used for base 12 system, 
	// a & b, A & B, t & e, or T & E
	if (s1.base12String != null) {
	    pcSet1.base12String = new String(s1.base12String);
	    base12String = new String(s1.base12String);
	} else {
	    pcSet1.base12String = new String("AB");
	    pcSet2.base12String = new String("AB");
	    base12String = new String("AB");
	}
    }
    // The end of constructors








    // Check out if it fits under transposition, or Tn.
    public String listFitTns() {

	StringBuffer relations = new StringBuffer();

	// The cardinality of pcSet1.set is always 
	// greater than or equal to that of pcSet2.set.

	int [][] tns = new int[SCALE_STEPS][pcSet1.set.length];
	// tns[i][j]: pc-sets at all the transpositions. 
	tns = pcSet1.getTns(0, SCALE_STEPS);

	int h = 0, i = 0, j = 0, jj = 0;
	// i: that of Ti. j: a pitch-class
	while(i < SCALE_STEPS) { 
	    while(jj < pcSet2.set.length && j < pcSet1.set.length) {
		if(pcSet2.set[jj] == tns[i][j]) {
		    jj++;
		    j++;
		} else {
		    jj = 0;
		    j++;
		}
	    }

	    if(jj == pcSet2.set.length) {
		relations.append("    T"
				 + i 
				 + ": <"
				 + pcSetArrayToString(pcSet1.set)
				 + ">  -->  <"
				 + pcSetArrayToString(tns[i])
				 + "> (P" 
				 + i 
				 + ")\n");

		h++;
		i++;
	    } else
		i++;

	    j = 0;
	    jj = 0;
	}

	StringBuffer f = new StringBuffer();

	if(h == 0)
	    f.append("noRelationsUnderT");
	else {
	    if(pcSet1.set.length != pcSet2.set.length) {
		f.append(" <" 
			 + pcSetArrayToString(pcSet2.set) 
			 + "> is a subsequence of <" 
			 + pcSetArrayToString(pcSet1.set) 
			 + "> under: \n");
	    } else {
		f.append(" <" 
			 + pcSetArrayToString(pcSet2.set) 
			 + "> is transformed from <" 
			 + pcSetArrayToString(pcSet1.set) 
			 + "> by: \n");
	    }
	    f.append(relations.toString());
	}
	return f.toString();
    }
    // The end of Tn











    // Check out if it fits under transposition and rotation, or rTn.
    public String listFitrTns() {

	StringBuffer relations = new StringBuffer();

	// The cardinality of pcSet1.set is always 
	// greater than or equal to that of pcSet2.set.

	int [][] tns = pcSet1.getTns(0, SCALE_STEPS);
	// tns[i][j]: pc-sets at all the transpositional levels. 
	PcSet q = new PcSet(pcSet1.set.length);
	int[][] rotatedTns = new int[pcSet1.set.length][pcSet1.set.length];

	int h = 0, i = 0, j = 0, ii = 0, jj = 0;
	while(i < SCALE_STEPS){ // i is that of Ti. j is a pitch-class.
	    for(j = 0; j < pcSet1.set.length; j++)
		q.set[j] = tns[i][j];
		
	    j = 0;

	    rotatedTns = q.rotateSequence(0, pcSet1.set.length);
	
	    // ii: that of rii. j: a pitch-class.
	    while(ii < pcSet1.set.length){
		while(jj < pcSet2.set.length && j < pcSet1.set.length) {
		    if(pcSet2.set[jj] == rotatedTns[ii][j]) {
			jj++;
			j++;
		    } else {
			jj = 0;
			j++;
		    }
		}

		if(jj == pcSet2.set.length) {
		    relations.append("    r"
				     + ii 
				     + "T" 
				     + i 
				     + ": <"
				     + pcSetArrayToString(pcSet1.set)
				     + ">  -->  <");
		    int[] rttd = rotateSeqByN(tns[i], ii);
		    relations.append(pcSetArrayToString(rttd)
				     + "> (P" 
				     + i 
				     + ")\n");
			
		    h++;
		    ii++;
		} else
		    ii++;
		
		j = 0;
		jj = 0;
	    }
	    
	    i++;
	    ii = 0;
	    j = 0;
	    jj = 0;
	}

	StringBuffer f = new StringBuffer();

	if(h == 0)
	    f.append("noRelationsUnderrT");
	else {
	    if(pcSet1.set.length != pcSet2.set.length) {
		f.append(" <" 
			 + pcSetArrayToString(pcSet2.set) 
			 + "> is a subsequence of <" 
			 + pcSetArrayToString(pcSet1.set) 
			 + "> under: \n");
	    } else {
		f.append(" <" 
			 + pcSetArrayToString(pcSet2.set) 
			 + "> is transformed from <" 
			 + pcSetArrayToString(pcSet1.set) 
			 + "> by: \n");
	    }
	    f.append(relations.toString());
	}
	return f.toString();
    }
    // The end of rTn










    // Check out if it fits under inversion, or TnI.
    public String listFitTnIs() {

	StringBuffer relations = new StringBuffer();

	// The cardinality of pcSet1.set is always 
	// greater than or equal to that of pcSet2.set.

	int [][] tnis = new int[SCALE_STEPS][pcSet1.set.length];
	// tnis[i][j]: pc-sets at all the transpositions/inversions. 
	tnis = pcSet1.getTnIs(0, SCALE_STEPS);
    
	int h = 0, i = 0, j = 0, jj = 0;
	// tnis[i][j]: pc-sets at all the transpositions. 
	while(i < SCALE_STEPS) { // i: that of Ti. j: a pitch-class.
	    while(jj < pcSet2.set.length && j < pcSet1.set.length) {
		if(pcSet2.set[jj] == tnis[i][j]) {
		    jj++;
		    j++;
		} else {
		    jj=0;
		    j++;
		}
	    }

	    if(jj == pcSet2.set.length) {
		relations.append("    T" 
				 + i 
				 + "I: <"
				 + pcSetArrayToString(pcSet1.set)
				 + ">  -->  <"
				 + pcSetArrayToString(tnis[i])
				 + "> (I" 
				 + tnItoIn(pcSet1.set, tnis[i])
				 + ")\n");
		h++;
		i++;
	    } else
		i++;

	    j=0;
	    jj=0;
	}

	StringBuffer f = new StringBuffer();

	if(h == 0)
	    f.append("noRelationsUnderTI");
	else {
	    if(pcSet1.set.length != pcSet2.set.length) {
		f.append(" <" 
			 + pcSetArrayToString(pcSet2.set) 
			 + "> is a subsequence of <" 
			 + pcSetArrayToString(pcSet1.set)
			 + "> under: \n");
	    } else {
		f.append(" <" 
			 + pcSetArrayToString(pcSet2.set) 
			 + "> is transformed from <" 
			 + pcSetArrayToString(pcSet1.set) 
			 + "> by: \n");
	    }
	    f.append(relations.toString());
	}
	return f.toString();
    }
    // The end of TnI










    // Check out if it fits under inversion and rotation, or rTnI.
    public String listFitrTnIs() {

	StringBuffer relations = new StringBuffer();

	// The cardinality of pcSet1.set is always 
	// greater than or equal to that of pcSet2.set.

	int [][] tnis = pcSet1.getTnIs(0, SCALE_STEPS);
	// tnis[i][j]: pc-sets at all the inverted transpositions. 
	PcSet q = new PcSet(pcSet1.set.length);
	int[][] totatedTnIs = new int[pcSet1.set.length][pcSet1.set.length];

	int h = 0, i = 0, j = 0, ii = 0, jj = 0;
	while(i < SCALE_STEPS) { // is that of Ti. j is a pitch-class.

	    for(j = 0; j < pcSet1.set.length; j++)
		q.set[j] = tnis[i][j];
			
	    j = 0;
	    
	    totatedTnIs = q.rotateSequence(0, pcSet1.set.length);
	
	    // ii is that of rii. j is a pitch-class.
	    while(ii < pcSet1.set.length) {
		while(jj < pcSet2.set.length && j < pcSet1.set.length)
		    if(pcSet2.set[jj] == totatedTnIs[ii][j]) {
			jj++;
			j++;
		    } else {
			jj=0;
			j++;
		    }

		if(jj == pcSet2.set.length) {
		    relations.append("    r" 
				     + ii 
				     + "T" 
				     + i 
				     + "I: <"
				     + pcSetArrayToString(pcSet1.set)
				     + ">  -->  <");
		    int[] rttd = rotateSeqByN(tnis[i], ii);
		    relations.append(pcSetArrayToString(rttd)
				     + "> (I" 
				     + tnItoIn(pcSet1.set, tnis[i])
				     + ")\n");

		    h++;
		    ii++;
		} else
		    ii++;

		j = 0;
		jj = 0;
	    }
		
	    i++;
	    ii = 0;
	    j = 0;
	    jj = 0;
	}

	StringBuffer f = new StringBuffer();

	if(h == 0)
	    f.append("noRelationsUnderrTI");
	else {
	    if(pcSet1.set.length != pcSet2.set.length) {
		f.append(" <" 
			 + pcSetArrayToString(pcSet2.set) 
			 + "> is a subsequence of <" 
			 + pcSetArrayToString(pcSet1.set) 
			 + "> under: \n");
	    } else {
		f.append(" <" 
			 + pcSetArrayToString(pcSet2.set) 
			 + "> is transformed from <" 
			 + pcSetArrayToString(pcSet1.set) 
			 + "> by: \n");
	    }
	    f.append(relations.toString());
	}
	return f.toString();
    }
    // The endo of rTnI











    // Check out if it fits under retrograde and transposition, or RTn.
    public String listFitRTs() {

	StringBuffer relations = new StringBuffer();

	// The cardinality of pcSet1.set is always 
	// greater than or equal to that of pcSet2.set.

	int [][] RTns = new int[SCALE_STEPS][pcSet1.set.length];
	// RTns[i][j]: pc-sets at all the transpositions. 
	RTns = pcSet1.getRTns(0, SCALE_STEPS);

	int h = 0, i = 0, j = 0, jj =0;
	while(i < SCALE_STEPS) { // i is that of Ti. j is a pc.
	    while(jj < pcSet2.set.length && j < pcSet1.set.length) {
		if(pcSet2.set[jj] == RTns[i][j]) {
		    jj++;
		    j++;
		} else {
		    jj=0;
		    j++;
		}
	    }

	    if(jj == pcSet2.set.length) {
		relations.append("    RT" 
				 + i 
				 + ": <"
				 + pcSetArrayToString(pcSet1.set)
				 + ">  -->  <"
				 + pcSetArrayToString(RTns[i])
				 + "> (R" 
				 + i 
				 + ")\n");
		h++;
		i++;
	    } else
		i++;

	    j = 0;
	    jj = 0;
	}

	StringBuffer f = new StringBuffer();

	if(h == 0)
	    f.append("noRelationsUnderRT");
	else {
	    if(pcSet1.set.length != pcSet2.set.length) {
		f.append(" <" 
			 + pcSetArrayToString(pcSet2.set) 
			 + "> is a subsequence of <" 
			 + pcSetArrayToString(pcSet1.set) 
			 + "> under: \n");
	    } else {
		f.append(" <" 
			 + pcSetArrayToString(pcSet2.set) 
			 + "> is transformed from <" 
			 + pcSetArrayToString(pcSet1.set) 
			 + "> by: \n");
	    }
	    f.append(relations.toString());
	}
	return f.toString();
    }
    // The end of RTn.







	


    // Check out if it fits under rRTn.
    public String listFitrRTs() {

	StringBuffer relations = new StringBuffer();

	// The cardinality of pcSet1.set is always 
	// greater than or equal to that of pcSet2.set.

	int [][] RTns = pcSet1.getRTns(0, SCALE_STEPS);
	// RTns[i][j]: pc-sets at all the transpositions. 
	PcSet q = new PcSet(pcSet1.set.length);
	int[][] rotatedRTns = new int[pcSet1.set.length][pcSet1.set.length];

	int h = 0, i = 0, j = 0, ii = 0, jj=0;
	while(i < SCALE_STEPS) { // is that of T(i). j is a pc.
	    for(j = 0; j < pcSet1.set.length; j++)
		q.set[j] = RTns[i][j];
		
	    j = 0;
		
	    rotatedRTns = q.rotateSequence(0, pcSet1.set.length);
	
	    // ii: that of r(ii). j: a pc.
	    while(ii < pcSet1.set.length) {
		while(jj < pcSet2.set.length && j < pcSet1.set.length) {
		    if(pcSet2.set[jj]==rotatedRTns[ii][j]) {
			jj++;
			j++;
		    } else {
			jj = 0;
			j++;
		    }
		}

		if(jj == pcSet2.set.length) {
		    relations.append("    r" 
				     + ii 
				     + "RT" 
				     + i 
				     + ": <"
				     + pcSetArrayToString(pcSet1.set)
				     + ">  -->  <");
		    int[] rttd = rotateSeqByN(RTns[i], ii);
		    relations.append(pcSetArrayToString(rttd)
				     + "> (R" 
				     + i 
				     + ")\n");
			
		    h++;
		    ii++;
		} else
		    ii++;

		j = 0;
		jj = 0;
	    }

	    i++;
	    ii = 0;
	    j = 0;
	    jj = 0;
	}

	StringBuffer f = new StringBuffer();

	if(h == 0)
	    f.append("noRelationsUnderrRT");
	else {
	    if(pcSet1.set.length != pcSet2.set.length) {
		f.append(" <" 
			 + pcSetArrayToString(pcSet2.set) 
			 + "> is a subsequence of <" 
			 + pcSetArrayToString(pcSet1.set) 
			 + "> under: \n");
	    } else {
		f.append(" <" 
			 + pcSetArrayToString(pcSet2.set) 
			 + "> is transformed from <" 
			 + pcSetArrayToString(pcSet1.set) 
			 + "> by: \n");
	    }
	    f.append(relations.toString());
	}
	return f.toString();
    }
    // The end of rRTn.











    // Check out if it fits under retrograde and inversion, or RTnI.
    public String listFitRTnIs() {

	StringBuffer relations = new StringBuffer();

	// The cardinality of pcSet1.set is always 
	// greater than or equal to that of pcSet2.set.

	int [][] RTnIs = new int[SCALE_STEPS][pcSet1.set.length];
	// RTnIs[i][j]: pc-sets at all the transpositions. 
	RTnIs = pcSet1.getRTnIs(0, SCALE_STEPS);
	
	int h = 0, i = 0, j = 0, jj = 0;
	while(i < SCALE_STEPS) { // i is that of Ti. j is a pc.
	    while(jj < pcSet2.set.length && j < pcSet1.set.length)
		if(pcSet2.set[jj] == RTnIs[i][j]) {
		    jj++;
		    j++;
		} else {
		    jj = 0;
		    j++;
		}

	    if(jj == pcSet2.set.length) {
		relations.append("    RT" 
				 + i 
				 + "I: <"
				 + pcSetArrayToString(pcSet1.set)
				 + ">  -->  <"
				 + pcSetArrayToString(RTnIs[i])
				 + "> (RI" 
		 + ((SCALE_STEPS * 2) + i - 2 * pcSet1.set[0]) % SCALE_STEPS 
				 + ")\n");
			
		h++;
		i++;
	    }
	    else
		i++;

	    j = 0;
	    jj = 0;
	}

	StringBuffer f = new StringBuffer();

	if(h == 0)
	    f.append("noRelationsUnderRTI");
	else {
	    if(pcSet1.set.length != pcSet2.set.length) {
		f.append(" <" 
			 + pcSetArrayToString(pcSet2.set) 
			 + "> is a subsequence of <" 
			 + pcSetArrayToString(pcSet1.set) 
			 + "> under: \n");
	    } else {
		f.append(" <" 
			 + pcSetArrayToString(pcSet2.set) 
			 + "> is transformed from <" 
			 + pcSetArrayToString(pcSet1.set) 
			 + "> by: \n");
	    }
	    f.append(relations.toString());
	}
	return f.toString();
    }
    // The end of RTnI









    // Check out if it fits under rRTnI.
    public String listFitrRTnIs() {

	StringBuffer relations = new StringBuffer();

	// The cardinality of pcSet1.set is always 
	// greater than or equal to that of pcSet2.set.

	int [][] RTnIs = pcSet1.getRTnIs(0, SCALE_STEPS);
	// RTnIs[i][j]: pc-sets at all the transpositions. 
	PcSet q = new PcSet(pcSet1.set.length);
	int[][] rotatedRTnIs = new int[pcSet1.set.length][pcSet1.set.length];

	int h = 0, i = 0, j = 0, ii = 0, jj = 0;
	while(i < SCALE_STEPS){	// i: that of T(i). j: a pitch class.
	    for(j = 0; j < pcSet1.set.length; j++)
		q.set[j] = RTnIs[i][j];

	    j = 0;
		
	    rotatedRTnIs = q.rotateSequence(0, pcSet1.set.length);
	
	    // ii is that of rii. j is a pithc class.
	    while(ii < pcSet1.set.length) {
		while(jj < pcSet2.set.length && j < pcSet1.set.length) {
		    if(pcSet2.set[jj] == rotatedRTnIs[ii][j]) {
			jj++;
			j++;
		    } else {
			jj = 0;
			j++;
		    }
		}

		if(jj == pcSet2.set.length) {
		    relations.append("    r" 
				     + ii 
				     + "RT" 
				     + i 
				     + ": <"
				     + pcSetArrayToString(pcSet1.set)
				     + ">  -->  <");
		    int[] rttd = rotateSeqByN(RTnIs[i], ii);
		    relations.append(pcSetArrayToString(rttd)
				     + "> (RI" 
	     + ((SCALE_STEPS * 2) + i - 2 * pcSet1.set[0]) % SCALE_STEPS 
				     + ")\n");
			
		    h++;
		    ii++;
		} else
		    ii++;

		j = 0;
		jj=0;
	    }
	    i++;
	    ii = 0;
	    j = 0;
	    jj = 0;
	}

	StringBuffer f = new StringBuffer();

	if(h == 0)
	    f.append("noRelationsUnderrRTI");
	else {
	    if(pcSet1.set.length != pcSet2.set.length) {
		f.append(" <" 
			 + pcSetArrayToString(pcSet2.set) 
			 + "> is a subsequence of <" 
			 + pcSetArrayToString(pcSet1.set) 
			 + "> under: \n");
	    } else {
		f.append(" <" 
			 + pcSetArrayToString(pcSet2.set) 
			 + "> is transformed from <" 
			 + pcSetArrayToString(pcSet1.set) 
			 + "> by: \n");
	    }
	    f.append(relations.toString());
	}
	return f.toString();
    }
    // The end of rRTnI.










    // Check out if it fits under multiplication, or Mn.
    public String listFitMns() {

	StringBuffer relations = new StringBuffer();

	// The cardinality of pcSet1.set is always 
	// greater than or equal to that of pcSet2.set.

	int[][] mns = new int[SCALE_STEPS][pcSet1.set.length];
	// mns[i][j]: pc-sets at all the multiplications. 
	mns = pcSet1.getMns(0, SCALE_STEPS);

	int h = 0, i = 0, j = 0, jj = 0;
	while(i < SCALE_STEPS) { // i is that of Mi. j is a pc.
	    while(jj < pcSet2.set.length && j < pcSet1.set.length)
		if(pcSet2.set[jj] == mns[i][j]) {
		    jj++;
		    j++;
		} else {
		    jj = 0;
		    j++;
		}

	    if(jj == pcSet2.set.length) {
		relations.append("    M" 
				 + i 
				 + ": <"
				 + pcSetArrayToString(pcSet1.set)
				 + ">  -->  <"
				 + pcSetArrayToString(mns[i])
				 + ">\n");
		h++;
		i++;
	    } else
		i++;

	    j = 0;
	    jj = 0;
	}

	StringBuffer f = new StringBuffer();

	if(h == 0)
	    f.append("noRelationsUnderM");
	else {
	    if(pcSet1.set.length != pcSet2.set.length) {
		f.append(" <" 
			 + pcSetArrayToString(pcSet2.set) 
			 + "> is a subsequence of <" 
			 + pcSetArrayToString(pcSet1.set) 
			 + "> under: \n");
	    } else {
		f.append(" <" 
			 + pcSetArrayToString(pcSet2.set) 
			 + "> is transformed from <" 
			 + pcSetArrayToString(pcSet1.set) 
			 + "> by: \n");
	    }
	    f.append(relations.toString());
	}
	return f.toString();
    }
    // The end of Mn









	// Check out if it fits under verticals
/*	
  public String listFitVerticals() {

		StringBuffer relations = new StringBuffer();


		return f.toString();
	}
*/
    // The end of verticals
}

