// OrderedSetLists.java, used for
// JDubiel.java: computer program for pc-set analysis
// by Akira Takaoka, akira.takaoka@columbia.edu
// Copyright (C) 1994-2017 Akira Takaoka
// All rights reserved.





// package jdubielApplication;

import java.lang.reflect.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;


// To list ordered pc-sets, or sequences, transformed by various operations.
public class OrderedSetLists extends JFrame 
                             implements ActionListener {

    static final int xOffset = 50, yOffset = 120;
    static int openFrameCount = 0;

    JScrollPane listsScrollPane;
    JPanel sequencePanel;
    JPanel buttonPanel;

    JEditorPane editorPane;
    JScrollPane editorScrollPane;

    JTextField listsTextField;

    JCheckBox tnCheckBox;
    JCheckBox tnICheckBox;
    JCheckBox mnCheckBox;
    JCheckBox RTnCheckBox; 
    JCheckBox RTnICheckBox; 

    JCheckBox tnInvarCheckBox;
    JCheckBox tnIInvarCheckBox;
    JCheckBox mnInvarCheckBox;
    JCheckBox RTnInvarCheckBox;
    JCheckBox RTnIInvarCheckBox;
    JCheckBox rTnInvarCheckBox;
    JCheckBox rTnIInvarCheckBox;
    JCheckBox rRTnInvarCheckBox;
    JCheckBox rRTnIInvarCheckBox;

    JCheckBox rotationalArrayPCheckBox;
    JCheckBox rotationalArrayRCheckBox;
    JCheckBox rotationalArrayICheckBox;
    JCheckBox rotationalArrayRICheckBox;
    JCheckBox rotationalArrayIRCheckBox;

    JSpinner tnSpinner;
    JSpinner tkSpinner;
    JSpinner tnISpinner;
    JSpinner tkISpinner;
    JSpinner mnSpinner;
    JSpinner mkSpinner;
    JSpinner RTnSpinner; 
    JSpinner RTkSpinner; 
    JSpinner RTnISpinner; 
    JSpinner RTkISpinner; 

    JSpinner tnInvarSpinner;
    JSpinner tkInvarSpinner;
    JSpinner tnIInvarSpinner;
    JSpinner tkIInvarSpinner;
    JSpinner mnInvarSpinner;
    JSpinner mkInvarSpinner;
    JSpinner RTnInvarSpinner;
    JSpinner RTkInvarSpinner;
    JSpinner RTnIInvarSpinner;
    JSpinner RTkIInvarSpinner;
    JSpinner rTnInvarSpinner;
    JSpinner rTkInvarSpinner;
    JSpinner rTnIInvarSpinner;
    JSpinner rTkIInvarSpinner;
    JSpinner rRTnInvarSpinner;
    JSpinner rRTkInvarSpinner;
    JSpinner rRTnIInvarSpinner;
    JSpinner rRTkIInvarSpinner;

    JSpinner rotationalArrayPSpinner;
    JSpinner rotationalArrayRSpinner;
    JSpinner rotationalArrayISpinner;
    JSpinner rotationalArrayRISpinner;
    JSpinner rotationalArrayIRSpinner;


    SpinnerNumberModel tnModel;
    SpinnerNumberModel tkModel;
    SpinnerNumberModel tnIModel;
    SpinnerNumberModel tkIModel;
    SpinnerNumberModel mnModel;
    SpinnerNumberModel mkModel;
    SpinnerNumberModel RTnModel; 
    SpinnerNumberModel RTkModel; 
    SpinnerNumberModel RTnIModel; 
    SpinnerNumberModel RTkIModel; 

    SpinnerNumberModel tnInvarModel;
    SpinnerNumberModel tkInvarModel;
    SpinnerNumberModel tnIInvarModel;
    SpinnerNumberModel tkIInvarModel;
    SpinnerNumberModel mnInvarModel;
    SpinnerNumberModel mkInvarModel;
    SpinnerNumberModel RTnInvarModel;
    SpinnerNumberModel RTkInvarModel;
    SpinnerNumberModel RTnIInvarModel;
    SpinnerNumberModel RTkIInvarModel;
    SpinnerNumberModel rTnInvarModel;
    SpinnerNumberModel rTkInvarModel;
    SpinnerNumberModel rTnIInvarModel;
    SpinnerNumberModel rTkIInvarModel;
    SpinnerNumberModel rRTnInvarModel;
    SpinnerNumberModel rRTkInvarModel;
    SpinnerNumberModel rRTnIInvarModel;
    SpinnerNumberModel rRTkIInvarModel;

    SpinnerNumberModel rotationalArrayPModel;
    SpinnerNumberModel rotationalArrayRModel;
    SpinnerNumberModel rotationalArrayIModel;
    SpinnerNumberModel rotationalArrayRIModel;
    SpinnerNumberModel rotationalArrayIRModel;

    StringBuffer results;






    // Constructor
    public OrderedSetLists() {

	createWidgets();

	//Put everything together.
	JPanel devidedPane = new JPanel(new BorderLayout());
	devidedPane.add(sequencePanel, BorderLayout.NORTH);
	devidedPane.add(buttonPanel, BorderLayout.WEST);
	devidedPane.add(listsScrollPane, BorderLayout.CENTER);
	devidedPane.add(editorScrollPane, BorderLayout.SOUTH);

	getContentPane().add(devidedPane, BorderLayout.CENTER);

	setSize(405,600);
	openFrameCount++;
	setTitle("Ordered Set Lists " + openFrameCount);
	setLocation(xOffset + 10 * openFrameCount, 
		    yOffset + 2 * openFrameCount);
    }

	


    // Create and add the widgets.
    private void createWidgets() {

	// add a function to generate cyclic sets.

	JLabel sequenceLabel = new JLabel("Ordered PC-Set: ", 
					  JLabel.TRAILING);
	listsTextField = new JTextField(24);
	listsTextField.setBackground(Color.white);
	listsTextField.setDragEnabled(true);
	listsTextField.setToolTipText("Insert a space(s) to segment the "
				      + "sequence. Ex. 0293 476A8 B15 ");
	sequenceLabel.setLabelFor(listsTextField);
	sequenceLabel.setToolTipText("Insert a space(s) to segment the "
				     + "sequence. Ex. 0293 476A8 B15 ");

	sequencePanel = new JPanel(new GridLayout(1, 2));
	sequencePanel.add(sequenceLabel);
	sequencePanel.add(listsTextField);
	sequencePanel.setBorder(
		BorderFactory.createCompoundBorder(
		   BorderFactory.createTitledBorder(""),
		   BorderFactory.createEmptyBorder(5,5,5,5)));



	// Find button
	JButton findButton = new JButton("Find");
	findButton.setActionCommand("find");
	findButton.addActionListener(this);
	findButton.setBackground(Color.lightGray);
	findButton.setFont(new Font("SansSerif", Font.BOLD, 12));

	// Clear button
	JButton clearButton = new JButton("Clear");
	clearButton.setActionCommand("clear");
	clearButton.addActionListener(this);
	clearButton.setBackground(Color.lightGray);

	JPanel buttonPanel1 = new JPanel(new SpringLayout());
	buttonPanel1.add(findButton);
	buttonPanel1.add(clearButton);
	SpringUtilities.makeCompactGrid(buttonPanel1,
					2, 1,  //rows, cols
					1, 1,  //initX, initY
					1, 1); //xPad, yPad

	buttonPanel = new JPanel(new BorderLayout());
	buttonPanel.add(buttonPanel1, BorderLayout.NORTH);



	// Tn check box and Tn/Tk spinner
	tnCheckBox = new JCheckBox("Tk to Tn:               ");
	tnCheckBox.setToolTipText("Lists ordered pc-sets under "
				  + "transpositions by k to n.");
	tnCheckBox.setActionCommand("tn");
	tnCheckBox.addActionListener(this);
	tnModel = new SpinnerNumberModel(
					 11, //initial value
					 0, //min
					 11, //max
					 1); //step
	tnSpinner = new JSpinner(tnModel);
	tnSpinner.setEditor(new JSpinner.NumberEditor(tnSpinner));
	tnSpinner.setToolTipText("The value of k.");
	Dimension d2 = new Dimension();
	d2.setSize(0.1, 0.1);
	tnSpinner.setSize(d2);

	tkModel = new SpinnerNumberModel(
					 0, //initial value
					 0, //min
					 11, //max
					 1); //step
	tkSpinner = new JSpinner(tkModel);
	tkSpinner.setEditor(new JSpinner.NumberEditor(tkSpinner));
	tkSpinner.setToolTipText("The value of n.");
	tkSpinner.setSize(d2);


	// TnI
	tnICheckBox = new JCheckBox("TkI to TnI:");
	tnICheckBox.setToolTipText("Lists ordered pc-sets under "
				   + "inversions by k to n.");
	tnICheckBox.setActionCommand("tni");
	tnICheckBox.addActionListener(this);
	tnIModel = new SpinnerNumberModel(
					  11, //initial value
					  0, //min
					  11, //max
					  1); //step
	tnISpinner = new JSpinner(tnIModel);
	tnISpinner.setEditor(new JSpinner.NumberEditor(tnISpinner));
	tnISpinner.setToolTipText("The value of k.");
	tnISpinner.setSize(d2);

	tkIModel = new SpinnerNumberModel(
					  0, //initial value
					  0, //min
					  11, //max
					  1); //step
	tkISpinner = new JSpinner(tkIModel);
	tkISpinner.setEditor(new JSpinner.NumberEditor(tkISpinner));
	tkISpinner.setToolTipText("The value of n.");
	tkISpinner.setSize(d2);


	// Mn
	mnCheckBox = new JCheckBox("Mk to Mn:");
	mnCheckBox.setToolTipText("Lists ordered pc-sets under "
				  + "multiplications by k to n.");
	mnCheckBox.setActionCommand("mn");
	mnCheckBox.addActionListener(this);
	mnModel = new SpinnerNumberModel(
					 11, //initial value
					 1, //min
					 11, //max
					 1); //step
	mnSpinner = new JSpinner(mnModel);
	mnSpinner.setEditor(new JSpinner.NumberEditor(mnSpinner));
	mnSpinner.setToolTipText("The value of k.");
	mnSpinner.setSize(d2);

	mkModel = new SpinnerNumberModel(
					 1, //initial value
					 1, //min
					 11, //max
					 1); //step
	mkSpinner = new JSpinner(mkModel);
	mkSpinner.setEditor(new JSpinner.NumberEditor(mkSpinner));
	mkSpinner.setToolTipText("The value of n.");
	mkSpinner.setSize(d2);


	// RTn
	RTnCheckBox = new JCheckBox("RTk to RTn:"); 
	RTnCheckBox.setToolTipText("Lists ordered pc-sets under "
				   + "retrograde transpositions by k to n.");
	RTnCheckBox.setActionCommand("RTn");
	RTnCheckBox.addActionListener(this);
	RTnModel = new SpinnerNumberModel(
					  11, //initial value
					  0, //min
					  11, //max
					  1); //step
	RTnSpinner = new JSpinner(RTnModel);
	RTnSpinner.setEditor(new JSpinner.NumberEditor(RTnSpinner));
	RTnSpinner.setToolTipText("The value of k.");
	RTnSpinner.setSize(d2);

	RTkModel = new SpinnerNumberModel(
					  0, //initial value
					  0, //min
					  11, //max
					  1); //step
	RTkSpinner = new JSpinner(RTkModel);
	RTkSpinner.setEditor(new JSpinner.NumberEditor(RTkSpinner));
	RTkSpinner.setToolTipText("The value of n.");
	RTkSpinner.setSize(d2);


	// RTnI (retrograde inversion)
	RTnICheckBox = new JCheckBox("RTkI to RTnI:"); 
	RTnICheckBox.setToolTipText("Lists ordered pc-sets under "
				    + "retrograde inversions by k to n.");
	RTnICheckBox.setActionCommand("RTnI");
	RTnICheckBox.addActionListener(this);
	RTnIModel = new SpinnerNumberModel(
					   11, //initial value
					   0, //min
					   11, //max
					   1); //step
	RTnISpinner = new JSpinner(RTnIModel);
	RTnISpinner.setEditor(new JSpinner.NumberEditor(RTnISpinner));
	RTnISpinner.setToolTipText("The value of k.");
	RTnISpinner.setSize(d2);

	RTkIModel = new SpinnerNumberModel(
					   0, //initial value
					   0, //min
					   11, //max
					   1); //step
	RTkISpinner = new JSpinner(RTkIModel);
	RTkISpinner.setEditor(new JSpinner.NumberEditor(RTkISpinner));
	RTkISpinner.setToolTipText("The value of n.");
	RTkISpinner.setSize(d2);



	//Add the widgets to the container.
	JPanel listsPanel1 = new JPanel(new SpringLayout());

	JLabel space1 = new JLabel(" ", JLabel.TRAILING);
	JLabel n1 = new JLabel("n", JLabel.CENTER);
	JLabel k = new JLabel("k", JLabel.CENTER);

	listsPanel1.add(space1);
	listsPanel1.add(k);
	listsPanel1.add(n1);

	listsPanel1.add(tnCheckBox);
	listsPanel1.add(tkSpinner);
	listsPanel1.add(tnSpinner);

	listsPanel1.add(tnICheckBox);
	listsPanel1.add(tkISpinner);
	listsPanel1.add(tnISpinner);

	listsPanel1.add(mnCheckBox);
	listsPanel1.add(mkSpinner);
	listsPanel1.add(mnSpinner);

	listsPanel1.add(RTnCheckBox);
	listsPanel1.add(RTkSpinner);
	listsPanel1.add(RTnSpinner);

	listsPanel1.add(RTnICheckBox);
	listsPanel1.add(RTkISpinner);
	listsPanel1.add(RTnISpinner);

	//Lay out the panel.
	SpringUtilities.makeCompactGrid(listsPanel1,
					6, 3,  //rows, cols
					1, 1,  //initX, initY
					1, 1); //xPad, yPad
	listsPanel1.setBorder(
	      BorderFactory.createCompoundBorder(
			 BorderFactory.createTitledBorder("pc-sets under"),
			 BorderFactory.createEmptyBorder(0,0,0,0)));








	// Invariant sequences under Tn
	tnInvarCheckBox = new JCheckBox("T of sizes k to n:  ");
	tnInvarCheckBox.setToolTipText("Lists invariant pitch-class sequences "
				       + "under transpositions by k to n.");
	tnInvarCheckBox.setActionCommand("tnInvar");
	tnInvarCheckBox.addActionListener(this);
	tnInvarModel = new SpinnerNumberModel(
					      3, //initial value
					      2, //min
					      10, //max
					      1); //step
	tnInvarSpinner = new JSpinner(tnInvarModel);
	tnInvarSpinner.setEditor(new JSpinner.NumberEditor(tnInvarSpinner));
	tnInvarSpinner.setToolTipText("The value of n.");
	tnInvarSpinner.setSize(d2);

	tkInvarModel = new SpinnerNumberModel(3, //initial value
					      2, //min
					      10, //max
					      1); //step
	tkInvarSpinner = new JSpinner(tkInvarModel);
	tkInvarSpinner.setEditor(new JSpinner.NumberEditor(tkInvarSpinner));
	tkInvarSpinner.setToolTipText("The value of k.");
	tkInvarSpinner.setSize(d2);


	// Invariant sequences under TnI
	tnIInvarCheckBox = new JCheckBox("TI of sizes k to n:  ");
	tnIInvarCheckBox.setToolTipText(
				"Lists invariant pitch-class sequences "
				+ "under inversions by k to n.");
	tnIInvarCheckBox.setActionCommand("tnIInvar");
	tnIInvarCheckBox.addActionListener(this);
	tnIInvarModel = new SpinnerNumberModel(
					       3, //initial value
					       2, //min
					       10, //max
					       1); //step
	tnIInvarSpinner = new JSpinner(tnIInvarModel);
	tnIInvarSpinner.setEditor(new JSpinner.NumberEditor(tnIInvarSpinner));
	tnIInvarSpinner.setToolTipText("The value of n.");
	tnIInvarSpinner.setSize(d2);

	tkIInvarModel = new SpinnerNumberModel(3, //initial value
					       2, //min
					       10, //max
					       1); //step
	tkIInvarSpinner = new JSpinner(tkIInvarModel);
	tkIInvarSpinner.setEditor(new JSpinner.NumberEditor(tkIInvarSpinner));
	tkIInvarSpinner.setToolTipText("The value of k.");
	tkIInvarSpinner.setSize(d2);


	// Invariant sequences under Mn
	mnInvarCheckBox = new JCheckBox("M of sizes k to n:  ");
	mnInvarCheckBox.setToolTipText(
				       "Lists invariant pitch-class sequences "
				       + "under multiplications by k to n.");
	mnInvarCheckBox.setActionCommand("mnInvar");
	mnInvarCheckBox.addActionListener(this);
	mnInvarModel = new SpinnerNumberModel(3, //initial value
					      2, //min
					      10, //max
					      1); //step
	mnInvarSpinner = new JSpinner(mnInvarModel);
	mnInvarSpinner.setEditor(new JSpinner.NumberEditor(mnInvarSpinner));
	mnInvarSpinner.setToolTipText("The value of n.");
	mnInvarSpinner.setSize(d2);

	mkInvarModel = new SpinnerNumberModel(3, //initial value
					      2, //min
					      10, //max
					      1); //step
	mkInvarSpinner = new JSpinner(mkInvarModel);
	mkInvarSpinner.setEditor(new JSpinner.NumberEditor(mkInvarSpinner));
	mkInvarSpinner.setToolTipText("The value of k.");
	mkInvarSpinner.setSize(d2);


	// Invariant sequences under RTn
	RTnInvarCheckBox = new JCheckBox("RT of sizes k to n:  "); 
	RTnInvarCheckBox.setToolTipText(
			"Lists invariant pitch-class sequences "
			+ "under retrograde transpositions by k to n.");
	RTnInvarCheckBox.setActionCommand("RTnInvar");
	RTnInvarCheckBox.addActionListener(this);
	RTnInvarModel = new SpinnerNumberModel(3, //initial value
					       2, //min
					       10, //max
					       1); //step
	RTnInvarSpinner = new JSpinner(RTnInvarModel);
	RTnInvarSpinner.setEditor(new JSpinner.NumberEditor(RTnInvarSpinner));
	RTnInvarSpinner.setSize(d2);

	RTkInvarModel = new SpinnerNumberModel(3, //initial value
					       2, //min
					       10, //max
					       1); //step
	RTkInvarSpinner = new JSpinner(RTkInvarModel);
	RTkInvarSpinner.setEditor(new JSpinner.NumberEditor(RTkInvarSpinner));
	RTkInvarSpinner.setSize(d2);


	// Invariant sequences under RIn (retrograde inversion)
	RTnIInvarCheckBox = new JCheckBox("RTI of sizes k to n:  "); 
	RTnIInvarCheckBox.setToolTipText(
				 "Lists invariant pitch-class sequences "
				 + "under retrograde inversions by k to n.");
	RTnIInvarCheckBox.setActionCommand("RTnIInvar");
	RTnIInvarCheckBox.addActionListener(this);
	RTnIInvarModel = new SpinnerNumberModel(3, //initial value
						2, //min
						10, //max
						1); //step
	RTnIInvarSpinner = new JSpinner(RTnIInvarModel);
	RTnIInvarSpinner.setEditor(
			   new JSpinner.NumberEditor(RTnIInvarSpinner));
	RTnIInvarSpinner.setSize(d2);

	RTkIInvarModel = new SpinnerNumberModel(3, //initial value
						2, //min
						10, //max
						1); //step
	RTkIInvarSpinner = new JSpinner(RTkIInvarModel);
	RTkIInvarSpinner.setEditor(
			   new JSpinner.NumberEditor(RTkIInvarSpinner));
	RTkIInvarSpinner.setSize(d2);



	// Invariant sequences under rTn
	rTnInvarCheckBox = new JCheckBox("rT of sizes k to n: ");
	rTnInvarCheckBox.setToolTipText(
				"Lists invariant pitch-class sequences "
				+ "under rotations and transpositions.");
	rTnInvarCheckBox.setActionCommand("rTnInvar");
	rTnInvarCheckBox.addActionListener(this);
	rTnInvarModel = new SpinnerNumberModel(3, //initial value
					       2, //min
					       10, //max
					       1); //step
	rTnInvarSpinner = new JSpinner(rTnInvarModel);
	rTnInvarSpinner.setEditor(new JSpinner.NumberEditor(rTnInvarSpinner));
	rTnInvarSpinner.setToolTipText("The value of n.");
	rTnInvarSpinner.setSize(d2);

	rTkInvarModel = new SpinnerNumberModel(3, //initial value
					       2, //min
					       10, //max
					       1); //step
	rTkInvarSpinner = new JSpinner(rTkInvarModel);
	rTkInvarSpinner.setEditor(
				  new JSpinner.NumberEditor(rTkInvarSpinner));
	rTkInvarSpinner.setToolTipText("The value of k.");
	rTkInvarSpinner.setSize(d2);


	// Invariant sequences under rTnI
	rTnIInvarCheckBox = new JCheckBox("rTI of sizes k to n: ");
	rTnIInvarCheckBox.setToolTipText(
				 "Lists invariant pitch-class sequences "
				 + "under rotations and inversions.");
	rTnIInvarCheckBox.setActionCommand("rTnIInvar");
	rTnIInvarCheckBox.addActionListener(this);
	rTnIInvarModel = new SpinnerNumberModel(3, //initial value
						2, //min
						10, //max
						1); //step
	rTnIInvarSpinner = new JSpinner(rTnIInvarModel);
	rTnIInvarSpinner.setEditor(
			   new JSpinner.NumberEditor(rTnIInvarSpinner));
	rTnIInvarSpinner.setToolTipText("The value of n.");
	rTnIInvarSpinner.setSize(d2);

	rTkIInvarModel = new SpinnerNumberModel(3, //initial value
						2, //min
						10, //max
						1); //step
	rTkIInvarSpinner = new JSpinner(rTkIInvarModel);
	rTkIInvarSpinner.setEditor(
			   new JSpinner.NumberEditor(rTkIInvarSpinner));
	rTkIInvarSpinner.setToolTipText("The value of k.");
	rTkIInvarSpinner.setSize(d2);


	// Invariant sequences under rRTn
	rRTnInvarCheckBox = new JCheckBox("rRT of sizes k to n: "); 
	rRTnInvarCheckBox.setToolTipText(
				 "Lists invariant pitch-class sequences "
				 + "under rotations and "
				 + "retrograde transpositions.");
	rRTnInvarCheckBox.setActionCommand("rRTnInvar");
	rRTnInvarCheckBox.addActionListener(this);
	rRTnInvarModel = new SpinnerNumberModel(3, //initial value
						2, //min
						10, //max
						1); //step
	rRTnInvarSpinner = new JSpinner(rRTnInvarModel);
	rRTnInvarSpinner.setEditor(
			   new JSpinner.NumberEditor(rRTnInvarSpinner));
	rRTnInvarSpinner.setSize(d2);

	rRTkInvarModel = new SpinnerNumberModel(3, //initial value
						2, //min
						10, //max
						1); //step
	rRTkInvarSpinner = new JSpinner(rRTkInvarModel);
	rRTkInvarSpinner.setEditor(
			   new JSpinner.NumberEditor(rRTkInvarSpinner));
	rRTkInvarSpinner.setSize(d2);


	// Invariant sequences under rRIn (retrograde inversion)
	rRTnIInvarCheckBox = new JCheckBox("rRTI of sizes k to n: "); 
	rRTnIInvarCheckBox.setToolTipText(
			  "Lists invariant pitch-class sequences "
			  + "under rotations and retrograde inversions.");
	rRTnIInvarCheckBox.setActionCommand("rRTnIInvar");
	rRTnIInvarCheckBox.addActionListener(this);
	rRTnIInvarModel = new SpinnerNumberModel(3, //initial value
						 2, //min
						 10, //max
						 1); //step
	rRTnIInvarSpinner = new JSpinner(rRTnIInvarModel);
	rRTnIInvarSpinner.setEditor(
			    new JSpinner.NumberEditor(rRTnIInvarSpinner));
	rRTnIInvarSpinner.setSize(d2);

	rRTkIInvarModel = new SpinnerNumberModel(3, //initial value
						 2, //min
						 10, //max
						 1); //step
	rRTkIInvarSpinner = new JSpinner(rRTkIInvarModel);
	rRTkIInvarSpinner.setEditor(
			    new JSpinner.NumberEditor(rRTkIInvarSpinner));
	rRTkIInvarSpinner.setSize(d2);



	JPanel listsPanel2 = new JPanel(new SpringLayout());

	listsPanel2.add(tnInvarCheckBox);
	listsPanel2.add(tkInvarSpinner);
	listsPanel2.add(tnInvarSpinner);

	listsPanel2.add(tnIInvarCheckBox);
	listsPanel2.add(tkIInvarSpinner);
	listsPanel2.add(tnIInvarSpinner);

	listsPanel2.add(mnInvarCheckBox);
	listsPanel2.add(mkInvarSpinner);
	listsPanel2.add(mnInvarSpinner);

	listsPanel2.add(RTnInvarCheckBox);
	listsPanel2.add(RTkInvarSpinner);
	listsPanel2.add(RTnInvarSpinner);

	listsPanel2.add(RTnIInvarCheckBox);
	listsPanel2.add(RTkIInvarSpinner);
	listsPanel2.add(RTnIInvarSpinner);

	listsPanel2.add(rTnInvarCheckBox);
	listsPanel2.add(rTkInvarSpinner);
	listsPanel2.add(rTnInvarSpinner);

	listsPanel2.add(rTnIInvarCheckBox);
	listsPanel2.add(rTkIInvarSpinner);
	listsPanel2.add(rTnIInvarSpinner);

	listsPanel2.add(rRTnInvarCheckBox);
	listsPanel2.add(rRTkInvarSpinner);
	listsPanel2.add(rRTnInvarSpinner);

	listsPanel2.add(rRTnIInvarCheckBox);
	listsPanel2.add(rRTkIInvarSpinner);
	listsPanel2.add(rRTnIInvarSpinner);

	//Lay out the panel.
	SpringUtilities.makeCompactGrid(listsPanel2,
					9, 3,  //rows, cols
					1, 1,  //initX, initY
					1, 1); //xPad, yPad
	listsPanel2.setBorder(
		      BorderFactory.createCompoundBorder(
				 BorderFactory.createTitledBorder(
					  "invariant subsequences under"),
				 BorderFactory.createEmptyBorder(0,0,0,0)));






	// P rotational array
	rotationalArrayPCheckBox 
	    = new JCheckBox("P rotational array of size:");
	rotationalArrayPCheckBox.setToolTipText(
				"Lists the P rotational array of size n.");
	rotationalArrayPCheckBox.setActionCommand("rotationalArrayP");
	rotationalArrayPCheckBox.addActionListener(this);
	rotationalArrayPModel = new SpinnerNumberModel(6, //initial value
						       3, //min
						       12, //max
						       1); //step
	rotationalArrayPSpinner = new JSpinner(rotationalArrayPModel);
	rotationalArrayPSpinner.setEditor(
			  new JSpinner.NumberEditor(rotationalArrayPSpinner));
	rotationalArrayPSpinner.setToolTipText(
			       "The number of the layers of the array.");
	rotationalArrayPSpinner.setSize(d2);



	// R rotational array
	rotationalArrayRCheckBox 
	    = new JCheckBox("R rotational array of size:");
	rotationalArrayRCheckBox.setToolTipText(
				"Lists the R rotational array of size n.");
	rotationalArrayRCheckBox.setActionCommand("rotationalArrayR");
	rotationalArrayRCheckBox.addActionListener(this);
	rotationalArrayRModel = new SpinnerNumberModel(6, //initial value
						       3, //min
						       12, //max
						       1); //step
	rotationalArrayRSpinner = new JSpinner(rotationalArrayRModel);
	rotationalArrayRSpinner.setEditor(
			  new JSpinner.NumberEditor(rotationalArrayRSpinner));
	rotationalArrayRSpinner.setToolTipText(
			       "The number of the layers of the array.");
	rotationalArrayRSpinner.setSize(d2);



	// I rotational array
	rotationalArrayICheckBox 
	    = new JCheckBox("I rotational array of size:");
	rotationalArrayICheckBox.setToolTipText(
				"Lists the I rotational array of size n.");
	rotationalArrayICheckBox.setActionCommand("rotationalArrayI");
	rotationalArrayICheckBox.addActionListener(this);
	rotationalArrayIModel = new SpinnerNumberModel(6, //initial value
						       3, //min
						       12, //max
						       1); //step
	rotationalArrayISpinner = new JSpinner(rotationalArrayIModel);
	rotationalArrayISpinner.setEditor(
			  new JSpinner.NumberEditor(rotationalArrayISpinner));
	rotationalArrayISpinner.setToolTipText(
			       "The number of the layers of the array.");
	rotationalArrayISpinner.setSize(d2);



	// RI rotational array
	rotationalArrayRICheckBox 
	    = new JCheckBox("RI rotational array of size:  ");
	rotationalArrayRICheckBox.setToolTipText(
				 "Lists the RI rotational array of size n.");
	rotationalArrayRICheckBox.setActionCommand("rotationalArrayRI");
	rotationalArrayRICheckBox.addActionListener(this);
	rotationalArrayRIModel = new SpinnerNumberModel(6, //initial value
							3, //min
							12, //max
							1); //step
	rotationalArrayRISpinner = new JSpinner(rotationalArrayRIModel);
	rotationalArrayRISpinner.setEditor(
		   new JSpinner.NumberEditor(rotationalArrayRISpinner));
	rotationalArrayRISpinner.setToolTipText(
				"The number of the layers of the array.");
	rotationalArrayRISpinner.setSize(d2);



	// IR rotational array
	rotationalArrayIRCheckBox 
	    = new JCheckBox("IR rotational array of size:  ");
	rotationalArrayIRCheckBox.setToolTipText(
				 "Lists the IR rotational array of size n.");
	rotationalArrayIRCheckBox.setActionCommand("rotationalArrayIR");
	rotationalArrayIRCheckBox.addActionListener(this);
	rotationalArrayIRModel = new SpinnerNumberModel(6, //initial value
							3, //min
							12, //max
							1); //step
	rotationalArrayIRSpinner = new JSpinner(rotationalArrayIRModel);
	rotationalArrayIRSpinner.setEditor(
		   new JSpinner.NumberEditor(rotationalArrayIRSpinner));
	rotationalArrayIRSpinner.setToolTipText(
				"The number of the layers of the array.");
	rotationalArrayIRSpinner.setSize(d2);




	JPanel listsPanel3 = new JPanel(new SpringLayout());

	listsPanel3.add(rotationalArrayPCheckBox);
	listsPanel3.add(rotationalArrayPSpinner);

	listsPanel3.add(rotationalArrayRCheckBox);
	listsPanel3.add(rotationalArrayRSpinner);

	listsPanel3.add(rotationalArrayICheckBox);
	listsPanel3.add(rotationalArrayISpinner);

	listsPanel3.add(rotationalArrayRICheckBox);
	listsPanel3.add(rotationalArrayRISpinner);

	listsPanel3.add(rotationalArrayIRCheckBox);
	listsPanel3.add(rotationalArrayIRSpinner);

	//Lay out the panel.
	SpringUtilities.makeCompactGrid(listsPanel3,
					5, 2,  //rows, cols
					1, 1,  //initX, initY
					1, 1); //xPad, yPad
	listsPanel3.setBorder(
	      BorderFactory.createCompoundBorder(
				 BorderFactory.createTitledBorder(""),
				 BorderFactory.createEmptyBorder(0,0,0,0)));



	JPanel listsPanel = new JPanel(new SpringLayout());
	listsPanel.add(listsPanel1);
	listsPanel.add(listsPanel2);
	listsPanel.add(listsPanel3);
	//Lay out the panel.
	SpringUtilities.makeCompactGrid(listsPanel,
					3, 1,  //rows, cols
					1, 1,  //initX, initY
					1, 1); //xPad, yPad
	listsPanel.setBorder(
			     BorderFactory.createCompoundBorder(
				BorderFactory.createTitledBorder(""),
				BorderFactory.createEmptyBorder(5,5,5,5)));

	listsScrollPane = new JScrollPane(listsPanel);
	listsScrollPane.setVerticalScrollBarPolicy(
			   JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
	listsScrollPane.setHorizontalScrollBarPolicy(
			     JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
	listsScrollPane.setPreferredSize(new Dimension(330, 180));
	listsScrollPane.setMinimumSize(new Dimension(330, 180));







	// "Found Relations" area
	editorPane = new JEditorPane();
	editorScrollPane = new JScrollPane(editorPane);
	editorScrollPane.setVerticalScrollBarPolicy(
			    JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
	editorScrollPane.setHorizontalScrollBarPolicy(
			      JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
	editorScrollPane.setPreferredSize(new Dimension(250, 300));
	editorScrollPane.setMinimumSize(new Dimension(250, 300));
	editorScrollPane.setBorder(
		   BorderFactory.createCompoundBorder(
		      BorderFactory.createTitledBorder("Found: "),
		      BorderFactory.createEmptyBorder(5,5,5,5)));
	editorScrollPane.getBorder();
    }







    public void actionPerformed(ActionEvent e) {

	if ("find".equals(e.getActionCommand())) {
	    editorPane.setText("");

	    results = new StringBuffer();
	    PcSet pq = new PcSet(); // just for the methods used below
			
	    // Get the typed-in orderd pc-set and
	    // remove space(s) at the beginning and the end of it.
	    String typedSequenceString 
		= pq.removeFistLastSpaces(listsTextField.getText());

	    // Remove consecutive spaces.
	    String noConsecutiveSpacesString 
	    	= pq.removeConsecutiveSpaces(typedSequenceString);

	    // Eliminate duplicates
	    // String duplicatesFreeSequenceString 
	    //	= pq.removeDuplicatesInString(noConsecutiveSpacesString);

	    // Check if there are non-numerical characters
	    String illegalChars 
		= pq.checkNonNumerals(noConsecutiveSpacesString);

	    // Puts each subsequence of the segmented tone row in an array.
	    char[] sequenceChars 
		= new char[noConsecutiveSpacesString.length()];
	    noConsecutiveSpacesString.getChars(
			       0, 
			       noConsecutiveSpacesString.length(), 
			       sequenceChars, 
			       0);

	    char[][] tmpSegmentsCharArray = new char[24][24];
	    int[] sizeOfEachSegment = new int[24];
	    int numberOfSegments = 0;

	    for (int i = 0; i < sequenceChars.length; i++) {
		while (sequenceChars[i] == ' ' && i < sequenceChars.length)
		    i++;

		sizeOfEachSegment[numberOfSegments] = 0;
		while (sequenceChars[i] != ' ') {
   tmpSegmentsCharArray[numberOfSegments][sizeOfEachSegment[numberOfSegments]] 
       = sequenceChars[i];
		    sizeOfEachSegment[numberOfSegments]++;
		    i++;

		    if (i > sequenceChars.length - 1)
			break;
		}
		numberOfSegments++;
	    }

	    char[][] segmentsCharArray = new char[numberOfSegments][];
	    for (int i = 0; i < segmentsCharArray.length; i++) {
		segmentsCharArray[i] = new char[sizeOfEachSegment[i]];
		System.arraycopy(tmpSegmentsCharArray[i], 
				 0, 
				 segmentsCharArray[i], 
				 0, 
				 sizeOfEachSegment[i]);
	    }



	    // Check errors.
	    boolean goAhead = true;

	    if (noConsecutiveSpacesString.length() == 0) {
		JOptionPane.showMessageDialog(
				      sequencePanel,
				      "The size of each sequence must be " 
				      + "at least 2.",
				      "Warning",
				      JOptionPane.WARNING_MESSAGE);
		goAhead = false;
	    }

	    // Check if the size of each segmented subsequence is at least 2.
	    for (int iiii = 0; iiii < numberOfSegments; iiii++) {
		if (sizeOfEachSegment[iiii] < 2 
		    || 12 < sizeOfEachSegment[iiii]) {
		    JOptionPane.showMessageDialog(
		      sequencePanel,
		      "The size of each sequence must be \n" 
		      + "between 2 and 12 inclusive.",
			  "Warning",
			  JOptionPane.WARNING_MESSAGE);
		    goAhead = false;
		}
	    }

	    // At least one operation is chosen?
	    if (tnCheckBox.getSelectedObjects() == null &&
		tnICheckBox.getSelectedObjects() == null &&
		mnCheckBox.getSelectedObjects() == null &&
		tnInvarCheckBox.getSelectedObjects() == null &&
		tnIInvarCheckBox.getSelectedObjects() == null &&
		mnInvarCheckBox.getSelectedObjects() == null &&
		RTnCheckBox.getSelectedObjects() == null &&
		RTnICheckBox.getSelectedObjects() == null &&
		tnInvarCheckBox.getSelectedObjects() == null &&
		tnIInvarCheckBox.getSelectedObjects() == null &&
		mnInvarCheckBox.getSelectedObjects() == null &&
		RTnInvarCheckBox.getSelectedObjects() == null &&
		RTnIInvarCheckBox.getSelectedObjects() == null &&
		rTnInvarCheckBox.getSelectedObjects() == null &&
		rTnIInvarCheckBox.getSelectedObjects() == null &&
		rRTnInvarCheckBox.getSelectedObjects() == null &&
		rRTnIInvarCheckBox.getSelectedObjects() == null &&
		rotationalArrayPCheckBox.getSelectedObjects() == null &&
		rotationalArrayRCheckBox.getSelectedObjects() == null &&
		rotationalArrayICheckBox.getSelectedObjects() == null &&
		rotationalArrayRICheckBox.getSelectedObjects() == null &&
		rotationalArrayIRCheckBox.getSelectedObjects() == null) {
		JOptionPane.showMessageDialog(sequencePanel, 
					      "You need to choose at least "
					      + "one of the operations.\n",
					      "Warning",
					      JOptionPane.WARNING_MESSAGE);
		goAhead = false;
	    } else if (illegalChars.length() > 1) {
		JOptionPane.showMessageDialog(
		      sequencePanel, 
		      "Non-numerical characters are found in Ordered PC-Set:"
		      + illegalChars.toString(),
		      "Warning",
		      JOptionPane.WARNING_MESSAGE);
		goAhead = false;
	    } else {
		if (goAhead == false)
		    goAhead = false;
		else
		    goAhead = true;
	    }



	    // If no error is found, the following will be carried out.
	    if (goAhead == true) {

		PcSet [] orderedPcSets = new PcSet[numberOfSegments];

		for (int i = 0; i < orderedPcSets.length; i++) {
		    orderedPcSets[i] = new PcSet(sizeOfEachSegment[i]);
		    orderedPcSets[i].set 
		    = orderedPcSets[i].charArrayToIntOne(segmentsCharArray[i]);
		}


		// Tk to Tn
		if (tnCheckBox.getSelectedObjects() != null) {
		    int k = tnModel.getNumber().intValue();
		    int n = tkModel.getNumber().intValue();

		    // For each segment of Ordered PC-Set
		    for (int ii = 0; ii < orderedPcSets.length; ii++) {

			StringBuffer eachSegStrBuffer = new StringBuffer("");
			for (int h = 0; h < sizeOfEachSegment[ii]; h++)
			    eachSegStrBuffer.append(segmentsCharArray[ii][h]);

			String eachSegString = eachSegStrBuffer.toString();
			results.append("Tk to Tn: <"
				       + eachSegString
				       + ">\n");

			int [][] pcsList 
	    = new int[Math.abs(n - k) + 1][orderedPcSets[ii].set.length];

			if (k <= n)
			    pcsList = orderedPcSets[ii].getTns(k, n);
			else
			    pcsList = orderedPcSets[ii].getTns(n, k);
		
			// For each "n" of Tn
			for (int i = 0; i < pcsList.length; i++) {
			    Set s = new Set(pcsList[i]);
			    s.base12String = orderedPcSets[ii].base12String;
			    char [] c = s.intArrayToCharOne(s.set);

			    if (k <= n)
				results.append("    T" 
					       + (k + i) 
					       + ": <"
					       + eachSegString
					       + ">  -->  <");
			    else
				results.append("    T" 
					       + (n + i) 
					       + ": <"
					       + eachSegString
					       + ">  -->  <");
				
			    for (int j = 0; j < sizeOfEachSegment[ii]; j++)
				results.append(c[j]);

			    results.append(">\n");
			}
			results.append("\n");
		    }
		}


		// TkI to TnI
		if (tnICheckBox.getSelectedObjects() != null) {
		    int k = tnIModel.getNumber().intValue();
		    int n = tkIModel.getNumber().intValue();

		    // For each segment of Ordered PC-Set
		    for (int ii = 0; ii < orderedPcSets.length; ii++) {

			StringBuffer eachSegStrBuffer = new StringBuffer("");
			for (int h = 0; h < sizeOfEachSegment[ii]; h++)
			    eachSegStrBuffer.append(segmentsCharArray[ii][h]);

			String eachSegString = eachSegStrBuffer.toString();
			results.append("TkI to TnI: <"
				       + eachSegString
				       + ">\n");

			int [][] pcsList 
			= new int[Math.abs(n - k) + 1][orderedPcSets[ii].set.length];

			if (k <= n)
			    pcsList = orderedPcSets[ii].getTnIs(k, n);
			else
			    pcsList = orderedPcSets[ii].getTnIs(n, k);

			// For each "n" of TnI
			for (int i = 0; i < pcsList.length; i++) {
			    Set s = new Set(pcsList[i]);
			    s.base12String = orderedPcSets[ii].base12String;
			    char [] c = s.intArrayToCharOne(s.set);

			    int nn 
				= s.tnItoIn(orderedPcSets[ii].set, pcsList[i]);
				
			    if (k <= n)
				results.append("    T" 
					       + (k + i) 
					       + "I: <"
					       + eachSegString
					       + ">  -->  <");
			    else
				results.append("    T" 
					       + (n + i) 
					       + "I: <"
					       + eachSegString
					       + ">  -->  <");


			    for (int j = 0; j < sizeOfEachSegment[ii]; j++)
				results.append(c[j]);
			    
			    results.append(">"
					   + " (I" 
					   + nn 
					   + ")\n");
			}
			results.append("\n");
		    }
		}


		// Mk to Mn
		if (mnCheckBox.getSelectedObjects() != null) {
		    int k = mnModel.getNumber().intValue();
		    int n = mkModel.getNumber().intValue();

		    // For each segment of Ordered PC-Set
		    for (int ii = 0; ii < orderedPcSets.length; ii++) {

			StringBuffer eachSegStrBuffer = new StringBuffer("");
			for (int h = 0; h < sizeOfEachSegment[ii]; h++)
			    eachSegStrBuffer.append(segmentsCharArray[ii][h]);

			String eachSegString = eachSegStrBuffer.toString();
			results.append("Mk to Mn: <"
				       + eachSegString
				       + ">\n");

			int [][] pcsList 
			    = new int[Math.abs(n - k) + 1][orderedPcSets[ii].set.length];

			if (k <= n)
			    pcsList = orderedPcSets[ii].getMns(k, n);
			else
			    pcsList = orderedPcSets[ii].getMns(n, k);

			// For each "n" of Mn
			for (int i = 0; i < pcsList.length; i++) {
			    Set s = new Set(pcsList[i]);
			    s.base12String = orderedPcSets[ii].base12String;
			    char [] c = s.intArrayToCharOne(s.set);

			    if (k <= n)
				results.append("    M" 
					       + (k + i) 
					       + ": <"
					       + eachSegString
					       + ">  -->  <");
			    else
				results.append("    M" 
					       + (n + i) 
					       + ": <"
					       + eachSegString
					       + ">  -->  <");
			    
			    for (int j = 0; j < sizeOfEachSegment[ii]; j++)
				results.append(c[j]);

			    results.append(">\n");
			}
			results.append("\n");
		    }
		}



		// RTk to RTn
		if (RTnCheckBox.getSelectedObjects() != null) {
		    int k = RTnModel.getNumber().intValue();
		    int n = RTkModel.getNumber().intValue();

		    // For each segment of Ordered PC-Set
		    for (int ii = 0; ii < orderedPcSets.length; ii++) {

			StringBuffer eachSegStrBuffer = new StringBuffer("");
			for (int h = 0; h < sizeOfEachSegment[ii]; h++)
			    eachSegStrBuffer.append(segmentsCharArray[ii][h]);

			String eachSegString = eachSegStrBuffer.toString();
			results.append("RTk to RTn: <"
				       + eachSegString
				       + ">\n");

			PcSet RorderedPcSet = new PcSet(sizeOfEachSegment[ii]);
			for (int h = 0; h < sizeOfEachSegment[ii]; h++)
			    RorderedPcSet.set[h] = orderedPcSets[ii].set[h];

			int [][] pcsList 
			 = new int[Math.abs(n - k) + 1][sizeOfEachSegment[ii]];

			if (k <= n)
			    pcsList = RorderedPcSet.getRTns(k, n);
			else
			    pcsList = RorderedPcSet.getRTns(n, k);
					
			// For each "n" of RTn
			for (int i = 0; i < pcsList.length; i++) {
			    Set s = new Set(pcsList[i]);
			    s.base12String = orderedPcSets[ii].base12String;
			    char [] c = s.intArrayToCharOne(s.set);

			    if (k <= n)
				results.append("    RT" 
					       + (k + i) 
					       + ": <"
					       + eachSegString
					       + ">  -->  <");
			    else
				results.append("    RT" 
					       + (n + i) 
					       + ": <"
					       + eachSegString
					       + ">  -->  <");

			    for (int j = 0; j < sizeOfEachSegment[ii]; j++)
				results.append(c[j]);

			    results.append(">\n");
			}
			results.append("\n");
		    }
		}



		// RTkI to RTnI
		if (RTnICheckBox.getSelectedObjects() != null) {
		    int k = RTnIModel.getNumber().intValue();
		    int n = RTkIModel.getNumber().intValue();

		    // For each segment of Ordered PC-Set
		    for (int ii = 0; ii < orderedPcSets.length; ii++) {

			StringBuffer eachSegStrBuffer = new StringBuffer("");
			for (int h = 0; h < sizeOfEachSegment[ii]; h++)
			    eachSegStrBuffer.append(segmentsCharArray[ii][h]);

			String eachSegString = eachSegStrBuffer.toString();
			results.append("RTkI to RTnI: <"
				       + eachSegString
				       + ">\n");

			PcSet RorderedPcSet = new PcSet(sizeOfEachSegment[ii]);
			for (int h = 0; h < sizeOfEachSegment[ii]; h++)
			    RorderedPcSet.set[h] = orderedPcSets[ii].set[h];

			int [][] pcsList 
			    = new int[Math.abs(n - k) + 1][orderedPcSets[ii].set.length];

			if (k <= n)
			    pcsList = RorderedPcSet.getRTnIs(k, n);
			else
			    pcsList = RorderedPcSet.getRTnIs(n, k);
					
			// For each "n" of RTnI
			for (int i = 0; i < pcsList.length; i++) {
			    Set s = new Set(pcsList[i]);
			    s.base12String = orderedPcSets[ii].base12String;
			    char [] c = s.intArrayToCharOne(s.set);

			    if (k <= n)
				results.append("    RT" 
					       + (k + i) 
					       + "I: <"
					       + eachSegString
					       + ">  -->  <");
			    else
				results.append("    RT" 
					       + (n + i) 
					       + ":I <"
					       + eachSegString
					       + ">  -->  <");

			    for (int j = 0; j < sizeOfEachSegment[ii]; j++)
				results.append(c[j]);

			    results.append(">\n");
			}
			results.append("\n");
		    }
		}





		// To find invariant subsequences
		if (tnInvarCheckBox.getSelectedObjects() != null ||
		    tnIInvarCheckBox.getSelectedObjects() != null ||
		    mnInvarCheckBox.getSelectedObjects() != null ||
		    RTnInvarCheckBox.getSelectedObjects() != null ||
		    RTnIInvarCheckBox.getSelectedObjects() != null ||
		    rTnInvarCheckBox.getSelectedObjects() != null ||
		    rTnIInvarCheckBox.getSelectedObjects() != null ||
		    rRTnInvarCheckBox.getSelectedObjects() != null ||
		    rRTnIInvarCheckBox.getSelectedObjects() != null) {

		    results.append("\n");
			
		    if (tnInvarCheckBox.getSelectedObjects() != null) {
			int k = tkInvarModel.getNumber().intValue();
			int n = tnInvarModel.getNumber().intValue();

			if (k > n) {
			    int tmp = n;
			    n = k;
			    k = tmp;
			}

			for (int ii = 0; ii < orderedPcSets.length; ii++) {
			    if (n > sizeOfEachSegment[ii])
				JOptionPane.showMessageDialog(
				      sequencePanel, 
				      "The sizes of invariants must be \n"
				      + "smaller than that of each "
				      + "subsequence.\n",
				      "Warning",
				      JOptionPane.WARNING_MESSAGE);
			    else {
				InvariantsList invr= new InvariantsList(
							orderedPcSets[ii], 
							sizeOfEachSegment[ii],
							0, 
							11);
				invr.base12String 
				    = orderedPcSets[ii].base12String;

				results.append(invr.orderedTnInvariants(k, n));
			    }
			}
		    }


		    if (tnIInvarCheckBox.getSelectedObjects() != null) {
			int k = tkIInvarModel.getNumber().intValue();
			int n = tnIInvarModel.getNumber().intValue();

			if (k > n) {
			    int tmp = n;
			    n = k;
			    k = tmp;
			}

			for (int ii = 0; ii < orderedPcSets.length; ii++) {
			    if (n > sizeOfEachSegment[ii])
				JOptionPane.showMessageDialog(
					sequencePanel, 
					"The sizes of invariants must be \n"
					+ "smaller than that of each "
					+ "subsequence.\n",
					"Warning",
					JOptionPane.WARNING_MESSAGE);
			    else {
				InvariantsList invr= new InvariantsList(
							orderedPcSets[ii], 
							sizeOfEachSegment[ii],
							0, 
							11);
				invr.base12String 
				    = orderedPcSets[ii].base12String;

				results.append(
				       invr.orderedTnIInvariants(k, n));
			    }
			}
		    }


		    if (mnInvarCheckBox.getSelectedObjects() != null) {
			int k = mkInvarModel.getNumber().intValue(); 
			int n = mnInvarModel.getNumber().intValue();

			if (k > n) {
			    int tmp = n;
			    n = k;
			    k = tmp;
			}

			for (int ii = 0; ii < orderedPcSets.length; ii++) {
			    if (n > sizeOfEachSegment[ii])
				JOptionPane.showMessageDialog(
					sequencePanel, 
					"The sizes of invariants must be \n"
					+ "smaller than that of each "
					+ "subsequence.\n",
					"Warning",
					JOptionPane.WARNING_MESSAGE);
			    else {
				InvariantsList invr= new InvariantsList(
							orderedPcSets[ii], 
							sizeOfEachSegment[ii],
							0, 
							11);
				invr.base12String 
				    = orderedPcSets[ii].base12String;

				results.append(invr.orderedMnInvariants(k, n));
			    }
			}
		    }


		    if (RTnInvarCheckBox.getSelectedObjects() != null) {
			int k = RTkInvarModel.getNumber().intValue();
			int n = RTnInvarModel.getNumber().intValue();

			if (k > n) {
			    int tmp = n;
			    n = k;
			    k = tmp;
			}

			for (int ii = 0; ii < orderedPcSets.length; ii++) {
			    if (n > sizeOfEachSegment[ii])
				JOptionPane.showMessageDialog(
					sequencePanel, 
					"The sizes of invariants must be \n"
					+ "smaller than that of each "
					+ "subsequence.\n",
					"Warning",
					JOptionPane.WARNING_MESSAGE);
			    else {
				InvariantsList invr= new InvariantsList(
							orderedPcSets[ii], 
							sizeOfEachSegment[ii],
							0, 
							11);
				invr.base12String 
				    = orderedPcSets[ii].base12String;

				results.append(
				       invr.orderedRTnInvariants(k, n));
			    }
			}
		    }


		    if (RTnIInvarCheckBox.getSelectedObjects() != null) {
			int k = RTkIInvarModel.getNumber().intValue();
			int n = RTnIInvarModel.getNumber().intValue();

			if (k > n) {
			    int tmp = n;
			    n = k;
			    k = tmp;
			}

			for (int ii = 0; ii < orderedPcSets.length; ii++) {
			    if (n > sizeOfEachSegment[ii])
				JOptionPane.showMessageDialog(
					sequencePanel, 
					"The sizes of invariants must be \n"
					+ "smaller than that of each "
					+ "subsequence.\n",
					"Warning",
					JOptionPane.WARNING_MESSAGE);
			    else {
				InvariantsList invr= new InvariantsList(
							orderedPcSets[ii], 
							sizeOfEachSegment[ii],
							0, 
							11);
				invr.base12String 
				    = orderedPcSets[ii].base12String;

				results.append(
				       invr.orderedRTnIInvariants(k, n));
			    }
			}
		    }

    
		    if (rTnInvarCheckBox.getSelectedObjects() != null) {
			int k = rTkInvarModel.getNumber().intValue();
			int n = rTnInvarModel.getNumber().intValue();

			if (k > n) {
			    int tmp = n;
			    n = k;
			    k = tmp;
			}

			for (int ii = 0; ii < orderedPcSets.length; ii++) {
			    if (n > sizeOfEachSegment[ii])
				JOptionPane.showMessageDialog(
					sequencePanel, 
					"The sizes of invariants must be \n"
					+ "smaller than that of each "
					+ "subsequence.\n",
					"Warning",
					JOptionPane.WARNING_MESSAGE);
			    else {
				InvariantsList invr= new InvariantsList(
							orderedPcSets[ii], 
							sizeOfEachSegment[ii],
							0, 
							11);
				invr.base12String 
				    = orderedPcSets[ii].base12String;

				results.append(
				       invr.orderedrTnInvariants(k, n));
			    }
			}
		    }


		    if (rTnIInvarCheckBox.getSelectedObjects() != null) {
			int k = rTkIInvarModel.getNumber().intValue();
			int n = rTnIInvarModel.getNumber().intValue();

			if (k > n) {
			    int tmp = n;
			    n = k;
			    k = tmp;
			}

			for (int ii = 0; ii < orderedPcSets.length; ii++) {
			    if (n > sizeOfEachSegment[ii])
				JOptionPane.showMessageDialog(
					sequencePanel, 
					"The sizes of invariants must be \n"
					+ "smaller than that of each "
					+ "subsequence.\n",
					"Warning",
					JOptionPane.WARNING_MESSAGE);
			    else {
				InvariantsList invr= new InvariantsList(
							orderedPcSets[ii], 
							sizeOfEachSegment[ii],
							0, 
							11);
				invr.base12String 
				    = orderedPcSets[ii].base12String;

				results.append(
				       invr.orderedrTnIInvariants(k, n));
			    }
			}
		    }


		    if (rRTnInvarCheckBox.getSelectedObjects() != null) {
			int k = rRTkInvarModel.getNumber().intValue();
			int n = rRTnInvarModel.getNumber().intValue();

			if (k > n) {
			    int tmp = n;
			    n = k;
			    k = tmp;
			}

			for (int ii = 0; ii < orderedPcSets.length; ii++) {
			    if (n > sizeOfEachSegment[ii])
				JOptionPane.showMessageDialog(
					sequencePanel, 
					"The sizes of invariants must be \n"
					+ "smaller than that of each "
					+ "subsequence.\n",
					"Warning",
					JOptionPane.WARNING_MESSAGE);
			    else {
				InvariantsList invr= new InvariantsList(
							orderedPcSets[ii], 
							sizeOfEachSegment[ii],
							0, 
							11);
				invr.base12String 
				    = orderedPcSets[ii].base12String;

				results.append(
				       invr.orderedrRTnInvariants(k, n));
			    }
			}
		    }


		    if (rRTnIInvarCheckBox.getSelectedObjects() != null) {
			int k = rRTkIInvarModel.getNumber().intValue();
			int n = rRTnIInvarModel.getNumber().intValue();

			if (k > n) {
			    int tmp = n;
			    n = k;
			    k = tmp;
			}

			for (int ii = 0; ii < orderedPcSets.length; ii++) {
			    if (n > sizeOfEachSegment[ii])
				JOptionPane.showMessageDialog(
					sequencePanel, 
					"The sizes of invariants must be \n"
					+ "smaller than that of each "
					+ "subsequence.\n",
					"Warning",
					JOptionPane.WARNING_MESSAGE);
			    else {
				InvariantsList invr= new InvariantsList(
							orderedPcSets[ii], 
							sizeOfEachSegment[ii],
							0, 
							11);
				invr.base12String 
				    = orderedPcSets[ii].base12String;

				results.append(
				       invr.orderedrRTnIInvariants(k, n));
			    }
			}
		    }
		}






		// To list rotational arrays in five forms
		if (rotationalArrayPCheckBox.getSelectedObjects() != null) {
		    results.append("\n");
		    int n = rotationalArrayPModel.getNumber().intValue();

		    for (int ii = 0; ii < orderedPcSets.length; ii++) {

			if (sizeOfEachSegment[ii] < n)
			    JOptionPane.showMessageDialog(
				  null, 
				  "The number of the layers of "
				  + "the rotational array must be \n"
				  + "less than or equal to the size of "
				  + "each sequence.\n",
				  "Warning",
				  JOptionPane.WARNING_MESSAGE);
			else {
			    RotationalArray rary 
				= new RotationalArray(orderedPcSets[ii], n);
			    results.append(rary.printRotationalArrayP());
			}
		    }
		}


		if (rotationalArrayRCheckBox.getSelectedObjects() != null) {
		    results.append("\n");
		    int n = rotationalArrayRModel.getNumber().intValue();

		    for (int ii = 0; ii < orderedPcSets.length; ii++) {
			if (sizeOfEachSegment[ii] < n)
			    JOptionPane.showMessageDialog(
				  null, 
				  "The number of the layers of "
				  + "the rotational array must be \n"
				  + "less than or equal to the size of "
				  + "each sequence.\n",
				  "Warning",
				  JOptionPane.WARNING_MESSAGE);
			else {
			    RotationalArray rary 
				= new RotationalArray(orderedPcSets[ii], n);
			    results.append(rary.printRotationalArrayR());
			}
		    }
		}


		if (rotationalArrayICheckBox.getSelectedObjects() != null) {
		    results.append("\n");
		    int n = rotationalArrayIModel.getNumber().intValue();
		    
		    for (int ii = 0; ii < orderedPcSets.length; ii++) {
			if (sizeOfEachSegment[ii] < n)
			    JOptionPane.showMessageDialog(
				  null, 
				  "The number of the layers of "
				  + "the rotational array must be \n"
				  + "less than or equal to the size of "
				  + "each sequence.\n",
				  "Warning",
				  JOptionPane.WARNING_MESSAGE);
			else {
			    RotationalArray rary 
				= new RotationalArray(orderedPcSets[ii], n);
			    results.append(rary.printRotationalArrayI());
			}
		    }
		}


		if (rotationalArrayRICheckBox.getSelectedObjects() != null) {
		    results.append("\n");
		    int n = rotationalArrayRIModel.getNumber().intValue();

		    for (int ii = 0; ii < orderedPcSets.length; ii++) {
			if (sizeOfEachSegment[ii] < n)
			    JOptionPane.showMessageDialog(
				  null, 
				  "The number of the layers of "
				  + "the rotational array must be \n"
				  + "less than or equal to the size of "
				  + "each sequence.\n",
				  "Warning",
				  JOptionPane.WARNING_MESSAGE);
			else {
			    RotationalArray rary 
				= new RotationalArray(orderedPcSets[ii], n);
			    results.append(rary.printRotationalArrayRI());
			}
		    }
		}


		if (rotationalArrayIRCheckBox.getSelectedObjects() != null) {
		    results.append("\n");
		    int n = rotationalArrayIRModel.getNumber().intValue();

		    for (int ii = 0; ii < orderedPcSets.length; ii++) {
			if (sizeOfEachSegment[ii] < n)
			    JOptionPane.showMessageDialog(
				  null, 
				  "The number of the layers of "
				  + "the rotational array must be \n"
				  + "less than or equal to the size of "
				  + "each sequence.\n",
				  "Warning",
				  JOptionPane.WARNING_MESSAGE);
			else {
			    RotationalArray rary 
				= new RotationalArray(orderedPcSets[ii], n);
			    results.append(rary.printRotationalArrayIR());
			}
		    }
		}

		editorPane.setText(results.toString());
	    }
	} 




	if ("clear".equals(e.getActionCommand())) {
	    listsTextField.setText("");
	    editorPane.setText("");
	}
    }
}
