// OrderedSetRelations.java, used for
// JDubiel.java: computer program for pc-set analysis
// by Akira Takaoka, akira.takaoka@columbia.edu
// Copyright (C) 1994-2017 Akira Takaoka
// All rights reserved.





// package jdubielApplication;

import java.lang.reflect.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;




// To find if an ordered pc-set is a subset of another.
public class OrderedSetRelations extends JFrame 
                                 implements ActionListener {

    static final int xOffset = 30, yOffset = 100;
    static int openFrameCount = 0;

    JScrollPane orderedRelationScrollPane;
    JPanel orderedSetRelationsPanel;
    JPanel buttonPanel;

    JTextField pcSet1TextField;
    JComboBox oderUnorderJComboBox;
    JTextField pcSet2TextField;

    JEditorPane editorPane;
    JScrollPane editorScrollPane;

    JCheckBox tnCheckBox;
    JCheckBox tnICheckBox;
    JCheckBox mnCheckBox;
    JCheckBox rkTnCheckBox;
    JCheckBox rkTnICheckBox;
    JCheckBox RTnCheckBox;
    JCheckBox RTnICheckBox;
    JCheckBox rkRTnCheckBox;
    JCheckBox rkRTnICheckBox;

    JCheckBox pVerticalsCheckBox;
    JCheckBox rVerticalsCheckBox;
    JCheckBox iVerticalsCheckBox;
    JCheckBox riVerticalsCheckBox;
    JCheckBox irVerticalsCheckBox;

    SpinnerNumberModel pVerticalsModel;
    SpinnerNumberModel rVerticalsModel;
    SpinnerNumberModel iVerticalsModel;
    SpinnerNumberModel riVerticalsModel;
    SpinnerNumberModel irVerticalsModel;

    StringBuffer results;




    public OrderedSetRelations() {

	createWidgets();

	//Put everything together.
	JPanel devidedPane = new JPanel(new BorderLayout());
	devidedPane.add(orderedSetRelationsPanel, 
			BorderLayout.NORTH);
	devidedPane.add(buttonPanel,
			BorderLayout.WEST);
	devidedPane.add(orderedRelationScrollPane,
			BorderLayout.CENTER);
	devidedPane.add(editorScrollPane,
			BorderLayout.SOUTH);

	getContentPane().add(devidedPane, BorderLayout.CENTER);

	setSize(455,600);
	openFrameCount++;
	setTitle("Ordered Set Relations " + openFrameCount);
	setLocation(xOffset + 20 * openFrameCount, 
		    yOffset + 20 * openFrameCount);
    }



    private void createWidgets() {

	//Lay out the pc-set labels on a panel.
	pcSet1TextField = new JTextField(12);
	pcSet1TextField.setBackground(Color.white);
	pcSet1TextField.setDragEnabled(true);
	JLabel pcSet1Label = new JLabel("PC-Set 1: ", 
					SwingConstants.RIGHT);
	pcSet1Label.setLabelFor(pcSet1TextField);

	pcSet2TextField = new JTextField(12);
	pcSet2TextField.setBackground(Color.white);
	pcSet2TextField.setDragEnabled(true);
	JLabel pcSet2Label = new JLabel("PC-Set 2: ", 
					SwingConstants.RIGHT);
	pcSet2Label.setLabelFor(pcSet2TextField);


	orderedSetRelationsPanel = new JPanel(new GridLayout(1, 4));
	orderedSetRelationsPanel.add(pcSet1Label);
	orderedSetRelationsPanel.add(pcSet1TextField);
	orderedSetRelationsPanel.add(pcSet2Label);
	orderedSetRelationsPanel.add(pcSet2TextField);
	orderedSetRelationsPanel.setBorder(
					   BorderFactory.createCompoundBorder(
									      BorderFactory.createTitledBorder(""),
									      BorderFactory.createEmptyBorder(5,5,5,5)));



	// Find button
	JButton findButton = new JButton("Find");
	findButton.setActionCommand("find");
	findButton.addActionListener(this);
	findButton.setBackground(Color.lightGray);
	findButton.setFont(new Font("SansSerif", Font.BOLD, 12));

	// Clear button
	JButton clearButton = new JButton("Clear");
	clearButton.setActionCommand("clear");
	clearButton.addActionListener(this);
	clearButton.setBackground(Color.lightGray);

	JPanel buttonPanel1 = new JPanel(new SpringLayout());
	buttonPanel1.add(findButton);
	buttonPanel1.add(clearButton);
	SpringUtilities.makeCompactGrid(buttonPanel1,
					2, 1,  //rows, cols
					1, 1,  //initX, initY
					1, 1); //xPad, yPad

	buttonPanel = new JPanel(new BorderLayout());
	buttonPanel.add(buttonPanel1, BorderLayout.NORTH);



	// Check buttons on a panel.
	tnCheckBox = new JCheckBox(" T ");
	tnCheckBox.setToolTipText("Transposition");
	tnCheckBox.setActionCommand("tn");
	tnCheckBox.addActionListener(this);

	tnICheckBox = new JCheckBox(" TI ");
	tnICheckBox.setToolTipText("Inversion");
	tnICheckBox.setActionCommand("tnI");
	tnICheckBox.addActionListener(this);

	mnCheckBox = new JCheckBox(" M ");
	mnCheckBox.setToolTipText("Multiplication");
	mnCheckBox.setActionCommand("mn");
	mnCheckBox.addActionListener(this);

	rkTnCheckBox = new JCheckBox(" rT "); 
	rkTnCheckBox.setToolTipText("Rotated transposition");
	rkTnCheckBox.setActionCommand("rkTn");
	rkTnCheckBox.addActionListener(this);

	rkTnICheckBox = new JCheckBox(" rTI "); 
	rkTnICheckBox.setToolTipText("Rotated inversion");
	rkTnICheckBox.setActionCommand("rkTnI");
	rkTnICheckBox.addActionListener(this);

	RTnCheckBox = new JCheckBox(" RT "); 
	RTnCheckBox.setToolTipText("Retrograde");
	RTnCheckBox.setActionCommand(" RTn ");
	RTnCheckBox.addActionListener(this);

	// RIn (retrograde inversion)
	RTnICheckBox = new JCheckBox(" RTI "); 
	RTnICheckBox.setToolTipText("Retrograde inversion");
	RTnICheckBox.setActionCommand("RTnI");
	RTnICheckBox.addActionListener(this);

	rkRTnCheckBox = new JCheckBox(" rRT "); 
	rkRTnCheckBox.setToolTipText("Rotated retrograde");
	rkRTnCheckBox.setActionCommand("rkRTn");
	rkRTnCheckBox.addActionListener(this);

	rkRTnICheckBox = new JCheckBox(" rRTI "); 
	rkRTnICheckBox.setToolTipText("Rotated retrograde inversion");
	rkRTnICheckBox.setActionCommand("rkRTnI");
	rkRTnICheckBox.addActionListener(this);

	JPanel checkBoxesPanel1 = new JPanel(new GridLayout(3, 3));
	checkBoxesPanel1.add(tnCheckBox);
	checkBoxesPanel1.add(tnICheckBox);
	checkBoxesPanel1.add(mnCheckBox);
	checkBoxesPanel1.add(rkTnCheckBox);
	checkBoxesPanel1.add(rkTnICheckBox);
	checkBoxesPanel1.add(RTnCheckBox);
	checkBoxesPanel1.add(RTnICheckBox);
	checkBoxesPanel1.add(rkRTnCheckBox);
	checkBoxesPanel1.add(rkRTnICheckBox);

	checkBoxesPanel1.setBorder(
	   BorderFactory.createCompoundBorder(
		      BorderFactory.createTitledBorder(
			      "their relations under: "),
			      BorderFactory.createEmptyBorder(5,5,5,5)));



	pVerticalsCheckBox 
	    = new JCheckBox("P rotational array of PC-Set 1. Size: "); 
	pVerticalsCheckBox.setActionCommand("pVerticals");
	pVerticalsCheckBox.addActionListener(this);

	pVerticalsModel = new SpinnerNumberModel(
						 6, //initial value
						 3, //min
						 12, //max
						 1); //step
	JSpinner pVerticalsSpinner = new JSpinner(pVerticalsModel);
	pVerticalsSpinner.setEditor(
			    new JSpinner.NumberEditor(pVerticalsSpinner));
	pVerticalsSpinner.setToolTipText(
				 "The number of the layers of the array.");
	Dimension d2 = new Dimension();
	d2.setSize(0.1, 0.1);
	pVerticalsSpinner.setSize(10,10);



	rVerticalsCheckBox 
	    = new JCheckBox("R rotational array of PC-Set 1. Size: "); 
	rVerticalsCheckBox.setActionCommand("rVerticals");
	rVerticalsCheckBox.addActionListener(this);

	rVerticalsModel = new SpinnerNumberModel(
						 6, //initial value
						 3, //min
						 12, //max
						 1); //step
	JSpinner rVerticalsSpinner = new JSpinner(rVerticalsModel);
	rVerticalsSpinner.setEditor(
			    new JSpinner.NumberEditor(rVerticalsSpinner));
	rVerticalsSpinner.setToolTipText(
				 "The number of the layers of the array.");
	d2.setSize(0.1, 0.1);
	rVerticalsSpinner.setSize(10,10);



	iVerticalsCheckBox 
	    = new JCheckBox("I rotational array of PC-Set 1. Size:  "); 
	iVerticalsCheckBox.setActionCommand("iVerticals");
	iVerticalsCheckBox.addActionListener(this);

	iVerticalsModel = new SpinnerNumberModel(
						 6, //initial value
						 3, //min
						 12, //max
						 1); //step
	JSpinner iVerticalsSpinner = new JSpinner(iVerticalsModel);
	iVerticalsSpinner.setEditor(
			    new JSpinner.NumberEditor(iVerticalsSpinner));
	iVerticalsSpinner.setToolTipText(
				 "The number of the layers of the array.");
	d2.setSize(0.1, 0.1);
	iVerticalsSpinner.setSize(10,10);



	riVerticalsCheckBox 
	    = new JCheckBox("RI rotational array of PC-Set 1. Size:"); 
	riVerticalsCheckBox.setActionCommand("riVerticals");
	riVerticalsCheckBox.addActionListener(this);

	riVerticalsModel = new SpinnerNumberModel(
						  6, //initial value
						  3, //min
						  12, //max
						  1); //step
	JSpinner riVerticalsSpinner = new JSpinner(riVerticalsModel);
	riVerticalsSpinner.setEditor(
			     new JSpinner.NumberEditor(riVerticalsSpinner));
	riVerticalsSpinner.setToolTipText(
				  "The number of the layers of the array.");
	d2.setSize(0.1, 0.1);
	riVerticalsSpinner.setSize(10,10);



	irVerticalsCheckBox 
	    = new JCheckBox("IR rotational array of PC-Set 1. Size:"); 
	irVerticalsCheckBox.setActionCommand("irVerticals");
	irVerticalsCheckBox.addActionListener(this);

	irVerticalsModel = new SpinnerNumberModel(
						  6, //initial value
						  3, //min
						  12, //max
						  1); //step
	JSpinner irVerticalsSpinner = new JSpinner(irVerticalsModel);
	irVerticalsSpinner.setEditor(
			     new JSpinner.NumberEditor(irVerticalsSpinner));
	irVerticalsSpinner.setToolTipText(
				  "The number of the layers of the array.");
	d2.setSize(0.1, 0.1);
	irVerticalsSpinner.setSize(10,10);



	JPanel checkBoxesPanelP = new JPanel();
	checkBoxesPanelP.setLayout(
			   new BoxLayout(checkBoxesPanelP, BoxLayout.X_AXIS));
	checkBoxesPanelP.add(pVerticalsCheckBox);
	checkBoxesPanelP.add(pVerticalsSpinner);

	JPanel checkBoxesPanelR = new JPanel();
	checkBoxesPanelR.setLayout(
			   new BoxLayout(checkBoxesPanelR, BoxLayout.X_AXIS));
	checkBoxesPanelR.add(rVerticalsCheckBox);
	checkBoxesPanelR.add(rVerticalsSpinner);

	JPanel checkBoxesPanelI = new JPanel();
	checkBoxesPanelI.setLayout(
			   new BoxLayout(checkBoxesPanelI, BoxLayout.X_AXIS));
	checkBoxesPanelI.add(iVerticalsCheckBox);
	checkBoxesPanelI.add(iVerticalsSpinner);

	JPanel checkBoxesPanelRI = new JPanel();
	checkBoxesPanelRI.setLayout(
		    new BoxLayout(checkBoxesPanelRI, BoxLayout.X_AXIS));
	checkBoxesPanelRI.add(riVerticalsCheckBox);
	checkBoxesPanelRI.add(riVerticalsSpinner);

	JPanel checkBoxesPanelIR = new JPanel();
	checkBoxesPanelIR.setLayout(
		    new BoxLayout(checkBoxesPanelIR, BoxLayout.X_AXIS));
	checkBoxesPanelIR.add(irVerticalsCheckBox);
	checkBoxesPanelIR.add(irVerticalsSpinner);


	JPanel checkBoxesPanel2 = new JPanel(new GridLayout(5, 1));
	checkBoxesPanel2.add(checkBoxesPanelP);
	checkBoxesPanel2.add(checkBoxesPanelR);
	checkBoxesPanel2.add(checkBoxesPanelI);
	checkBoxesPanel2.add(checkBoxesPanelRI);
	checkBoxesPanel2.add(checkBoxesPanelIR);
	checkBoxesPanel2.setBorder(
	   BorderFactory.createCompoundBorder(
		      BorderFactory.createTitledBorder(
			      "unordered PC-Set 2 in the verticals of"),
			      BorderFactory.createEmptyBorder(5,5,5,5)));



	//Lay out the panel.
	JPanel checkBoxesPanel = new JPanel(new SpringLayout());
	checkBoxesPanel.add(checkBoxesPanel1);
	checkBoxesPanel.add(checkBoxesPanel2);
	SpringUtilities.makeCompactGrid(checkBoxesPanel,
					2, 1,  //rows, cols
					1, 1,  //initX, initY
					1, 1); //xPad, yPad
	checkBoxesPanel.setBorder(
	  BorderFactory.createCompoundBorder(
		     BorderFactory.createTitledBorder(""),
		     BorderFactory.createEmptyBorder(5,5,5,5)));

	orderedRelationScrollPane = new JScrollPane(checkBoxesPanel);
	orderedRelationScrollPane.setVerticalScrollBarPolicy(
				     JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
	orderedRelationScrollPane.setHorizontalScrollBarPolicy(
			       JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
	orderedRelationScrollPane.setPreferredSize(new Dimension(380, 150));
	orderedRelationScrollPane.setMinimumSize(new Dimension(380, 150));




	// "Found Relations" area
	editorPane = new JEditorPane();
	editorScrollPane = new JScrollPane(editorPane);
	editorScrollPane.setVerticalScrollBarPolicy(
			    JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
	editorScrollPane.setHorizontalScrollBarPolicy(
			    JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
	editorScrollPane.setPreferredSize(new Dimension(300, 300));
	editorScrollPane.setMinimumSize(new Dimension(300, 300));
	editorScrollPane.setBorder(
	   BorderFactory.createCompoundBorder(
		      BorderFactory.createTitledBorder("Found: "),
		      BorderFactory.createEmptyBorder(5,5,5,5)));
	editorScrollPane.getBorder();
    }









    public void actionPerformed(ActionEvent e) {

	if("find".equals(e.getActionCommand())) {
	    editorPane.setText("");		

	    results = new StringBuffer();
			
	    // Get typed-in pcSet1 and pcSet2.
	    String readPcSetString1 = pcSet1TextField.getText();
	    String readPcSetString2 = pcSet2TextField.getText();

	    // Eliminate spaces
	    StringBuffer withoutSpaceStringBuffer1 = new StringBuffer();
	    for (int i = 0; i < readPcSetString1.length(); i++)
		if (readPcSetString1.charAt(i) != ' ')
		    withoutSpaceStringBuffer1.append(
					     readPcSetString1.charAt(i));

	    StringBuffer withoutSpaceStringBuffer2 = new StringBuffer();
	    for (int i = 0; i < readPcSetString2.length(); i++)
		if (readPcSetString2.charAt(i) != ' ')
		    withoutSpaceStringBuffer2.append(
					     readPcSetString2.charAt(i));

	    // Eliminate duplicates
	    PcSet pq = new PcSet();
	    String pcSet1String = pq.removeDuplicatesInString(
				      withoutSpaceStringBuffer1.toString());
	    String pcSet2String = pq.removeDuplicatesInString(
				      withoutSpaceStringBuffer2.toString());


	    // Check if there are non-numerical characters in PC-Set 1.
	    String illegalChars1 
		= pq.checkNonNumerals(pcSet1String);

	    // Check if there are non-numerical characters in PC-Set 2.
	    String illegalChars2 
		= pq.checkNonNumerals(pcSet2String);

	    if (illegalChars1.length() > 1 || illegalChars2.length() > 1) {
		if (illegalChars1.length() > 1 && illegalChars2.length() > 1)
		    JOptionPane.showMessageDialog(
			  orderedSetRelationsPanel, 
			  "Non-numerical characters typed in PC-Set 1: "
			  + illegalChars1.toString()
			  + "\nand in PC-Set 2: "
			  + illegalChars2.toString(),
			  "Warning",
			  JOptionPane.WARNING_MESSAGE);

		if (illegalChars1.length() > 1)
		    JOptionPane.showMessageDialog(
				  orderedSetRelationsPanel, 
				  "Non-numerical characters typed in PC-Set 1:"
				  + illegalChars1.toString(),
				  "Warning",
				  JOptionPane.WARNING_MESSAGE);

		if (illegalChars2.length() > 1)
		    JOptionPane.showMessageDialog(
				  orderedSetRelationsPanel, 
				  "Non-numerical characters typed in PC-Set 2:"
				  + illegalChars2.toString(),
				  "Warning",
				  JOptionPane.WARNING_MESSAGE);
	    } else if(pcSet1String.length() 
		      != withoutSpaceStringBuffer1.length()) {
		JOptionPane.showMessageDialog(
				      orderedSetRelationsPanel, 
				      "There are duplicates in PC-Set 1.",
				      "Warning",
				      JOptionPane.WARNING_MESSAGE);
	    } else if(pcSet2String.length() 
		      != withoutSpaceStringBuffer2.length()) {
		JOptionPane.showMessageDialog(
				      orderedSetRelationsPanel, 
				      "There are duplicates in PC-Set 2.",
				      "Warning",
				      JOptionPane.WARNING_MESSAGE);
	    } else if (pcSet1String.length() < 2) {
		JOptionPane.showMessageDialog(
			      orderedSetRelationsPanel, 
			      "The size of a pc-set must be at least 2.",
			      "Warning",
			      JOptionPane.WARNING_MESSAGE);
	    } else if (pcSet2String.length() < 2) {
		JOptionPane.showMessageDialog(
			      orderedSetRelationsPanel, 
			      "The size of a pc-set must be at least 2.",
				      "Warning",
				      JOptionPane.WARNING_MESSAGE);
	    } else if (tnCheckBox.getSelectedObjects() == null &&
		       tnICheckBox.getSelectedObjects() == null &&
		       mnCheckBox.getSelectedObjects() == null &&
		       rkTnCheckBox.getSelectedObjects() == null &&
		       rkTnICheckBox.getSelectedObjects() == null &&
		       RTnCheckBox.getSelectedObjects() == null &&
		       RTnICheckBox.getSelectedObjects() == null &&
		       rkRTnCheckBox.getSelectedObjects() == null &&
		       rkRTnICheckBox.getSelectedObjects() == null &&
		       pVerticalsCheckBox.getSelectedObjects() == null &&
		       rVerticalsCheckBox.getSelectedObjects() == null &&
		       iVerticalsCheckBox.getSelectedObjects() == null &&
		       riVerticalsCheckBox.getSelectedObjects() == null &&
		       irVerticalsCheckBox.getSelectedObjects() == null
		       ) {
		JOptionPane.showMessageDialog(
					      orderedSetRelationsPanel, 
					      "You need to choose at least "
					      + "one of the operations.\n",
					      "Warning",
					      JOptionPane.WARNING_MESSAGE);
	    } else {
		// No input errors found

		char[] pcSet1Chars = new char[pcSet1String.length()];
		pcSet1String.getChars(0, 
				      pcSet1String.length(), 
				      pcSet1Chars, 
				      0);
		Set pcSet1 = new Set(pcSet1Chars.length);
		pcSet1.set = pcSet1.charArrayToIntOne(pcSet1Chars);

		char[] pcSet2Chars = new char[pcSet2String.length()];
		pcSet2String.getChars(0, 
				      pcSet2String.length(), 
				      pcSet2Chars, 
				      0);
		Set pcSet2 = new Set(pcSet2Chars.length);
		pcSet2.set = pcSet2.charArrayToIntOne(pcSet2Chars);


		// Check base 12 characters to use
		if (!pcSet1.base12String.equals("NotUsed")) {
		    pcSet2.base12String = pcSet1.base12String;
		} else if (!pcSet2.base12String.equals("NotUsed")
			   && pcSet1.base12String.equals("NotUsed")) {
		    pcSet1.base12String = pcSet2.base12String;
		} else {
		    pcSet1.base12String = "AB";
		    pcSet2.base12String = "AB";
		}



		OrderedInclusion oi = new OrderedInclusion(pcSet1, pcSet2);

		if (tnCheckBox.getSelectedObjects() != null) {
		    int noRelations = 0;
		    int someRelations = 0;

		    results.append("\nInvariant subsets under T:\n\n");

		    String nrut = oi.listFitTns();
		    if (nrut.equals("noRelationsUnderT"))
			noRelations++;
		    else {
			results.append(nrut);
			someRelations++;
		    }
		    if (noRelations != 0 && someRelations == 0)
			results.append("    No invariants under T.\n\n");
		}

			    
		if (tnICheckBox.getSelectedObjects() != null) {
		    int noRelations = 0;
		    int someRelations = 0;

		    results.append("\nInvariant subsets under TI:\n\n");

		    String nrut = oi.listFitTnIs();
		    if (nrut.equals("noRelationsUnderTI"))
			noRelations++;
		    else {
			results.append(nrut);
			someRelations++;
		    }
		    if (noRelations != 0 && someRelations == 0)
			results.append("    No invariants under TI.\n\n");
		}

		
		if (mnCheckBox.getSelectedObjects() != null) {
		    int noRelations = 0;
		    int someRelations = 0;

		    results.append("\nInvariant subsets under M:\n\n");

		    String nrut = oi.listFitMns();
		    if (nrut.equals("noRelationsUnderM"))
			noRelations++;
		    else {
			results.append(nrut);
			someRelations++;
		    }
		    if (noRelations != 0 && someRelations == 0)
			results.append("    No invariants under M.\n\n");
		}


		if (rkTnCheckBox.getSelectedObjects() != null) {
		    int noRelations = 0;
		    int someRelations = 0;

		    results.append("\nInvariant subsets under rT:\n\n");

		    String nrut = oi.listFitrTns();
		    if (nrut.equals("noRelationsUnderrT"))
			noRelations++;
		    else {
			results.append(nrut);
			someRelations++;
		    }
		    if (noRelations != 0 && someRelations == 0)
			results.append("    No invariants under rT.\n\n");
		}


		if (rkTnICheckBox.getSelectedObjects() != null) {
		    int noRelations = 0;
		    int someRelations = 0;

		    results.append("\nInvariant subsets under rTI:\n\n");

		    String nrut = oi.listFitrTnIs();
		    if (nrut.equals("noRelationsUnderrTI"))
			noRelations++;
		    else {
			results.append(nrut);
			someRelations++;
		    }
		    if (noRelations != 0 && someRelations == 0)
			results.append("    No invariants under rTI.\n\n");
		}


		if (RTnCheckBox.getSelectedObjects() != null) {
		    int noRelations = 0;
		    int someRelations = 0;

		    results.append("\nInvariant subsets under RT:\n\n");

		    String nrut = oi.listFitRTs();
		    if (nrut.equals("noRelationsUnderRT"))
			noRelations++;
		    else {
			results.append(nrut);
			someRelations++;
		    }
		    if (noRelations != 0 && someRelations == 0)
			results.append("    No invariants under RT.\n\n");
		}


		if (RTnICheckBox.getSelectedObjects() != null) {
		    int noRelations = 0;
		    int someRelations = 0;

		    results.append("\nInvariant subsets under RTI:\n\n");

		    String nrut = oi.listFitRTnIs();
		    if (nrut.equals("noRelationsUnderRTI"))
			noRelations++;
		    else {
			results.append(nrut);
			someRelations++;
		    }
		    if (noRelations != 0 && someRelations == 0)
			results.append("    No invariants under RTI.\n\n");
		}


		if (rkRTnCheckBox.getSelectedObjects() != null) {
		    int noRelations = 0;
		    int someRelations = 0;

		    results.append("\nInvariant subsets under rRT:\n\n");

		    String nrut = oi.listFitrRTs();
		    if (nrut.equals("noRelationsUnderrRT"))
			noRelations++;
		    else {
			results.append(nrut);
			someRelations++;
		    }
		    if (noRelations != 0 && someRelations == 0)
			results.append("    No invariants under rRT.\n\n");
		}


		if (rkRTnICheckBox.getSelectedObjects() != null) {
		    int noRelations = 0;
		    int someRelations = 0;

		    results.append("\nInvariant subsets under rRTI:\n\n");

		    String nrut = oi.listFitrRTnIs();
		    if (nrut.equals("noRelationsUnderrRTI"))
			noRelations++;
		    else {
			results.append(nrut);
			someRelations++;
		    }
		    if (noRelations != 0 && someRelations == 0)
			results.append("    No invariants under rRTI.\n\n");
		}


		if (pVerticalsCheckBox.getSelectedObjects() != null) {
		    int n = pVerticalsModel.getNumber().intValue();
		    if (pcSet1.set.length < n || n < pcSet2.set.length)
			JOptionPane.showMessageDialog(
			      null, 
			      "The number of the layers of the rotational "
			      + "array must be \n"
			      + "less than or equal to "
			      + "the size of PC-Set 1 and \n"
			      + "greater than "
			      + "or equal to that of PC-Set 2.\n",
			      "Warning",
			      JOptionPane.WARNING_MESSAGE);
		    else {
			results.append("\n");
			RotationalArray rarry = new RotationalArray(pcSet1, n);
			results.append(rarry.printRotationalArrayP());
			results.append(rarry.listFitVerticalsP(pcSet2.set));
		    }
		}
		    
		if (rVerticalsCheckBox.getSelectedObjects() != null) {
		    int n = rVerticalsModel.getNumber().intValue();
		    if (pcSet1.set.length < n || n < pcSet2.set.length)
			JOptionPane.showMessageDialog(
			      null, 
			      "The number of the layers of the rotational "
			      + "array must be \n"
			      + "less than or equal to "
			      + "the size of PC-Set 1 and \n"
			      + "greater than "
			      + "or equal to that of PC-Set 2.\n",
			      "Warning",
			      JOptionPane.WARNING_MESSAGE);
		    else {
			results.append("\n");
			RotationalArray rarry = new RotationalArray(pcSet1, n);
			results.append(rarry.printRotationalArrayR());
			results.append(rarry.listFitVerticalsR(pcSet2.set));
		    }
		}

		if (iVerticalsCheckBox.getSelectedObjects() != null) {
		    int n = iVerticalsModel.getNumber().intValue();
		    if (pcSet1.set.length < n || n < pcSet2.set.length)
			JOptionPane.showMessageDialog(
			      null, 
			      "The number of the layers of the rotational "
			      + "array must be \n"
			      + "less than or equal to "
			      + "the size of PC-Set 1 and \n"
			      + "greater than "
			      + "or equal to that of PC-Set 2.\n",
			      "Warning",
			      JOptionPane.WARNING_MESSAGE);
		    else {
			results.append("\n");
			RotationalArray rarry = new RotationalArray(pcSet1, n);
			results.append(rarry.printRotationalArrayI());
			results.append(rarry.listFitVerticalsI(pcSet2.set));
		    }
		}

		if (riVerticalsCheckBox.getSelectedObjects() != null) {
		    int n = riVerticalsModel.getNumber().intValue();
		    if (pcSet1.set.length < n || n < pcSet2.set.length)
			JOptionPane.showMessageDialog(
			      null,
			      "The number of the layers of the rotational "
			      + "array must be \n"
			      + "less than or equal to "
			      + "the size of PC-Set 1 and \n"
			      + "greater than "
			      + "or equal to that of PC-Set 2.\n",
			      "Warning",
			      JOptionPane.WARNING_MESSAGE);
		    else {
			results.append("\n");
			RotationalArray rarry = new RotationalArray(pcSet1, n);
			results.append(rarry.printRotationalArrayRI());
			results.append(rarry.listFitVerticalsRI(pcSet2.set));
		    }
		}

		if (irVerticalsCheckBox.getSelectedObjects() != null) {
		    int n = irVerticalsModel.getNumber().intValue();
		    if (pcSet1.set.length < n || n < pcSet2.set.length)
			JOptionPane.showMessageDialog(
			      null,
			      "The number of the layers of the rotational "
			      + "array must be \n"
			      + "less than or equal to "
			      + "the size of PC-Set 1 and \n"
			      + "greater than "
			      + "or equal to that of PC-Set 2.\n",
			      "Warning",
			      JOptionPane.WARNING_MESSAGE);
		    else {
			results.append("\n");
			RotationalArray rarry = new RotationalArray(pcSet1, n);
			results.append(rarry.printRotationalArrayIR());
			results.append(rarry.listFitVerticalsIR(pcSet2.set));
		    }
		}

	    }
				
	    editorPane.setText(results.toString());
	}






	if ("clear".equals(e.getActionCommand())) {
	    pcSet1TextField.setText("");
	    pcSet2TextField.setText("");
	    editorPane.setText("");		
	}
    }
}
