// RotationalArray.java, used for
// JDubiel.java: computer program for pc-set analysis
// by Akira Takaoka, akira.takaoka@columbia.edu
// Copyright (C) 1994-2017 Akira Takaoka
// All rights reserved.





// package jdubielApplication;

import java.lang.reflect.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;


public class RotationalArray extends Set {

    int [] givenOrderedSet;
    int numOfRotation;

    int [][] rArrayP;
    int [][] rArrayR;
    int [][] rArrayI;
    int [][] rArrayRI;
    int [][] rArrayIR;


    // Constructor 1
    // Initializes a rotational array of given pc-set givenOrderedSet.
    public RotationalArray(int [] gOS, int nOfR) {

	givenOrderedSet = gOS;
	numOfRotation = nOfR;

	rArrayP = new int[numOfRotation][givenOrderedSet.length];
	rArrayR = new int[numOfRotation][givenOrderedSet.length];
	rArrayI = new int[numOfRotation][givenOrderedSet.length];
	rArrayRI = new int[numOfRotation][givenOrderedSet.length];
	rArrayIR = new int[numOfRotation][givenOrderedSet.length];


	for(int i = 0; i < givenOrderedSet.length; i++){
	    rArrayP[0][i] = givenOrderedSet[i];
	    rArrayR[0][i] = givenOrderedSet[(givenOrderedSet.length - 1) - i];
	    rArrayI[0][i] = ((12 + (givenOrderedSet[0] * 2)) 
			     - givenOrderedSet[i]) % 12;
	    rArrayRI[0][i] = ((12 + (rArrayR[0][0] * 2)) - rArrayR[0][i])%12;
	}

	for (int i = 0; i < givenOrderedSet.length; i++)
	    rArrayIR[0][i] = rArrayI[0][(givenOrderedSet.length - 1) - i];

	int [] dffrncP = new int[givenOrderedSet.length];
	int [] dffrncR = new int[givenOrderedSet.length];
	int [] dffrncI = new int[givenOrderedSet.length];
	int [] dffrncRI = new int[givenOrderedSet.length];
	int [] dffrncIR = new int[givenOrderedSet.length];

	for(int i = 0; i < givenOrderedSet.length; i++){
	    dffrncP[i]=rArrayP[0][0]-rArrayP[0][i];
	    dffrncR[i]=rArrayR[0][0]-rArrayR[0][i];
	    dffrncI[i]=rArrayI[0][0]-rArrayI[0][i];
	    dffrncRI[i]=rArrayRI[0][0]-rArrayRI[0][i];
	    dffrncIR[i]=rArrayIR[0][0]-rArrayIR[0][i];
	}

	for(int i = 1; i < numOfRotation; i++){
	    for(int ii = 0; ii < givenOrderedSet.length; ii++){
		rArrayP[i][ii] 
		    = ((12 + rArrayP[0][(ii + i) % givenOrderedSet.length]) 
		       + dffrncP[i]) % 12;
		rArrayR[i][ii]
		    = ((12 + rArrayR[0][(ii + i) % givenOrderedSet.length]) 
		       + dffrncR[i]) % 12;
		rArrayI[i][ii]
		    = ((12 + rArrayI[0][(ii + i) % givenOrderedSet.length]) 
		       + dffrncI[i]) % 12;
		rArrayRI[i][ii]
		    = ((12 + rArrayRI[0][(ii + i) % givenOrderedSet.length]) 
		       + dffrncRI[i]) % 12;
		rArrayIR[i][ii]
		    = ((12 + rArrayIR[0][(ii + i) % givenOrderedSet.length]) 
		       + dffrncIR[i]) % 12;
	    }
	}
    }
    // Constructor 2
    // Initializes a rotational array of given pc-set givenOrderedSet.
    public RotationalArray(Set rotatedSeq, int nOfR) {

	givenOrderedSet = rotatedSeq.set;
	numOfRotation = nOfR;

	rArrayP = new int[numOfRotation][givenOrderedSet.length];
	rArrayR = new int[numOfRotation][givenOrderedSet.length];
	rArrayI = new int[numOfRotation][givenOrderedSet.length];
	rArrayRI = new int[numOfRotation][givenOrderedSet.length];
	rArrayIR = new int[numOfRotation][givenOrderedSet.length];


	for(int i = 0; i < givenOrderedSet.length; i++){
	    rArrayP[0][i] = givenOrderedSet[i];
	    rArrayR[0][i] = givenOrderedSet[(givenOrderedSet.length - 1) - i];
	    rArrayI[0][i] = ((12 + (givenOrderedSet[0] * 2)) 
			     - givenOrderedSet[i]) % 12;
	    rArrayRI[0][i] = ((12 + (rArrayR[0][0] * 2)) - rArrayR[0][i])%12;
	}

	for (int i = 0; i < givenOrderedSet.length; i++)
	    rArrayIR[0][i] = rArrayI[0][(givenOrderedSet.length - 1) - i];

	int [] dffrncP = new int[givenOrderedSet.length];
	int [] dffrncR = new int[givenOrderedSet.length];
	int [] dffrncI = new int[givenOrderedSet.length];
	int [] dffrncRI = new int[givenOrderedSet.length];
	int [] dffrncIR = new int[givenOrderedSet.length];

	for(int i = 0; i < givenOrderedSet.length; i++){
	    dffrncP[i]=rArrayP[0][0]-rArrayP[0][i];
	    dffrncR[i]=rArrayR[0][0]-rArrayR[0][i];
	    dffrncI[i]=rArrayI[0][0]-rArrayI[0][i];
	    dffrncRI[i]=rArrayRI[0][0]-rArrayRI[0][i];
	    dffrncIR[i]=rArrayIR[0][0]-rArrayIR[0][i];
	}

	for(int i = 1; i < numOfRotation; i++){
	    for(int ii = 0; ii < givenOrderedSet.length; ii++){
		rArrayP[i][ii] 
		    = ((12 + rArrayP[0][(ii + i) % givenOrderedSet.length]) 
		       + dffrncP[i]) % 12;
		rArrayR[i][ii]
		    = ((12 + rArrayR[0][(ii + i) % givenOrderedSet.length]) 
		       + dffrncR[i]) % 12;
		rArrayI[i][ii]
		    = ((12 + rArrayI[0][(ii + i) % givenOrderedSet.length]) 
		       + dffrncI[i]) % 12;
		rArrayRI[i][ii]
		    = ((12 + rArrayRI[0][(ii + i) % givenOrderedSet.length]) 
		       + dffrncRI[i]) % 12;
		rArrayIR[i][ii]
		    = ((12 + rArrayIR[0][(ii + i) % givenOrderedSet.length]) 
		       + dffrncIR[i]) % 12;
	    }
	}

	// Characters used for base 12 system, 
	// a & b, A & B, t & e, or T & E
	if (rotatedSeq.base12String != null)
	    base12String = new String(rotatedSeq.base12String);
	else
	    base12String = new String("AB");
    }
    // Constructor 3
    // Initializes a rotational array of given pc-set givenOrderedSet.
    public RotationalArray(Set rotatedSeq, int setLength, int nOfR) {

	givenOrderedSet = new int[setLength];
	System.arraycopy(givenOrderedSet, 0, rotatedSeq.set, 0, setLength);
	numOfRotation = nOfR;

	rArrayP = new int[numOfRotation][givenOrderedSet.length];
	rArrayR = new int[numOfRotation][givenOrderedSet.length];
	rArrayI = new int[numOfRotation][givenOrderedSet.length];
	rArrayRI = new int[numOfRotation][givenOrderedSet.length];
	rArrayIR = new int[numOfRotation][givenOrderedSet.length];


	for(int i = 0; i < givenOrderedSet.length; i++){
	    rArrayP[0][i] = givenOrderedSet[i];
	    rArrayR[0][i] = givenOrderedSet[(givenOrderedSet.length - 1) - i];
	    rArrayI[0][i] = ((12 + (givenOrderedSet[0] * 2)) 
			     - givenOrderedSet[i]) % 12;
	    rArrayRI[0][i] = ((12 + (rArrayR[0][0] * 2)) - rArrayR[0][i])%12;
	}

	for (int i = 0; i < givenOrderedSet.length; i++)
	    rArrayIR[0][i] = rArrayI[0][(givenOrderedSet.length - 1) - i];

	int [] dffrncP = new int[givenOrderedSet.length];
	int [] dffrncR = new int[givenOrderedSet.length];
	int [] dffrncI = new int[givenOrderedSet.length];
	int [] dffrncRI = new int[givenOrderedSet.length];
	int [] dffrncIR = new int[givenOrderedSet.length];

	for(int i = 0; i < givenOrderedSet.length; i++){
	    dffrncP[i]=rArrayP[0][0]-rArrayP[0][i];
	    dffrncR[i]=rArrayR[0][0]-rArrayR[0][i];
	    dffrncI[i]=rArrayI[0][0]-rArrayI[0][i];
	    dffrncRI[i]=rArrayRI[0][0]-rArrayRI[0][i];
	    dffrncIR[i]=rArrayIR[0][0]-rArrayIR[0][i];
	}

	for(int i = 1; i < numOfRotation; i++){
	    for(int ii = 0; ii < givenOrderedSet.length; ii++){
		rArrayP[i][ii] 
		    = ((12 + rArrayP[0][(ii + i) % givenOrderedSet.length]) 
		       + dffrncP[i]) % 12;
		rArrayR[i][ii]
		    = ((12 + rArrayR[0][(ii + i) % givenOrderedSet.length]) 
		       + dffrncR[i]) % 12;
		rArrayI[i][ii]
		    = ((12 + rArrayI[0][(ii + i) % givenOrderedSet.length]) 
		       + dffrncI[i]) % 12;
		rArrayRI[i][ii]
		    = ((12 + rArrayRI[0][(ii + i) % givenOrderedSet.length]) 
		       + dffrncRI[i]) % 12;
		rArrayIR[i][ii]
		    = ((12 + rArrayIR[0][(ii + i) % givenOrderedSet.length]) 
		       + dffrncIR[i]) % 12;
	    }
	}

	// Characters used for base 12 system, 
	// a & b, A & B, t & e, or T & E
	if (rotatedSeq.base12String != null)
	    base12String = new String(rotatedSeq.base12String);
	else
	    base12String = new String("AB");
    }
    // The end of constructors







    // Print a P rotational array 
    public String printRotationalArrayP() {

	StringBuffer arrayStringBuffer = new StringBuffer();

	// Rotational array P
	arrayStringBuffer.append("\nP rotational array of <");
	arrayStringBuffer.append(pcSetArrayToString(givenOrderedSet));
	arrayStringBuffer.append(">:\n\n");
	for(int i = 0; i < rArrayP.length; i++) {
	    arrayStringBuffer.append("    ");
	    arrayStringBuffer.append(pcSetArrayToString(rArrayP[i]));
	    arrayStringBuffer.append("\n");
	}
	return arrayStringBuffer.toString();
    }
    // The end of PrintRotationalArrayP()



    // Print an R rotational array
    public String printRotationalArrayR() {

	StringBuffer arrayStringBuffer = new StringBuffer();

	// Rotational array R
	arrayStringBuffer.append("\nR rotational array of <");
	arrayStringBuffer.append(pcSetArrayToString(givenOrderedSet));
	arrayStringBuffer.append(">:\n\n");
	for(int i = 0; i < rArrayR.length; i++) {
	    arrayStringBuffer.append("    ");
	    arrayStringBuffer.append(pcSetArrayToString(rArrayR[i]));
	    arrayStringBuffer.append("\n");
	}
	return arrayStringBuffer.toString();
    }
    // The end of PrintRotationalArrayR()



    // Print an I rotational array
    public String printRotationalArrayI() {

	StringBuffer arrayStringBuffer = new StringBuffer();

	// Rotational array I
	arrayStringBuffer.append("\nI rotational array of <");
	arrayStringBuffer.append(pcSetArrayToString(givenOrderedSet));
	arrayStringBuffer.append(">:\n\n");
	for(int i = 0; i < rArrayI.length; i++) {
	    arrayStringBuffer.append("    ");
	    arrayStringBuffer.append(pcSetArrayToString(rArrayI[i]));
	    arrayStringBuffer.append("\n");
	}
	return arrayStringBuffer.toString();
    }
    // The end of PrintRotationalArrayI()



    // Print an RI rotational array
    public String printRotationalArrayRI() {

	StringBuffer arrayStringBuffer = new StringBuffer();

	// Rotational array RI
	arrayStringBuffer.append("\nRI rotational array of <");
	arrayStringBuffer.append(pcSetArrayToString(givenOrderedSet));
	arrayStringBuffer.append(">:\n\n");
	for(int i = 0; i < rArrayRI.length; i++) {
	    arrayStringBuffer.append("    ");
	    arrayStringBuffer.append(pcSetArrayToString(rArrayRI[i]));
	    arrayStringBuffer.append("\n");
	}
	return arrayStringBuffer.toString();
    }
    // The end of PrintRotationalArrayRI()



    // Print an IR rotational array
    public String printRotationalArrayIR() {

	StringBuffer arrayStringBuffer = new StringBuffer();

	// Rotational array IR
	arrayStringBuffer.append("\nIR rotational array of <");
	arrayStringBuffer.append(pcSetArrayToString(givenOrderedSet));
	arrayStringBuffer.append(">:\n\n");
	for(int i = 0; i < rArrayIR.length; i++) {
	    arrayStringBuffer.append("    ");
	    arrayStringBuffer.append(pcSetArrayToString(rArrayIR[i]));
	    arrayStringBuffer.append("\n");
	}
	return arrayStringBuffer.toString();
    }
    // The end of PrintRotationalArrayIR()







    // Finds if a given pc-set is contained in the verticals of 
    // P rotational array.
    public String listFitVerticalsP(int [] givenPC) {

	// Make verticals out of rArrays

	int [][] verticalsP = new int[rArrayP[0].length][rArrayP.length];
	for (int i = 0; i < rArrayP[0].length; i++) {
	    for (int ii = 0; ii < rArrayP.length; ii++) {
		verticalsP[i][ii] = rArrayP[ii][i];
	    }
	}

	StringBuffer listFitVStringBuffer = new StringBuffer();

	// Now check verticals made of the array P.
	int h = 0;
	for(int i = 0; i < verticalsP.length; i++) {

	    int j = 0, jj = 0;
	    while(jj < givenPC.length && j < verticalsP[i].length)
		if(givenPC[jj] == verticalsP[i][j]) {
		    jj++;
		    j = 0;
		} else
		    j++;

	    if(jj == givenPC.length) {
		listFitVStringBuffer.append("\n    {");
		listFitVStringBuffer.append(pcSetArrayToString(givenPC));
		listFitVStringBuffer.append("}");
		listFitVStringBuffer.append(" is a subset of vertical");
		listFitVStringBuffer.append(" P" + (i + 1) + ": {");
		listFitVStringBuffer.append(pcSetArrayToString(verticalsP[i]));
		listFitVStringBuffer.append("}.");
		h++;
	    }
	}
		
	if(h == 0) {
	    listFitVStringBuffer.append("\n    {");
	    listFitVStringBuffer.append(pcSetArrayToString(givenPC));
	    listFitVStringBuffer.append("}");
	    listFitVStringBuffer.append(" is not a subset of any of the P verticals.\n");
	}

	listFitVStringBuffer.append("\n");
	return listFitVStringBuffer.toString();
    }
    // The end of listIfFitVerticalsP()  ----------------*/







    // Finds if a given pc-set is contained in the verticals of 
    // R rotational array.
    public String listFitVerticalsR(int [] givenPC) {

	// Make verticals out of rArrays

	int [][] verticalsR = new int[rArrayR[0].length][rArrayR.length];
	for (int i = 0; i < rArrayR[0].length; i++) {
	    for (int ii = 0; ii < rArrayR.length; ii++) {
		verticalsR[i][ii] = rArrayR[ii][i];
	    }
	}

	StringBuffer listFitVStringBuffer = new StringBuffer();

	// Now check verticals made of the array P.
	int h = 0;
	for(int i = 0; i < verticalsR.length; i++) {

	    int j = 0, jj = 0;
	    while(jj < givenPC.length && j < verticalsR[i].length)
		if(givenPC[jj] == verticalsR[i][j]) {
		    jj++;
		    j = 0;
		} else
		    j++;

	    if(jj == givenPC.length) {
		listFitVStringBuffer.append("\n    {");
		listFitVStringBuffer.append(pcSetArrayToString(givenPC));
		listFitVStringBuffer.append("}");
		listFitVStringBuffer.append(" is a subset of vertical");
		listFitVStringBuffer.append(" R" + (i + 1) + ": {");
		listFitVStringBuffer.append(pcSetArrayToString(verticalsR[i]));
		listFitVStringBuffer.append("}.");
		h++;
	    }
	}
		
	if(h == 0) {
	    listFitVStringBuffer.append("\n    {");
	    listFitVStringBuffer.append(pcSetArrayToString(givenPC));
	    listFitVStringBuffer.append("}");
	    listFitVStringBuffer.append(" is not a subset of any of the R verticals.\n");
	}

	listFitVStringBuffer.append("\n");
	return listFitVStringBuffer.toString();
    }
    // The end of listIfFitVerticalsR()  ----------------*/







    // Finds if a given pc-set is contained in the verticals of 
    // I rotational array.
    public String listFitVerticalsI(int [] givenPC) {

	// Make verticals out of rArrays

	int [][] verticalsI = new int[rArrayI[0].length][rArrayI.length];
	for (int i = 0; i < rArrayI[0].length; i++) {
	    for (int ii = 0; ii < rArrayI.length; ii++) {
		verticalsI[i][ii] = rArrayI[ii][i];
	    }
	}

	StringBuffer listFitVStringBuffer = new StringBuffer();

	// Now check verticals made of the array I.
	int h = 0;
	for(int i = 0; i < verticalsI.length; i++) {

	    int j = 0, jj = 0;
	    while(jj < givenPC.length && j < verticalsI[i].length)
		if(givenPC[jj] == verticalsI[i][j]) {
		    jj++;
		    j = 0;
		} else
		    j++;

	    if(jj == givenPC.length) {
		listFitVStringBuffer.append("\n    {");
		listFitVStringBuffer.append(pcSetArrayToString(givenPC));
		listFitVStringBuffer.append("}");
		listFitVStringBuffer.append(" is a subset of vertical");
		listFitVStringBuffer.append(" I" + (i + 1) + ": {");
		listFitVStringBuffer.append(pcSetArrayToString(verticalsI[i]));
		listFitVStringBuffer.append("}.");
		h++;
	    }
	}
		
	if(h == 0) {
	    listFitVStringBuffer.append("\n    {");
	    listFitVStringBuffer.append(pcSetArrayToString(givenPC));
	    listFitVStringBuffer.append("}");
	    listFitVStringBuffer.append(" is not a subset of any of the I verticals.\n");
	}

	listFitVStringBuffer.append("\n");
	return listFitVStringBuffer.toString();
    }
    // The end of listIfFitVerticalsI()  ----------------*/







    // Finds if a given pc-set is contained in the verticals of 
    // RI rotational array.
    public String listFitVerticalsRI(int [] givenPC) {

	// Make verticals out of rArrays

	int [][] verticalsRI = new int[rArrayRI[0].length][rArrayRI.length];
	for (int i = 0; i < rArrayRI[0].length; i++) {
	    for (int ii = 0; ii < rArrayRI.length; ii++) {
		verticalsRI[i][ii] = rArrayRI[ii][i];
	    }
	}

	StringBuffer listFitVStringBuffer = new StringBuffer();

	// Now check verticals made of the array RI.
	int h = 0;
	for(int i = 0; i < verticalsRI.length; i++) {

	    int j = 0, jj = 0;
	    while(jj < givenPC.length && j < verticalsRI[i].length)
		if(givenPC[jj] == verticalsRI[i][j]) {
		    jj++;
		    j = 0;
		} else
		    j++;

	    if(jj == givenPC.length) {
		listFitVStringBuffer.append("\n    {");
		listFitVStringBuffer.append(pcSetArrayToString(givenPC));
		listFitVStringBuffer.append("}");
		listFitVStringBuffer.append(" is a subset of vertical");
		listFitVStringBuffer.append(" RI" + (i + 1) + ": {");
		listFitVStringBuffer.append(pcSetArrayToString(verticalsRI[i]));
		listFitVStringBuffer.append("}.");
		h++;
	    }
	}
		
	if(h == 0) {
	    listFitVStringBuffer.append("\n    {");
	    listFitVStringBuffer.append(pcSetArrayToString(givenPC));
	    listFitVStringBuffer.append("}");
	    listFitVStringBuffer.append(" is not a subset of any of the RI verticals.\n");
	}

	listFitVStringBuffer.append("\n");
	return listFitVStringBuffer.toString();
    }
    // The end of listIfFitVerticalsRI()  ----------------*/







    // Finds if a given pc-set is contained in the verticals of 
    // IR rotational array.
    public String listFitVerticalsIR(int [] givenPC) {

	// Make verticals out of rArrays

	int [][] verticalsIR = new int[rArrayIR[0].length][rArrayIR.length];
	for (int i = 0; i < rArrayIR[0].length; i++) {
	    for (int ii = 0; ii < rArrayIR.length; ii++) {
		verticalsIR[i][ii] = rArrayIR[ii][i];
	    }
	}

	StringBuffer listFitVStringBuffer = new StringBuffer();

	// Now check verticals made of the array IR.
	int h = 0;
	for(int i = 0; i < verticalsIR.length; i++) {

	    int j = 0, jj = 0;
	    while(jj < givenPC.length && j < verticalsIR[i].length)
		if(givenPC[jj] == verticalsIR[i][j]) {
		    jj++;
		    j = 0;
		} else
		    j++;

	    if(jj == givenPC.length) {
		listFitVStringBuffer.append("\n    {");
		listFitVStringBuffer.append(pcSetArrayToString(givenPC));
		listFitVStringBuffer.append("}");
		listFitVStringBuffer.append(" is a subset of vertical");
		listFitVStringBuffer.append(" IR" + (i + 1) + ": {");
		listFitVStringBuffer.append(pcSetArrayToString(verticalsIR[i]));
		listFitVStringBuffer.append("}.");
		h++;
	    }
	}
		
	if(h == 0) {
	    listFitVStringBuffer.append("\n    {");
	    listFitVStringBuffer.append(pcSetArrayToString(givenPC));
	    listFitVStringBuffer.append("}");
	    listFitVStringBuffer.append(" is not a subset of any of the IR verticals.\n");
	}

	listFitVStringBuffer.append("\n");
	return listFitVStringBuffer.toString();
    }
    // The end of listIfFitVerticalsIR()  ----------------*/

}
