// SchoenbergFrame.java, used for
// JDubiel.java: computer program for pc-set analysis
// by Akira Takaoka, akira.takaoka@columbia.edu
// Copyright (C) 1994-2021 Akira Takaoka
// All rights reserved.





// package jdubielApplication;

import java.lang.reflect.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;



public class SchoenbergFrame extends JFrame implements ActionListener {

    static final int xOffset = 350, yOffset = 70;
    static int openFrameCount = 0;

    JPanel schoenbergPanel;
    JPanel buttonPanel;
    JScrollPane schoenbergScrollPane;

    JComboBox<String> pieceJComboBox;
    JTextField toneRowTextField;

    JComboBox<String> oderUnorderJComboBox;
    JTextField pcSetTextField;

    JEditorPane editorPane;
    JScrollPane editorScrollPane;

    JCheckBox tnCheckBox;
    JCheckBox tnICheckBox;
    JCheckBox mnCheckBox;
    JCheckBox RTnCheckBox;
    JCheckBox RTnICheckBox;

    JCheckBox primeFormsCheckBox;
    JCheckBox matrixCheckBox;

    StringBuffer results;







    public SchoenbergFrame() {

	//...Create the GUI and put it in the window...
	createWidgets();

	//Put everything together.
	JPanel devidedPane = new JPanel(new BorderLayout());
	devidedPane.add(schoenbergPanel, BorderLayout.NORTH);
	devidedPane.add(buttonPanel, BorderLayout.WEST);
	devidedPane.add(schoenbergScrollPane, BorderLayout.CENTER);
	devidedPane.add(editorScrollPane, BorderLayout.SOUTH);

	getContentPane().add(devidedPane, BorderLayout.CENTER);

	setSize(440, 620);
	openFrameCount++;
	setTitle("Schoenberg's Twelve-Tone Compositions " + openFrameCount);
	setLocation(xOffset + 20 * openFrameCount,
		    yOffset + 20 * openFrameCount);
    }
	



    // Create and add the widgets.
    private void createWidgets() {

	JLabel piecesLabel = new JLabel("Choose a piece: ", 
					SwingConstants.RIGHT);


	String[] pieceStrings = {
	    " Five Piano Pieces, Op. 23, No. 5 (1920/1923) ",
	    " Serenade, Op. 24, Movement 4, \"Sonett\" (1922/1923) ",
	    " Serenade, Op. 24, Movement 5, \"Tanzscene\" (1920/1923) ",
	    " Suite for Piano, Op. 25 (1921/1923) ",
	    " Wind Quintet, Op. 26 (1924) ",
	    " Four Pieces for Mixed Chorus, Op. 27, No. 1 (1925) ",
	    " Four Pieces for Mixed Chorus, Op. 27, No. 2 (1925) ",
	    " Four Pieces for Mixed Chorus, Op. 27, No. 3 (1925) ",
	    " Four Pieces for Mixed Chorus, Op. 27, No. 4 (1925) ",
	    " Three Satires for Mixed Chorus, Op. 28, No. 1 (1925/1926) ",
	    " Three Satires for Mixed Chorus, Op. 28, No. 3 (1925/1926) ",
	    " Suite, Op. 29 (1925) ",
	    " Third String Quartet, Op. 30 (1927) ",
	    " Variations for Orchestra, Op. 31 (1926/1928) ",
	    " Von Heute auf Morgen, Op. 32 (1929) ",
	    " Two Piano Pieces, Op. 33A, No. 1 (1928) ",
	    " Two Piano Pieces, Op. 33B, No. 2 (1931) ",
	    " Accompaniment to a Film Scene, Op. 34 (1930) ",
	    " Six Pieces for Male Chorus, Op. 35, No. 1 (1930) ",
	    " Six Pieces for Male Chorus, Op. 35, No. 2 (1930) ",
	    " Six Pieces for Male Chorus, Op. 35, No. 3 (1930) ",
	    " Six Pieces for Male Chorus, Op. 35, No. 5 (1930) ",
	    " Concerto for Violin and Orchestra, Op. 36 (1934/1936) ",
	    " Fourth String Quartet, Op. 37 (1936) ",
	    " Fragment Of Phantasia for Piano (Four Hands) (date unknown) ",
	    " Fragment of Sonata for Organ (date unknown) ",
	    " Fragment for Piano (Four Hands) (date unknown) ",
	    " Ode To Napoleon Buonaparte, Op. 41 (1942) ",
	    " Concerto for Piano and Orchestra, Op. 42 (1942) ",
	    " Die Jakobsleiter (1917/1922) ",
	    " Prelude to a Suite from \"Genesis\", Op. 44 (1945) ",
	    " String Trio, Op. 45 (1946) ",
	    " A Survivor from Warsaw, Op. 46 (1947) ",
	    " Fantasy for Violin and Piano, Op. 47 (1949) ",
	    " Three Songs, Op. 48, No. 1, \"Sommermud\" (1933) ",
	    " Three Songs, Op. 48, No. 2, \"Tot\" (1933) ",
	    " Three Songs, Op. 48, No. 3, \"Madchenlied\" (1933) ",
	    " Israel Exists Again (1949) ",
	    " Three Times a Thousand Years, Op. 50A (1949) ",
	    " Psalm 130, De Profundis (unfinished), Op. 50B (1950) ",
	    " Modern Psalms, Op. 50C, First Psalm (unfinished) (1950) ",
	    " Moses and Aron (1930-1932) "
	};


	pieceJComboBox = new JComboBox<>(pieceStrings);
	pieceJComboBox.setSelectedIndex(41);
	pieceJComboBox.setBackground(Color.white);
	pieceJComboBox.setFont(new Font("SansSerif", Font.ITALIC, 14));
	pieceJComboBox.setActionCommand("piece");
	pieceJComboBox.addActionListener(this);
	piecesLabel.setLabelFor(pieceJComboBox);

	JLabel toneRowLabel = new JLabel("Its tone row: ", 
					 SwingConstants.RIGHT);
	toneRowLabel.setToolTipText("Insert a space(s) to segment the "
				    + "tone row. Ex. 9A4231 7568B0 ");
	toneRowTextField = new JTextField(12);
	toneRowTextField.setBackground(Color.white);
	toneRowLabel.setLabelFor(toneRowTextField);
	toneRowTextField.setText(" 9A42317568B0 ");
	toneRowTextField.setToolTipText("Insert a space(s) to segment the "
					+ "tone row. Ex. 9A4231 7568B0 ");
	toneRowTextField.setDragEnabled(true);

	//Add the widgets to the container.
	schoenbergPanel = new JPanel(new GridLayout(2, 2));

	schoenbergPanel.add(piecesLabel);
	schoenbergPanel.add(pieceJComboBox);
	schoenbergPanel.add(toneRowLabel);
	schoenbergPanel.add(toneRowTextField);
	schoenbergPanel.setBorder(BorderFactory.createCompoundBorder(
                                BorderFactory.createTitledBorder(
					"Schoenberg's Twelve-Tone composition: "),
                                BorderFactory.createEmptyBorder(5,5,5,5)));







	// Find button
	JButton findButton = new JButton("Find");
	findButton.setActionCommand("find");
	findButton.addActionListener(this);
	findButton.setBackground(Color.lightGray);
	findButton.setFont(new Font("SansSerif", Font.BOLD, 12));

	// Clear button
	JButton clearButton = new JButton("Clear");
	clearButton.setActionCommand("clear");
	clearButton.addActionListener(this);
	clearButton.setBackground(Color.lightGray);

	JPanel buttonPanel1 = new JPanel(new SpringLayout());
	buttonPanel1.add(findButton);
	buttonPanel1.add(clearButton);
	SpringUtilities.makeCompactGrid(buttonPanel1,
					2, 1,  //rows, cols
					1, 1,  //initX, initY
					1, 1); //xPad, yPad

	buttonPanel = new JPanel(new BorderLayout());
	buttonPanel.add(buttonPanel1, BorderLayout.NORTH);



	String[] oderUnorderStrings = {"Ordered PC-Set:", "Unordered PC-Set:"};
	oderUnorderJComboBox = new JComboBox<>(oderUnorderStrings);
	oderUnorderJComboBox.setSelectedIndex(0);
	oderUnorderJComboBox.setAlignmentY(RIGHT_ALIGNMENT);
	oderUnorderJComboBox.setBackground(Color.white);
	pcSetTextField = new JTextField(10);
	pcSetTextField.setBackground(Color.white);
	pcSetTextField.setDragEnabled(true);

	JPanel pcSetPanel = new JPanel(new SpringLayout());
	pcSetPanel.add(oderUnorderJComboBox);
	pcSetPanel.add(pcSetTextField);
	SpringUtilities.makeCompactGrid(pcSetPanel,
					1, 2,  //rows, cols
					1, 1,  //initX, initY
					1, 1); //xPad, yPad



	// Check buttons on a panel.
	tnCheckBox = new JCheckBox(" T ");
	tnCheckBox.setToolTipText("Pn: Transposition by n");
	tnCheckBox.setActionCommand("tn");
	tnCheckBox.addActionListener(this);

	tnICheckBox = new JCheckBox(" TI ");
	tnICheckBox.setToolTipText("In: inversion at n");
	tnICheckBox.setActionCommand("tnI");
	tnICheckBox.addActionListener(this);

	mnCheckBox = new JCheckBox(" M ");
	mnCheckBox.setToolTipText("Mn: multiplication by n");
	mnCheckBox.setActionCommand("mn");
	mnCheckBox.addActionListener(this);

	RTnCheckBox = new JCheckBox(" RT "); 
	RTnCheckBox.setToolTipText("Rn: retrograde");
	RTnCheckBox.setActionCommand("RTn");
	RTnCheckBox.addActionListener(this);

	// RIn (retrograde inversion)
	RTnICheckBox = new JCheckBox(" RTI "); 
	RTnICheckBox.setToolTipText("RIn: retrograde inversion");
	RTnICheckBox.setActionCommand("RTnI");
	RTnICheckBox.addActionListener(this);


	JPanel checkBoxesPanel1 = new JPanel(new GridLayout(1, 5));
	checkBoxesPanel1.add(tnCheckBox);
	checkBoxesPanel1.add(tnICheckBox);
	checkBoxesPanel1.add(mnCheckBox);
	checkBoxesPanel1.add(RTnCheckBox);
	checkBoxesPanel1.add(RTnICheckBox);

	checkBoxesPanel1.setBorder(
			BorderFactory.createCompoundBorder(
				      BorderFactory.createTitledBorder(
					"in the tone row under: "),
                                BorderFactory.createEmptyBorder(5,5,5,5)));





	primeFormsCheckBox = new JCheckBox("Tn/TnI types of subsequences"); 
	primeFormsCheckBox.setToolTipText(
		"Shows the names of the Tn/TnI types of subsequences of the tone row.");
	primeFormsCheckBox.setActionCommand("primeForms");
	primeFormsCheckBox.addActionListener(this);





	matrixCheckBox = new JCheckBox("Matrix of the tone row"); 
	matrixCheckBox.setToolTipText(
			     "Shows the forty-eight forms of the tone row.");
	matrixCheckBox.setActionCommand("matrix");
	matrixCheckBox.addActionListener(this);




	//Lay out the panel.
	JPanel checkBoxesPanel = new JPanel(new SpringLayout());
	checkBoxesPanel.add(pcSetPanel);
	checkBoxesPanel.add(checkBoxesPanel1);
	checkBoxesPanel.add(primeFormsCheckBox);
	checkBoxesPanel.add(matrixCheckBox);
	SpringUtilities.makeCompactGrid(checkBoxesPanel,
					4, 1,  //rows, cols
					1, 1,  //initX, initY
					1, 1); //xPad, yPad
	checkBoxesPanel.setBorder(
			BorderFactory.createCompoundBorder(
                                BorderFactory.createTitledBorder(
					""),
                                BorderFactory.createEmptyBorder(5,5,5,5)));

	schoenbergScrollPane = new JScrollPane(checkBoxesPanel);
	schoenbergScrollPane.setVerticalScrollBarPolicy(
		JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
	schoenbergScrollPane.setHorizontalScrollBarPolicy(
		JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
	schoenbergScrollPane.setPreferredSize(new Dimension(390, 200));
	schoenbergScrollPane.setMinimumSize(new Dimension(390, 200));







	// "Found Relations" area
	editorPane = new JEditorPane();
	editorScrollPane = new JScrollPane(editorPane);
	editorScrollPane.setVerticalScrollBarPolicy(
				JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
	editorScrollPane.setHorizontalScrollBarPolicy(
			JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
	editorScrollPane.setPreferredSize(new Dimension(250, 350));
	editorScrollPane.setMinimumSize(new Dimension(250, 350));
	editorScrollPane.setBorder(
			BorderFactory.createCompoundBorder(
				BorderFactory.createTitledBorder(
					"Found: "),
				BorderFactory.createEmptyBorder(5,5,5,5)));
	editorScrollPane.getBorder();
    }









    public int[] initSeries(String piece) { // Initialize a 12-tone row.

        // Schoenberg's 12-tone series

	if(piece.equals(" Five Piano Pieces, Op. 23, No. 5 (1920/1923) ")) {
	    int [] toneRow01 = {1,9,11,7,8,6,10,2,4,3,0,5};
	    return toneRow01;
	} else if(piece.equals(" Serenade, Op. 24, Movement 4, \"Sonett\" (1922/1923) ")) {
	    int [] toneRow02 = {4,2,3,11,0,1,8,6,9,5,7,10};
	    return toneRow02;
	} else if(piece.equals(" Serenade, Op. 24, Movement 5, \"Tanzscene\" (1920/1923) ")) {
	    int [] toneRow03 = {9,10,0,3,4,6,5,7,8,11,1,2};
	    return toneRow03;
	} else if(piece.equals(" Suite for Piano, Op. 25 (1921/1923) ")) {
	    int [] toneRow04 = {4,5,7,1,6,3,8,2,11,0,9,10};
	    return toneRow04;
	} else if(piece.equals(" Wind Quintet, Op. 26 (1924) ")) {
	    int [] toneRow05 = {3,7,9,11,1,0,10,2,4,6,8,5};
	    return toneRow05;
	} else if(piece.equals(" Four Pieces for Mixed Chorus, Op. 27, No. 1 (1925) ")) {
	    int [] toneRow06 = {6,5,2,8,7,1,3,4,10,9,11,0};
	    return toneRow06;
	} else if(piece.equals(" Four Pieces for Mixed Chorus, Op. 27, No. 2 (1925) ")) {
	    int [] toneRow07 = {0,11,4,10,2,8,3,7,6,5,9,1};
	    return toneRow07;
	} else if(piece.equals(" Four Pieces for Mixed Chorus, Op. 27, No. 3 (1925) ")) {
	    int [] toneRow08 = {7,6,2,4,5,3,11,0,8,10,9,1};
	    return toneRow08;
	} else if(piece.equals(" Four Pieces for Mixed Chorus, Op. 27, No. 4 (1925) ")) {
	    int [] toneRow09 = {1,3,10,6,8,4,11,0,2,9,5,7};
	    return toneRow09;
	} else if(piece.equals(" Three Satires for Mixed Chorus, Op. 28, No. 1 (1925/1926) ")) {
	    int [] toneRow10 = {0,4,7,1,9,11,5,3,2,6,8,10};
	    return toneRow10;
	} else if(piece.equals(" Three Satires for Mixed Chorus, Op. 28, No. 3 (1925/1926) ")) {
	    int [] toneRow11 = {5,6,4,8,2,10,7,9,3,11,1,0};
	    return toneRow11;
	} else if(piece.equals(" Suite, Op. 29 (1925) ")) {
	    int [] toneRow12 = {3,7,6,10,2,11,0,9,8,4,5,1};
	    return toneRow12;
	} else if(piece.equals(" Third String Quartet, Op. 30 (1927) ")) {
	    int [] toneRow13 = {7,4,3,9,0,5,6,11,10,1,8,2};
	    return toneRow13;
	} else if(piece.equals(" Variations for Orchestra, Op. 31 (1926/1928) ")) {
	    int [] toneRow14 = {10,4,6,3,5,9,2,1,7,8,11,0};
	    return toneRow14;
	} else if(piece.equals(" Von Heute auf Morgen, Op. 32 (1929) ")) {
	    int [] toneRow15 = {2,3,9,1,11,5,8,7,4,0,10,6};
	    return toneRow15;
	} else if(piece.equals(" Two Piano Pieces, Op. 33A, No. 1 (1928) ")) {
	    int [] toneRow16 = {10,5,0,11,9,6,1,3,7,8,2,4}; 
	    return toneRow16;
	} else if(piece.equals(" Two Piano Pieces, Op. 33B, No. 2 (1931) ")) {
	    int [] toneRow17 = {11,1,5,3,9,8,6,10,7,4,0,2};
	    return toneRow17;
	} else if(piece.equals(" Accompaniment to a Film Scene, Op. 34 (1930) ")) {
	    int [] toneRow18 = {3,6,2,4,1,0,9,11,10,8,5,7};
	    return toneRow18;
	} else if(piece.equals(" Six Pieces for Male Chorus, Op. 35, No. 1 (1930) ")) {
	    int [] toneRow19 = {2,11,3,5,4,1,8,10,9,6,0,7};
	    return toneRow19;
	} else if(piece.equals(" Six Pieces for Male Chorus, Op. 35, No. 2 (1930) ")) {
	    int [] toneRow20 = {6,9,7,1,0,2,5,11,10,3,4,8};
	    return toneRow20;
	} else if(piece.equals(" Six Pieces for Male Chorus, Op. 35, No. 3 (1930) ")) {
	    int [] toneRow21 = {3,6,7,8,5,0,9,10,4,11,2,1};
	    return toneRow21;
	} else if(piece.equals(" Six Pieces for Male Chorus, Op. 35, No. 5 (1930) ")) {
	    int [] toneRow22 = {1,7,10,2,3,11,8,4,0,6,5,9};
	    return toneRow22;
	} else if(piece.equals(" Concerto for Violin and Orchestra, Op. 36 (1934/1936) ")) {
	    int [] toneRow23 = {9,10,3,11,4,6,0,1,7,8,2,5};
	    return toneRow23;
	} else if(piece.equals(" Fourth String Quartet, Op. 37 (1936) ")) {
	    int [] toneRow24 = {2,1,9,10,5,3,4,0,8,7,6,11};
	    return toneRow24;
	} else if(piece.equals(" Fragment Of Phantasia for Piano (Four Hands) (date unknown) ")) {
	    int [] toneRow25 = {1,5,3,6,4,8,0,11,2,9,10,7};
	    return toneRow25;
	} else if(piece.equals(" Fragment of Sonata for Organ (date unknown) ")) {
	    int [] toneRow26 = {1,7,11,3,9,2,8,6,10,5,0,4};
	    return toneRow26;
	} else if(piece.equals(" Fragment for Piano (Four Hands) (date unknown) ")) {
	    int [] toneRow27 = {6,9,0,7,1,2,8,11,5,10,4,3};
	    return toneRow27;
	} else if(piece.equals(" Ode To Napoleon Buonaparte, Op. 41 (1942) ")) {
	    int [] toneRow28 = {1,0,4,5,9,8,3,2,6,7,11,10};
	    return toneRow28;
	} else if(piece.equals(" Concerto for Piano and Orchestra, Op. 42 (1942) ")) {
	    int [] toneRow29 = {3,10,2,5,4,0,6,8,1,9,11,7};
	    return toneRow29;
	} else if(piece.equals(" Die Jakobsleiter (1917/1922) ")) {
	    int [] toneRow30 = {1,2,5,4,8,7,0,3,11,10,6,9};
	    return toneRow30;
	} else if(piece.equals(" Prelude to a Suite from \"Genesis\", Op. 44 (1945) ")) {
	    int [] toneRow31 = {10,6,2,5,4,0,11,8,1,3,9,7};
	    return toneRow31;
	} else if(piece.equals(" String Trio, Op. 45 (1946) ")) {
	    int [] toneRow32 = {2,10,3,9,4,1,11,8,6,7,5,0};
	    return toneRow32;
	} else if(piece.equals(" A Survivor from Warsaw, Op. 46 (1947) ")) {
	    int [] toneRow33 = {6,7,0,8,4,3,11,10,5,9,1,2};
	    return toneRow33;
	} else if(piece.equals(" Fantasy for Violin and Piano, Op. 47 (1949) ")) {
	    int [] toneRow34 = {10,9,1,11,5,7,3,4,0,2,8,6};
	    return toneRow34;
	} else if(piece.equals(" Three Songs, Op. 48, No. 1, \"Sommermud\" (1933) ")) {
	    int [] toneRow35 = {1,2,0,6,3,5,4,10,11,7,9,8};
	    return toneRow35;
	} else if(piece.equals(" Three Songs, Op. 48, No. 2, \"Tot\" (1933) ")) {
	    int [] toneRow36 = {2,3,9,1,10,4,8,7,0,11,5,6};
	    return toneRow36;
	} else if(piece.equals(" Three Songs, Op. 48, No. 3, \"Madchenlied\" (1933) ")) {
	    int [] toneRow37 = {1,7,9,11,3,5,10,6,4,0,8,2};
	    return toneRow37;
	} else if(piece.equals(" Israel Exists Again (1949) ")) {
	    int [] toneRow38 = {0,3,4,9,11,5,2,1,10,8,6,7};
	    return toneRow38;
	} else if(piece.equals(" Three Times a Thousand Years, Op. 50A (1949) ")) {
	    int [] toneRow39 = {7,9,6,4,5,11,10,2,0,1,3,8};
	    return toneRow39;
	} else if(piece.equals(" Psalm 130, De Profundis (unfinished), Op. 50B (1950) ")) {
	    int [] toneRow40 = {3,9,8,4,2,10,7,11,0,6,5,1};
	    return toneRow40;
	} else if(piece.equals(" Modern Psalms, Op. 50C, First Psalm (unfinished) (1950) ")) {
	    int [] toneRow41 = {4,3,0,8,11,7,5,9,6,10,1,2};
	    return toneRow41;
	} else if(piece.equals(" Moses and Aron (1930-1932) ")) {
	    int [] toneRow42 = {9,10,4,2,3,1,7,5,6,8,11,0};
	    return toneRow42;
	} else {
	    int [] toneRow43 = {};
	    return toneRow43;
	}
    }
    // The end of initSeries()





    public void actionPerformed(ActionEvent e) {

	if("piece".equals(e.getActionCommand())) {
	    Set s = new Set();
	    toneRowTextField.setText(" " 
	     + s.pcSetArrayToString(initSeries((String) pieceJComboBox.getSelectedItem())));
	}


	if("find".equals(e.getActionCommand())) {
	    editorPane.setText("");		

	    results = new StringBuffer();
	    PcSet pq = new PcSet();

	    // Get a twelve-tone row and a typed-in pc set and
	    // remove space(s) at the beginning and the end of the row.
	    String typedToneRowString 
		= pq.removeFistLastSpaces(toneRowTextField.getText());

	    // Remove consecutive spaces.
	    String noConsecutiveSpacesString 
		= pq.removeConsecutiveSpaces(typedToneRowString);

	    // Eliminate duplicates
	    String duplicatesFreeToneRowString 
		= pq.removeDuplicatesInString(noConsecutiveSpacesString);

	    // Check if there are non-numerical characters
	    String illegalChars 
		= pq.checkNonNumerals(duplicatesFreeToneRowString);



	    // Puts each subsequence of the segmented tone row in an array.
	    char[] toneRowChars 
		= new char[duplicatesFreeToneRowString.length()];
	    duplicatesFreeToneRowString.getChars(
				 0, 
				 duplicatesFreeToneRowString.length(), 
				 toneRowChars, 
				 0);

	    char[][] tmpSegmentsCharArray = new char[24][24];
	    int[] sizeOfEachSegment = new int[24];
	    int numberOfSegments = 0;

	    for(int i = 0; i < toneRowChars.length; i++) {
		while(toneRowChars[i] == ' ' && i < toneRowChars.length)
		    i++;

		sizeOfEachSegment[numberOfSegments] = 0;
		while(toneRowChars[i] != ' ') {
   tmpSegmentsCharArray[numberOfSegments][sizeOfEachSegment[numberOfSegments]] 
       = toneRowChars[i];
		    sizeOfEachSegment[numberOfSegments]++;
		    i++;

		    if(i > toneRowChars.length - 1)
			break;
		}
		numberOfSegments++;
	    }

	    char[][] rowSegmentsCharArray = new char[numberOfSegments][];
	    for (int i = 0; i < rowSegmentsCharArray.length; i++) {
		rowSegmentsCharArray[i] = new char[sizeOfEachSegment[i]];
		System.arraycopy(tmpSegmentsCharArray[i], 
				 0, 
				 rowSegmentsCharArray[i], 
				 0, 
				 sizeOfEachSegment[i]);
	    }


			
	    // Get typed-in pc-set.
	    String readPcSetString = pcSetTextField.getText();

	    // Eliminate spaces
	    StringBuffer withoutSpaceStringBuffer = new StringBuffer();
	    for (int i = 0; i < readPcSetString.length(); i++)
		if (readPcSetString.charAt(i) != ' ')
		    withoutSpaceStringBuffer.append(readPcSetString.charAt(i));

	    // Eliminate duplicates
	    String pcSet2String = pq.removeDuplicatesInString(
				     withoutSpaceStringBuffer.toString());

	    // Check if there are non-numerical characters
	    String pcSetIllegalChars
		= pq.checkNonNumerals(pcSet2String);



	    // Check errors.
	    boolean goAhead = true;

	    // Check if the size of each segmented subsequence is at least 3.
	    for (int iiii = 0; iiii < numberOfSegments; iiii++)
		if (sizeOfEachSegment[iiii] < 3) {
		    JOptionPane.showMessageDialog(
			schoenbergPanel, 
			"The size of each subsequence must be at least 3.",
			"Warning",
			JOptionPane.WARNING_MESSAGE);
		goAhead = false;
		}

	    // Any operations chosen?
	    if (tnCheckBox.getSelectedObjects() == null &&
		tnICheckBox.getSelectedObjects() == null &&
		mnCheckBox.getSelectedObjects() == null &&
		RTnCheckBox.getSelectedObjects() == null &&
		RTnICheckBox.getSelectedObjects() == null &&
		primeFormsCheckBox.getSelectedObjects() == null &&
		matrixCheckBox.getSelectedObjects() == null) {
		JOptionPane.showMessageDialog(
				schoenbergPanel,
				"You need to choose at least "
				+ "one operation.\n",
				"Warning",
				JOptionPane.WARNING_MESSAGE);
		goAhead = false;
	    } else if (pcSet2String.length() < 2
		       && (tnCheckBox.getSelectedObjects() != null
			   || tnICheckBox.getSelectedObjects() != null
			   || mnCheckBox.getSelectedObjects() != null
			   || RTnCheckBox.getSelectedObjects() != null
			   || RTnICheckBox.getSelectedObjects() != null)) {
		JOptionPane.showMessageDialog(
			schoenbergPanel, 
			"The size of the pc-set must be at least 2.",
			"Warning",
			JOptionPane.WARNING_MESSAGE);
		goAhead = false;
	    } else if (illegalChars.length() > 1) {
		JOptionPane.showMessageDialog(
		      schoenbergPanel, 
	      "Non-numerical characters are found in the twelve-tone row:"
		      + illegalChars.toString(),
		      "Warning",
		      JOptionPane.WARNING_MESSAGE);
		goAhead = false;
	    } else if (pq.numberOfNonSpaces(duplicatesFreeToneRowString) 
		      != pq.numberOfNonSpaces(noConsecutiveSpacesString)) {
		    JOptionPane.showMessageDialog(
				  schoenbergPanel, 
				"There are duplicates in the tone row.",
				"Warning",
				JOptionPane.WARNING_MESSAGE);
		goAhead = false;
	    } else if (pq.numberOfNonSpaces(duplicatesFreeToneRowString) 
		       != 12) {
		JOptionPane.showMessageDialog(
			schoenbergPanel, 
			"The size of the tone row is not 12.",
			"Warning",
			JOptionPane.WARNING_MESSAGE);
	    } else if (pcSetIllegalChars.length() > 1) {
		JOptionPane.showMessageDialog(
		      schoenbergPanel,
		      "Non-numerical characters are found:"
		      + pcSetIllegalChars.toString(),
		      "Warning",
		      JOptionPane.WARNING_MESSAGE);
		goAhead = false;
	    } else if(pcSet2String.length() 
			  != withoutSpaceStringBuffer.length()) {
		    JOptionPane.showMessageDialog(
				  schoenbergPanel, 
				"There are duplicates in the pc-set.",
				"Warning",
				JOptionPane.WARNING_MESSAGE);
		goAhead = false;
	    } else {
		if (goAhead == false)
		    goAhead = false;
		else
		    goAhead = true;
	    }


	    
	    // If no error is found, the following will be carried out.
	    if (goAhead == true) {

		PcSet [] rowSegmentsPcSets = new PcSet[numberOfSegments];

		for (int i = 0; i < rowSegmentsPcSets.length; i++) {
		    rowSegmentsPcSets[i] = new PcSet(sizeOfEachSegment[i]);
		    rowSegmentsPcSets[i].set 
	    = rowSegmentsPcSets[i].charArrayToIntOne(rowSegmentsCharArray[i]);
		}


		// Gets an ordered or unordered PC-Set 2.
		char[] pcSet2Chars = new char[pcSet2String.length()];
		pcSet2String.getChars(0, 
				      pcSet2String.length(), 
				      pcSet2Chars, 
				      0);
		PcSet pcSet2 = new PcSet(pcSet2Chars.length);
		pcSet2.set = pcSet2.charArrayToIntOne(pcSet2Chars);


		if (tnCheckBox.getSelectedObjects() != null ||
		    tnICheckBox.getSelectedObjects() != null ||
		    mnCheckBox.getSelectedObjects() != null ||
		    RTnCheckBox.getSelectedObjects() != null ||
		    RTnICheckBox.getSelectedObjects() != null) {

		    // Is PC-Set 2 ordered or unordered?
		    String ordered 
			= (String)oderUnorderJComboBox.getSelectedItem();

		    if(ordered.equals("Ordered PC-Set:")) {
			OrderedInclusion[] inclusion 
			    = new OrderedInclusion[numberOfSegments];
		
			for(int ii = 0; ii < numberOfSegments; ii++)
			    inclusion[ii] 
				= new OrderedInclusion(rowSegmentsPcSets[ii],
						       pcSet2);


			if (tnCheckBox.getSelectedObjects() != null) {
			    int noRelations = 0;
			    int someRelations = 0;
			    for(int ii = 0; ii < inclusion.length; ii++) {
				String nru = inclusion[ii].listFitTns();
				if (nru.equals("noRelationsUnderT"))
				    noRelations++;
				else {
				    results.append(nru);
				    someRelations++;
				}
			    }
			    results.append("\n");
			    if (noRelations != 0 && someRelations == 0)
				results.append(
				       "No relations under T.\n\n");
			}


			if (tnICheckBox.getSelectedObjects() != null) {
			    int noRelations = 0;
			    int someRelations = 0;
			    for(int ii = 0; ii < inclusion.length; ii++) {
				String nru = inclusion[ii].listFitTnIs();
				if (nru.equals("noRelationsUnderTI"))
				    noRelations++;
				else {
				    results.append(nru);
				    someRelations++;
				}
			    }
			    results.append("\n");
			    if (noRelations != 0 && someRelations == 0)
				results.append(
				       "No relations under TI.\n\n");
			}
			    

			if (mnCheckBox.getSelectedObjects() != null) {
			    int noRelations = 0;
			    int someRelations = 0;
			    for(int ii = 0; ii < inclusion.length; ii++) {
				String nru = inclusion[ii].listFitMns();
				if (nru.equals("noRelationsUnderM"))
				    noRelations++;
				else {
				    results.append(nru);
				    someRelations++;
				}
			    }
			    results.append("\n");
			    if (noRelations != 0 && someRelations == 0)
				results.append(
				       "No relations under M.\n\n");
			}
		    

			if (RTnCheckBox.getSelectedObjects() != null) {
			    int noRelations = 0;
			    int someRelations = 0;
			    for(int ii = 0; ii < inclusion.length; ii++) {
				String nru = inclusion[ii].listFitRTs();
				if (nru.equals("noRelationsUnderRT"))
				    noRelations++;
				else {
				    results.append(nru);
				    someRelations++;
				}
			    }
			    results.append("\n");
			    if (noRelations != 0 && someRelations == 0)
				results.append(
				       "No relations under RT.\n\n");
			}


			if (RTnICheckBox.getSelectedObjects() != null) {
			    int noRelations = 0;
			    int someRelations = 0;
			    for(int ii = 0; ii < inclusion.length; ii++) {
				String nru = inclusion[ii].listFitRTnIs();
				if (nru.equals("noRelationsUnderRTI"))
				    noRelations++;
				else {
				    results.append(nru);
				    someRelations++;
				}
			    }
			    results.append("\n");
			    if (noRelations != 0 && someRelations == 0)
				results.append(
				       "No relations under RTI.\n\n");
			}

		    } else {
			if(rowSegmentsPcSets.length == 1)
			    JOptionPane.showMessageDialog(
				schoenbergPanel,
				"You need to segment the tone row\n by inserting a space(s)\n",
				"Warning",
				JOptionPane.WARNING_MESSAGE);
			else {
			    UnorderedInclusion[] inclusion 
				= new UnorderedInclusion[numberOfSegments];
		    
			    for(int ii = 0; ii < numberOfSegments; ii++)
				inclusion[ii] 
				    = new UnorderedInclusion(
						     rowSegmentsPcSets[ii],
						     pcSet2);

			    if (tnCheckBox.getSelectedObjects() != null)
				for(int i = 0; i < inclusion.length; i++)
				    results.append(inclusion[i].listFitTns());
			    
			    if (tnICheckBox.getSelectedObjects() != null)
				for(int i = 0; i < inclusion.length; i++)
				    results.append(inclusion[i].listFitTnIs());

			    if (mnCheckBox.getSelectedObjects() != null)
				for(int i = 0; i < inclusion.length; i++)
				    results.append(inclusion[i].listFitMns());
			}
			if (RTnCheckBox.getSelectedObjects() != null ||
			    RTnICheckBox.getSelectedObjects() != null)
			    JOptionPane.showMessageDialog(
				  schoenbergPanel,
				"Retrograde can not be "
				+ "applied to an unordered pc-set.\n",
				"Warning",
				JOptionPane.WARNING_MESSAGE);
		    }
		}
	




		// Chooses operations.
		if (primeFormsCheckBox.getSelectedObjects() != null) {

		    if (rowSegmentsPcSets.length < 2)
			JOptionPane.showMessageDialog(
			schoenbergPanel,
			"Insert a space(s) in the twelve-tone row to make subsequences.\n",
			"Warning",
			JOptionPane.WARNING_MESSAGE);
		else {
		    TnTnIType[] subSequences 
			= new TnTnIType[rowSegmentsPcSets.length];
		    Set[] prime = new Set[rowSegmentsPcSets.length];
		    for(int i = 0; i < rowSegmentsPcSets.length; i++) {
			subSequences[i] 
			    = new TnTnIType(rowSegmentsPcSets[i].set);
			subSequences[i].base12String 
			    = rowSegmentsPcSets[i].base12String;
			subSequences[i].getTnType();
			subSequences[i].getTnIType();
			subSequences[i].getTnTnIType();
			prime[i] = subSequences[i].getPrimeByTnTnI();
		    }

		    results.append("\n");
		    results.append("Subsequence: ");
		    for(int i = 0; i < rowSegmentsPcSets.length; i++) {
			results.append("<");
			results.append(
	       subSequences[i].pcSetArrayToString(rowSegmentsPcSets[i].set));
			results.append("> ");
		    }

		    results.append("\n");
		    results.append("Tn/TnI type:      ");
		    for(int i = 0; i < rowSegmentsPcSets.length; i++) {
			results.append("[");
			results.append(subSequences[i].tntnitypeToString());
			results.append("]   ");
		    }

		    results.append("\n");
		    results.append("I-Vector:          ");
		    for(int i = 0; i < rowSegmentsPcSets.length; i++) {
			results.append("<");
			results.append(prime[i].ivectorToString());
			results.append("> ");
		    }

		    results.append("\n");
		    results.append("Forte Name:             ");
		    for(int i = 0; i < rowSegmentsPcSets.length; i++) {
			results.append(prime[i].name);
			results.append("           ");
		    }

		    results.append("\n");
		}
	    }
		






	    // Chooses operations.
	    if (matrixCheckBox.getSelectedObjects() != null) {

		    results.append("\n");
		    results.append("Matrix: \n\n");
		    Set st = new Set();
		    for(int i = 0; i < rowSegmentsPcSets.length; i++) {
			st.base12String = rowSegmentsPcSets[i].base12String;

			int[][] mtrx = st.matrix(rowSegmentsPcSets[i].set);

			results.append("        ");
			for(int h = 0; h < mtrx.length; h++) {
			    if (10 == (((mtrx[0][h] + 12) - mtrx[0][0]) % 12))
				results.append("IA  ");
			    else if (11
			     == (((mtrx[0][h] + 12) - mtrx[0][0]) % 12))
				results.append("IB  ");
			    else
				results.append("I"
			       + (((mtrx[0][h] + 12) - mtrx[0][0]) % 12)
					       + "  ");
			}
			results.append("\n");
			results.append("        -----------------------------");
			results.append("\n");
			
			for(int ii = 0; ii < mtrx.length; ii++) {
			    if (10 == (((mtrx[ii][0] + 12) - mtrx[0][0]) % 12))
				results.append("PA: ");
			    else if (11
			     == (((mtrx[ii][0] + 12) - mtrx[0][0]) % 12))
				results.append("PB: ");
			    else
				results.append("P"
			       + (((mtrx[ii][0] + 12) - mtrx[0][0]) % 12)
					       + ": ");
			    
			    for(int iii = 0; iii < mtrx.length; iii++) {
				results.append("   ");
				results.append(st.intToChar(mtrx[ii][iii]));
			    }
			    results.append("\n");
			}
			results.append("\n");
		    }
		}
	    }

	    editorPane.setText(results.toString());
	}

	if ("clear".equals(e.getActionCommand())) {
	    pcSetTextField.setText("");
	    editorPane.setText("");		
	}
    }
}

