// StravinskyFrame.java, used for
// JDubiel.java: computer program for pc-set analysis
// by Akira Takaoka, akira.takaoka@columbia.edu
// Copyright (C) 1994-2021 Akira Takaoka
// All rights reserved.





// package jdubielApplication;

import java.lang.reflect.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;



public class StravinskyFrame extends JFrame implements ActionListener {

    static final int xOffset = 300, yOffset = 20;
    static int openFrameCount = 0;

    JScrollPane stravinskyScrollPane;
    JPanel stravinskyPanel;
    JPanel buttonPanel;

    JComboBox<String> pieceJComboBox;
    JTextField toneRowTextField;

    JComboBox<String> oderUnorderJComboBox;
    JTextField pcSetTextField;

    JEditorPane editorPane;
    JScrollPane editorScrollPane;

    JCheckBox tnCheckBox;
    JCheckBox tnICheckBox;
    JCheckBox mnCheckBox;
    JCheckBox rkTnCheckBox;
    JCheckBox rkTnICheckBox;
    JCheckBox RTnCheckBox;
    JCheckBox RTnICheckBox;
    JCheckBox rkRTnCheckBox;
    JCheckBox rkRTnICheckBox;

    JCheckBox pVerticalsCheckBox;
    JCheckBox rVerticalsCheckBox;
    JCheckBox iVerticalsCheckBox;
    JCheckBox riVerticalsCheckBox;
    JCheckBox irVerticalsCheckBox;

    SpinnerNumberModel pVerticalsModel;
    SpinnerNumberModel rVerticalsModel;
    SpinnerNumberModel iVerticalsModel;
    SpinnerNumberModel riVerticalsModel;
    SpinnerNumberModel irVerticalsModel;

    JCheckBox primeFormsCheckBox;
    JCheckBox matrixCheckBox;

    StringBuffer results;







    public StravinskyFrame() {

	//...Create the GUI and put it in the window...
	createWidgets();

	//Put everything together.
	JPanel devidedPane = new JPanel(new BorderLayout());
	devidedPane.add(stravinskyPanel, BorderLayout.NORTH);
	devidedPane.add(buttonPanel, BorderLayout.WEST);
	devidedPane.add(stravinskyScrollPane, BorderLayout.CENTER);
	devidedPane.add(editorScrollPane, BorderLayout.SOUTH);

	getContentPane().add(devidedPane, BorderLayout.CENTER);

	setSize(430,620);
	openFrameCount++;
	setTitle("Stravinsky's Twelve-Tone Compositions " + openFrameCount);
	setLocation(xOffset + 20 * openFrameCount, 
		    yOffset + 20 * openFrameCount);
    }
	



	// Create and add the widgets.
	private void createWidgets() {

		JLabel piecesLabel = new JLabel("Choose a piece: ", 
						SwingConstants.RIGHT);
		String[] pieceStrings = {
			"Agon P1",
			"Agon P2",
			"Agon P3",
			"Agon P4",
			"Canticum Sacrum P1",
			"Canticum Sacrum P2",
			"Threni",
			"Movements",
			"Epitaphium",
			"Double Canon",
			"A Sermon, a Narrative, and a Prayer",
			"Anthem: The Dove Descending",
			"The Flood",
			"Abraham and Isaac",
			"Elegy J.F.K.",
			"Variations",
			"Introitus",
			"Requiem Canticles P1",
			"Requiem Canticles P2",
			"The Owl and the Pussycat"};
		pieceJComboBox = new JComboBox<>(pieceStrings);
		pieceJComboBox.setSelectedIndex(7);
		pieceJComboBox.setBackground(Color.white);
		pieceJComboBox.setFont(new Font("SansSerif", 
						Font.ITALIC, 14));
		pieceJComboBox.setActionCommand("piece");
		pieceJComboBox.addActionListener(this);
		piecesLabel.setLabelFor(pieceJComboBox);

		JLabel toneRowLabel = new JLabel("Its tone row: ", 
						SwingConstants.RIGHT);
		toneRowLabel.setToolTipText(
					"Insert a space(s) to segment the "
					+ "tone row. Ex. 34A892 0B1675 ");
		toneRowTextField = new JTextField(12);
		toneRowTextField.setBackground(Color.white);
		toneRowLabel.setLabelFor(toneRowTextField);
		toneRowTextField.setText(" 34A8920B1675 ");
		toneRowTextField.setToolTipText(
					"Insert a space(s) to segment the "
					+ "tone row. Ex. 34A892 0B1675 ");
		toneRowTextField.setDragEnabled(true);

		//Add the widgets to the container.
		stravinskyPanel = new JPanel(new GridLayout(2, 2));
		stravinskyPanel.add(piecesLabel);
		stravinskyPanel.add(pieceJComboBox);
		stravinskyPanel.add(toneRowLabel);
		stravinskyPanel.add(toneRowTextField);
		stravinskyPanel.setBorder(
			BorderFactory.createCompoundBorder(
                                BorderFactory.createTitledBorder(
					"Stravinsky's Twelve-Tone composition: "),
                                BorderFactory.createEmptyBorder(5,5,5,5)));







		// Find button
		JButton findButton = new JButton("Find");
		findButton.setActionCommand("find");
		findButton.addActionListener(this);
		findButton.setBackground(Color.lightGray);
		findButton.setFont(new Font("SansSerif", Font.BOLD, 12));

		// Clear button
		JButton clearButton = new JButton("Clear");
		clearButton.setActionCommand("clear");
		clearButton.addActionListener(this);
		clearButton.setBackground(Color.lightGray);

		JPanel buttonPanel1 = new JPanel(new SpringLayout());
		buttonPanel1.add(findButton);
		buttonPanel1.add(clearButton);
		SpringUtilities.makeCompactGrid(buttonPanel1,
						2, 1,  //rows, cols
						1, 1,  //initX, initY
						1, 1); //xPad, yPad

		buttonPanel = new JPanel(new BorderLayout());
		buttonPanel.add(buttonPanel1, BorderLayout.NORTH);







		String[] oderUnorderStrings = {"Ordered PC-Set:",
					       "Unordered PC-Set:"};
		oderUnorderJComboBox = new JComboBox<>(oderUnorderStrings);
		oderUnorderJComboBox.setSelectedIndex(0);
		oderUnorderJComboBox.setAlignmentY(RIGHT_ALIGNMENT);
		oderUnorderJComboBox.setBackground(Color.white);
		pcSetTextField = new JTextField(10);
		pcSetTextField.setBackground(Color.white);
		pcSetTextField.setDragEnabled(true);

		JPanel pcSetPanel = new JPanel(new SpringLayout());
		pcSetPanel.add(oderUnorderJComboBox);
		pcSetPanel.add(pcSetTextField);
		SpringUtilities.makeCompactGrid(pcSetPanel,
						1, 2,  //rows, cols
						1, 1,  //initX, initY
						1, 1); //xPad, yPad





		// Check buttons on a panel.
		tnCheckBox = new JCheckBox(" T ");
		tnCheckBox.setToolTipText("Pn: Transposition by n");
		tnCheckBox.setActionCommand("tn");
		tnCheckBox.addActionListener(this);

		tnICheckBox = new JCheckBox(" TI ");
		tnICheckBox.setToolTipText("In: inversion at n");
		tnICheckBox.setActionCommand("tnI");
		tnICheckBox.addActionListener(this);

		mnCheckBox = new JCheckBox(" M ");
		mnCheckBox.setToolTipText("Mn: multiplication by n");
		mnCheckBox.setActionCommand("mn");
		mnCheckBox.addActionListener(this);

		rkTnCheckBox = new JCheckBox(" rT "); 
		rkTnCheckBox.setToolTipText("Rotated Pn: Pn rotated by k");
		rkTnCheckBox.setActionCommand("rkTn");
		rkTnCheckBox.addActionListener(this);

		rkTnICheckBox = new JCheckBox(" rTI "); 
		rkTnICheckBox.setToolTipText(
				     "Rotated In: inversion rotated by k");
		rkTnICheckBox.setActionCommand("rkTnI");
		rkTnICheckBox.addActionListener(this);

		RTnCheckBox = new JCheckBox(" RT "); 
		RTnCheckBox.setToolTipText("Rn: retrograde");
		RTnCheckBox.setActionCommand("RTn");
		RTnCheckBox.addActionListener(this);

		// RIn (retrograde inversion)
		RTnICheckBox = new JCheckBox(" RTI "); 
		RTnICheckBox.setToolTipText("RIn: retrograde inversion");
		RTnICheckBox.setActionCommand("RTnI");
		RTnICheckBox.addActionListener(this);

		rkRTnCheckBox = new JCheckBox(" rRT "); 
		rkRTnCheckBox.setToolTipText(
			"Rotated Rn: retrograde rotated by k");
		rkRTnCheckBox.setActionCommand("rkRTn");
		rkRTnCheckBox.addActionListener(this);

		rkRTnICheckBox = new JCheckBox(" rRTI "); 
		rkRTnICheckBox.setToolTipText(
			"Rotated RIn: retrograde inversion rotated by k");
		rkRTnICheckBox.setActionCommand("rkRTnI");
		rkRTnICheckBox.addActionListener(this);

		JPanel checkBoxesPanel1 = new JPanel(new GridLayout(3, 3));
		checkBoxesPanel1.add(tnCheckBox);
		checkBoxesPanel1.add(tnICheckBox);
		checkBoxesPanel1.add(mnCheckBox);
		checkBoxesPanel1.add(rkTnCheckBox);
		checkBoxesPanel1.add(rkTnICheckBox);
		checkBoxesPanel1.add(RTnCheckBox);
		checkBoxesPanel1.add(RTnICheckBox);
		checkBoxesPanel1.add(rkRTnCheckBox);
		checkBoxesPanel1.add(rkRTnICheckBox);

		checkBoxesPanel1.setBorder(
			BorderFactory.createCompoundBorder(
                                BorderFactory.createTitledBorder(
					"in the tone row under: "),
                                BorderFactory.createEmptyBorder(5,5,5,5)));



		pVerticalsCheckBox = new JCheckBox("P rotational array of size:         "); 
		pVerticalsCheckBox.setActionCommand("pVerticals");
		pVerticalsCheckBox.addActionListener(this);

		pVerticalsModel = new SpinnerNumberModel(
			6, //initial value
			3, //min
			12, //max
			1); //step
		JSpinner pVerticalsSpinner = new JSpinner(pVerticalsModel);
		pVerticalsSpinner.setEditor(
			new JSpinner.NumberEditor(pVerticalsSpinner));
		pVerticalsSpinner.setToolTipText("The number of the layers of the array.");
		Dimension d2 = new Dimension();
		d2.setSize(0.1, 0.1);
		pVerticalsSpinner.setSize(10,10);



		rVerticalsCheckBox = new JCheckBox("R rotational array of size:         "); 
		rVerticalsCheckBox.setActionCommand("rVerticals");
		rVerticalsCheckBox.addActionListener(this);

		rVerticalsModel = new SpinnerNumberModel(
			6, //initial value
			3, //min
			12, //max
			1); //step
		JSpinner rVerticalsSpinner = new JSpinner(rVerticalsModel);
		rVerticalsSpinner.setEditor(
			new JSpinner.NumberEditor(rVerticalsSpinner));
		rVerticalsSpinner.setToolTipText("The number of the layers of the array.");
		d2.setSize(0.1, 0.1);
		rVerticalsSpinner.setSize(10,10);



		iVerticalsCheckBox = new JCheckBox("I rotational array of size:          "); 
		iVerticalsCheckBox.setActionCommand("iVerticals");
		iVerticalsCheckBox.addActionListener(this);

		iVerticalsModel = new SpinnerNumberModel(
			6, //initial value
			3, //min
			12, //max
			1); //step
		JSpinner iVerticalsSpinner = new JSpinner(iVerticalsModel);
		iVerticalsSpinner.setEditor(
			new JSpinner.NumberEditor(iVerticalsSpinner));
		iVerticalsSpinner.setToolTipText("The number of the layers of the array.");
		d2.setSize(0.1, 0.1);
		iVerticalsSpinner.setSize(10,10);



		riVerticalsCheckBox = new JCheckBox("RI rotational array of size:        "); 
		riVerticalsCheckBox.setActionCommand("riVerticals");
		riVerticalsCheckBox.addActionListener(this);

		riVerticalsModel = new SpinnerNumberModel(
			6, //initial value
			3, //min
			12, //max
			1); //step
		JSpinner riVerticalsSpinner = new JSpinner(riVerticalsModel);
		riVerticalsSpinner.setEditor(
			new JSpinner.NumberEditor(riVerticalsSpinner));
		riVerticalsSpinner.setToolTipText("The number of the layers of the array.");
		d2.setSize(0.1, 0.1);
		riVerticalsSpinner.setSize(10,10);



		irVerticalsCheckBox = new JCheckBox("IR rotational array of size:        "); 
		irVerticalsCheckBox.setActionCommand("irVerticals");
		irVerticalsCheckBox.addActionListener(this);

		irVerticalsModel = new SpinnerNumberModel(
			6, //initial value
			3, //min
			12, //max
			1); //step
		JSpinner irVerticalsSpinner = new JSpinner(irVerticalsModel);
		irVerticalsSpinner.setEditor(
			new JSpinner.NumberEditor(irVerticalsSpinner));
		irVerticalsSpinner.setToolTipText("The number of the layers of the array.");
		d2.setSize(0.1, 0.1);
		irVerticalsSpinner.setSize(10,10);



		JPanel checkBoxesPanelP = new JPanel();
		checkBoxesPanelP.setLayout(new BoxLayout(checkBoxesPanelP, BoxLayout.X_AXIS));
		checkBoxesPanelP.add(pVerticalsCheckBox);
		checkBoxesPanelP.add(pVerticalsSpinner);

		JPanel checkBoxesPanelR = new JPanel();
		checkBoxesPanelR.setLayout(new BoxLayout(checkBoxesPanelR, BoxLayout.X_AXIS));
		checkBoxesPanelR.add(rVerticalsCheckBox);
		checkBoxesPanelR.add(rVerticalsSpinner);

		JPanel checkBoxesPanelI = new JPanel();
		checkBoxesPanelI.setLayout(new BoxLayout(checkBoxesPanelI, BoxLayout.X_AXIS));
		checkBoxesPanelI.add(iVerticalsCheckBox);
		checkBoxesPanelI.add(iVerticalsSpinner);

		JPanel checkBoxesPanelRI = new JPanel();
		checkBoxesPanelRI.setLayout(new BoxLayout(checkBoxesPanelRI, BoxLayout.X_AXIS));
		checkBoxesPanelRI.add(riVerticalsCheckBox);
		checkBoxesPanelRI.add(riVerticalsSpinner);

		JPanel checkBoxesPanelIR = new JPanel();
		checkBoxesPanelIR.setLayout(new BoxLayout(checkBoxesPanelIR, BoxLayout.X_AXIS));
		checkBoxesPanelIR.add(irVerticalsCheckBox);
		checkBoxesPanelIR.add(irVerticalsSpinner);


		JPanel checkBoxesPanel2 = new JPanel(new GridLayout(5, 1));
		checkBoxesPanel2.add(checkBoxesPanelP);
		checkBoxesPanel2.add(checkBoxesPanelR);
		checkBoxesPanel2.add(checkBoxesPanelI);
		checkBoxesPanel2.add(checkBoxesPanelRI);
		checkBoxesPanel2.add(checkBoxesPanelIR);
		checkBoxesPanel2.setBorder(
			BorderFactory.createCompoundBorder(
                                BorderFactory.createTitledBorder(
					"in the verticals of"),
                                BorderFactory.createEmptyBorder(5,5,5,5)));







		primeFormsCheckBox = new JCheckBox("Tn/TnI types of subsequences"); 
		primeFormsCheckBox.setToolTipText(
			"Shows Forte names, Tn/TnI types, and i-vectors of subsequences.");
		primeFormsCheckBox.setActionCommand("primeForms");
		primeFormsCheckBox.addActionListener(this);





		matrixCheckBox = new JCheckBox("Matrix of the tone row"); 
		matrixCheckBox.setToolTipText(
			"Shows the forty-eight forms of the tone row.");
		matrixCheckBox.setActionCommand("matrix");
		matrixCheckBox.addActionListener(this);




		//Lay out the panel.
		JPanel checkBoxesPanel = new JPanel(new SpringLayout());
		checkBoxesPanel.add(pcSetPanel);
		checkBoxesPanel.add(checkBoxesPanel1);
		checkBoxesPanel.add(checkBoxesPanel2);
		checkBoxesPanel.add(primeFormsCheckBox);
		checkBoxesPanel.add(matrixCheckBox);
		SpringUtilities.makeCompactGrid(checkBoxesPanel,
						5, 1,  //rows, cols
						1, 1,  //initX, initY
						1, 1); //xPad, yPad
		checkBoxesPanel.setBorder(
			BorderFactory.createCompoundBorder(
                                BorderFactory.createTitledBorder(
					""),
                                BorderFactory.createEmptyBorder(5,5,5,5)));

		stravinskyScrollPane = new JScrollPane(checkBoxesPanel);
		stravinskyScrollPane.setVerticalScrollBarPolicy(
                        JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		stravinskyScrollPane.setHorizontalScrollBarPolicy(
			JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		stravinskyScrollPane.setPreferredSize(new Dimension(330, 150));
		stravinskyScrollPane.setMinimumSize(new Dimension(330, 150));








		// "Found Relations" area
		editorPane = new JEditorPane();
		editorScrollPane = new JScrollPane(editorPane);
		editorScrollPane.setVerticalScrollBarPolicy(
                        JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		editorScrollPane.setHorizontalScrollBarPolicy(
			JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		editorScrollPane.setPreferredSize(new Dimension(250, 350));
		editorScrollPane.setMinimumSize(new Dimension(250, 350));
		editorScrollPane.setBorder(
			BorderFactory.createCompoundBorder(
				BorderFactory.createTitledBorder(
					"Found: "),
				BorderFactory.createEmptyBorder(5,5,5,5)));
		editorScrollPane.getBorder();
	}









    public int[] initSeries(String piece) { // Initialize a 12-tone row.

	if (piece.equals("Agon P1")) {
	    int[] toneRow00 = {3,2,4,5,0,11,1,10,8,9,7,6}; 
	    return toneRow00;
	} else if (piece.equals("Agon P2")) {
	    int[] toneRow01 = {9,11,10,0,1,3,4,6,5,7,2,8}; 
	    return toneRow01;
	} else if (piece.equals("Agon P3")) {
	    int[] toneRow02 = {0,2,3,5,4,9,7,8,10,11,1,6}; 
	    return toneRow02;
	} else if (piece.equals("Agon P4")) {
	    int[] toneRow03 = {5,6,9,8,7,10,11,2,1,4,3,0}; 
	    return toneRow03;
	} else if (piece.equals("Canticum Sacrum P1")) {
	    int[] toneRow04 = {8,7,5,2,6,4,3,1,10,0,11,9}; 
	    return toneRow04;
	} else if (piece.equals("Canticum Sacrum P2")) {
	    int[] toneRow05 = {9,8,10,0,1,11,4,3,6,2,5,7}; 
	    return toneRow05;
	} else if (piece.equals("Threni")) {
	    int[] toneRow06 = {3,8,7,10,1,9,2,11,4,0,5,6};
	    return toneRow06;
	} else if (piece.equals("Movements")) {
	    int[] toneRow07 = {3,4,10,8,9,2,0,11,1,6,7,5};
	    return toneRow07;
	} else if (piece.equals("Epitaphium")) {
	    int[] toneRow08 = {1,10,3,4,0,11,6,5,2,7,8,9};
	    return toneRow08;
	} else if (piece.equals("Double Canon")) {
	    int[] toneRow09 = {6,5,9,8,7,2,0,3,4,1,11,10};
	    return toneRow09;
	} else if (piece.equals("A Sermon, a Narrative, and a Prayer")) {
	    int[] toneRow10 = {3,4,0,2,1,10,11,6,7,9,8,5};
	    return toneRow10;
	} else if (piece.equals("Anthem: The Dove Descending")) {
	    int[] toneRow11 = {5,8,3,1,11,0,10,9,7,4,6,2};
	    return toneRow11;
	} else if (piece.equals("The Flood")) {
	    int[] toneRow12 = {8,7,9,10,2,4,5,3,6,0,11,1};
	    return toneRow12;
	} else if (piece.equals("Abraham and Isaac")) {
	    int[] toneRow13 = {7,8,10,0,1,9,11,3,2,4,6,5};
	    return toneRow13;
	} else if (piece.equals("Elegy J.F.K.")) {
	    int[] toneRow14 = {8,2,0,10,4,5,11,9,7,6,3,1};
	    return toneRow14;
	} else if (piece.equals("Variations")) {
	    int[] toneRow15 = {2,0,9,11,4,10,8,1,3,7,6,5};
	    return toneRow15;
	} else if (piece.equals("Introitus")) {
	    int[] toneRow16 = {8,1,2,0,4,5,11,10,9,6,7,3};
	    return toneRow16;
	} else if (piece.equals("Requiem Canticles P1")) {
	    int[] toneRow17 = {5,0,11,9,10,2,1,3,8,6,4,7};
	    return toneRow17;
	} else if (piece.equals("Requiem Canticles P2")) {
	    int[] toneRow18 = {5,7,3,4,6,1,11,0,2,9,8,10};
	    return toneRow18;
	} else if (piece.equals("The Owl and the Pussycat")) {
	    int[] toneRow19 = {2,4,11,1,10,8,7,9,0,3,6,5};
	    return toneRow19;
	} else {
	    int[] toneRow20 = {};
	    return toneRow20;
	}
    }
    // The end of initSeries()





    public void actionPerformed(ActionEvent e) {

	if("piece".equals(e.getActionCommand())) {
	    Set s = new Set();
	    toneRowTextField.setText(" " 
		+ s.pcSetArrayToString(initSeries((String) pieceJComboBox.getSelectedItem())));
	}


	if("find".equals(e.getActionCommand())) {
	    editorPane.setText("");		

	    results = new StringBuffer();
	    PcSet pq = new PcSet();

	    // Get a twelve-tone row and a typed-in pc set and
	    // remove space(s) at the beginning and the end of the row.
	    String typedToneRowString 
		= pq.removeFistLastSpaces(toneRowTextField.getText());

	    // Remove consecutive spaces.
	    String noConsecutiveSpacesString 
		= pq.removeConsecutiveSpaces(typedToneRowString);

	    // Eliminate duplicates
	    String duplicatesFreeToneRowString 
		= pq.removeDuplicatesInString(noConsecutiveSpacesString);

	    // Check if there are non-numerical characters
	    String illegalChars 
		= pq.checkNonNumerals(duplicatesFreeToneRowString);



	    // Puts each subsequence of the segmented tone row in an array.
	    char[] toneRowChars 
		= new char[duplicatesFreeToneRowString.length()];
	    duplicatesFreeToneRowString.getChars(
				     0, 
				     duplicatesFreeToneRowString.length(), 
				     toneRowChars, 
				     0);

	    char[][] tmpSegmentsCharArray = new char[24][24];
	    int[] sizeOfEachSegment = new int[24];
	    int numberOfSegments = 0;

	    for(int i = 0; i < toneRowChars.length; i++) {
		while(toneRowChars[i] == ' ' && i < toneRowChars.length)
		    i++;

		sizeOfEachSegment[numberOfSegments] = 0;
		while(toneRowChars[i] != ' ') {
   tmpSegmentsCharArray[numberOfSegments][sizeOfEachSegment[numberOfSegments]] 
       = toneRowChars[i];
		    sizeOfEachSegment[numberOfSegments]++;
		    i++;

		    if(i > toneRowChars.length - 1)
			break;
		}
		numberOfSegments++;
	    }

	    char[][] rowSegmentsCharArray = new char[numberOfSegments][];
	    for (int i = 0; i < rowSegmentsCharArray.length; i++) {
		rowSegmentsCharArray[i] = new char[sizeOfEachSegment[i]];
		System.arraycopy(tmpSegmentsCharArray[i], 
				 0, 
				 rowSegmentsCharArray[i], 
				 0, 
				 sizeOfEachSegment[i]);
	    }



	    // Get typed-in pc-set.
	    String readPcSetString = pcSetTextField.getText();

	    // Eliminate spaces
	    StringBuffer withoutSpaceStringBuffer = new StringBuffer();
	    for (int i = 0; i < readPcSetString.length(); i++)
		if (readPcSetString.charAt(i) != ' ')
		    withoutSpaceStringBuffer.append(readPcSetString.charAt(i));

	    // Eliminate duplicates
	    String pcSet2String = pq.removeDuplicatesInString(
				     withoutSpaceStringBuffer.toString());

	    // Check if there are non-numerical characters
	    String pcSetIllegalChars
		= pq.checkNonNumerals(pcSet2String);



	    // Check errors.
	    boolean goAhead = true;

	    // Check if the size of each segmented subsequence is at least 3.
	    for (int iiii = 0; iiii < numberOfSegments; iiii++)
		if (sizeOfEachSegment[iiii] < 3) {
		    JOptionPane.showMessageDialog(
			stravinskyPanel, 
			"The size of each subsequence must be at least 3.",
			"Warning",
			JOptionPane.WARNING_MESSAGE);
		goAhead = false;
		}

	    // Any operations chosen?
	    if (tnCheckBox.getSelectedObjects() == null &&
		tnICheckBox.getSelectedObjects() == null &&
		mnCheckBox.getSelectedObjects() == null &&
		rkTnCheckBox.getSelectedObjects() == null &&
		rkTnICheckBox.getSelectedObjects() == null &&
		RTnCheckBox.getSelectedObjects() == null &&
		RTnICheckBox.getSelectedObjects() == null &&
		rkRTnCheckBox.getSelectedObjects() == null &&
		rkRTnICheckBox.getSelectedObjects() == null &&
		pVerticalsCheckBox.getSelectedObjects() == null &&
		rVerticalsCheckBox.getSelectedObjects() == null &&
		iVerticalsCheckBox.getSelectedObjects() == null &&
		riVerticalsCheckBox.getSelectedObjects() == null &&
		irVerticalsCheckBox.getSelectedObjects() == null &&
		primeFormsCheckBox.getSelectedObjects() == null &&
		matrixCheckBox.getSelectedObjects() == null) {
		JOptionPane.showMessageDialog(
				stravinskyPanel,
				"You need to choose at least "
				+ "one operation.\n",
				"Warning",
				JOptionPane.WARNING_MESSAGE);
		goAhead = false;
	    } else if (pcSet2String.length() < 2 
		       && (tnCheckBox.getSelectedObjects() != null
			   || tnICheckBox.getSelectedObjects() != null
			   || mnCheckBox.getSelectedObjects() != null
			   || rkTnCheckBox.getSelectedObjects() != null
			   || rkTnICheckBox.getSelectedObjects() != null
			   || RTnCheckBox.getSelectedObjects() != null
			   || RTnICheckBox.getSelectedObjects() != null
			   || rkRTnCheckBox.getSelectedObjects() != null
			   || rkRTnICheckBox.getSelectedObjects() != null
			   || pVerticalsCheckBox.getSelectedObjects() != null
			   || rVerticalsCheckBox.getSelectedObjects() != null
			   || iVerticalsCheckBox.getSelectedObjects() != null
			   || riVerticalsCheckBox.getSelectedObjects() != null
			   || irVerticalsCheckBox.getSelectedObjects() 
			   != null)) {
		JOptionPane.showMessageDialog(
			stravinskyPanel, 
			"The size of the pc-set must be at least 2.",
			"Warning",
			JOptionPane.WARNING_MESSAGE);
		goAhead = false;
	    } else if (illegalChars.length() > 1) {
		JOptionPane.showMessageDialog(
		      stravinskyPanel, 
	      "Non-numerical characters are found in the twelve-tone row:"
		      + illegalChars.toString(),
		      "Warning",
		      JOptionPane.WARNING_MESSAGE);
		goAhead = false;
	    } else if (pq.numberOfNonSpaces(duplicatesFreeToneRowString) 
		      != pq.numberOfNonSpaces(noConsecutiveSpacesString)) {
		    JOptionPane.showMessageDialog(
				  stravinskyPanel, 
				"There are duplicates in the tone row.",
				"Warning",
				JOptionPane.WARNING_MESSAGE);
		goAhead = false;
	    } else if (pq.numberOfNonSpaces(duplicatesFreeToneRowString) 
		       != 12) {
		JOptionPane.showMessageDialog(
			stravinskyPanel, 
			"The size of the tone row is not 12.",
			"Warning",
			JOptionPane.WARNING_MESSAGE);
	    } else if (pcSetIllegalChars.length() > 1) {
		JOptionPane.showMessageDialog(
		      stravinskyPanel,
		      "Non-numerical characters are found:"
		      + pcSetIllegalChars.toString(),
		      "Warning",
		      JOptionPane.WARNING_MESSAGE);
		goAhead = false;
	    } else if(pcSet2String.length() 
			  != withoutSpaceStringBuffer.length()) {
		    JOptionPane.showMessageDialog(
				  stravinskyPanel,
				"There are duplicates in Ordered PC-Set.",
				"Warning",
				JOptionPane.WARNING_MESSAGE);
		goAhead = false;
	    } else {
		if (goAhead == false)
		    goAhead = false;
		else
		    goAhead = true;
	    }



	    // If no error is found, the following will be carried out.
	    if (goAhead == true) {

		PcSet [] rowSegmentsPcSets = new PcSet[numberOfSegments];

		for (int i = 0; i < rowSegmentsPcSets.length; i++) {
		    rowSegmentsPcSets[i] = new PcSet(sizeOfEachSegment[i]);
		    rowSegmentsPcSets[i].set 
	    = rowSegmentsPcSets[i].charArrayToIntOne(rowSegmentsCharArray[i]);
		}


		// Gets an ordered or unordered PC-Set 2.
		char[] pcSet2Chars = new char[pcSet2String.length()];
		pcSet2String.getChars(0, 
				      pcSet2String.length(), 
				      pcSet2Chars, 
				      0);
		PcSet pcSet2 = new PcSet(pcSet2Chars.length);
		pcSet2.set = pcSet2.charArrayToIntOne(pcSet2Chars);


		if (tnCheckBox.getSelectedObjects() != null ||
		    tnICheckBox.getSelectedObjects() != null ||
		    mnCheckBox.getSelectedObjects() != null ||
		    rkTnCheckBox.getSelectedObjects() != null ||
		    rkTnICheckBox.getSelectedObjects() != null ||
		    RTnCheckBox.getSelectedObjects() != null ||
		    RTnICheckBox.getSelectedObjects() != null ||
		    rkRTnCheckBox.getSelectedObjects() != null ||
		    rkRTnICheckBox.getSelectedObjects() != null) {

		    // Is PC-Set 2 ordered or unordered?
		    String ordered 
			= (String)oderUnorderJComboBox.getSelectedItem();

		    if(ordered.equals("Ordered PC-Set:")) {
			OrderedInclusion[] inclusion 
			    = new OrderedInclusion[numberOfSegments];
		
			for(int ii = 0; ii < numberOfSegments; ii++) {
			    inclusion[ii] 
				= new OrderedInclusion(rowSegmentsPcSets[ii],
						       pcSet2);
			}

			if (tnCheckBox.getSelectedObjects() != null) {
			    int noRelations = 0;
			    int someRelations = 0;
			    for(int ii = 0; ii < inclusion.length; ii++) {
				String nru = inclusion[ii].listFitTns();
				if (nru.equals("noRelationsUnderT"))
				    noRelations++;
				else {
				    results.append(nru);
				    someRelations++;
				}
			    }
			    results.append("\n");
			    if (noRelations != 0 && someRelations == 0)
				results.append(
				       "No relations under T.\n\n");
			}


			if (tnICheckBox.getSelectedObjects() != null) {
			    int noRelations = 0;
			    int someRelations = 0;
			    for(int ii = 0; ii < inclusion.length; ii++) {
				String nru = inclusion[ii].listFitTnIs();
				if (nru.equals("noRelationsUnderTI"))
				    noRelations++;
				else {
				    results.append(nru);
				    someRelations++;
				}
			    }
			    results.append("\n");
			    if (noRelations != 0 && someRelations == 0)
				results.append(
				       "No relations under TI.\n\n");
			}


			if (mnCheckBox.getSelectedObjects() != null) {
			    int noRelations = 0;
			    int someRelations = 0;
			    for(int ii = 0; ii < inclusion.length; ii++) {
				String nru = inclusion[ii].listFitMns();
				if (nru.equals("noRelationsUnderM"))
				    noRelations++;
				else {
				    results.append(nru);
				    someRelations++;
				}
			    }
			    results.append("\n");
			    if (noRelations != 0 && someRelations == 0)
				results.append(
				       "No relations under M.\n\n");
			}
		    

			if (rkTnCheckBox.getSelectedObjects() != null) {
			    int noRelations = 0;
			    int someRelations = 0;
			    for(int ii = 0; ii < inclusion.length; ii++) {
				String nru = inclusion[ii].listFitrTns();
				if (nru.equals("noRelationsUnderrT"))
				    noRelations++;
				else {
				    results.append(nru);
				    someRelations++;
				}
			    }
			    results.append("\n");
			    if (noRelations != 0 && someRelations == 0)
				results.append(
				       "No relations under rT.\n\n");
			}

			if (rkTnICheckBox.getSelectedObjects() != null) {
			    int noRelations = 0;
			    int someRelations = 0;
			    for(int ii = 0; ii < inclusion.length; ii++) {
				String nru = inclusion[ii].listFitrTnIs();
				if (nru.equals("noRelationsUnderrTI"))
				    noRelations++;
				else {
				    results.append(nru);
				    someRelations++;
				}
			    }
			    results.append("\n");
			    if (noRelations != 0 && someRelations == 0)
				results.append(
				       "No relations under rTI.\n\n");
			}


			if (RTnCheckBox.getSelectedObjects() != null) {
			    int noRelations = 0;
			    int someRelations = 0;
			    for(int ii = 0; ii < inclusion.length; ii++) {
				String nru = inclusion[ii].listFitRTs();
				if (nru.equals("noRelationsUnderRT"))
				    noRelations++;
				else {
				    results.append(nru);
				    someRelations++;
				}
			    }
			    results.append("\n");
			    if (noRelations != 0 && someRelations == 0)
				results.append(
				       "No relations under RT.\n\n");
			}


			if (RTnICheckBox.getSelectedObjects() != null) {
			    int noRelations = 0;
			    int someRelations = 0;
			    for(int ii = 0; ii < inclusion.length; ii++) {
				String nru = inclusion[ii].listFitRTnIs();
				if (nru.equals("noRelationsUnderRTI"))
				    noRelations++;
				else {
				    results.append(nru);
				    someRelations++;
				}
			    }
			    results.append("\n");
			    if (noRelations != 0 && someRelations == 0)
				results.append(
				       "No relations under RTI.\n\n");
			}


			if (rkRTnCheckBox.getSelectedObjects() != null) {
			    int noRelations = 0;
			    int someRelations = 0;
			    for(int ii = 0; ii < inclusion.length; ii++) {
				String nru = inclusion[ii].listFitrRTs();
				if (nru.equals("noRelationsUnderrRT"))
				    noRelations++;
				else {
				    results.append(nru);
				    someRelations++;
				}
			    }
			    results.append("\n");
			    if (noRelations != 0 && someRelations == 0)
				results.append(
				       "No relations under rRT.\n\n");
			}


			if (rkRTnICheckBox.getSelectedObjects() != null) {
			    int noRelations = 0;
			    int someRelations = 0;
			    for(int ii = 0; ii < inclusion.length; ii++) {
				String nru = inclusion[ii].listFitrRTnIs();
				if (nru.equals("noRelationsUnderrRTI"))
				    noRelations++;
				else {
				    results.append(nru);
				    someRelations++;
				}
			    }
			    results.append("\n");
			    if (noRelations != 0 && someRelations == 0)
				results.append(
				       "No relations under rRTI.\n\n");
			}

		    } else {
			if(rowSegmentsPcSets.length == 1)
			    JOptionPane.showMessageDialog(
				stravinskyPanel,
				"You need to segment the tone row\n by inserting a space(s)\n",
				"Warning",
				JOptionPane.WARNING_MESSAGE);
			else {
			    UnorderedInclusion[] inclusion 
				= new UnorderedInclusion[numberOfSegments];
		    
			    for(int ii = 0; ii < numberOfSegments; ii++)
				inclusion[ii] 
				    = new UnorderedInclusion(
						     rowSegmentsPcSets[ii],
						     pcSet2);

			    if (tnCheckBox.getSelectedObjects() != null)
				for(int i = 0; i < inclusion.length; i++)
				    results.append(inclusion[i].listFitTns());
			    
			    if (tnICheckBox.getSelectedObjects() != null)
				for(int i = 0; i < inclusion.length; i++)
				    results.append(inclusion[i].listFitTnIs());

			    if (mnCheckBox.getSelectedObjects() != null)
				for(int i = 0; i < inclusion.length; i++)
				    results.append(inclusion[i].listFitMns());
			}
			if (rkTnCheckBox.getSelectedObjects() != null ||
			    rkTnICheckBox.getSelectedObjects() != null ||
			    RTnCheckBox.getSelectedObjects() != null ||
			    RTnICheckBox.getSelectedObjects() != null ||
			    rkRTnCheckBox.getSelectedObjects() != null ||
			    rkRTnICheckBox.getSelectedObjects() != null)
			    JOptionPane.showMessageDialog(
				  stravinskyPanel,
				"Rotation and retrograde can not be "
				+ "applied to an unordered pc-set.\n",
				"Warning",
				JOptionPane.WARNING_MESSAGE);
		    }
		}


		// Chooses operations on verticals.
		if (pVerticalsCheckBox.getSelectedObjects() != null ||
		    rVerticalsCheckBox.getSelectedObjects() != null ||
		    iVerticalsCheckBox.getSelectedObjects() != null ||
		    riVerticalsCheckBox.getSelectedObjects() != null ||
		    irVerticalsCheckBox.getSelectedObjects() != null) {

		RotationalArray[] rarry 
		    = new RotationalArray[rowSegmentsPcSets.length];


		if (pVerticalsCheckBox.getSelectedObjects() != null) {
		    int arraySize = pVerticalsModel.getNumber().intValue();
		    for(int i = 0; i < rarry.length; i++) {
			if(rowSegmentsPcSets[i].set.length < arraySize ||
			   arraySize < pcSet2.set.length) {
			    JOptionPane.showMessageDialog(
				stravinskyPanel,
				"The number of the layers of the rotational \n"
				+ "array must be smaller than or equal to \n"
				+ "the size of the subsequences and greater \n"
				+ "than or equal to the size of PC-Set 2. \n",
				"Warning",
				JOptionPane.WARNING_MESSAGE);
			} else {
			    rarry[i] 
			    = new RotationalArray(rowSegmentsPcSets[i], arraySize);
			    results.append("\n");
			    results.append(rarry[i].printRotationalArrayP());
			    results.append(rarry[i].listFitVerticalsP(pcSet2.set));
			}
		    }
		}


		if (rVerticalsCheckBox.getSelectedObjects() != null) {
		    int arraySize = rVerticalsModel.getNumber().intValue();
		    for(int i = 0; i < rarry.length; i++) {
			if(rowSegmentsPcSets[i].set.length < arraySize ||
			   arraySize < pcSet2.set.length) {
			    JOptionPane.showMessageDialog(
				stravinskyPanel,
				"The number of the layers of the rotational \n"
				+ "array must be smaller than or equal to \n"
				+ "the size of the subsequences and greater \n"
				+ "than or equal to the size of PC-Set 2. \n",
				"Warning",
				JOptionPane.WARNING_MESSAGE);
			} else {
			    rarry[i] 
			    = new RotationalArray(rowSegmentsPcSets[i], arraySize);
			    results.append("\n");
			    results.append(rarry[i].printRotationalArrayR());
			    results.append(rarry[i].listFitVerticalsR(pcSet2.set));
			}
		    }
		}


		if (iVerticalsCheckBox.getSelectedObjects() != null) {
		    int arraySize = iVerticalsModel.getNumber().intValue();
		    for(int i = 0; i < rarry.length; i++) {
			if(rowSegmentsPcSets[i].set.length < arraySize ||
			   arraySize < pcSet2.set.length) {
			    JOptionPane.showMessageDialog(
				stravinskyPanel,
				"The number of the layers of I rotational \n"
				+ "array must be smaller than or equal to \n"
				+ "the size of the subsequences and greater \n"
				+ "than or equal to the size of PC-Set 2. \n",
				"Warning",
				JOptionPane.WARNING_MESSAGE);
			} else {
			    rarry[i] 
			    = new RotationalArray(rowSegmentsPcSets[i], arraySize);
			    results.append("\n");
			    results.append(rarry[i].printRotationalArrayI());
			    results.append(rarry[i].listFitVerticalsI(pcSet2.set));
			}
		    }
		}


		if (riVerticalsCheckBox.getSelectedObjects() != null) {
		    int arraySize = riVerticalsModel.getNumber().intValue();
		    for(int i = 0; i < rarry.length; i++) {
			if(rowSegmentsPcSets[i].set.length < arraySize ||
			   arraySize < pcSet2.set.length) {
			    JOptionPane.showMessageDialog(
				stravinskyPanel,
				"The number of the layers of the rotational \n"
				+ "array must be smaller than or equal to \n"
				+ "the size of the subsequences and greater \n"
				+ "than or equal to the size of PC-Set 2. \n",
				"Warning",
				JOptionPane.WARNING_MESSAGE);
			} else {
			    rarry[i] 
			    = new RotationalArray(rowSegmentsPcSets[i], arraySize);
			    results.append("\n");
			    results.append(rarry[i].printRotationalArrayRI());
			    results.append(rarry[i].listFitVerticalsRI(pcSet2.set));
			}
		    }
		}


		if (irVerticalsCheckBox.getSelectedObjects() != null) {
		    int arraySize = irVerticalsModel.getNumber().intValue();
		    for(int i = 0; i < rarry.length; i++) {
			if(rowSegmentsPcSets[i].set.length < arraySize ||
			   arraySize < pcSet2.set.length) {
			    JOptionPane.showMessageDialog(
				stravinskyPanel,
				"The number of the layers of the rotational \n"
				+ "array must be smaller than or equal to \n"
				+ "the size of the subsequences and greater \n"
				+ "than or equal to the size of PC-Set 2. \n",
				"Warning",
				JOptionPane.WARNING_MESSAGE);
			} else {
			    rarry[i] 
			    = new RotationalArray(rowSegmentsPcSets[i], arraySize);
			    results.append("\n");
			    results.append(rarry[i].printRotationalArrayIR());
			    results.append(rarry[i].listFitVerticalsIR(pcSet2.set));
			}
		    }
		}


	    }





	    // Chooses operations.
	    if (primeFormsCheckBox.getSelectedObjects() != null) {

		if (rowSegmentsPcSets.length < 2)
		    JOptionPane.showMessageDialog(
			stravinskyPanel,
			"Insert a space(s) in the twelve-tone row to make subsequences.\n",
			"Warning",
			JOptionPane.WARNING_MESSAGE);
		else {
		    TnTnIType[] subSequences 
			= new TnTnIType[rowSegmentsPcSets.length];
		    Set[] prime = new Set[rowSegmentsPcSets.length];
		    for(int i = 0; i < rowSegmentsPcSets.length; i++) {
			subSequences[i] 
			    = new TnTnIType(rowSegmentsPcSets[i].set);
			subSequences[i].base12String 
			    = rowSegmentsPcSets[i].base12String;
			subSequences[i].getTnType();
			subSequences[i].getTnIType();
			subSequences[i].getTnTnIType();
			prime[i] = subSequences[i].getPrimeByTnTnI();
		    }

		    results.append("\n");
		    results.append("Subsequences: ");
		    for(int i = 0; i < rowSegmentsPcSets.length; i++) {
			results.append("<");
			results.append(
	       subSequences[i].pcSetArrayToString(rowSegmentsPcSets[i].set));
			results.append("> ");
		    }

		    results.append("\n");
		    results.append("Tn/TnI type:      ");
		    for(int i = 0; i < rowSegmentsPcSets.length; i++) {
			results.append("[");
			results.append(subSequences[i].tntnitypeToString());
			results.append("]   ");
		    }

		    results.append("\n");
		    results.append("I-Vector:          ");
		    for(int i = 0; i < rowSegmentsPcSets.length; i++) {
			results.append("<");
			results.append(prime[i].ivectorToString());
			results.append("> ");
		    }

		    results.append("\n");
		    results.append("Forte Name:             ");
		    for(int i = 0; i < rowSegmentsPcSets.length; i++) {
			results.append(prime[i].name);
			results.append("           ");
		    }

		    results.append("\n");
		}
	    }







	    // Chooses operations.
	    if (matrixCheckBox.getSelectedObjects() != null) {

		    results.append("\n");
		    results.append("Matrix: \n\n");
		    Set st = new Set();
		    for(int i = 0; i < rowSegmentsPcSets.length; i++) {
			st.base12String = rowSegmentsPcSets[i].base12String;

			int[][] mtrx = st.matrix(rowSegmentsPcSets[i].set);

			results.append("        ");
			for(int h = 0; h < mtrx.length; h++) {
			    if (10 == (((mtrx[0][h] + 12) - mtrx[0][0]) % 12))
				results.append("IA  ");
			    else if (11
			     == (((mtrx[0][h] + 12) - mtrx[0][0]) % 12))
				results.append("IB  ");
			    else
				results.append("I"
			       + (((mtrx[0][h] + 12) - mtrx[0][0]) % 12)
					       + "  ");
			}
			results.append("\n");
			results.append("        -----------------------------");
			results.append("\n");
			
			for(int ii = 0; ii < mtrx.length; ii++) {
			    if (10 == (((mtrx[ii][0] + 12) - mtrx[0][0]) % 12))
				results.append("PA: ");
			    else if (11
			     == (((mtrx[ii][0] + 12) - mtrx[0][0]) % 12))
				results.append("PB: ");
			    else
				results.append("P"
			       + (((mtrx[ii][0] + 12) - mtrx[0][0]) % 12)
					       + ": ");
			    
			    for(int iii = 0; iii < mtrx.length; iii++) {
				results.append("   ");
				results.append(st.intToChar(mtrx[ii][iii]));
			    }
			    results.append("\n");
			}
			results.append("\n");
		    }
		}
	    }

	    editorPane.setText(results.toString());
	}

	if ("clear".equals(e.getActionCommand())) {
	    pcSetTextField.setText("");
	    editorPane.setText("");		
	}
    }
}
