// TwelveTones.java, used for
// JDubiel.java: computer program for pc-set analysis
// by Akira Takaoka, akira.takaoka@columbia.edu
// Copyright (C) 1994-2021 Akira Takaoka
// All rights reserved.





// package jdubielApplication;

import java.lang.reflect.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;



public class TwelveTones extends JFrame implements ActionListener {

    static final int xOffset = 70, yOffset = 70;
    static int openFrameCount = 0;

    JScrollPane toneRowScrollPane;
    JPanel twelveToneRowPanel;
    JPanel buttonPanel;

    JEditorPane editorPane;
    JScrollPane editorScrollPane;

    JTextField toneRowTextField;
    JTextField pcSetTextField;
    JComboBox<String> oderUnorderJComboBox;

    JCheckBox tnCheckBox;
    JCheckBox tnICheckBox;
    JCheckBox mnCheckBox;
    JCheckBox rkTnCheckBox;
    JCheckBox rkTnICheckBox;
    JCheckBox RTnCheckBox;
    JCheckBox RTnICheckBox;
    JCheckBox rkRTnCheckBox;
    JCheckBox rkRTnICheckBox;

    JCheckBox pVerticalsCheckBox;
    JCheckBox rVerticalsCheckBox;
    JCheckBox iVerticalsCheckBox;
    JCheckBox riVerticalsCheckBox;
    JCheckBox irVerticalsCheckBox;

    SpinnerNumberModel pVerticalsModel;
    SpinnerNumberModel rVerticalsModel;
    SpinnerNumberModel iVerticalsModel;
    SpinnerNumberModel riVerticalsModel;
    SpinnerNumberModel irVerticalsModel;

    JCheckBox primeFormsCheckBox;
    JCheckBox matrixCheckBox;

    StringBuffer results;







    public TwelveTones() {

	createWidgets();

	//Put everything together.
	JPanel devidedPane = new JPanel(new BorderLayout());
	devidedPane.add(twelveToneRowPanel, BorderLayout.NORTH);
	devidedPane.add(buttonPanel, BorderLayout.WEST);
	devidedPane.add(toneRowScrollPane, BorderLayout.CENTER);
	devidedPane.add(editorScrollPane, BorderLayout.SOUTH);

	getContentPane().add(devidedPane, BorderLayout.CENTER);

	setSize(450, 600);
	openFrameCount++;
	setTitle("Twelve-Tone Operations " + openFrameCount);
	setLocation(xOffset + 10 * openFrameCount, 
		    yOffset + 2 * openFrameCount);
    }
	


	// Create and add the widgets.
	private void createWidgets() {

                // add a function to generate cyclic sets.

		JLabel toneRowLabel = new JLabel("Twelve-Tone Row: ", 
							SwingConstants.RIGHT);
		toneRowLabel.setToolTipText("Insert a space(s) to segment "
					+ "the tone row. Ex. 0293 476A B158 ");
		toneRowTextField = new JTextField(12);
		toneRowTextField.setBackground(Color.white);
		toneRowTextField.setToolTipText(
					"Insert a space(s) to segment the "
					+ "tone row. Ex. 0293 476A B158 ");
		toneRowTextField.setDragEnabled(true);
		toneRowLabel.setLabelFor(toneRowTextField);

		//Lay out the panel1.
		twelveToneRowPanel = new JPanel(new GridLayout(1, 2));
		twelveToneRowPanel.add(toneRowLabel);
		twelveToneRowPanel.add(toneRowTextField);
		twelveToneRowPanel.setBorder(
			BorderFactory.createCompoundBorder(
                                BorderFactory.createTitledBorder(""),
                                BorderFactory.createEmptyBorder(5,5,5,5)));





		// Find button
		JButton findButton = new JButton("Find");
		findButton.setActionCommand("find");
		findButton.addActionListener(this);
		findButton.setBackground(Color.lightGray);
		findButton.setFont(new Font("SansSerif", Font.BOLD, 12));

		// Clear button
		JButton clearButton = new JButton("Clear");
		clearButton.setActionCommand("clear");
		clearButton.addActionListener(this);
		clearButton.setBackground(Color.lightGray);

		JPanel buttonPanel1 = new JPanel(new SpringLayout());
		buttonPanel1.add(findButton);
		buttonPanel1.add(clearButton);

		SpringUtilities.makeCompactGrid(buttonPanel1,
						2, 1,  //rows, cols
						1, 1,  //initX, initY
						1, 1); //xPad, yPad

		buttonPanel = new JPanel(new BorderLayout());
		buttonPanel.add(buttonPanel1, BorderLayout.NORTH);









		String[] oderUnorderStrings = {"Ordered PC-Set:",
					       "Unordered PC-Set:"};
		oderUnorderJComboBox = new JComboBox<>(oderUnorderStrings);
		oderUnorderJComboBox.setSelectedIndex(0);
		oderUnorderJComboBox.setAlignmentY(RIGHT_ALIGNMENT);
		oderUnorderJComboBox.setBackground(Color.white);
		pcSetTextField = new JTextField(12);
		pcSetTextField.setBackground(Color.white);
		pcSetTextField.setDragEnabled(true);

		JPanel pcSetPanel = new JPanel(new SpringLayout());
		pcSetPanel.add(oderUnorderJComboBox);
		pcSetPanel.add(pcSetTextField);
		SpringUtilities.makeCompactGrid(pcSetPanel,
						1, 2,  //rows, cols
						1, 1,  //initX, initY
						1, 1); //xPad, yPad



		// Check buttons on a panel.
		tnCheckBox = new JCheckBox(" T ");
		tnCheckBox.setToolTipText(
			"Pn: Transposition by n");
		tnCheckBox.setActionCommand("tn");
		tnCheckBox.addActionListener(this);

		tnICheckBox = new JCheckBox(" TI ");
		tnICheckBox.setToolTipText(
			"In: inversion at n");
		tnICheckBox.setActionCommand("tnI");
		tnICheckBox.addActionListener(this);

		mnCheckBox = new JCheckBox(" M ");
		mnCheckBox.setToolTipText(
			"Mn: multiplication by n");
		mnCheckBox.setActionCommand("mn");
		mnCheckBox.addActionListener(this);

		rkTnCheckBox = new JCheckBox(" rT "); 
		rkTnCheckBox.setToolTipText(
			"Rotated Pn: Pn rotated by k");
		rkTnCheckBox.setActionCommand("rkTn");
		rkTnCheckBox.addActionListener(this);

		rkTnICheckBox = new JCheckBox(" rTI "); 
		rkTnICheckBox.setToolTipText(
			"Rotated In: inversion rotated by k");
		rkTnICheckBox.setActionCommand("rkTnI");
		rkTnICheckBox.addActionListener(this);

		RTnCheckBox = new JCheckBox(" RT "); 
		RTnCheckBox.setToolTipText(
			"Rn: retrograde");
		RTnCheckBox.setActionCommand(" RT ");
		RTnCheckBox.addActionListener(this);

		// RIn (retrograde inversion)
		RTnICheckBox = new JCheckBox(" RTI "); 
		RTnICheckBox.setToolTipText(
			"RIn: retrograde inversion");
		RTnICheckBox.setActionCommand("RTnI");
		RTnICheckBox.addActionListener(this);

		rkRTnCheckBox = new JCheckBox(" rRT "); 
		rkRTnCheckBox.setToolTipText(
			"Rotated Rn: retrograde rotated by k");
		rkRTnCheckBox.setActionCommand("rkRTn");
		rkRTnCheckBox.addActionListener(this);

		rkRTnICheckBox = new JCheckBox(" rRTI "); 
		rkRTnICheckBox.setToolTipText(
			"Rotated RIn: retrograde inversion rotated by k");
		rkRTnICheckBox.setActionCommand("rkRTnI");
		rkRTnICheckBox.addActionListener(this);

		JPanel checkBoxesPanel1 = new JPanel(new GridLayout(3, 3));
		checkBoxesPanel1.add(tnCheckBox);
		checkBoxesPanel1.add(tnICheckBox);
		checkBoxesPanel1.add(mnCheckBox);
		checkBoxesPanel1.add(rkTnCheckBox);
		checkBoxesPanel1.add(rkTnICheckBox);
		checkBoxesPanel1.add(RTnCheckBox);
		checkBoxesPanel1.add(RTnICheckBox);
		checkBoxesPanel1.add(rkRTnCheckBox);
		checkBoxesPanel1.add(rkRTnICheckBox);

		checkBoxesPanel1.setBorder(
			BorderFactory.createCompoundBorder(
                                BorderFactory.createTitledBorder(
					"in the tone row under: "),
                                BorderFactory.createEmptyBorder(5,5,5,5)));



		pVerticalsCheckBox = new JCheckBox("P rotational array of size:              "); 
		pVerticalsCheckBox.setActionCommand("pVerticals");
		pVerticalsCheckBox.addActionListener(this);

		pVerticalsModel = new SpinnerNumberModel(
			6, //initial value
			3, //min
			12, //max
			1); //step
		JSpinner pVerticalsSpinner = new JSpinner(pVerticalsModel);
		pVerticalsSpinner.setEditor(
			new JSpinner.NumberEditor(pVerticalsSpinner));
		pVerticalsSpinner.setToolTipText("The number of the layers of the array.");
		Dimension d2 = new Dimension();
		d2.setSize(0.1, 0.1);
		pVerticalsSpinner.setSize(10,10);



		rVerticalsCheckBox = new JCheckBox("R rotational array of size:              "); 
		rVerticalsCheckBox.setActionCommand("rVerticals");
		rVerticalsCheckBox.addActionListener(this);

		rVerticalsModel = new SpinnerNumberModel(
			6, //initial value
			3, //min
			12, //max
			1); //step
		JSpinner rVerticalsSpinner = new JSpinner(rVerticalsModel);
		rVerticalsSpinner.setEditor(
			new JSpinner.NumberEditor(rVerticalsSpinner));
		rVerticalsSpinner.setToolTipText("The number of the layers of the array.");
		d2.setSize(0.1, 0.1);
		rVerticalsSpinner.setSize(10,10);



		iVerticalsCheckBox = new JCheckBox("I rotational array of size:               "); 
		iVerticalsCheckBox.setActionCommand("iVerticals");
		iVerticalsCheckBox.addActionListener(this);

		iVerticalsModel = new SpinnerNumberModel(
			6, //initial value
			3, //min
			12, //max
			1); //step
		JSpinner iVerticalsSpinner = new JSpinner(iVerticalsModel);
		iVerticalsSpinner.setEditor(
			new JSpinner.NumberEditor(iVerticalsSpinner));
		iVerticalsSpinner.setToolTipText("The number of the layers of the array.");
		d2.setSize(0.1, 0.1);
		iVerticalsSpinner.setSize(10,10);



		riVerticalsCheckBox = new JCheckBox("RI rotational array of size:             "); 
		riVerticalsCheckBox.setActionCommand("riVerticals");
		riVerticalsCheckBox.addActionListener(this);

		riVerticalsModel = new SpinnerNumberModel(
			6, //initial value
			3, //min
			12, //max
			1); //step
		JSpinner riVerticalsSpinner = new JSpinner(riVerticalsModel);
		riVerticalsSpinner.setEditor(
			new JSpinner.NumberEditor(riVerticalsSpinner));
		riVerticalsSpinner.setToolTipText("The number of the layers of the array.");
		d2.setSize(0.1, 0.1);
		riVerticalsSpinner.setSize(10,10);



		irVerticalsCheckBox = new JCheckBox("IR rotational array of size:             "); 
		irVerticalsCheckBox.setActionCommand("irVerticals");
		irVerticalsCheckBox.addActionListener(this);

		irVerticalsModel = new SpinnerNumberModel(
			6, //initial value
			3, //min
			12, //max
			1); //step
		JSpinner irVerticalsSpinner = new JSpinner(irVerticalsModel);
		irVerticalsSpinner.setEditor(
			new JSpinner.NumberEditor(irVerticalsSpinner));
		irVerticalsSpinner.setToolTipText("The number of the layers of the array.");
		d2.setSize(0.1, 0.1);
		irVerticalsSpinner.setSize(10,10);



		JPanel checkBoxesPanelP = new JPanel();
		checkBoxesPanelP.setLayout(new BoxLayout(checkBoxesPanelP, BoxLayout.X_AXIS));
		checkBoxesPanelP.add(pVerticalsCheckBox);
		checkBoxesPanelP.add(pVerticalsSpinner);

		JPanel checkBoxesPanelR = new JPanel();
		checkBoxesPanelR.setLayout(new BoxLayout(checkBoxesPanelR, BoxLayout.X_AXIS));
		checkBoxesPanelR.add(rVerticalsCheckBox);
		checkBoxesPanelR.add(rVerticalsSpinner);

		JPanel checkBoxesPanelI = new JPanel();
		checkBoxesPanelI.setLayout(new BoxLayout(checkBoxesPanelI, BoxLayout.X_AXIS));
		checkBoxesPanelI.add(iVerticalsCheckBox);
		checkBoxesPanelI.add(iVerticalsSpinner);

		JPanel checkBoxesPanelRI = new JPanel();
		checkBoxesPanelRI.setLayout(new BoxLayout(checkBoxesPanelRI, BoxLayout.X_AXIS));
		checkBoxesPanelRI.add(riVerticalsCheckBox);
		checkBoxesPanelRI.add(riVerticalsSpinner);

		JPanel checkBoxesPanelIR = new JPanel();
		checkBoxesPanelIR.setLayout(new BoxLayout(checkBoxesPanelIR, BoxLayout.X_AXIS));
		checkBoxesPanelIR.add(irVerticalsCheckBox);
		checkBoxesPanelIR.add(irVerticalsSpinner);


		JPanel checkBoxesPanel2 = new JPanel(new GridLayout(5, 1));
		checkBoxesPanel2.add(checkBoxesPanelP);
		checkBoxesPanel2.add(checkBoxesPanelR);
		checkBoxesPanel2.add(checkBoxesPanelI);
		checkBoxesPanel2.add(checkBoxesPanelRI);
		checkBoxesPanel2.add(checkBoxesPanelIR);
		checkBoxesPanel2.setBorder(
			BorderFactory.createCompoundBorder(
                                BorderFactory.createTitledBorder(
					"in the verticals of"),
                                BorderFactory.createEmptyBorder(5,5,5,5)));






		primeFormsCheckBox = new JCheckBox("Tn/TnI types of subsequences"); 
		primeFormsCheckBox.setToolTipText(
			"Shows names of Tn/TnI types of subsequences of the tone row.");
		primeFormsCheckBox.setActionCommand("primeForms");
		primeFormsCheckBox.addActionListener(this);





		matrixCheckBox = new JCheckBox("Matrix of the tone row"); 
		matrixCheckBox.setToolTipText(
			"Shows the forty-eight forms of the tone row.");
		matrixCheckBox.setActionCommand("matrix");
		matrixCheckBox.addActionListener(this);





		//Lay out the panel.
		JPanel checkBoxesPanel = new JPanel(new SpringLayout());
		checkBoxesPanel.add(pcSetPanel);
		checkBoxesPanel.add(checkBoxesPanel1);
		checkBoxesPanel.add(checkBoxesPanel2);
		checkBoxesPanel.add(primeFormsCheckBox);
		checkBoxesPanel.add(matrixCheckBox);
		SpringUtilities.makeCompactGrid(checkBoxesPanel,
						5, 1,  //rows, cols
						1, 1,  //initX, initY
						1, 1); //xPad, yPad

		checkBoxesPanel.setBorder(
			BorderFactory.createCompoundBorder(
                                BorderFactory.createTitledBorder(
					""),
                                BorderFactory.createEmptyBorder(5,5,5,5)));

		toneRowScrollPane = new JScrollPane(checkBoxesPanel);
		toneRowScrollPane.setVerticalScrollBarPolicy(
                        JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		toneRowScrollPane.setHorizontalScrollBarPolicy(
			JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		toneRowScrollPane.setPreferredSize(new Dimension(340, 200));
		toneRowScrollPane.setMinimumSize(new Dimension(340, 200));






		// "Found Relations" area
		editorPane = new JEditorPane();
		editorScrollPane = new JScrollPane(editorPane);
		editorScrollPane.setVerticalScrollBarPolicy(
                        JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		editorScrollPane.setHorizontalScrollBarPolicy(
			JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		editorScrollPane.setPreferredSize(new Dimension(200, 300));
		editorScrollPane.setMinimumSize(new Dimension(200, 300));
		editorScrollPane.setBorder(
			BorderFactory.createCompoundBorder(
				BorderFactory.createTitledBorder(
					"Found: "),
				BorderFactory.createEmptyBorder(5,5,5,5)));
		editorScrollPane.getBorder();
	}







    public void actionPerformed(ActionEvent e) {

	if("find".equals(e.getActionCommand())) {
	    editorPane.setText("");		

	    results = new StringBuffer();
	    PcSet pq = new PcSet(); // just for the methods used below

	    // Get a twelve-tone row and a typed-in pc set and
	    // remove space(s) at the beginning and the end of the row.
	    String typedToneRowString 
		= pq.removeFistLastSpaces(toneRowTextField.getText());

	    // Remove consecutive spaces.
	    String noConsecutiveSpacesString 
		= pq.removeConsecutiveSpaces(typedToneRowString);

	    // Eliminate duplicates
	    String duplicatesFreeToneRowString 
		= pq.removeDuplicatesInString(noConsecutiveSpacesString);

	    // Check if there are non-numerical characters
	    String illegalChars 
		= pq.checkNonNumerals(duplicatesFreeToneRowString);



	    // Puts each subsequence of the segmented tone row in an array.
	    char[] toneRowChars 
		= new char[duplicatesFreeToneRowString.length()];
	    duplicatesFreeToneRowString.getChars(
				     0, 
				     duplicatesFreeToneRowString.length(), 
				     toneRowChars, 
				     0);

	    char[][] tmpSegmentsCharArray = new char[24][24];
	    int[] sizeOfEachSegment = new int[24];
	    int numberOfSegments = 0;

	    for(int i = 0; i < toneRowChars.length; i++) {
		while(toneRowChars[i] == ' ' && i < toneRowChars.length)
		    i++;

		sizeOfEachSegment[numberOfSegments] = 0;
		while(toneRowChars[i] != ' ') {
   tmpSegmentsCharArray[numberOfSegments][sizeOfEachSegment[numberOfSegments]] 
       = toneRowChars[i];
                    sizeOfEachSegment[numberOfSegments]++;
		    i++;

		    if(i > toneRowChars.length - 1)
			break;
		}
		numberOfSegments++;
	    }

	    char[][] rowSegmentsCharArray = new char[numberOfSegments][];
	    for (int i = 0; i < rowSegmentsCharArray.length; i++) {
		rowSegmentsCharArray[i] = new char[sizeOfEachSegment[i]];
		System.arraycopy(tmpSegmentsCharArray[i], 
				 0, 
				 rowSegmentsCharArray[i], 
				 0, 
				 sizeOfEachSegment[i]);
	    }
		

	
	    // Get typed-in pc-set.
	    String readPcSetString = pcSetTextField.getText();

	    // Eliminate spaces
	    StringBuffer withoutSpaceStringBuffer = new StringBuffer();
	    for (int i = 0; i < readPcSetString.length(); i++)
		if (readPcSetString.charAt(i) != ' ')
		    withoutSpaceStringBuffer.append(readPcSetString.charAt(i));

	    // Eliminate duplicates
	    String pcSet2String = pq.removeDuplicatesInString(
				     withoutSpaceStringBuffer.toString());

	    // Check if there are non-numerical characters
	    String pcSetIllegalChars
		= pq.checkNonNumerals(pcSet2String);




	    // Check errors.
	    boolean goAhead = true;

	    // Check if the size of each segmented subsequence is at least 3.
	    for (int iiii = 0; iiii < numberOfSegments; iiii++) {
		if (sizeOfEachSegment[iiii] < 3) {
		    JOptionPane.showMessageDialog(
				  twelveToneRowPanel,
			"The size of each subsequence must be at least 3.",
				"Warning",
				JOptionPane.WARNING_MESSAGE);
		goAhead = false;
		}
	    }

	    // At least one operation is chosen?
	    if (tnCheckBox.getSelectedObjects() == null &&
		tnICheckBox.getSelectedObjects() == null &&
		mnCheckBox.getSelectedObjects() == null &&
		rkTnCheckBox.getSelectedObjects() == null &&
		rkTnICheckBox.getSelectedObjects() == null &&
		RTnCheckBox.getSelectedObjects() == null &&
		RTnICheckBox.getSelectedObjects() == null &&
		rkRTnCheckBox.getSelectedObjects() == null &&
		rkRTnICheckBox.getSelectedObjects() == null &&
		pVerticalsCheckBox.getSelectedObjects() == null &&
		rVerticalsCheckBox.getSelectedObjects() == null &&
		iVerticalsCheckBox.getSelectedObjects() == null &&
		riVerticalsCheckBox.getSelectedObjects() == null &&
		irVerticalsCheckBox.getSelectedObjects() == null &&
		primeFormsCheckBox.getSelectedObjects() == null &&
		matrixCheckBox.getSelectedObjects() == null) {
		JOptionPane.showMessageDialog(
				  twelveToneRowPanel,
				  "You need to choose at least "
				  + "one of the operations.\n",
				  "Warning",
				  JOptionPane.WARNING_MESSAGE);
		goAhead = false;
	    } else if (pcSet2String.length() < 2
		       && (tnCheckBox.getSelectedObjects() != null
			   || tnICheckBox.getSelectedObjects() != null
			   || mnCheckBox.getSelectedObjects() != null
			   || rkTnCheckBox.getSelectedObjects() != null
			   || rkTnICheckBox.getSelectedObjects() != null
			   || RTnCheckBox.getSelectedObjects() != null
			   || RTnICheckBox.getSelectedObjects() != null
			   || rkRTnCheckBox.getSelectedObjects() != null
			   || rkRTnICheckBox.getSelectedObjects() != null
			   || pVerticalsCheckBox.getSelectedObjects() != null
			   || rVerticalsCheckBox.getSelectedObjects() != null
			   || iVerticalsCheckBox.getSelectedObjects() != null
			   || riVerticalsCheckBox.getSelectedObjects() != null
			   || irVerticalsCheckBox.getSelectedObjects() 
			   != null)) {
		JOptionPane.showMessageDialog(
			twelveToneRowPanel,
			"The size of the pc-set must be at least 2.",
			"Warning",
			JOptionPane.WARNING_MESSAGE);
		goAhead = false;
	    } else if (illegalChars.length() > 1) {
		JOptionPane.showMessageDialog(
		      twelveToneRowPanel, 
	      "Non-numerical characters are found in Twelve-Tone Row:"
		      + illegalChars.toString(),
		      "Warning",
		      JOptionPane.WARNING_MESSAGE);
		goAhead = false;
	    } else if (pq.numberOfNonSpaces(duplicatesFreeToneRowString) 
		      != pq.numberOfNonSpaces(noConsecutiveSpacesString)) {
		    JOptionPane.showMessageDialog(
				twelveToneRowPanel, 
				"There are duplicates in Twelve-Tone Row.",
				"Warning",
				JOptionPane.WARNING_MESSAGE);
		goAhead = false;
	    } else if (pq.numberOfNonSpaces(duplicatesFreeToneRowString) 
		       != 12) {
		JOptionPane.showMessageDialog(
			twelveToneRowPanel, 
			"The size of the tone row is not 12.",
			"Warning",
			JOptionPane.WARNING_MESSAGE);
	    } else if (pcSetIllegalChars.length() > 1) {
		JOptionPane.showMessageDialog(
		      twelveToneRowPanel,
		      "Non-numerical characters are found:"
		      + pcSetIllegalChars.toString(),
		      "Warning",
		      JOptionPane.WARNING_MESSAGE);
		goAhead = false;
	    } else if(pcSet2String.length() 
		      != withoutSpaceStringBuffer.length()) {
		JOptionPane.showMessageDialog(
				twelveToneRowPanel,
				"There are duplicates in the pc-set.",
				"Warning",
				JOptionPane.WARNING_MESSAGE);
		goAhead = false;
	    } else {
		if (goAhead == false)
		    goAhead = false;
		else
		    goAhead = true;
	    }



	    // If no error is found, the following will be carried out.
	    if (goAhead == true) {

		PcSet [] rowSegmentsPcSets = new PcSet[numberOfSegments];

		for (int i = 0; i < rowSegmentsPcSets.length; i++) {
		    rowSegmentsPcSets[i] = new PcSet(sizeOfEachSegment[i]);
		    rowSegmentsPcSets[i].set 
	    = rowSegmentsPcSets[i].charArrayToIntOne(rowSegmentsCharArray[i]);
		}



		// Gets a char-array of the PC-Set 2 String.
		char[] pcSet2Chars = new char[pcSet2String.length()];
		pcSet2String.getChars(0, 
				      pcSet2String.length(), 
				      pcSet2Chars, 
				      0);
		PcSet pcSet2 = new PcSet(pcSet2Chars.length);
		pcSet2.set = pcSet2.charArrayToIntOne(pcSet2Chars);

		
		if (tnCheckBox.getSelectedObjects() != null ||
		    tnICheckBox.getSelectedObjects() != null ||
		    mnCheckBox.getSelectedObjects() != null ||
		    rkTnCheckBox.getSelectedObjects() != null ||
		    rkTnICheckBox.getSelectedObjects() != null ||
		    RTnCheckBox.getSelectedObjects() != null ||
		    RTnICheckBox.getSelectedObjects() != null ||
		    rkRTnCheckBox.getSelectedObjects() != null ||
		    rkRTnICheckBox.getSelectedObjects() != null) {

		    // Is PC-Set 2 ordered or unordered?
		    // Check if PC-Set is Ordered or Unordered
		    String ordered 
			= (String)oderUnorderJComboBox.getSelectedItem();

		    if(ordered.equals("Ordered PC-Set:")) {
			OrderedInclusion[] inclusion 
			    = new OrderedInclusion[numberOfSegments];
		
			for(int ii = 0; ii < numberOfSegments; ii++) {
			    inclusion[ii] 
				= new OrderedInclusion(rowSegmentsPcSets[ii],
						       pcSet2);
			}

			if (tnCheckBox.getSelectedObjects() != null) {
			    int noRelations = 0;
			    int someRelations = 0;
			    for(int ii = 0; ii < inclusion.length; ii++) {
				String nru = inclusion[ii].listFitTns();
				if (nru.equals("noRelationsUnderT"))
				    noRelations++;
				else {
				    results.append(nru);
				    someRelations++;
				}
			    }
			    results.append("\n");
			    if (noRelations != 0 && someRelations == 0)
				results.append(
				       "No relations under T.\n\n");
			}


			if (tnICheckBox.getSelectedObjects() != null) {
			    int noRelations = 0;
			    int someRelations = 0;
			    for(int ii = 0; ii < inclusion.length; ii++) {
				String nru = inclusion[ii].listFitTnIs();
				if (nru.equals("noRelationsUnderTI"))
				    noRelations++;
				else {
				    results.append(nru);
				    someRelations++;
				}
			    }
			    results.append("\n");
			    if (noRelations != 0 && someRelations == 0)
				results.append(
				       "No relations under TI.\n\n");
			}


			if (mnCheckBox.getSelectedObjects() != null) {
			    int noRelations = 0;
			    int someRelations = 0;
			    for(int ii = 0; ii < inclusion.length; ii++) {
				String nru = inclusion[ii].listFitMns();
				if (nru.equals("noRelationsUnderM"))
				    noRelations++;
				else {
				    results.append(nru);
				    someRelations++;
				}
			    }
			    results.append("\n");
			    if (noRelations != 0 && someRelations == 0)
				results.append(
				       "No relations under M.\n\n");
			}
		    

			if (rkTnCheckBox.getSelectedObjects() != null) {
			    int noRelations = 0;
			    int someRelations = 0;
			    for(int ii = 0; ii < inclusion.length; ii++) {
				String nru = inclusion[ii].listFitrTns();
				if (nru.equals("noRelationsUnderrT"))
				    noRelations++;
				else {
				    results.append(nru);
				    someRelations++;
				}
			    }
			    results.append("\n");
			    if (noRelations != 0 && someRelations == 0)
				results.append(
				       "No relations under rT.\n\n");
			}

			if (rkTnICheckBox.getSelectedObjects() != null) {
			    int noRelations = 0;
			    int someRelations = 0;
			    for(int ii = 0; ii < inclusion.length; ii++) {
				String nru = inclusion[ii].listFitrTnIs();
				if (nru.equals("noRelationsUnderrTI"))
				    noRelations++;
				else {
				    results.append(nru);
				    someRelations++;
				}
			    }
			    results.append("\n");
			    if (noRelations != 0 && someRelations == 0)
				results.append(
				       "No relations under rTI.\n\n");
			}


			if (RTnCheckBox.getSelectedObjects() != null) {
			    int noRelations = 0;
			    int someRelations = 0;
			    for(int ii = 0; ii < inclusion.length; ii++) {
				String nru = inclusion[ii].listFitRTs();
				if (nru.equals("noRelationsUnderRT"))
				    noRelations++;
				else {
				    results.append(nru);
				    someRelations++;
				}
			    }
			    results.append("\n");
			    if (noRelations != 0 && someRelations == 0)
				results.append(
				       "No relations under RT.\n\n");
			}


			if (RTnICheckBox.getSelectedObjects() != null) {
			    int noRelations = 0;
			    int someRelations = 0;
			    for(int ii = 0; ii < inclusion.length; ii++) {
				String nru = inclusion[ii].listFitRTnIs();
				if (nru.equals("noRelationsUnderRTI"))
				    noRelations++;
				else {
				    results.append(nru);
				    someRelations++;
				}
			    }
			    results.append("\n");
			    if (noRelations != 0 && someRelations == 0)
				results.append(
				       "No relations under RTI.\n\n");
			}


			if (rkRTnCheckBox.getSelectedObjects() != null) {
			    int noRelations = 0;
			    int someRelations = 0;
			    for(int ii = 0; ii < inclusion.length; ii++) {
				String nru = inclusion[ii].listFitrRTs();
				if (nru.equals("noRelationsUnderrRT"))
				    noRelations++;
				else {
				    results.append(nru);
				    someRelations++;
				}
			    }
			    results.append("\n");
			    if (noRelations != 0 && someRelations == 0)
				results.append(
				       "No relations under rRT.\n\n");
			}


			if (rkRTnICheckBox.getSelectedObjects() != null) {
			    int noRelations = 0;
			    int someRelations = 0;
			    for(int ii = 0; ii < inclusion.length; ii++) {
				String nru = inclusion[ii].listFitrRTnIs();
				if (nru.equals("noRelationsUnderrRTI"))
				    noRelations++;
				else {
				    results.append(nru);
				    someRelations++;
				}
			    }
			    results.append("\n");
			    if (noRelations != 0 && someRelations == 0)
				results.append(
				       "No relations under rRTI.\n\n");
			}

		    } else {
			if(rowSegmentsPcSets.length == 1)
			    JOptionPane.showMessageDialog(
				  twelveToneRowPanel,
		"You need to segment the tone row\n by inserting a space(s) into it.\n",
				  "Warning",
				  JOptionPane.WARNING_MESSAGE);
			else {
			    UnorderedInclusion[] inclusion 
				= new UnorderedInclusion[numberOfSegments];
		    
			    for(int ii = 0; ii < numberOfSegments; ii++)
				inclusion[ii] 
				    = new UnorderedInclusion(
						     rowSegmentsPcSets[ii],
						     pcSet2);

			    if (tnCheckBox.getSelectedObjects() != null)
				for(int i = 0; i < inclusion.length; i++)
				    results.append(inclusion[i].listFitTns());
			    
			    if (tnICheckBox.getSelectedObjects() != null)
				for(int i = 0; i < inclusion.length; i++)
				    results.append(inclusion[i].listFitTnIs());

			    if (mnCheckBox.getSelectedObjects() != null)
				for(int i = 0; i < inclusion.length; i++)
				    results.append(inclusion[i].listFitMns());
			}
			if (rkTnCheckBox.getSelectedObjects() != null ||
			    rkTnICheckBox.getSelectedObjects() != null ||
			    RTnCheckBox.getSelectedObjects() != null ||
			    RTnICheckBox.getSelectedObjects() != null ||
			    rkRTnCheckBox.getSelectedObjects() != null ||
			    rkRTnICheckBox.getSelectedObjects() != null)
			    JOptionPane.showMessageDialog(
				  twelveToneRowPanel,
				  "Rotation and retrograde can not be "
				  + "applied to an unordered pc-set.\n",
				  "Warning",
				  JOptionPane.WARNING_MESSAGE);
		    }
		
		}






		// Chooses operations on verticals.
		if (pVerticalsCheckBox.getSelectedObjects() != null ||
		    rVerticalsCheckBox.getSelectedObjects() != null ||
		    iVerticalsCheckBox.getSelectedObjects() != null ||
		    riVerticalsCheckBox.getSelectedObjects() != null ||
		    irVerticalsCheckBox.getSelectedObjects() != null) {

		    RotationalArray[] rarry 
			= new RotationalArray[numberOfSegments];

		    if (pVerticalsCheckBox.getSelectedObjects() != null) {
			int arraySize = pVerticalsModel.getNumber().intValue();
			for(int i = 0; i < rarry.length; i++) {
			    if(rowSegmentsPcSets[i].set.length < arraySize ||
			       arraySize < pcSet2.set.length) {
				JOptionPane.showMessageDialog(
				twelveToneRowPanel,
			"The number of the layers of the rotational array \n"
				+ "must be smaller than or equal to the \n"
				+ "size of the subsequences and greater \n"
				+ "than or equal to the size of the PC-Set. \n",
				"Warning",
				JOptionPane.WARNING_MESSAGE);
			    } else {
				rarry[i] 
				    = new RotationalArray(
						  rowSegmentsPcSets[i], 
						  arraySize);
				results.append("\n");
				results.append(
				       rarry[i].printRotationalArrayP());
				results.append(
				       rarry[i].listFitVerticalsP(pcSet2.set));
			    }
			}
		    }


		    if (rVerticalsCheckBox.getSelectedObjects() != null) {
			int arraySize = rVerticalsModel.getNumber().intValue();
			for(int i = 0; i < rarry.length; i++) {
			    if(rowSegmentsPcSets[i].set.length < arraySize ||
			       arraySize < pcSet2.set.length) {
				JOptionPane.showMessageDialog(
				twelveToneRowPanel,
			"The number of the layers of the rotational array \n"
				+ "must be smaller than or equal to the \n"
				+ "size of the subsequences and greater \n"
				+ "than or equal to the size of the PC-Set. \n",
				"Warning",
				JOptionPane.WARNING_MESSAGE);
			    } else {
				rarry[i] 
				    = new RotationalArray(
						  rowSegmentsPcSets[i], 
						  arraySize);
				results.append("\n");
				results.append(
				       rarry[i].printRotationalArrayR());
				results.append(
				       rarry[i].listFitVerticalsR(pcSet2.set));
			    }
			}
		    }


		    if (iVerticalsCheckBox.getSelectedObjects() != null) {
			int arraySize = iVerticalsModel.getNumber().intValue();
			for(int i = 0; i < rarry.length; i++) {
			    if(rowSegmentsPcSets[i].set.length < arraySize ||
			       arraySize < pcSet2.set.length) {
				JOptionPane.showMessageDialog(
				twelveToneRowPanel,
			"The number of the layers of the rotational array \n"
				+ "must be smaller than or equal to the \n"
				+ "size of the subsequences and greater \n"
				+ "than or equal to the size of the PC-Set. \n",
				"Warning",
				JOptionPane.WARNING_MESSAGE);
			    } else {
				rarry[i] 
				    = new RotationalArray(
						  rowSegmentsPcSets[i], 
						  arraySize);
				results.append("\n");
				results.append(
				       rarry[i].printRotationalArrayI());
				results.append(
				       rarry[i].listFitVerticalsI(pcSet2.set));
			    }
			}
		    }


		    if (riVerticalsCheckBox.getSelectedObjects() != null) {
			int arraySize 
			    = riVerticalsModel.getNumber().intValue();
			for(int i = 0; i < rarry.length; i++) {
			    if(rowSegmentsPcSets[i].set.length < arraySize ||
			       arraySize < pcSet2.set.length) {
				JOptionPane.showMessageDialog(
				twelveToneRowPanel,
			"The number of the layers of the rotational array\n"
				+ "must be smaller than or equal to the \n"
				+ "size of the subsequences and greater \n"
				+ "than or equal to the size of the PC-Set. \n",
				"Warning",
				JOptionPane.WARNING_MESSAGE);
			    } else {
				rarry[i] 
				    = new RotationalArray(
						  rowSegmentsPcSets[i], 
						  arraySize);
				results.append("\n");
				results.append(
				       rarry[i].printRotationalArrayRI());
				results.append(
			              rarry[i].listFitVerticalsRI(pcSet2.set));
			    }
			}
		    }


		    if (irVerticalsCheckBox.getSelectedObjects() != null) {
			int arraySize 
			    = irVerticalsModel.getNumber().intValue();
			for(int i = 0; i < rarry.length; i++) {
			    if(rowSegmentsPcSets[i].set.length < arraySize ||
			       arraySize < pcSet2.set.length) {
				JOptionPane.showMessageDialog(
				twelveToneRowPanel,
			"The number of the layers of the rotational array\n"
				+ "must be smaller than or equal to the \n"
				+ "size of the subsequences and greater \n"
				+ "than or equal to the size of the PC-Set. \n",
				"Warning",
				JOptionPane.WARNING_MESSAGE);
			    } else {
				rarry[i] 
				    = new RotationalArray(
						  rowSegmentsPcSets[i], 
						  arraySize);
				results.append("\n");
				results.append(
				       rarry[i].printRotationalArrayIR());
				results.append(
				      rarry[i].listFitVerticalsIR(pcSet2.set));
			    }
			}
		    }

		}




		// Chooses operations.
		if (primeFormsCheckBox.getSelectedObjects() != null) {

		    if (rowSegmentsPcSets.length < 2)
			JOptionPane.showMessageDialog(
			      twelveToneRowPanel,
	 "Insert a space(s) into the twelve-tone row to make subsequences.\n",
				      "Warning",
				      JOptionPane.WARNING_MESSAGE);
		    else {
			TnTnIType[] subSequences 
			    = new TnTnIType[rowSegmentsPcSets.length];
			Set[] prime = new Set[rowSegmentsPcSets.length];
			for(int i = 0; i < rowSegmentsPcSets.length; i++) {
			    subSequences[i] 
				= new TnTnIType(rowSegmentsPcSets[i].set);
			    subSequences[i].base12String 
				= rowSegmentsPcSets[i].base12String;
			    subSequences[i].getTnType();
			    subSequences[i].getTnIType();
			    subSequences[i].getTnTnIType();
			    prime[i] = subSequences[i].getPrimeByTnTnI();
			}

			results.append("\n");
			results.append("Subsequence: ");
			for(int i = 0; i < rowSegmentsPcSets.length; i++) {
			    results.append("<");
			    results.append(
		 subSequences[i].pcSetArrayToString(rowSegmentsPcSets[i].set));
			    results.append("> ");
			}

			results.append("\n");
			results.append("Tn/TnI type:      ");
			for(int i = 0; i < rowSegmentsPcSets.length; i++) {
			    results.append("[");
			    results.append(
				   subSequences[i].tntnitypeToString());
			    results.append("]   ");
			}

			results.append("\n");
			results.append("I-Vector:          ");
			for(int i = 0; i < rowSegmentsPcSets.length; i++) {
			    results.append("<");
			    results.append(prime[i].ivectorToString());
			    results.append("> ");
			}

			results.append("\n");
			results.append("Forte Name:             ");
			for(int i = 0; i < rowSegmentsPcSets.length; i++) {
			    results.append(prime[i].name);
			    results.append("           ");
			}

			results.append("\n");
		    }
		}
		






		// Chooses operations.
		if (matrixCheckBox.getSelectedObjects() != null) {

		    results.append("\n");
		    results.append("Matrix: \n");
		    Set st = new Set();
		    for(int i = 0; i < rowSegmentsPcSets.length; i++) {
			st.base12String = rowSegmentsPcSets[i].base12String;
			int[][] mtrx = st.matrix(rowSegmentsPcSets[i].set);
			for(int ii = 0; ii < mtrx.length; ii++) {
			    for(int iii = 0; iii < mtrx.length; iii++) {
				results.append(" ");
				results.append(st.intToChar(mtrx[ii][iii]));
			    }
			    results.append("\n");
			}
			results.append("\n");
		    }
		}
	    }

	    editorPane.setText(results.toString());
	}


	if ("clear".equals(e.getActionCommand())) {
	    toneRowTextField.setText("");
	    pcSetTextField.setText("");
	    editorPane.setText("");		
	}
    }
}
