// UnorderedInclusion.java, used for
// JDubiel.java: Computer program for pc-set analysis
// by Akira Takaoka, akira.takaoka@columbia.edu
// Copyright (C) 1994-2017 Akira Takaoka
// All rights reserved.







// package jdubielApplication;


public class UnorderedInclusion extends PcSet {

    // include a method to check out a cyclic subset.

    PcSet pcSet1, pcSet2;


    // Constructor 1
    public UnorderedInclusion(int[] pc1, int[] pc2) {
	if(pc1.length >= pc2.length) {
	    pcSet1 = new PcSet(pc1.length);
	    pcSet2 = new PcSet(pc2.length);
	    for (int i = 0; i < pcSet1.set.length; i++)
		pcSet1.set[i] = pc1[i];
	    for (int i = 0; i < pcSet2.set.length; i++)
		pcSet2.set[i] = pc2[i];
	} else {
	    pcSet1 = new PcSet(pc2.length);
	    pcSet2 = new PcSet(pc1.length);
	    for (int i = 0; i < pcSet1.set.length; i++)
		pcSet1.set[i] = pc2[i];
	    for (int i = 0; i < pcSet2.set.length; i++)
		pcSet2.set[i] = pc1[i];

	    // So the size of pcSet1.set is always greater than 
	    // or equal to that of pcSet2.set.
	}
    }
    // Constructor 2
    public UnorderedInclusion(Set s1, Set s2) {
	if(s1.set.length >= s2.set.length) {
	    pcSet1 = new PcSet(s1.set.length);
	    pcSet2 = new PcSet(s2.set.length);
	    for (int i = 0; i < pcSet1.set.length; i++)
		pcSet1.set[i] = s1.set[i];
	    for (int i = 0; i < pcSet2.set.length; i++)
		pcSet2.set[i] = s2.set[i];
	} else {
	    pcSet1 = new PcSet(s2.set.length);
	    pcSet2 = new PcSet(s1.set.length);
	    for (int i = 0; i < pcSet1.set.length; i++)
		pcSet1.set[i] = s2.set[i];
	    for (int i = 0; i < pcSet2.set.length; i++)
		pcSet2.set[i] = s1.set[i];

	    // So the size of pcSet1.set is always greater than 
	    // or equal to that of pcSet2.set.
	}

	// Characters used for base 12 system, 
	// a & b, A & B, t & e, or T & E
	if (s1.base12String != null) {
	    pcSet1.base12String = new String(s1.base12String);
	    base12String = new String(s1.base12String);
	} else if (s2.base12String != null) {
	    pcSet2.base12String = new String(s2.base12String);
	    base12String = new String(s2.base12String);
	} else {
	    pcSet1.base12String = new String("AB");
	    pcSet2.base12String = new String("AB");
	    base12String = new String("AB");
	}
    }





    // Check out if it is included under transposition, or Tn.
    public String listFitTns() {

	// SCALE_STEPS is defined in Set.java
	int[][] tnPcSet1 = new int[SCALE_STEPS][pcSet1.set.length];
	int h = 0, i = 0, j = 0, jj = 0;

	StringBuffer relations = new StringBuffer();

	// tnPcSet1[i][j]: 
	// pcSet1.set at all the transpositional levels 
	tnPcSet1 = pcSet1.getTns(0, SCALE_STEPS);
	
	// i is that of Ti. j is a pitch class.
	while (i < SCALE_STEPS) {
	    while (jj < pcSet2.set.length 
		   && j < tnPcSet1[i].length) {
		if (pcSet2.set[jj] == tnPcSet1[i][j]) {
		    jj++;
		    j = 0;
		} else {
		    j++;
		}
	    }

	    if (jj == pcSet2.set.length) {
		relations.append("    T" 
				 + i 
				 + ": {"
				 + pcSetArrayToString(pcSet1.set)
				 + "}  -->  {");

		for (int g = 0; g < pcSet1.set.length; g++)
		    relations.append(intToChar(tnPcSet1[i][g]));

		relations.append("}\n");
			
		h++;
		i++;
	    } else
		i++;

	    j = 0;
	    jj = 0;
	}

	relations.append("\n");

	StringBuffer f = new StringBuffer();

	if(h == 0)
	    f.append("No relations under T.\n\n");
	else {
	    if(pcSet1.set.length != pcSet2.set.length) {
		f.append("{" 
			 + pcSetArrayToString(pcSet2.set) 
			 + "} is "
			 + "a subset of "
			 + "{" 
			 + pcSetArrayToString(pcSet1.set)
			 + "} under: \n");
	    } else {
		f.append("{" 
			 + pcSetArrayToString(pcSet2.set) 
			 + "} is "
			 + "transformed from "
			 + "{" 
			 + pcSetArrayToString(pcSet1.set) 
			 + "} by: \n");
	    }
	    f.append(relations.toString());
	}
	return f.toString();
    }






    // Check out if it's included under inversion, or TnI.
    public String listFitTnIs() {

	int[][] tniPcSet1 = new int[SCALE_STEPS][pcSet1.set.length];
	int h = 0, i = 0, j = 0, jj = 0;

	StringBuffer relations = new StringBuffer();
  
	tniPcSet1 = pcSet1.getTnIs(0, SCALE_STEPS);
	// tniPcSet1[i][j]: pc-sets at all the inversions. 
	// i is that of Ti. j is a pitch class.

	// i is that of Ti. j is a pitch class.
	while (i < SCALE_STEPS) {
	    while (jj < pcSet2.set.length 
		   && j < tniPcSet1[i].length) {
		if(pcSet2.set[jj] == tniPcSet1[i][j]) {
		    jj++;
		    j = 0;
		} else
		    j++;
	    }
    
	    if (jj == pcSet2.set.length) {

		relations.append("    T" 
				 + i 
				 + "I: {"
				 + pcSetArrayToString(pcSet1.set)
				 + "}  -->  {");

		for (int g = 0; g < pcSet1.set.length; g++)
		    relations.append(intToChar(tniPcSet1[i][g]));

		relations.append("}\n");

		h++;
		i++;
	    } else
		i++;
			
	    j = 0;
	    jj = 0;
	}

	relations.append("\n");

	StringBuffer f = new StringBuffer();

	if(h == 0)
	    f.append("No relations under TI.\n\n");
	else {
	    if(pcSet1.set.length != pcSet2.set.length) {
		f.append("{" 
			 + pcSetArrayToString(pcSet2.set) 
			 + "} is "
			 + "a subset of "
			 + "{" 
			 + pcSetArrayToString(pcSet1.set) 
			 + "} under: \n");
	    } else {
		f.append("{" 
			 + pcSetArrayToString(pcSet2.set) 
			 + "} is "
			 + "transformed from "
			 + "{" 
			 + pcSetArrayToString(pcSet1.set) 
			 + "} by: \n");
	    }
	    f.append(relations.toString());
	}
	return f.toString();
    }





    // Check out first if it's included under multiplication, or Mn.
    public String listFitMns() {

	int[][] mnPcSet1 = new int[SCALE_STEPS][pcSet1.set.length];
	int h = 0, i = 1, j = 0, jj = 0;

	StringBuffer relations = new StringBuffer();

	mnPcSet1 = pcSet1.getMns(0, SCALE_STEPS);
	// mnPcSet1[i][j]: pc-sets at all the multiplications. 
	// i is that of Mi. j is a pitch class.

	while (i < SCALE_STEPS) {
	    while (jj < pcSet2.set.length 
		   && j < mnPcSet1[i].length) {
		if(pcSet2.set[jj] == mnPcSet1[i][j]) {
		    jj++;
		    j = 0;
		}
		else
		    j++;
	    }

	    if (jj == pcSet2.set.length) {

		relations.append("    M" 
				 + i 
				 + ": {"
				 + pcSetArrayToString(pcSet1.set)
				 + "}  -->  {");

		for (int g = 0; g < pcSet1.set.length; g++)
		    relations.append(intToChar(mnPcSet1[i][g]));

		relations.append("}\n");
			
		h++;
		i++;
	    }
	    else
		i++;

	    j = 0;
	    jj = 0;
	}

	relations.append("\n");

	StringBuffer f = new StringBuffer();

	if(h == 0)
	    f.append("No relations under M.\n\n");
	else {
	    if(pcSet1.set.length != pcSet2.set.length) {
		f.append("{" 
			 + pcSetArrayToString(pcSet2.set) 
			 + "} is "
			 + "a subset of "
			 + "{" 
			 + pcSetArrayToString(pcSet1.set) 
			 + "} under: \n");
	    } else {
		f.append("{" 
			 + pcSetArrayToString(pcSet2.set) 
			 + "} is "
			 + "transformed from "
			 + "{" 
			 + pcSetArrayToString(pcSet1.set) 
			 + "} by: \n");
	    }
	    f.append(relations.toString());
	}
	return f.toString();
    }






    public String sharedSubSets(int k, int n) { 
	// Check out if there are shared subsets under Tn, TnI, and Mn.
	// The size of pcSet1.set is always greater than or equal to 
	// that of pcSet2.set.
	// k: minimum size, n: maximum

	SubSuperSetList ssl = new SubSuperSetList();
	// int numOfCombinations(int i, int j): 
	// iCj = i!/(j! * (i - j)!)
	int subsetSize;
	StringBuffer f = new StringBuffer();

	for (subsetSize = k; subsetSize <= n; subsetSize++) {

	    int numOfSubSets;
	    if (subsetSize == pcSet2.set.length)
		numOfSubSets = 1;
	    else
		numOfSubSets 
		    = ssl.numOfCombinations(pcSet2.set.length, subsetSize);

	    int [][] subSets = new int[numOfSubSets][subsetSize];
	    subSets = ssl.getSubSets(pcSet2.set, subsetSize);

	    int fTn = 0;  // The number of instances under Tn
	    int fTnI = 0; // The number of instances under TnI
	    int fMn = 0;  // The number of instances under Mn

	    for (int iii = 0; iii < subSets.length; iii++) {

		StringBuffer relations = new StringBuffer();


		// Check subsets under Tn first.
		// SCALE_STEPS is defined in Set.java
		int[][] tnPcSet1 = new int[SCALE_STEPS][pcSet1.set.length];
		int hTn = 0;
		int i = 0, j = 0, jj = 0;

		// tnPcSet1[i][j]: pc-sets at all the transpositional levels 
		tnPcSet1 = pcSet1.getTns(0, SCALE_STEPS);
	
		// i is that of Ti. j is a pitch class.
		while (i < SCALE_STEPS) {
		    while (jj < subSets[iii].length 
			   && j < tnPcSet1[i].length) {
			if(subSets[iii][jj] == tnPcSet1[i][j]) {
			    jj++;
			    j = 0;
			}
			else
			    j++;
		    }

		    if (jj == subSets[iii].length) {
			relations.append(
					 "    T" 
					 + i 
					 + ": {"
					 + pcSetArrayToString(pcSet1.set)
					 + "}  -->  ");
			relations.append("{");
			relations.append(
				 intArrayToCharOne(
					   sortPcSet(tnPcSet1[i])));
			relations.append("} \n");

			fTn++;
			hTn++;
			i++;
		    } else
			i++;

		    j = 0;
		    jj = 0;
		}



		// Check subsets under TnI next.
		int[][] tniPcSet1 = new int[SCALE_STEPS][pcSet1.set.length];
		int hTnI = 0;
		i = 0; 
		j = 0; 
		jj = 0;

		tniPcSet1 = pcSet1.getTnIs(0, SCALE_STEPS);
		// tniPcSet1[i][j]: pc-sets at all the inversions. 
		// i is that of Ti. j is a pitch class.

		// i is that of Ti. j is a pitch class.
		while (i < SCALE_STEPS) {
		    while (jj < subSets[iii].length 
			   && j < tniPcSet1[i].length) {
			if(subSets[iii][jj] == tniPcSet1[i][j]) {
			    jj++;
			    j = 0;
			} else
			    j++;
		    }
    
		    if (jj == subSets[iii].length) {
			relations.append(
					 "    T" 
					 + i 
					 + "I: {"
					 + pcSetArrayToString(pcSet1.set)
					 + "}  -->  ");
			relations.append("{");
			relations.append(
				 intArrayToCharOne(
					   sortPcSet(tniPcSet1[i])));
			relations.append("} \n");

			fTnI++;
			hTnI++;
			i++;
		    } else
			i++;
			
		    j = 0;
		    jj = 0;
		}



		// Then check subsets under Mn
		int[][] mnPcSet1 = new int[SCALE_STEPS][pcSet1.set.length];
		int hMn = 0;
		i = 1; 
		j = 0; 
		jj = 0;

		mnPcSet1 = pcSet1.getMns(0, SCALE_STEPS);
		// mnPcSet1[i][j]: pc-sets at all the multiplications. 
		// i is that of Mi. j is a pitch class.

		while (i < SCALE_STEPS) {
		    while (jj < subSets[iii].length 
			   && j < mnPcSet1[i].length) {
			if(subSets[iii][jj] == mnPcSet1[i][j]) {
			    jj++;
			    j = 0;
			}
			else
			    j++;
		    }

		    if (jj == subSets[iii].length) {
			relations.append(
					 "    M" 
					 + i 
					 + ": {"
					 + pcSetArrayToString(pcSet1.set)
					 + "}  -->  ");
			relations.append("{");
			relations.append(
				 intArrayToCharOne(
					   sortPcSet(mnPcSet1[i])));
			relations.append("} \n");
			
			fMn++;
			hMn++;
			i++;
		    }
		    else
			i++;

		    j = 0;
		    jj = 0;
		}


		if (hTn != 0 || hTnI !=0 || hMn != 0) {
		    f.append("{" 
			     + pcSetArrayToString(sortPcSet(subSets[iii])) 
			     + "}, a subset of " 
			     + "{" 
			     + pcSetArrayToString(pcSet2.set) 
			     + "}, "
			     + "is shared with: \n");
		    f.append(relations.toString());
		    f.append(" \n");
		}
	    }

	    if (fTn == 0 && fTnI == 0 && fMn == 0) {
		f.append("There is no shared subset of size " 
			 + subsetSize + ". \n");
	    }
	}

	return f.toString();
    }









    public String sharedSuperSets(int k, int n) { 
	// Check out if there are shared supersets under Tn, TnI, and Mn.
	// The size of pcSet1.set is always greater than or equal to 
	// that of pcSet2.set.
	// k: minimum size, n: maximum sizes of supersets

	SubSuperSetList ssl = new SubSuperSetList();
	// int numOfCombinations(int i, int j): 
	// iCj = i!/(j! * (i - j)!)
	int supersetSize;
	StringBuffer f = new StringBuffer();

	for (supersetSize = k; supersetSize <= n; supersetSize++) {

	    int numOfSuperSets;
	    if (supersetSize == pcSet1.set.length)
		numOfSuperSets = 1;
	    else
		numOfSuperSets
		    = ssl.numOfCombinations(SCALE_STEPS - pcSet1.set.length, 
					    supersetSize - pcSet1.set.length);

	    int [][] superSets = new int[numOfSuperSets][supersetSize];
	    superSets = ssl.getSuperSets(pcSet1.set, supersetSize);

	    int fTn = 0;  // The number of instances under Tn
	    int fTnI = 0; // The number of instances under TnI
	    int fMn = 0;  // The number of instances under Mn

	    for (int iii = 0; iii < superSets.length; iii++) {

		StringBuffer relations = new StringBuffer();

		// Check supersets under Tn first.
		// SCALE_STEPS is defined in Set.java
		int[][] tnPcSet2 = new int[SCALE_STEPS][pcSet2.set.length];
		int hTn = 0; 
		int i = 0, j = 0, jj = 0;

		// tnPcSet2[i][j]: pc-sets at all the transpositional levels 
		tnPcSet2 = pcSet2.getTns(0, SCALE_STEPS);
	
		// i is that of Ti. j is a pitch class.
		while (i < SCALE_STEPS) {
		    while (jj < tnPcSet2[i].length 
			   && j < superSets[iii].length) {

			if (tnPcSet2[i][jj] == superSets[iii][j]) {
			    jj++;
			    j = 0;
			}
			else
			    j++;
		    }

		    if (jj == tnPcSet2[i].length) {
			relations.append(
					 "    T" 
					 + i 
					 + ": {"
					 + pcSetArrayToString(pcSet2.set)
					 + "}  -->  {");
			relations.append(
				 intArrayToCharOne(
					   sortPcSet(tnPcSet2[i])));
			relations.append("} \n");


			fTn++;
			hTn++;
			i++;
		    } else
			i++;

		    j = 0;
		    jj = 0;
		}



		// Check supersets under TnI
		int[][] tniPcSet2 = new int[SCALE_STEPS][pcSet2.set.length];
		int hTnI = 0; 
		i = 0; 
		j = 0; 
		jj = 0;

		tniPcSet2 = pcSet2.getTnIs(0, SCALE_STEPS);
		// tniPcSet2[i][j]: pc-sets at all the inversions. 
		// i is that of Ti. j is a pitch class.

		// i is that of Ti. j is a pitch class.
		while (i < SCALE_STEPS) {
		    while (jj < tniPcSet2[i].length 
			   && j < superSets[iii].length) {

			if(tniPcSet2[i][jj] == superSets[iii][j]) {
			    jj++;
			    j = 0;
			} else
			    j++;
		    }
    
		    if (jj == tniPcSet2[i].length) {
			relations.append(
					 "    T" 
					 + i 
					 + "I: {"
					 + pcSetArrayToString(pcSet2.set)
					 + "}  -->  {");
			relations.append(
				 intArrayToCharOne(
					   sortPcSet(tniPcSet2[i])));
			relations.append("} \n");

			fTnI++;
			hTnI++;
			i++;
		    } else
			i++;
			
		    j = 0;
		    jj = 0;
		}



		// Check supersets under Mn
		int[][] mnPcSet2 = new int[SCALE_STEPS][pcSet2.set.length];

		int hMn = 0; 
		i = 1; 
		j = 0; 
		jj = 0;

		mnPcSet2 = pcSet2.getMns(0, SCALE_STEPS);
		// mnPcSet2[i][j]: pc-sets at all the multiplications. 
		// i is that of Mi. j is a pitch class.

		while (i < SCALE_STEPS) {
		    while (jj < mnPcSet2[i].length 
			   && j < superSets[iii].length) {

			if(mnPcSet2[i][jj] == superSets[iii][j]) {
			    jj++;
			    j = 0;
			}
			else
			    j++;
		    }

		    if (jj == mnPcSet2[i].length) {
			relations.append(
					 "    M" 
					 + i 
					 + ": {"
					 + pcSetArrayToString(pcSet2.set)
					 + "}  -->  {");
			relations.append(
				 intArrayToCharOne(
					   sortPcSet(mnPcSet2[i])));
			relations.append("} \n");
			
			fMn++;
			hMn++;
			i++;
		    }
		    else
			i++;

		    j = 0;
		    jj = 0;
		}

		if (hTn != 0 || hTnI !=0 || hMn != 0) {
		    f.append("{" 
			     + pcSetArrayToString(sortPcSet(superSets[iii]))
			     + "}, a superset of {"
			     + pcSetArrayToString(pcSet1.set)
			     + "}, is shared with: \n");
		    f.append(relations.toString());
		    f.append(" \n");
		}
	    }

	    if (fTn == 0 && fTnI == 0 && fMn == 0) {
		f.append("There is no shared superset of size " 
			 + supersetSize 
			 + ". \n\n");
	    }
	}

	return f.toString();
    }
}
