// UnorderedSetLists.java, used for
// JDubiel.java: computer program for pc-set analysis
// by Akira Takaoka, akira.takaoka@columbia.edu
// Copyright (C) 1994-2017 Akira Takaoka
// All rights reserved.




// package jdubielApplication;

import java.lang.reflect.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;





public class UnorderedSetLists extends JFrame 
                               implements ActionListener {

    static final int xOffset = 110, yOffset = 100;
    static int openFrameCount = 0;


    JScrollPane listsScrollPane;

    JPanel listsPanel;
    JPanel pcSetPanel;
    JPanel buttonPanel;

    JEditorPane editorPane;
    JScrollPane editorScrollPane;

    JTextField listsTextField;

    JCheckBox tnCheckBox;
    JCheckBox tnICheckBox;
    JCheckBox mnCheckBox;
    JCheckBox subSetsCheckBox;
    JCheckBox superSetsCheckBox;
    JCheckBox invarTnCheckBox;
    JCheckBox invarTnICheckBox;
    JCheckBox invarMnCheckBox;

    JSpinner tnSpinner;
    JSpinner tkSpinner;
    JSpinner tnISpinner;
    JSpinner tkISpinner;
    JSpinner mnSpinner;
    JSpinner mkSpinner;
    JSpinner invarTkSpinner;
    JSpinner invarTnSpinner;
    JSpinner invarTkISpinner;
    JSpinner invarTnISpinner;
    JSpinner invarMkSpinner;
    JSpinner invarMnSpinner;
    JSpinner subSetsSpinner;
    JSpinner superSetsSpinner;

    SpinnerNumberModel tnModel;
    SpinnerNumberModel tkModel;
    SpinnerNumberModel tnIModel;
    SpinnerNumberModel tkIModel;
    SpinnerNumberModel mnModel;
    SpinnerNumberModel mkModel;
    SpinnerNumberModel subSetsModel;
    SpinnerNumberModel superSetsModel;
    SpinnerNumberModel invarTkModel;
    SpinnerNumberModel invarTnModel;
    SpinnerNumberModel invarTkIModel;
    SpinnerNumberModel invarTnIModel;
    SpinnerNumberModel invarMkModel;
    SpinnerNumberModel invarMnModel;

    StringBuffer results;







    public UnorderedSetLists() {

	createWidgets();

	// Put everything together.
	JPanel devidedPane = new JPanel(new BorderLayout());
	devidedPane.add(pcSetPanel, BorderLayout.NORTH);
	devidedPane.add(buttonPanel, BorderLayout.WEST);
	devidedPane.add(listsScrollPane, BorderLayout.CENTER);
	devidedPane.add(editorScrollPane, BorderLayout.SOUTH);

	getContentPane().add(devidedPane, BorderLayout.CENTER);

	setSize(360, 600);
	openFrameCount++;
	setTitle("Unordered Set Lists " + openFrameCount);
	setLocation(xOffset + 10 * openFrameCount, 
		    yOffset + 10 * openFrameCount);
    }
	


    // Create and add the widgets.
    private void createWidgets() {

	// add a function to generate cyclic sets.

	// Typed pc-set
	JLabel pcSetLabel = new JLabel("Unordered PC-Set: ", JLabel.TRAILING);
	listsTextField = new JTextField(12);
	listsTextField.setBackground(Color.white);
	listsTextField.setDragEnabled(true);
	pcSetLabel.setLabelFor(listsTextField);

	pcSetPanel = new JPanel();
	pcSetPanel.add(pcSetLabel);
	pcSetPanel.add(listsTextField);
	pcSetPanel.setBorder(
		     BorderFactory.createCompoundBorder(
				BorderFactory.createTitledBorder(""),
                                BorderFactory.createEmptyBorder(0,0,0,0)));



	// Find button
	JButton findButton = new JButton("Find");
	findButton.setActionCommand("find");
	findButton.addActionListener(this);
	findButton.setBackground(Color.lightGray);
	findButton.setFont(new Font("SansSerif", Font.BOLD, 12));

	// Clear button
	JButton clearButton = new JButton("Clear");
	clearButton.setActionCommand("clear");
	clearButton.addActionListener(this);
	clearButton.setBackground(Color.lightGray);
	
	JPanel buttonPanel1 = new JPanel(new SpringLayout());
	buttonPanel1.add(findButton);
	buttonPanel1.add(clearButton);
	SpringUtilities.makeCompactGrid(buttonPanel1,
					2, 1,  //rows, cols
					1, 1,  //initX, initY
					1, 1); //xPad, yPad

	buttonPanel = new JPanel(new BorderLayout());
	buttonPanel.add(buttonPanel1, BorderLayout.NORTH);





		// Tn check box and Tn/Tk spinner
		tnCheckBox = new JCheckBox("Tk to Tn:    ");
		tnCheckBox.setToolTipText("Lists pc-sets under "
					   + "transpositions by k to n.");

		tkModel = new SpinnerNumberModel(
			0, //initial value
			0, //min
			11, //max
			1); //step
		tkSpinner = new JSpinner(tkModel);
		tkSpinner.setEditor(new JSpinner.NumberEditor(tkSpinner));
		Dimension d2 = new Dimension();
		tkSpinner.setSize(d2);

		tnModel = new SpinnerNumberModel(
			11, //initial value
			0, //min
			11, //max
			1); //step
		tnSpinner = new JSpinner(tnModel);
		tnSpinner.setEditor(new JSpinner.NumberEditor(tnSpinner));
		d2.setSize(0.0, 0.0);
		tnSpinner.setSize(d2);


		// TnI check box & TnI and TkI spinners
		tnICheckBox = new JCheckBox("TkI to TnI:    ");
		tnICheckBox.setToolTipText("Lists pc-sets under "
					   + "inversions by k to n.");

		tkIModel = new SpinnerNumberModel(
			0, //initial value
			0, //min
			11, //max
			1); //step
		tkISpinner = new JSpinner(tkIModel);
		tkISpinner.setEditor(new JSpinner.NumberEditor(tkISpinner));
		tkISpinner.setSize(d2);

		tnIModel = new SpinnerNumberModel(
			11, //initial value
			0, //min
			11, //max
			1); //step
		tnISpinner = new JSpinner(tnIModel);
		tnISpinner.setEditor(new JSpinner.NumberEditor(tnISpinner));
		tnISpinner.setSize(d2);


		// Mn check box & Mn spinner
		mnCheckBox = new JCheckBox("Mk to Mn:    ");
		mnCheckBox.setToolTipText("Lists pc-sets under "
					   + "multiplications by k to n.");

		mkModel = new SpinnerNumberModel(
			1, //initial value
			1, //min
			11, //max
			1); //step
		mkSpinner = new JSpinner(mkModel);
		mkSpinner.setEditor(new JSpinner.NumberEditor(mkSpinner));
		mkSpinner.setSize(d2);

		mnModel = new SpinnerNumberModel(
			11, //initial value
			1, //min
			11, //max
			1); //step
		JSpinner mnSpinner = new JSpinner(mnModel);
		mnSpinner.setEditor(new JSpinner.NumberEditor(mnSpinner));
		mnSpinner.setSize(d2);


		// Invariant pcs under Tn check box
		invarTnCheckBox = new JCheckBox("Tk to Tn:    ");
		invarTnCheckBox.setToolTipText(
			"Lists the invariant pitch classes "
			+ "under transposition by n.");

		// Invariant pcs under Tk spinner
		invarTkModel = new SpinnerNumberModel(
			0, //initial value
			0, //min
			11, //max
			1); //step
		invarTkSpinner = new JSpinner(invarTkModel);
		invarTkSpinner.setEditor(
			new JSpinner.NumberEditor(invarTkSpinner));
		invarTkSpinner.setSize(d2);

		invarTnModel = new SpinnerNumberModel(
			11, //initial value
			0, //min
			11, //max
			1); //step
		invarTnSpinner = new JSpinner(invarTnModel);
		invarTnSpinner.setEditor(
			new JSpinner.NumberEditor(invarTnSpinner));
		invarTnSpinner.setSize(d2);


		// Invariant pcs under TnI check box
		invarTnICheckBox = new JCheckBox("TkI to TnI:    ");
		invarTnICheckBox.setToolTipText(
			"Lists the invariant pitch classes "
			+ "under inversion by n.");

		// Invariant pcs under TkI spinner
		invarTkIModel = new SpinnerNumberModel(
			0, //initial value
			0, //min
			11, //max
			1); //step
		invarTkISpinner = new JSpinner(invarTkIModel);
		invarTkISpinner.setEditor(
			new JSpinner.NumberEditor(invarTkISpinner));
		invarTkISpinner.setSize(d2);

		invarTnIModel = new SpinnerNumberModel(
			11, //initial value
			0, //min
			11, //max
			1); //step
		invarTnISpinner = new JSpinner(invarTnIModel);
		invarTnISpinner.setEditor(
			new JSpinner.NumberEditor(invarTnISpinner));
		invarTnISpinner.setSize(d2);


		// Invariant pcs under Mn check box
		invarMnCheckBox = new JCheckBox("Mk to Mn:    ");
		invarMnCheckBox.setToolTipText(
			"Lists the invariant pitch classes "
			+ "under multiplication by n.");

		// Invariant pcs under Mk spinner
		invarMkModel = new SpinnerNumberModel(
			1, //initial value
			1, //min
			11, //max
			1); //step
		invarMkSpinner = new JSpinner(invarMkModel);
		invarMkSpinner.setEditor(
			new JSpinner.NumberEditor(invarMkSpinner));
		invarMkSpinner.setSize(d2);

		invarMnModel = new SpinnerNumberModel(
			11, //initial value
			1, //min
			11, //max
			1); //step
		invarMnSpinner = new JSpinner(invarMnModel);
		invarMnSpinner.setEditor(
			new JSpinner.NumberEditor(invarMnSpinner));
		invarMnSpinner.setSize(d2);


		// Subsets check box
		subSetsCheckBox = new JCheckBox("subsets of size:     ");
		subSetsCheckBox.setToolTipText(
			"Lists all the subsets of cardinality n and their distribution.");

		subSetsModel = new SpinnerNumberModel(
			3, //initial value
			3, //min
			9, //max
			1); //step
		subSetsSpinner = new JSpinner(subSetsModel);
		subSetsSpinner.setEditor(
			new JSpinner.NumberEditor(subSetsSpinner));
		subSetsSpinner.setSize(d2);


		// Supersets check box
		superSetsCheckBox = new JCheckBox("supersets of size:     ");
		superSetsCheckBox.setToolTipText(
			"Lists all the supersets of size n and their distribution.");

		superSetsModel = new SpinnerNumberModel(
			7, //initial value
			4, //min
			9, //max
			1); //step
		superSetsSpinner = new JSpinner(superSetsModel);
		superSetsSpinner.setEditor(
			new JSpinner.NumberEditor(superSetsSpinner));
		superSetsSpinner.setSize(d2);








		//Add the widgets to the container.
		JPanel listsPanel1 = new JPanel(new SpringLayout());
		JLabel blankLabel1 = new JLabel(" ", JLabel.TRAILING);
		JLabel n = new JLabel("k", JLabel.CENTER);
		JLabel k = new JLabel("n", JLabel.CENTER);

        	listsPanel1.add(blankLabel1);
        	listsPanel1.add(n);
        	listsPanel1.add(k);

        	listsPanel1.add(tnCheckBox);
		listsPanel1.add(tkSpinner);
		listsPanel1.add(tnSpinner);

		listsPanel1.add(tnICheckBox);
		listsPanel1.add(tkISpinner);
		listsPanel1.add(tnISpinner);

		listsPanel1.add(mnCheckBox);
		listsPanel1.add(mkSpinner);
		listsPanel1.add(mnSpinner);

		//Lay out listsPanel1
		SpringUtilities.makeCompactGrid(listsPanel1,
						4, 3,  //rows, cols
						1, 1,  //initX, initY
						1, 1); //xPad, yPad
		listsPanel1.setBorder(
			BorderFactory.createCompoundBorder(
                                BorderFactory.createTitledBorder(
					"pc-sets under"),
                                BorderFactory.createEmptyBorder(0,0,0,0)));


		
		// listsPanel 2
		JPanel listsPanel2 = new JPanel(new SpringLayout());

		listsPanel2.add(invarTnCheckBox);
		listsPanel2.add(invarTkSpinner);
		listsPanel2.add(invarTnSpinner);

		listsPanel2.add(invarTnICheckBox);
		listsPanel2.add(invarTkISpinner);
		listsPanel2.add(invarTnISpinner);

		listsPanel2.add(invarMnCheckBox);
		listsPanel2.add(invarMkSpinner);
		listsPanel2.add(invarMnSpinner);

		//Lay out listsPanel 2
		SpringUtilities.makeCompactGrid(listsPanel2,
						3, 3,  //rows, cols
						1, 1,  //initX, initY
						1, 1); //xPad, yPad
		listsPanel2.setBorder(
			BorderFactory.createCompoundBorder(
                                BorderFactory.createTitledBorder(
					"invariant subsets under"),
                                BorderFactory.createEmptyBorder(0,0,0,0)));



		// Lists panel 3
		JPanel listsPanel3 = new JPanel(new SpringLayout());

		listsPanel3.add(subSetsCheckBox);
		listsPanel3.add(subSetsSpinner);
		listsPanel3.add(superSetsCheckBox);
		listsPanel3.add(superSetsSpinner);

		//Lay out lists panel 3
		SpringUtilities.makeCompactGrid(listsPanel3,
						2, 2,  //rows, cols
						1, 1,  //initX, initY
						1, 1); //xPad, yPad
/*
		listsPanel3.setBorder(
			BorderFactory.createCompoundBorder(
                                BorderFactory.createTitledBorder(
					""),
                                BorderFactory.createEmptyBorder(0,0,0,0)));
*/


		listsPanel = new JPanel(new SpringLayout());
		listsPanel.add(listsPanel1);
		listsPanel.add(listsPanel2);
		listsPanel.add(listsPanel3);
		SpringUtilities.makeCompactGrid(listsPanel,
						3, 1,  //rows, cols
						1, 1,  //initX, initY
						1, 1); //xPad, yPad
		listsPanel.setBorder(
			BorderFactory.createCompoundBorder(
                                BorderFactory.createTitledBorder(
					""),
                                BorderFactory.createEmptyBorder(5,5,5,5)));


		listsScrollPane = new JScrollPane(listsPanel);
		listsScrollPane.setVerticalScrollBarPolicy(
                        JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		listsScrollPane.setHorizontalScrollBarPolicy(
			JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		listsScrollPane.setPreferredSize(new Dimension(250, 280));
		listsScrollPane.setMinimumSize(new Dimension(250, 280));


		// "Found Relations" area
		editorPane = new JEditorPane();
		editorScrollPane = new JScrollPane(editorPane);
		editorScrollPane.setVerticalScrollBarPolicy(
                        JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		editorScrollPane.setHorizontalScrollBarPolicy(
			JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		editorScrollPane.setPreferredSize(new Dimension(250, 270));
		editorScrollPane.setMinimumSize(new Dimension(250, 270));
		editorScrollPane.setBorder(
			BorderFactory.createCompoundBorder(
				BorderFactory.createTitledBorder(
					"Found: "),
				BorderFactory.createEmptyBorder(5,5,5,5)));
		editorScrollPane.getBorder();
	}







    public void actionPerformed(ActionEvent e) {

	if("find".equals(e.getActionCommand())) {
	    editorPane.setText("");

	    results = new StringBuffer();
	    PcSet pq = new PcSet(); // just for the methods used below
			
	    // Get the typed-in orderd pc-set and
	    // remove space(s) at the beginning and the end of it.
	    String readPcSetString 
		= pq.removeFistLastSpaces(listsTextField.getText());

	    // Remove spaces.
	    StringBuffer withoutSpaceStringBuffer = new StringBuffer();
	    for (int i = 0; i < readPcSetString.length(); i++)
		if (readPcSetString.charAt(i) != ' ')
		    withoutSpaceStringBuffer.append(readPcSetString.charAt(i));

	    String withoutSpaceString = withoutSpaceStringBuffer.toString();

	    // Remove duplicates
	    String pcSetString 
		= pq.removeDuplicatesInString(withoutSpaceString);

	    // Check if there are non-numerical characters
	    String illegalChars = pq.checkNonNumerals(pcSetString);

	    if (illegalChars.length() > 1) {
		JOptionPane.showMessageDialog(
			      pcSetPanel, 
			      "Non-numerical characters typed:"
			      + illegalChars.toString(),
			      "Warning",
			      JOptionPane.WARNING_MESSAGE);
		} else if(pcSetString.length() 
			  != withoutSpaceString.length()) {
		    JOptionPane.showMessageDialog(
				pcSetPanel, 
				"There are duplicates in the pc-set.",
				"Warning",
				JOptionPane.WARNING_MESSAGE);
	    } else if(pcSetString.length() < 3) {
		JOptionPane.showMessageDialog(
				pcSetPanel, 
				"The size of a set must be \n"
				+"equal to or greater than 3.",
				"Warning",
				JOptionPane.WARNING_MESSAGE);
	    } else if(9 < pcSetString.length()) {
		JOptionPane.showMessageDialog(
				pcSetPanel, 
				"The size of a set must be "
				+"equal to or smaller than 9.",
				"Warning",
				JOptionPane.WARNING_MESSAGE);
	    } else {
		char[] pcSetChars = new char[pcSetString.length()];
		pcSetString.getChars(
			0, 
			pcSetString.length(), 
			pcSetChars, 
			0);
		PcSet pcSet = new PcSet(pcSetChars.length);
		pcSet.set = pcSet.charArrayToIntOne(pcSetChars);

		if (tnCheckBox.getSelectedObjects() == null &&
		    tnICheckBox.getSelectedObjects() == null &&
		    mnCheckBox.getSelectedObjects() == null &&
		    invarTnCheckBox.getSelectedObjects() == null &&
		    invarTnICheckBox.getSelectedObjects() == null &&
		    invarMnCheckBox.getSelectedObjects() == null &&
		    subSetsCheckBox.getSelectedObjects() == null &&
		    superSetsCheckBox.getSelectedObjects() == null) {
		    JOptionPane.showMessageDialog(
					pcSetPanel, 
					"You need to choose at least "
					+ "one of the operations.\n",
					"Warning",
					JOptionPane.WARNING_MESSAGE);
		} else {

		    StringBuffer pcSetStringBuffer = new StringBuffer("");
		    for (int h = 0; h < pcSetChars.length; h++)
			pcSetStringBuffer.append(pcSetChars[h]);

		    String inPcSetString = pcSetStringBuffer.toString();

		    if (tnCheckBox.getSelectedObjects() != null) {
			int k = tnModel.getNumber().intValue();
			int n = tkModel.getNumber().intValue();
			int [][] pcsList 
			    = new int[Math.abs(n - k) + 1][pcSet.set.length];

			if (k <= n)
			    pcsList = pcSet.getTns(k, n);
			else
			    pcsList = pcSet.getTns(n, k);
					
			results.append("\n");
			results.append("Tk to Tn: <"
				       + inPcSetString
				       + ">\n");
					
			for (int i = 0; i < pcsList.length; i++) {
			    Set s = new Set(pcsList[i]);
			    s.base12String = pcSet.base12String;
			    char [] c = s.intArrayToCharOne(s.set);

			    if (k <= n)
				    results.append("    T" 
						   + (k + i) 
						   + ": {"
						   + inPcSetString
						   + "}  -->  {");
			    else
				    results.append("    T" 
						   + (n + i) 
						   + ": {"
						   + inPcSetString
						   + "}  -->  {");

			    for(int j = 0; j < c.length; j++)
				results.append(c[j]);

			    results.append("}\n");
			}
		    }




		    if (tnICheckBox.getSelectedObjects() != null) {
			int k = tnIModel.getNumber().intValue();
			int n = tkIModel.getNumber().intValue();
			int [][] pcsList 
			    = new int[Math.abs(n - k) + 1][pcSet.set.length];
					
			if (k <= n)
			    pcsList = pcSet.getTnIs(k, n);
			else
			    pcsList = pcSet.getTnIs(n, k);

			results.append("\n");
			results.append("TkI to TnI: <"
				       + inPcSetString
				       + ">\n");

			for (int i = 0; i < pcsList.length; i++) {
			    Set s = new Set(pcsList[i]);
			    s.base12String = pcSet.base12String;
			    char [] c = s.intArrayToCharOne(s.set);

			    if (k <= n)
				    results.append("    T" 
						   + (k + i) 
						   + "I: {"
						   + inPcSetString
						   + "}  -->  {");
			    else
				    results.append("    T" 
						   + (n + i) 
						   + "I: {"
						   + inPcSetString
						   + "}  -->  {");

			    for(int j = 0; j < c.length; j++)
				results.append(c[j]);

			    results.append("}\n");
			}
		    }




		    if (mnCheckBox.getSelectedObjects() != null) {
			int k = mnModel.getNumber().intValue();
			int n = mkModel.getNumber().intValue();
			int [][] pcsList 
			    = new int[Math.abs(n - k) + 1][pcSet.set.length];

			if (k <= n)
			    pcsList = pcSet.getMns(k, n);
			else
			    pcsList = pcSet.getMns(n, k);

			results.append("\n");
			results.append("Mk to Mn: <"
				       + inPcSetString
				       + ">\n");

			for (int i = 0; i < pcsList.length; i++) {
			    Set s = new Set(pcsList[i]);
			    s.base12String = pcSet.base12String;
			    char [] c = s.intArrayToCharOne(s.set);

			    if (k <= n)
				    results.append("    M" 
						   + (k + i) 
						   + ": {"
						   + inPcSetString
						   + "}  -->  {");
			    else
				    results.append("    M" 
						   + (n + i) 
						   + ": {"
						   + inPcSetString
						   + "}  -->  {");

			    for(int j = 0; j < c.length; j++)
				results.append(c[j]);

			    results.append("}\n");
			}
		    }



		    if (invarTnCheckBox.getSelectedObjects() != null ||
			invarTnICheckBox.getSelectedObjects() != null ||
			invarMnCheckBox.getSelectedObjects() != null) {
			
			InvariantsList invr;

			if (invarTnCheckBox.getSelectedObjects() != null) {
			    int k = invarTkModel.getNumber().intValue();
			    int n = invarTnModel.getNumber().intValue();

			    if (k > n) {
				int tmp = k;
				k = n;
				n = tmp;
			    }

			    invr = new InvariantsList(pcSet, k, n);

			    results.append(invr.unorderedTnInvariants());
			    results.append("\n");
			}

			if (invarTnICheckBox.getSelectedObjects() != null) {
			    int k = invarTkIModel.getNumber().intValue();
			    int n = invarTnIModel.getNumber().intValue();

			    if (k <= n)
				invr = new InvariantsList(pcSet, k, n);
			    else
				invr = new InvariantsList(pcSet, n, k);

			    results.append(invr.unorderedTnIInvariants());
			    results.append("\n");
			}

			if (invarMnCheckBox.getSelectedObjects() != null) {
			    int k = invarMkModel.getNumber().intValue();
			    int n = invarMnModel.getNumber().intValue();

			    if (k <= n)
				invr = new InvariantsList(pcSet, k, n);
			    else
				invr = new InvariantsList(pcSet, n, k);

			    results.append(invr.unorderedMnInvariants());
			    results.append("\n");
			}
		    }



		    if (subSetsCheckBox.getSelectedObjects() != null) {
			int n = subSetsModel.getNumber().intValue();
			if (pcSet.set.length <= n ) {
			    JOptionPane.showMessageDialog(
					pcSetPanel, 
					"The cardinality is larger than "
					+ "that of the typed in pc-set.\n",
					"Warning",
					JOptionPane.WARNING_MESSAGE);
			} else {
			    SubSuperSetList ssl = new SubSuperSetList();
			    ssl.base12String = pcSet.base12String;
			    results.append(
				   ssl.printSubSets(
					    pcSet.set, ssl.getSubSets(
							      pcSet.set, n)));
			}
		    }



		    if (superSetsCheckBox.getSelectedObjects() != null) {
			int n = superSetsModel.getNumber().intValue();
			if (pcSet.set.length >= n ) {
			    JOptionPane.showMessageDialog(
					pcSetPanel, 
					"The cardinality is smaller than "
					+ "that of the typed in pc-set.\n",
					"Warning",
					JOptionPane.WARNING_MESSAGE);
			} else if (9 < n ) {
			    JOptionPane.showMessageDialog(
					pcSetPanel, 
					"The largest cardinality of "
					+ "super sets is 9.\n",
					"Warning",
					JOptionPane.WARNING_MESSAGE);
			} else {
			    SubSuperSetList ssl = new SubSuperSetList();
			    ssl.base12String = pcSet.base12String;
			    results.append(
				   ssl.printSuperSets(
					      pcSet.set, ssl.getSuperSets(
							pcSet.set, n)));
			}
		    }

		    editorPane.setText(results.toString());
		}
	    } 
	}







	if("clear".equals(e.getActionCommand())) {
	    listsTextField.setText("");
	    editorPane.setText("");
	}
    }
}    
