// UnorderedSetProperties.java, used for
// JDubiel.java: Computer Program for pc-set analysis
// by Akira Takaoka, akira.takaoka@columbia.edu
// Copyright (C) 1994-2017 Akira Takaoka
// All rights reserved.


// package jdubielApplication;

import java.lang.reflect.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;
import java.lang.reflect.*;




public class UnorderedSetProperties extends JFrame 
                                    implements ActionListener {

    static final int xOffset = 10, yOffset = 100;
    static int openFrameCount = 0;

    JPanel propertiesPanel;
    JPanel pcSetPanel;
    JPanel buttonPanel;

    JComboBox<String> throughJComboBox;
    JTextField throughTextField;
    JTextField tnTypeTextField;
    JTextField tnITypeTextField;
    JTextField tnTnITypeTextField;
    JTextField forteNameTextField;
    JTextField iVectorTextField;
    JTextField m7m7iTextField;
    JTextField symmetryTextField;
    JTextField combinatorialTextField;

    public UnorderedSetProperties() {

	createWidgets();

	// Put everything together.
	JPanel devidedPane = new JPanel(new BorderLayout());
	devidedPane.setOpaque(true);
	devidedPane.add(pcSetPanel, BorderLayout.NORTH);
	devidedPane.add(buttonPanel, BorderLayout.WEST);
	devidedPane.add(propertiesPanel, BorderLayout.CENTER);

	// ...Create the GUI and put it in the window...
	setContentPane(devidedPane);
	setSize(300,320);
	openFrameCount++;
	setTitle("Unordered Set Properties " + openFrameCount);
	setLocation(xOffset + 20 * openFrameCount, 
		    yOffset + 20 * openFrameCount);
    }
	


    // Create and add the widgets.
    private void createWidgets() {

	// add a function to find cyclic sets.

	// Typed pc-set panel
	String[] throughStrings = {"PC-Set", "Forte Name", "I-Vector"};
	throughJComboBox = new JComboBox<>(throughStrings);
	throughJComboBox.setToolTipText("Choose PC-Set, "
					+ "Forte name, "
					+ "or Inverval Vector "
					+ "and then type, e.g., "
					+ "3701, "
					+ "BA15, ba15 or 1ET5, "
					+ "4-z29, " 
					+ "or 111111.");
	throughJComboBox.setSelectedIndex(0);
	throughJComboBox.setBackground(Color.white);

	throughTextField = new JTextField(11);
	throughTextField.setBackground(Color.white);
	throughTextField.setToolTipText(
			"Type a, A, t, or T for pitch-class 10 "
			+ "and b, B, e, or E for pitch-class 11 .");
	throughTextField.setDragEnabled(true);

	pcSetPanel = new JPanel(); 
	pcSetPanel.add(throughJComboBox);
	pcSetPanel.add(throughTextField);
	pcSetPanel.setBorder(
			BorderFactory.createCompoundBorder(
				   BorderFactory.createTitledBorder(""),
				   BorderFactory.createEmptyBorder(0,0,0,0)));



		// Find button
		JButton findButton = new JButton("Find");
		findButton.setActionCommand("find");
		findButton.addActionListener(this);
		findButton.setBackground(Color.lightGray);
		findButton.setFont(new Font("SansSerif", Font.BOLD, 12));

		// Clear button
		JButton clearButton = new JButton("Clear");
		clearButton.setActionCommand("clear");
		clearButton.addActionListener(this);
		clearButton.setBackground(Color.lightGray);

		JPanel buttonPanel1 = new JPanel(new SpringLayout());
		buttonPanel1.add(findButton);
		buttonPanel1.add(clearButton);
		SpringUtilities.makeCompactGrid(buttonPanel1,
						2, 1,  //rows, cols
						1, 1,  //initX, initY
						1, 1); //xPad, yPad

		buttonPanel = new JPanel(new BorderLayout());
		buttonPanel.add(buttonPanel1, BorderLayout.NORTH);



		// Properties panel
		forteNameTextField = new JTextField(12);
		forteNameTextField.setBackground(Color.white);
		forteNameTextField.setDragEnabled(true);
		JLabel forteNameLabel = new JLabel("Forte Name:", JLabel.TRAILING);
		forteNameLabel.setLabelFor(forteNameTextField);
		forteNameLabel.setToolTipText("Forte name of the Tn/TnI type,"
					      + " or the set class");

		tnTypeTextField = new JTextField(12);
		tnTypeTextField.setBackground(Color.white);
		tnTypeTextField.setDragEnabled(true);
		JLabel tnTypeLabel = new JLabel("Tn Type:", JLabel.TRAILING);
		tnTypeLabel.setLabelFor(tnTypeTextField);

		tnITypeTextField = new JTextField(12);
		tnITypeTextField.setBackground(Color.white);
		tnITypeTextField.setDragEnabled(true);
		JLabel tnITypeLabel = new JLabel("TnI Type:", JLabel.TRAILING);
		tnITypeLabel.setLabelFor(tnITypeTextField);

		tnTnITypeTextField = new JTextField(12);
		tnTnITypeTextField.setBackground(Color.white);
		tnTnITypeTextField.setDragEnabled(true);
		JLabel tnTnITypeLabel = new JLabel("Tn/TnI Type:", 
						   JLabel.TRAILING);
		tnTnITypeLabel.setLabelFor(tnTnITypeTextField);
		tnTnITypeLabel.setToolTipText("Set class");

		iVectorTextField = new JTextField(12);
		iVectorTextField.setBackground(Color.white);
		iVectorTextField.setDragEnabled(true);
		JLabel iVectorLabel = new JLabel("I-Vector:", JLabel.TRAILING);
		iVectorLabel.setLabelFor(iVectorTextField);
		iVectorLabel.setToolTipText("Interval-class vector");

		m7m7iTextField = new JTextField(12);
		m7m7iTextField.setBackground(Color.white);
		m7m7iTextField.setDragEnabled(true);
		JLabel m7m7iLabel = new JLabel("M7/M7I:", JLabel.TRAILING);
		m7m7iLabel.setLabelFor(m7m7iTextField);
		m7m7iLabel.setToolTipText("Tn/TnI type, or set class, "
				       + "under multiplication by 7");

		symmetryTextField = new JTextField(12);
		symmetryTextField.setBackground(Color.white);
		symmetryTextField.setDragEnabled(true);
		JLabel symmetryLabel = new JLabel("Symmetry:",JLabel.TRAILING);
		symmetryLabel.setLabelFor(symmetryTextField);
		symmetryLabel.setToolTipText("Degree of symmetry: "
					     + "transpositional, inversional");

		combinatorialTextField = new JTextField(12);
		combinatorialTextField.setBackground(Color.white);
		combinatorialTextField.setDragEnabled(true);
		JLabel combinatorialLabel 
			= new JLabel("Combinatoriality:", JLabel.TRAILING);
		combinatorialLabel.setLabelFor(combinatorialTextField);
		combinatorialLabel.setToolTipText(
				  "Combinatoriality: P R I RI");


		//Add the widgets to the container.
		propertiesPanel = new JPanel(new SpringLayout());
		propertiesPanel.add(forteNameLabel);
		propertiesPanel.add(forteNameTextField);
		propertiesPanel.add(tnTypeLabel);
		propertiesPanel.add(tnTypeTextField);
		propertiesPanel.add(tnITypeLabel);
		propertiesPanel.add(tnITypeTextField);
		propertiesPanel.add(tnTnITypeLabel);
		propertiesPanel.add(tnTnITypeTextField);
		propertiesPanel.add(iVectorLabel);
		propertiesPanel.add(iVectorTextField);
		propertiesPanel.add(m7m7iLabel);
		propertiesPanel.add(m7m7iTextField);
		propertiesPanel.add(symmetryLabel);
		propertiesPanel.add(symmetryTextField);
		propertiesPanel.add(combinatorialLabel);
		propertiesPanel.add(combinatorialTextField);
		propertiesPanel.setBorder(
			BorderFactory.createCompoundBorder(
                                BorderFactory.createTitledBorder(
					""),
                                BorderFactory.createEmptyBorder(5,5,5,5)));


		//Lay out the panel.
		SpringUtilities.makeCompactGrid(propertiesPanel,
						8, 2, //rows, cols
						1, 1,        //initX, initY
						1, 1);       //xPad, yPad
	}



	public String[] getFTIM(String typedin, String source) {

		// Convert from a typed-in string to a char array.
		char[] typedInCharArray = new char[typedin.length()];
		typedin.getChars(0, typedin.length(), typedInCharArray, 0);

		TnTnIType typedIn = new TnTnIType(typedInCharArray);

		Set prime = new Set();

		String tntypeString;
		String tnitypeString;
		String tntnitypeString; 
		String ivectorString;
		String m7m7iString;
		String symmetryString;
		String combinatorialString;

		if (source == "PC-Set") {
			typedIn.charSetToSet(); // Convert char[] into int[].
			typedIn.getTnTnIType();
			prime = typedIn.getPrimeByTnTnI();
			tntypeString = typedIn.tntypeToString();
			tnitypeString = typedIn.tnitypeToString();
			tntnitypeString = typedIn.tntnitypeToString();
		} else if (source == "PC-Set (note names)") {
			// Convert a set of letter names into an int array.
			typedIn.letterSetToSet();
			typedIn.getTnType();
			typedIn.getTnIType();
			typedIn.getTnTnIType();
			prime = typedIn.getPrimeByTnTnI();
			tntypeString = typedIn.tntypeToString();
			tnitypeString = typedIn.tnitypeToString();
			tntnitypeString = typedIn.tntnitypeToString();
		} else if (source == "Forte Name") {
			prime = typedIn.getPrimeByName(typedin);
			tntypeString = "N/A";
			tnitypeString = "N/A";
			tntnitypeString = prime.tntnitypeToString();
		} else if (source == "I-Vector") {
			typedIn.charSetToSet();
			prime = typedIn.getPrimeByIVector(typedIn.set);
			tntypeString = "N/A";
			tnitypeString = "N/A";
			if (prime.tntnitypeToString() != null)
			    tntnitypeString = prime.tntnitypeToString();
			else
			    tntnitypeString = "N/A";
		} else {
			typedInCharArray = new char[0];
			tntypeString = "null";
			tnitypeString = "null";
			tntnitypeString = "null";
		}

		// I-vector
		ivectorString = prime.ivectorToString();

		// M7/M7I
		if (prime.set.length <= 6)
			m7m7iString = prime.m7setToString();
		else
			m7m7iString = "N/A";

		// Symmetry
		symmetryString = prime.symmetry[0] + ", " + prime.symmetry[1];

		// Combinatoriality
		if (prime.set.length == 6)
			combinatorialString = prime.combinatorialToString();
		else
			combinatorialString = "N/A";


		String[] properties = {prime.name,
				       tntypeString,
				       tnitypeString,
				       tntnitypeString,
				       ivectorString,
				       m7m7iString,
				       symmetryString,
				       combinatorialString};
		return properties;
	}



    public void actionPerformed(ActionEvent e) {

	if("find".equals(e.getActionCommand())) {
	    
	    String byMeansOf = (String) throughJComboBox.getSelectedItem();
	    String[] p = new String[8];
			
	    // Get the String of the typed-in pc-set.
	    String readPcSetString = throughTextField.getText();

	    // Eliminate spaces
	    StringBuffer withoutSpaceStringBuffer = new StringBuffer();
	    for (int i = 0; i < readPcSetString.length(); i++)
		if (readPcSetString.charAt(i) != ' ')
		    withoutSpaceStringBuffer.append(readPcSetString.charAt(i));

	    String pcSetString;
	    boolean goAhead = false;

	    if (byMeansOf == "PC-Set") {

	    // Eliminate duplicates
		PcSet pq = new PcSet();
		pcSetString = pq.removeDuplicatesInString(
				     withoutSpaceStringBuffer.toString());

		// Check if there are non-numerals in PC-Set.
		StringBuffer illegalChars = new StringBuffer(" ");
		for (int i = 0; i < pcSetString.length(); i++) {
		    if (pcSetString.charAt(i) != '0' &&
			pcSetString.charAt(i) != '1' &&
			pcSetString.charAt(i) != '2' &&
			pcSetString.charAt(i) != '3' &&
			pcSetString.charAt(i) != '4' &&
			pcSetString.charAt(i) != '5' &&
			pcSetString.charAt(i) != '6' &&
			pcSetString.charAt(i) != '7' &&
			pcSetString.charAt(i) != '8' &&
			pcSetString.charAt(i) != '9' &&
			pcSetString.charAt(i) != 'a' &&
			pcSetString.charAt(i) != 'A' &&
			pcSetString.charAt(i) != 'b' &&
			pcSetString.charAt(i) != 'B' &&
			pcSetString.charAt(i) != 't' &&
			pcSetString.charAt(i) != 'T' &&
			pcSetString.charAt(i) != 'e' &&
			pcSetString.charAt(i) != 'E') {
			illegalChars.append("\"" 
					    + pcSetString.charAt(i) 
					    + "\" ");
		    }
		}

		if (illegalChars.length() > 1) {
		    JOptionPane.showMessageDialog(
				propertiesPanel, 
				"Non-numeral(s) typed:"
				+ illegalChars.toString(),
				"Warning",
				JOptionPane.WARNING_MESSAGE);
		} else if(pcSetString.length() 
			  != withoutSpaceStringBuffer.length()) {
		    JOptionPane.showMessageDialog(
				propertiesPanel, 
				"There are duplicates in the pc-set.",
				"Warning",
				JOptionPane.WARNING_MESSAGE);
		} else if(pcSetString.length() < 3) {
		    JOptionPane.showMessageDialog(
						propertiesPanel, 
						"The size of a set must be \n"
						+"at least 3.",
						"Warning",
						JOptionPane.WARNING_MESSAGE);
		} else if(9 < pcSetString.length()) {
		    JOptionPane.showMessageDialog(
						propertiesPanel, 
						"The size of a set must be \n"
						+"equal to or smaller than 9.",
						"Warning",
						JOptionPane.WARNING_MESSAGE);
		} else {
		    // No input errors found
		    p = getFTIM(pcSetString, "PC-Set");
		    goAhead = true;
		}
	    } else if (byMeansOf == "PC-Set (note names)") {

		PcSet pq = new PcSet();
		pcSetString = pq.removeDuplicatesInString(
				     withoutSpaceStringBuffer.toString());

		if(pcSetString.length() < 3) {
		    JOptionPane.showMessageDialog(
						propertiesPanel, 
						"The size of a set must be \n"
						+"at least 3.",
						"Warning",
						JOptionPane.WARNING_MESSAGE);
		} else if(18 < pcSetString.length()) {
		    JOptionPane.showMessageDialog(
						propertiesPanel, 
						"The size of a set must be \n"
						+"equal to or smaller than 9.",
						"Warning",
						JOptionPane.WARNING_MESSAGE);
		} else {
		    p = getFTIM(pcSetString, "PC-Set (note names)");
		    goAhead = true;
		}
	    } else if (byMeansOf == "Forte Name") {

		PcSet pq = new PcSet();
		pcSetString = pq.removeDuplicatesInString(
				     withoutSpaceStringBuffer.toString());

		if(pcSetString.length() < 3 
		   || 5 < pcSetString.length()
		   || pcSetString.charAt(1) != '-') {
		    JOptionPane.showMessageDialog(
						propertiesPanel, 
						"The name is mistyped.",
						"Warning",
						JOptionPane.WARNING_MESSAGE);
		} else {
		    p = getFTIM(pcSetString, "Forte Name");
		    goAhead = true;
		}
	    } else if (byMeansOf == "I-Vector") {

		StringBuffer illegalChars = new StringBuffer(" ");
		for (int i = 0; i < withoutSpaceStringBuffer.length(); i++) {
		    if (withoutSpaceStringBuffer.charAt(i) != '0' &&
			withoutSpaceStringBuffer.charAt(i) != '1' &&
			withoutSpaceStringBuffer.charAt(i) != '2' &&
			withoutSpaceStringBuffer.charAt(i) != '3' &&
			withoutSpaceStringBuffer.charAt(i) != '4' &&
			withoutSpaceStringBuffer.charAt(i) != '5' &&
			withoutSpaceStringBuffer.charAt(i) != '6' &&
			withoutSpaceStringBuffer.charAt(i) != '7' &&
			withoutSpaceStringBuffer.charAt(i) != '8' &&
			withoutSpaceStringBuffer.charAt(i) != '9' &&
			withoutSpaceStringBuffer.charAt(i) != 'a' &&
			withoutSpaceStringBuffer.charAt(i) != 'A' &&
			withoutSpaceStringBuffer.charAt(i) != 'b' &&
			withoutSpaceStringBuffer.charAt(i) != 'B' &&
			withoutSpaceStringBuffer.charAt(i) != 't' &&
			withoutSpaceStringBuffer.charAt(i) != 'T' &&
			withoutSpaceStringBuffer.charAt(i) != 'e' &&
			withoutSpaceStringBuffer.charAt(i) != 'E') {
			illegalChars.append(
				    "\"" 
				    + withoutSpaceStringBuffer.charAt(i) 
				    + "\" ");
		    }
		}

		if (illegalChars.length() > 1) {
		    JOptionPane.showMessageDialog(
				propertiesPanel, 
				"Non-numerical characters typed:"
				+ illegalChars.toString(),
				"Warning",
				JOptionPane.WARNING_MESSAGE);
		} else if(withoutSpaceStringBuffer.length() != 6) {

		    System.out.println("withoutSpaceStringBuffer = " 
				       + withoutSpaceStringBuffer);
		    System.out.println("withoutSpaceStringBuffer.length() = " 
				       + withoutSpaceStringBuffer.length());

		    JOptionPane.showMessageDialog(
					propertiesPanel, 
					"The i-vector is mistyped",
					"Warning",
					JOptionPane.WARNING_MESSAGE);
		} else {
		    p = getFTIM(withoutSpaceStringBuffer.toString(), 
				"I-Vector");
		    goAhead = true;
		}
	    } else
		p[0] = "Error";

	    if (goAhead == true) {
		forteNameTextField.setText("  " + p[0]);
		tnTypeTextField.setText("  " + p[1]);
		tnITypeTextField.setText("  " + p[2]);
		tnTnITypeTextField.setText("  " + p[3]);
		iVectorTextField.setText("  " + p[4]);
		m7m7iTextField.setText("  " + p[5]);
		symmetryTextField.setText("  " + p[6]);
		combinatorialTextField.setText("  " + p[7]);
	    }
	} 

	if("clear".equals(e.getActionCommand())) {
	    throughTextField.setText("");
	    forteNameTextField.setText("");
	    tnTypeTextField.setText("");
	    tnITypeTextField.setText("");
	    tnTnITypeTextField.setText("");
	    iVectorTextField.setText("");
	    m7m7iTextField.setText("");
	    symmetryTextField.setText("");
	    combinatorialTextField.setText("");
	}
    }
}
