// UnorderedSetRelations.java, used for
// JDubiel.java: Computer program for pc-set analysis
// by Akira Takaoka, akira.takaoka@columbia.edu
// Copyright (C) 1994-2017 Akira Takaoka
// All rights reserved.





// package jdubielApplication;

import java.io.IOException;
import java.util.Locale;

import java.lang.reflect.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;


// To find if an unordered pc-set is a subset of another.
public class UnorderedSetRelations extends JFrame 
                                   implements ActionListener {

	static final int xOffset = 60, yOffset = 150;
	static int openFrameCount = 0;

	JPanel relationsPanel;
	JPanel buttonPanel;
	JPanel checkBoxesPanel;

        JEditorPane editorPane;
        JScrollPane editorScrollPane;

	JTextField pcSet1TextField;
	JTextField pcSet2TextField;

	JCheckBox tnCheckBox;
	JCheckBox tnICheckBox;
	JCheckBox mnCheckBox;

        JCheckBox subSetsCheckBox;
        JCheckBox superSetsCheckBox;

        JSpinner subsetNSpinner;
        JSpinner subsetKSpinner;
        SpinnerNumberModel subsetNModel;
        SpinnerNumberModel subsetKModel;

        JSpinner supersetNSpinner;
        JSpinner supersetKSpinner;
        SpinnerNumberModel supersetNModel;
        SpinnerNumberModel supersetKModel;

        StringBuffer results;


	public UnorderedSetRelations() {

		createWidgets();

		//Put everything together.
		JPanel devidedPane = new JPanel(new BorderLayout());
		devidedPane.setOpaque(true);
		devidedPane.add(relationsPanel, 
				BorderLayout.NORTH);
		devidedPane.add(buttonPanel,
			     BorderLayout.WEST);
		devidedPane.add(checkBoxesPanel,
			     BorderLayout.CENTER);
		devidedPane.add(editorScrollPane,
			     BorderLayout.SOUTH);

		getContentPane().add(devidedPane, BorderLayout.CENTER);

		setSize(445,480);
		openFrameCount++;
		setTitle("Unordered Set Relations " + openFrameCount);
		setLocation(xOffset + 10 * openFrameCount, 
			    yOffset + 10 * openFrameCount);
	}
	


	private void createWidgets() {
		
		//Lay out the pc-set labels on a panel.
		JLabel pcSet1Label 
			= new JLabel("PC-Set 1:", SwingConstants.RIGHT);
		pcSet1TextField = new JTextField(11);
		pcSet1TextField.setBackground(Color.white);
		pcSet1TextField.setToolTipText("Type in a pc-set in a "
						+ "numerical form.");
		pcSet1TextField.setDragEnabled(true);
		pcSet1Label.setLabelFor(pcSet1TextField);

		JLabel pcSet2Label 
			= new JLabel("PC-Set 2:", SwingConstants.RIGHT);
		pcSet2TextField = new JTextField(11);
		pcSet2TextField.setBackground(Color.white);
		pcSet2TextField.setToolTipText("Type in a pc-set in a "
						+ "numerical form.");
		pcSet2TextField.setDragEnabled(true);
		pcSet2Label.setLabelFor(pcSet2TextField);

		relationsPanel = new JPanel(new GridLayout(1, 4));
		relationsPanel.add(pcSet1Label);
		relationsPanel.add(pcSet1TextField);
		relationsPanel.add(pcSet2Label);
		relationsPanel.add(pcSet2TextField);
		relationsPanel.setBorder(
			BorderFactory.createCompoundBorder(
                                BorderFactory.createTitledBorder(
					""),
                                BorderFactory.createEmptyBorder(5,5,5,5)));



		// Find button
		JButton findButton = new JButton("Find");
		findButton.setActionCommand("find");
		findButton.addActionListener(this);
		findButton.setBackground(Color.lightGray);
		findButton.setFont(new Font("SansSerif", Font.BOLD, 12));

		// Clear button
		JButton clearButton = new JButton("Clear");
		clearButton.setActionCommand("clear");
		clearButton.addActionListener(this);
		clearButton.setBackground(Color.lightGray);

		JPanel buttonPanel1 = new JPanel(new SpringLayout());
		buttonPanel1.add(findButton);
		buttonPanel1.add(clearButton);
		SpringUtilities.makeCompactGrid(buttonPanel1,
						2, 1,  //rows, cols
						1, 1,  //initX, initY
						1, 1); //xPad, yPad

		buttonPanel = new JPanel(new BorderLayout());
		buttonPanel.add(buttonPanel1, BorderLayout.NORTH);


		// Check boxes on a panel.
		tnCheckBox = new JCheckBox("T    ");
		tnCheckBox.setToolTipText("Transposition");

		tnICheckBox = new JCheckBox("TI    ");
		tnICheckBox.setToolTipText("Inversion");

		mnCheckBox = new JCheckBox("M    ");
		mnCheckBox.setToolTipText("Multiplication");

//		checkBoxesPanel1 = new JPanel(new GridLayout(1, 3));
		JPanel checkBoxesPanel1 = new JPanel();
		checkBoxesPanel1.add(tnCheckBox);
		checkBoxesPanel1.add(tnICheckBox);
		checkBoxesPanel1.add(mnCheckBox);
//		checkBoxesPanel1.add(tnCheckBox, BorderLayout.WEST);
//		checkBoxesPanel1.add(tnICheckBox, BorderLayout.CENTER);
//		checkBoxesPanel1.add(mnCheckBox, BorderLayout.EAST);
		checkBoxesPanel1.setBorder(
			BorderFactory.createCompoundBorder(
                                BorderFactory.createTitledBorder(
					"their relations under: "),
                                BorderFactory.createEmptyBorder(0,0,0,0)));



		subSetsCheckBox 
		    = new JCheckBox("shared subsets of sizes k to n: "); 
		subSetsCheckBox.setToolTipText(
			"Shows the subsets shared by pc-set 1 and pc-set 2.");
		subsetNModel = new SpinnerNumberModel(
			3, //initial value
			3, //min
			9, //max
			1); //step
		subsetNSpinner = new JSpinner(subsetNModel);
		subsetNSpinner.setEditor(
				 new JSpinner.NumberEditor(subsetNSpinner));
		Dimension d2 = new Dimension();
		d2.setSize(0.0, 0.0);
		subsetNSpinner.setSize(d2);

		subsetKModel = new SpinnerNumberModel(
			3, //initial value
			3, //min
			9, //max
			1); //step
		subsetKSpinner = new JSpinner(subsetKModel);
		subsetKSpinner.setEditor(
				 new JSpinner.NumberEditor(subsetKSpinner));
		subsetKSpinner.setSize(d2);





		superSetsCheckBox 
		    = new JCheckBox("shared supersets of sizes k to n: "); 
		superSetsCheckBox.setToolTipText(
		       "Shows the supersets shared by pc-set 1 and pc-set 2.");
		supersetNModel = new SpinnerNumberModel(
			8, //initial value
			4, //min
			11, //max
			1); //step
		supersetNSpinner = new JSpinner(supersetNModel);
		supersetNSpinner.setEditor(
			   new JSpinner.NumberEditor(supersetNSpinner));
		supersetNSpinner.setSize(d2);

		supersetKModel = new SpinnerNumberModel(
			8, //initial value
			4, //min
			11, //max
			1); //step
		supersetKSpinner = new JSpinner(supersetKModel);
		supersetKSpinner.setEditor(
			   new JSpinner.NumberEditor(supersetKSpinner));
		supersetKSpinner.setSize(d2);





		//Add the widgets to the container.
		JPanel checkBoxesPanel2 = new JPanel(new SpringLayout());
		JLabel blankLabel1 = new JLabel(" ", JLabel.TRAILING);
		JLabel n = new JLabel("k ", JLabel.CENTER);
		JLabel k = new JLabel("n ", JLabel.CENTER);

        	checkBoxesPanel2.add(blankLabel1);
        	checkBoxesPanel2.add(n);
        	checkBoxesPanel2.add(k);

        	checkBoxesPanel2.add(subSetsCheckBox);
		checkBoxesPanel2.add(subsetNSpinner);
		checkBoxesPanel2.add(subsetKSpinner);

        	checkBoxesPanel2.add(superSetsCheckBox);
		checkBoxesPanel2.add(supersetNSpinner);
		checkBoxesPanel2.add(supersetKSpinner);

		//Lay out listsPanel2
		SpringUtilities.makeCompactGrid(checkBoxesPanel2,
						3, 3,  //rows, cols
						1, 1,  //initX, initY
						1, 1); //xPad, yPad





		checkBoxesPanel = new JPanel(new SpringLayout());
		checkBoxesPanel.add(checkBoxesPanel1);
		checkBoxesPanel.add(checkBoxesPanel2);
		SpringUtilities.makeCompactGrid(checkBoxesPanel,
						2, 1,  //rows, cols
						1, 1,  //initX, initY
						1, 1); //xPad, yPad

		checkBoxesPanel.setBorder(
			BorderFactory.createCompoundBorder(
                                BorderFactory.createTitledBorder(
					""),
                                BorderFactory.createEmptyBorder(5,5,5,5)));




		// "Found Relations" area
		editorPane = new JEditorPane();
		editorScrollPane = new JScrollPane(editorPane);
		editorScrollPane.setVerticalScrollBarPolicy(
                        JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		editorScrollPane.setHorizontalScrollBarPolicy(
			JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		editorScrollPane.setPreferredSize(new Dimension(250, 240));
		editorScrollPane.setMinimumSize(new Dimension(250, 240));
		editorScrollPane.setBorder(
			BorderFactory.createCompoundBorder(
				BorderFactory.createTitledBorder(
					"Found: "),
				BorderFactory.createEmptyBorder(5,5,5,5)));
		editorScrollPane.getBorder();
	}









    public void actionPerformed(ActionEvent e) {

	if("find".equals(e.getActionCommand())) {
	    editorPane.setText("");		

	    results = new StringBuffer();
			
	    // Get typed-in pcSet1 and pcSet2.
	    String readPcSetString1 = pcSet1TextField.getText();
	    String readPcSetString2 = pcSet2TextField.getText();

	    // Remove spaces
	    StringBuffer withoutSpaceStringBuffer1 = new StringBuffer();
	    for (int i = 0; i < readPcSetString1.length(); i++)
		if (readPcSetString1.charAt(i) != ' ')
		    withoutSpaceStringBuffer1.append(
					     readPcSetString1.charAt(i));

	    StringBuffer withoutSpaceStringBuffer2 = new StringBuffer();
	    for (int i = 0; i < readPcSetString2.length(); i++)
		if (readPcSetString2.charAt(i) != ' ')
		    withoutSpaceStringBuffer2.append(
					     readPcSetString2.charAt(i));

	    String withoutSpaceString1 = withoutSpaceStringBuffer1.toString();
	    String withoutSpaceString2 = withoutSpaceStringBuffer2.toString();

	    // Remove duplicates
	    PcSet pq = new PcSet();
	    String pcSet1String 
		= pq.removeDuplicatesInString(withoutSpaceString1);
	    String pcSet2String 
		= pq.removeDuplicatesInString(withoutSpaceString2);

	    // Check if there are non-numerals in PC-Set 1.
	    String illegalChars1 = pq.checkNonNumerals(pcSet1String);

	    // Check if there are non-numerals in PC-Set 2.
	    String illegalChars2 = pq.checkNonNumerals(pcSet2String);

	    if (illegalChars1.length() > 1) {
		JOptionPane.showMessageDialog(
			relationsPanel, 
			"Non-numeral(s) typed in PC-Set 1:"
			+ illegalChars1.toString(),
			"Warning",
			JOptionPane.WARNING_MESSAGE);
	    } else if (illegalChars2.length() > 1) {
		JOptionPane.showMessageDialog(
			relationsPanel, 
			"Non-numeral(s) typed in PC-Set 2:"
			+ illegalChars2.toString(),
			"Warning",
			JOptionPane.WARNING_MESSAGE);
	    } else if(pcSet1String.length() != withoutSpaceString1.length()) {
		JOptionPane.showMessageDialog(
			relationsPanel, 
			"There are duplicates in PC-Set1.",
			"Warning",
			JOptionPane.WARNING_MESSAGE);
	    } else if(pcSet2String.length() != withoutSpaceString2.length()) {
		JOptionPane.showMessageDialog(
			relationsPanel, 
			"There are duplicates in PC-Set2.",
			"Warning",
			JOptionPane.WARNING_MESSAGE);
	    } else if(pcSet1String.length() < 3 
	       || pcSet2String.length() < 3) {
		JOptionPane.showMessageDialog(
			relationsPanel, 
			"The size of a set must be at least 3.",
			"Warning",
			JOptionPane.WARNING_MESSAGE);
	    } else if(11 < pcSet1String.length()
		      || 11 < pcSet2String.length()) {
		JOptionPane.showMessageDialog(
			relationsPanel, 
			"The size of a set must be "
			+"equal to or smaller than 11.",
			"Warning",
			JOptionPane.WARNING_MESSAGE);
	    } else {
		// No input errors found

		char[] pcSet1Chars = new char[pcSet1String.length()];
		pcSet1String.getChars(
				0, 
				pcSet1String.length(), 
				pcSet1Chars, 
				0);
		Set pcSet1 = new Set(pcSet1Chars.length);
		pcSet1.set = pcSet1.charArrayToIntOne(pcSet1Chars);

		char[] pcSet2Chars = new char[pcSet2String.length()];
		pcSet2String.getChars(
				0, 
				pcSet2String.length(), 
				pcSet2Chars, 
				0);
		Set pcSet2 = new Set(pcSet2Chars.length);
		pcSet2.set = pcSet2.charArrayToIntOne(pcSet2Chars);



		if (!pcSet1.base12String.equals("NotUsed")) {
		    pcSet2.base12String = pcSet1.base12String;
		} else if (!pcSet2.base12String.equals("NotUsed")
			   && pcSet1.base12String.equals("NotUsed")) {
		    pcSet1.base12String = pcSet2.base12String;
		} else {
		    pcSet1.base12String = "AB";
		    pcSet2.base12String = "AB";
		}



		UnorderedInclusion uoi 
		    = new UnorderedInclusion(pcSet1, pcSet2);

		if (tnCheckBox.getSelectedObjects() == null &&
		    tnICheckBox.getSelectedObjects() == null &&
		    mnCheckBox.getSelectedObjects() == null &&
		    subSetsCheckBox.getSelectedObjects() == null &&
		    superSetsCheckBox.getSelectedObjects() == null) {
		    JOptionPane.showMessageDialog(
					relationsPanel, 
					"You need to choose at least "
					+ "one of the operations.\n",
					"Warning",
					JOptionPane.WARNING_MESSAGE);
		} else {
		    if (tnCheckBox.getSelectedObjects() != null)
			results.append(uoi.listFitTns());

		    if (tnICheckBox.getSelectedObjects() != null)
			results.append(uoi.listFitTnIs());

		    if (mnCheckBox.getSelectedObjects() != null)
			results.append(uoi.listFitMns());

		    if (subSetsCheckBox.getSelectedObjects() != null) {
			int k = subsetKModel.getNumber().intValue();
			int n = subsetNModel.getNumber().intValue();
			if (n < k) {
			    int tmp = k;
			    k = n;
			    n = tmp;
			} // So always k <= n

			if (pcSet1.set.length < n || pcSet2.set.length < n) {
			    JOptionPane.showMessageDialog(
				checkBoxesPanel,
				"The size of subsets must be smaller "
				+ "than or equal to those of PC-Set 1 and 2",
					"Warning",
					JOptionPane.WARNING_MESSAGE);
			} else {
			    UnorderedInclusion subsetuoi 
			      = new UnorderedInclusion(pcSet1, pcSet2);
			    results.append(subsetuoi.sharedSubSets(k, n));
			    results.append("\n");
			}
		    }

		    if (superSetsCheckBox.getSelectedObjects() != null) {
			int k = supersetKModel.getNumber().intValue();
			int n = supersetNModel.getNumber().intValue();
			if (n < k) {
			    int tmp = k;
			    k = n;
			    n = tmp;
			} // So always k <= n

			if (pcSet1.set.length > k || pcSet2.set.length > k) {
			    JOptionPane.showMessageDialog(
				checkBoxesPanel,
				"The size of supersets must be greater "
				+ "than or equal to those of PC-Set 1 and 2",
				"Warning",
				JOptionPane.WARNING_MESSAGE);
			} else {
			    UnorderedInclusion supersetuoi 
			      = new UnorderedInclusion(pcSet1, pcSet2);
			    results.append(supersetuoi.sharedSuperSets(k, n));
			    results.append("\n");
			}
		    }
		}
				
		editorPane.setText(results.toString());
	    }
	}

	if ("clear".equals(e.getActionCommand())) {
	    pcSet1TextField.setText("");
	    pcSet2TextField.setText("");
	    editorPane.setText("");		
	}
    }
}


